@@ -3,6 +3,7 @@
    Copyright (C) 2003 Christoph Cullmann <cullmann@kde.org>
    Copyright (C) 2001 Joseph Wenninger <jowenn@kde.org>
    Copyright (C) 2006 Dominik Haumann <dhdev@gmx.de>
+   Copyright (C) 2007 Mirko Stocker <me@misto.ch>
 
    Based on work of:
      Copyright (C) 1999 Jochen Wilhelmy <digisnap@cs.tu-berlin.de>
@@ -176,6 +177,11 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   setLayout(layout);
 }
 
+KateIndentConfigTab::~KateIndentConfigTab()
+{
+  delete ui;
+}
+
 void KateIndentConfigTab::showWhatsThis(const QString& text)
 {
   QWhatsThis::showText(QCursor::pos(), text);
@@ -272,6 +278,11 @@ KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
   setLayout(layout);
 }
 
+KateSelectConfigTab::~KateSelectConfigTab()
+{
+  delete ui;
+}
+
 void KateSelectConfigTab::apply ()
 {
   // nothing changed, no need to apply stuff
@@ -375,6 +386,11 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
   setLayout(layout);
 }
 
+KateEditConfigTab::~KateEditConfigTab()
+{
+  delete ui;
+}
+
 void KateEditConfigTab::apply ()
 {
   // try to update the rest of tabs
@@ -471,6 +487,7 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
 
 KateViewDefaultsConfig::~KateViewDefaultsConfig()
 {
+  delete ui;
 }
 
 void KateViewDefaultsConfig::apply ()
@@ -519,29 +536,30 @@ void KateViewDefaultsConfig::defaults (){;}
 
 //BEGIN KateEditKeyConfiguration
 
-KateEditKeyConfiguration::KateEditKeyConfiguration( QWidget* parent, KateDocument* doc )
+KateEditKeyConfiguration::KateEditKeyConfiguration( QWidget* parent)
   : KateConfigPage( parent )
+  , m_ready(false)
 {
-  m_doc = doc;
-  m_ready = false;
+  m_doc = new KateDocument();
+  m_view = new KateView(m_doc, 0);
+}
+
+KateEditKeyConfiguration::~KateEditKeyConfiguration()
+{
+  delete m_view;
+  delete m_doc;
 }
 
 void KateEditKeyConfiguration::showEvent ( QShowEvent * )
 {
-#ifdef __GNUC__
-#warning fixme, to work without a document object, perhaps create some own internally
-#endif
-  return ;
-
   if (!m_ready)
   {
-    QVBoxLayout *l=new QVBoxLayout(this);
-    KateView* view = (KateView*)m_doc->views().at(0);
-    m_ac = view->editActionCollection();
-    l->addWidget(m_shortcutsEditor = new KShortcutsEditor( m_ac, this, false ));
-    //is this really needed? if yes, I'll add it to KShortcutsEditor
-    //note that changes will immediately become active with KShortcutsEditor -- ahartmetz
-    //connect( m_shortcutsEditor, SIGNAL( keyChange() ), this, SLOT( slotChanged() ) );
+	m_actionCollection = m_view->editActionCollection();
+
+    QVBoxLayout *layout = new QVBoxLayout(this);
+    layout->addWidget(m_shortcutsEditor = new KShortcutsEditor( m_actionCollection, this, false ));
+    connect( m_shortcutsEditor, SIGNAL( keyChange() ), this, SLOT( slotChanged() ) );
+
     m_shortcutsEditor->show ();
 
     m_ready = true;
@@ -552,22 +570,15 @@ void KateEditKeyConfiguration::showEvent ( QShowEvent * )
 
 void KateEditKeyConfiguration::apply()
 {
-#ifdef __GNUC__
-#warning fixme, to work without a document object, perhaps create some own internally
-#endif
-  return ;
-
-  if ( ! hasChanged() )
+  if(!hasChanged())
     return;
   m_changed = false;
 
   if (m_ready)
   {
-#ifdef __GNUC__
-#warning: semantics of KKeyDialog changed from change/commit to change in-place/revert
-#endif
-    //m_keyChooser->commitChanges();
-    m_ac->writeSettings();
+    KateViewConfig::global()->configStart ();
+    m_actionCollection->writeSettings();
+    KateViewConfig::global()->configEnd ();
   }
 }
 //END KateEditKeyConfiguration
@@ -621,6 +632,11 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
   setLayout(layout);
 }
 
+KateSaveConfigTab::~KateSaveConfigTab()
+{
+  delete ui;
+}
+
 void KateSaveConfigTab::apply()
 {
   modeConfigPage->apply();
@@ -898,11 +914,12 @@ KateHlDownloadDialog::KateHlDownloadDialog(QWidget *parent, const char *name, bo
   list->setColumnCount(4);
   list->setHeaderLabels(QStringList() << "" << i18n("Name") << i18n("Installed") << i18n("Latest"));
   list->setSelectionMode(QAbstractItemView::MultiSelection);
-  // KDE4 replacement?
-  //list->setAllColumnsShowFocus(true);
+  list->setAllColumnsShowFocus(true);
+  list->setRootIsDecorated(false);
+  list->setColumnWidth(0, 22);
 
   new QLabel(i18n("<b>Note:</b> New versions are selected automatically."), vbox);
-  setButtonIcon(User1, KIcon("ok"));
+  setButtonIcon(User1, KIcon("dialog-ok"));
 
   transferJob = KIO::get(
     KUrl(QString(HLDOWNLOADPATH)
@@ -979,6 +996,7 @@ void KateHlDownloadDialog::listDataReceived(KIO::Job *, const QByteArray &data)
           entry->setIcon(0, SmallIcon(("get-hot-new-stuff")));
         }
       }
+      list->resizeColumnToContents(1);
     }
   }
 }
@@ -1129,6 +1147,7 @@ KateModOnHdPrompt::~KateModOnHdPrompt()
     delete m_diffFile;
     m_diffFile = 0;
   }
+  delete ui;
 }
 
 void KateModOnHdPrompt::slotDiff()
