@@ -81,7 +81,7 @@ QString KateSchemaManager::printingSchema ()
 }
 
 KateSchemaManager::KateSchemaManager ()
-    : m_config ("kateschemarc", KConfig::CascadeConfig)
+    : m_config ("kateschemarc", KConfig::NoGlobals)
 {
   update ();
 }
@@ -506,7 +506,7 @@ void KateSchemaConfigFontColorTab::schemaChanged (uint schema)
   p.setColor( QPalette::Text, _c );
   m_defaultStyles->viewport()->setPalette( p );
 
-  for ( int i = 0; i < KateHlManager::self()->defaultStyles(); i++ )
+  for ( uint i = 0; i < KateHlManager::self()->defaultStyles(); i++ )
   {
     m_defaultStyles->addItem( KateHlManager::self()->defaultStyleName(i, true), l->at( i ) );
   }
