@@ -1,6 +1,7 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2007 Sebastian Pipping <webmaster@hartwork.org>
    Copyright (C) 2007 Matthew Woehlke <mw_triad@users.sourceforge.net>
+   Copyright (C) 2007 Thomas Friedrichsmeier <thomas.friedrichsmeier@ruhr-uni-bochum.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -25,6 +26,8 @@
 #include "ui_searchbarincremental.h"
 #include "ui_searchbarpower.h"
 
+#include <kactioncollection.h>
+
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QComboBox>
 #include <QtGui/QCheckBox>
@@ -140,7 +143,7 @@ void KateSearchBar::findPrevious() {
 
 void KateSearchBar::highlight(const Range & range, const QColor & color) {
     SmartRange * const highlight = m_view->doc()->newSmartRange(range, m_topRange);
-    highlight->setInsertBehavior(SmartRange::ExpandRight);
+    highlight->setInsertBehavior(SmartRange::DoNotExpand);
     Attribute::Ptr attribute(new Attribute());
     attribute->setBackground(color);
     highlight->setAttribute(attribute);
@@ -161,10 +164,9 @@ void KateSearchBar::highlightReplacement(const Range & range) {
 
 
 void KateSearchBar::highlightAllMatches(const QString & pattern,
-        Search::SearchOptions searchOptions,
-        bool repaintHappensElsewhere) {
+        Search::SearchOptions searchOptions) {
     onForAll(pattern, m_view->doc()->documentRange(),
-            searchOptions, NULL, repaintHappensElsewhere);
+            searchOptions, NULL);
 }
 
 
@@ -361,7 +363,6 @@ void KateSearchBar::onIncPatternChanged(const QString & pattern, bool invokedByU
 
             // Kill highlight
             resetHighlights();
-            updateHighlights();
         }
 
         // Reset edit color
@@ -427,8 +428,7 @@ void KateSearchBar::onIncPatternChanged(const QString & pattern, bool invokedByU
 
         // Highlight all
         if (found && isChecked(m_incMenuHighlightAll)) {
-            const bool REPAINT_NOT_DONE_HERE = false;
-            highlightAllMatches(pattern, enabledOptions, REPAINT_NOT_DONE_HERE);
+            highlightAllMatches(pattern, enabledOptions);
         }
     }
 }
@@ -478,13 +478,11 @@ void KateSearchBar::onIncHighlightAllToggle(bool checked, bool invokedByUserActi
 
                 // Highlight them all
                 resetHighlights();
-                const bool REPAINT_DONE_HERE = true;
-                highlightAllMatches(pattern, enabledOptions, REPAINT_DONE_HERE);
+                highlightAllMatches(pattern, enabledOptions);
             }
         } else {
             resetHighlights();
         }
-        updateHighlights();
     }
 }
 
@@ -673,7 +671,7 @@ bool KateSearchBar::onStep(bool replace, bool forwards) {
                 // Selection is match -> replace
                 const QString replacement = m_powerUi->replacement->currentText();
                 afterReplace = m_view->doc()->newSmartRange(match);
-                afterReplace->setInsertBehavior(SmartRange::ExpandRight);
+                afterReplace->setInsertBehavior(SmartRange::ExpandRight | SmartRange::ExpandLeft);
                 replaceMatch(resultRanges, replacement);
 
                 // Find, second try after replaced text
@@ -743,8 +741,7 @@ bool KateSearchBar::onStep(bool replace, bool forwards) {
     if ((found && highlightAll) || (afterReplace != NULL)) {
         // Highlight all matches
         if (found && highlightAll) {
-            const bool REPAINT_DONE_HERE = true;
-            highlightAllMatches(pattern, enabledOptions, REPAINT_DONE_HERE);
+            highlightAllMatches(pattern, enabledOptions);
         }
 
         // Highlight replacement (on top if overlapping) if new match selected
@@ -757,7 +754,6 @@ bool KateSearchBar::onStep(bool replace, bool forwards) {
             highlightReplacement(*afterReplace);
         }
 
-        updateHighlights();
     }
 
     delete afterReplace;
@@ -930,8 +926,7 @@ void KateSearchBar::onPowerReplaceNext() {
 // replacement != NULL --> Replace and highlight all matches
 void KateSearchBar::onForAll(const QString & pattern, Range inputRange,
         Search::SearchOptions enabledOptions,
-        const QString * replacement,
-        bool repaintHappensElsewhere) {
+        const QString * replacement) {
     bool multiLinePattern = false;
     const bool regexMode = enabledOptions.testFlag(Search::Regex);
     if (regexMode) {
@@ -947,33 +942,25 @@ void KateSearchBar::onForAll(const QString & pattern, Range inputRange,
 
     // Before first match
     resetHighlights();
-    if (replacement != NULL) {
-        m_view->doc()->editBegin();
-    }
 
+    SmartRange * const workingRange = m_view->doc()->newSmartRange(inputRange);
     int matchCounter = 0;
     for (;;) {
-        const QVector<Range> resultRanges = m_view->doc()->searchText(inputRange, pattern, enabledOptions);
+        const QVector<Range> resultRanges = m_view->doc()->searchText(*workingRange, pattern, enabledOptions);
         Range match = resultRanges[0];
         if (!match.isValid()) {
-            // After last match
-            if (matchCounter > 0) {
-                if (replacement != NULL) {
-                    m_view->doc()->editEnd();
-                }
-
-                if (!repaintHappensElsewhere) {
-                    updateHighlights();
-                }
-            }
             break;
         }
 
         // Work with the match
         if (replacement != NULL) {
+            if (matchCounter == 0) {
+                m_view->doc()->editBegin();
+            }
+
             // Track replacement operation
             SmartRange * const afterReplace = m_view->doc()->newSmartRange(match);
-            afterReplace->setInsertBehavior(SmartRange::ExpandRight);
+            afterReplace->setInsertBehavior(SmartRange::ExpandRight | SmartRange::ExpandLeft);
 
             // Replace
             replaceMatch(resultRanges, *replacement, ++matchCounter);
@@ -988,40 +975,31 @@ void KateSearchBar::onForAll(const QString & pattern, Range inputRange,
         }
 
         // Continue after match
-        if (match.start() == match.end()) {
+        SmartCursor & workingStart = workingRange->smartStart();
+        workingStart.setPosition(match.end());
+        if (workingStart == match.start()) {
             // Can happen for regex patterns like "^".
             // If we don't advance here we will loop forever...
-            const int line = match.end().line();
-            const int col = match.end().column();
-            const int maxColWithNewline = m_view->doc()->lineLength(line);
-            if (col < maxColWithNewline) {
-                // Advance on same line
-                inputRange.setRange(Cursor(line, col + 1), inputRange.end());
-            } else {
-                // Advance to next line
-                const int maxLine = m_view->doc()->lines() - 1;
-                if (line < maxLine) {
-                    // Next line
-                    inputRange.setRange(Cursor(line + 1, 0), inputRange.end());
-                } else {
-                    // Already at last line
-                    break;
-                }
-            }
-        } else {
-            inputRange.setRange(match.end(), inputRange.end());
+            workingStart.advance(1);
+        } else if (regexMode && !multiLinePattern && workingStart.atEndOfLine()) {
+            // single-line regexps might match the naked line end
+            // therefore we better advance to the next line
+            workingStart.advance(1);
         }
 
-        // Single-line pattern workaround
-        if (regexMode && !multiLinePattern) {
-            const bool FORWARDS = true;
-            fixForSingleLine(inputRange, FORWARDS);
-        }
-
-        if (!inputRange.isValid()) {
+        if (!workingRange->isValid() || workingStart.atEndOfDocument()) {
             break;
         }
     }
+
+    // After last match
+    if (matchCounter > 0) {
+        if (replacement != NULL) {
+            m_view->doc()->editEnd();
+        }
+    }
+
+    delete workingRange;
 }
 
 
@@ -1071,8 +1049,7 @@ void KateSearchBar::onPowerReplaceAll() {
 
 
     // Pass on the hard work
-    const bool REPAINT_NOT_DONE_HERE = false;
-    onForAll(pattern, inputRange, enabledOptions, &replacement, REPAINT_NOT_DONE_HERE);
+    onForAll(pattern, inputRange, enabledOptions, &replacement);
 
 
     // Add to search history
@@ -1354,13 +1331,11 @@ void KateSearchBar::onPowerHighlightAllToggle(int state, bool invokedByUserActio
 
                 // Highlight them all
                 resetHighlights();
-                const bool REPAINT_DONE_HERE = true;
-                highlightAllMatches(pattern, enabledOptions, REPAINT_DONE_HERE);
+                highlightAllMatches(pattern, enabledOptions);
             }
         } else {
             resetHighlights();
         }
-        updateHighlights();
     }
 }
 
@@ -1524,6 +1499,12 @@ void KateSearchBar::onMutatePower() {
         QLineEdit * const replacementLineEdit = m_powerUi->pattern->lineEdit();
         Q_ASSERT(replacementLineEdit != NULL);
         replacementLineEdit->completer()->setCaseSensitivity(Qt::CaseSensitive);
+
+        // Shortcut hack
+        #if QT_VERSION < KDE_MAKE_VERSION(4,4,0) // Remove this once Qt 4.4 becomes necessary
+        foreach (QWidget* widget, m_widget->findChildren<QWidget*>())
+            m_view->actionCollection ()->addAssociatedWidget(widget);
+        #endif
     }
 
     // Guess settings from context
@@ -1591,7 +1572,9 @@ void KateSearchBar::onMutatePower() {
     }
 
     // Focus
-    m_powerUi->pattern->setFocus(Qt::MouseFocusReason);
+    if (m_widget->isVisible()) {
+        m_powerUi->pattern->setFocus(Qt::MouseFocusReason);
+    }
 }
 
 
@@ -1651,6 +1634,12 @@ void KateSearchBar::onMutateIncremental() {
 
         // Focus proxy
         centralWidget()->setFocusProxy(m_incUi->pattern);
+
+        // Shortcut hack
+        #if QT_VERSION < KDE_MAKE_VERSION(4,4,0) // Remove this once Qt 4.4 becomes necessary
+        foreach (QWidget* widget, m_widget->findChildren<QWidget*>())
+            m_view->actionCollection ()->addAssociatedWidget(widget);
+        #endif
     }
 
     // Guess settings from context
@@ -1698,7 +1687,9 @@ void KateSearchBar::onMutateIncremental() {
     }
 
     // Focus
-    m_incUi->pattern->setFocus(Qt::MouseFocusReason);
+    if (m_widget->isVisible()) {
+        m_incUi->pattern->setFocus(Qt::MouseFocusReason);
+    }
 }
 
 
@@ -1748,12 +1739,6 @@ void KateSearchBar::resetHighlights() {
 }
 
 
-void KateSearchBar::updateHighlights() {
-    const bool EVERYTHING = false;
-    m_view->repaintText(EVERYTHING);
-}
-
-
 
 void KateSearchBar::showEvent(QShowEvent * event) {
     // Update init cursor
