@@ -76,6 +76,7 @@
 #include <ktoggleaction.h>
 #include <kselectaction.h>
 #include <kactioncollection.h>
+#include <kdeversion.h>
 
 #include <QtGui/QFont>
 #include <QtCore/QFileInfo>
@@ -101,7 +102,6 @@ static void blockFix(KTextEditor::Range& range)
 
 KateView::KateView( KateDocument *doc, QWidget *parent )
     : KTextEditor::View( parent )
-    , m_destructing(false)
     , m_completionWidget(0)
     , m_editActions (0)
     , m_doc( doc )
@@ -212,17 +212,10 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
 
 KateView::~KateView()
 {
-  if (!m_destructing) {
-    m_destructing=true;
+  if (!m_doc->singleViewMode())
+    KatePartPluginManager::self()->removeView(this);
 
-    if (!m_doc->singleViewMode())
-      KatePartPluginManager::self()->removeView(this);
-
-    m_doc->removeView( this );
-  }
-
-  delete m_selection;
-  m_selection = 0L;
+  m_doc->removeView( this );
 
   foreach (KTextEditor::SmartRange* range, m_externalHighlights)
     removeExternalHighlight(range);
@@ -232,6 +225,10 @@ KateView::~KateView()
 
   delete m_viewInternal;
 
+  // after m_viewInternal to allow KateViewInternal to disconnect from signal signalRangeDeleted
+  delete m_selection;
+  m_selection = 0L;
+
   delete m_renderer;
 
   delete m_config;
@@ -259,7 +256,7 @@ void KateView::setupConnections()
 void KateView::setupActions()
 {
   KActionCollection *ac = this->actionCollection ();
-  QAction *a;
+  KAction *a;
 
   m_toggleWriteLock = 0;
 
@@ -476,6 +473,7 @@ void KateView::setupActions()
   list2.append(i18n("Follow &Line Numbers"));
   list2.append(i18n("&Always On"));
   m_setDynWrapIndicators->setItems(list2);
+  m_setDynWrapIndicators->setEnabled(m_toggleDynWrap->isChecked()); // only synced on real change, later
 
   a = toggleAction = m_toggleFoldingMarkers = new KToggleAction(i18n("Show Folding &Markers"), this);
   ac->addAction("view_folding_markers", a);
@@ -560,6 +558,14 @@ void KateView::setupActions()
 
   slotSelectionChanged ();
 
+  ac->addAssociatedWidget(m_viewInternal);
+  foreach (QAction* action, ac->actions())
+#if QT_VERSION < KDE_MAKE_VERSION(4,4,0)
+    action->setShortcutContext(Qt::WidgetShortcut); // remove after Qt4.4 becomes mandatory
+#else
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+#endif
+
   connect (this, SIGNAL(selectionChanged(KTextEditor::View*)), this, SLOT(slotSelectionChanged()));
 }
 
@@ -572,10 +578,9 @@ void KateView::setupEditActions()
 {
   m_editActions = new KActionCollection( static_cast<QObject*>(this) );
   m_editActions->setObjectName( "edit_actions" );
-  m_editActions->setAssociatedWidget(m_viewInternal);
   KActionCollection* ac = m_editActions;
 
-  QAction* a = ac->addAction("word_left");
+  KAction* a = ac->addAction("word_left");
   a->setText(i18n("Move Word Left"));
   a->setShortcuts(KStandardShortcut::backwardWord());
   connect(a, SIGNAL(triggered(bool)),  SLOT(wordLeft()));
@@ -609,12 +614,12 @@ void KateView::setupEditActions()
 
   a = ac->addAction("beginning_of_line");
   a->setText(i18n("Move to Beginning of Line"));
-  a->setShortcut(QKeySequence(Qt::Key_Home));
+  a->setShortcuts(KStandardShortcut::beginningOfLine());
   connect(a, SIGNAL(triggered(bool)), SLOT(home()));
 
   a = ac->addAction("beginning_of_document");
   a->setText(i18n("Move to Beginning of Document"));
-  a->setShortcuts(KStandardShortcut::home());
+  a->setShortcuts(KStandardShortcut::begin());
   connect(a, SIGNAL(triggered(bool)), SLOT(top()));
 
   a = ac->addAction("select_beginning_of_line");
@@ -630,7 +635,7 @@ void KateView::setupEditActions()
 
   a = ac->addAction("end_of_line");
   a->setText(i18n("Move to End of Line"));
-  a->setShortcut(QKeySequence(Qt::Key_End));
+  a->setShortcuts(KStandardShortcut::endOfLine());
   connect(a, SIGNAL(triggered(bool)), SLOT(end()));
 
   a = ac->addAction("end_of_document");
@@ -802,13 +807,21 @@ void KateView::setupEditActions()
     slotGotFocus();
   else
     slotLostFocus();
+
+  m_editActions->addAssociatedWidget(m_viewInternal);
+  foreach (QAction* action, m_editActions->actions())
+#if QT_VERSION < KDE_MAKE_VERSION(4,4,0)
+    action->setShortcutContext(Qt::WidgetShortcut); // remove after Qt4.4 becomes mandatory
+#else
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+#endif
 }
 
 void KateView::setupCodeFolding()
 {
   KActionCollection *ac=this->actionCollection();
 
-  QAction* a = ac->addAction("folding_toplevel");
+  KAction* a = ac->addAction("folding_toplevel");
   a->setText(i18n("Collapse Toplevel"));
   a->setShortcut(QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_Minus));
   connect(a, SIGNAL(triggered(bool)), m_doc->foldingTree(), SLOT(collapseToplevelNodes()));
@@ -956,11 +969,16 @@ bool KateView::setCursorPositionInternal( const KTextEditor::Cursor& position, u
   QString line_str = m_doc->line( position.line() );
 
   int x = 0;
-  for (int z = 0; z < line_str.length() && z < position.column(); z++) {
+  int z = 0;
+  for (; z < line_str.length() && z < position.column(); z++) {
     if (line_str[z] == QChar('\t')) x += tabwidth - (x % tabwidth); else x++;
   }
 
-  m_viewInternal->updateCursor( position, false, true, calledExternally );
+  if (blockSelectionMode())
+    if (z < position.column())
+      x += position.column() - z;
+
+  m_viewInternal->updateCursor( KTextEditor::Cursor(position.line(), x), false, true, calledExternally );
 
   return true;
 }
@@ -1251,6 +1269,8 @@ void KateView::updateRendererConfig()
 
   m_toggleWWMarker->setChecked( m_renderer->config()->wordWrapMarker()  );
 
+  m_viewInternal->updateBracketMarkAttributes();
+
   // update the text area
   m_viewInternal->updateView (true);
   m_viewInternal->repaint ();
@@ -1769,7 +1789,7 @@ void KateView::lineAsHTML (KateTextLine::Ptr line, int startCol, int length, QTe
 void KateView::exportAsHTML ()
 {
   KUrl url = KFileDialog::getSaveUrl(m_doc->documentName(), "text/html",
-                                     0, i18n("Export File as HTML"));
+                                     this, i18n("Export File as HTML"));
 
   if ( url.isEmpty() )
     return;
@@ -1867,24 +1887,6 @@ bool KateView::wrapCursor ()
 
 //END
 
-//BEGIN IM INPUT STUFF
-void KateView::setIMSelectionValue( const KTextEditor::Range& imRange, const KTextEditor::Range& imSelection, bool imComposeEvent )
-{
-  m_imRange = imRange;
-  m_imSelection = imSelection;
-  m_imComposeEvent = imComposeEvent;
-}
-
-const KTextEditor::Range& KateView::imSelection() const
-{
-  return m_imSelection;
-}
-
-const KTextEditor::Range& KateView::imEdit() const
-{
-  return m_imRange;
-}
-//END IM INPUT STUFF
 
 void KateView::slotTextInserted ( KTextEditor::View *view, const KTextEditor::Cursor &position, const QString &text)
 {
@@ -1904,14 +1906,14 @@ void KateView::addInternalHighlight( KTextEditor::SmartRange * topRange )
 {
   m_internalHighlights.append(topRange);
 
-  addHighlightRange(topRange);
+  m_viewInternal->addHighlightRange(topRange);
 }
 
 void KateView::removeInternalHighlight( KTextEditor::SmartRange * topRange )
 {
   m_internalHighlights.removeAll(topRange);
 
-  removeHighlightRange(topRange);
+  m_viewInternal->removeHighlightRange(topRange);
 }
 
 const QList< KTextEditor::SmartRange * > & KateView::internalHighlights( ) const
@@ -1940,7 +1942,7 @@ void KateView::addExternalHighlight( KTextEditor::SmartRange * topRange, bool su
     emit dynamicHighlightAdded(static_cast<KateSmartRange*>(topRange));
   }
 
-  addHighlightRange(topRange);
+  m_viewInternal->addHighlightRange(topRange);
 }
 
 void KateView::removeExternalHighlight( KTextEditor::SmartRange * topRange )
@@ -1958,7 +1960,7 @@ void KateView::removeExternalHighlight( KTextEditor::SmartRange * topRange )
     emit dynamicHighlightRemoved(static_cast<KateSmartRange*>(topRange));
   }
 
-  removeHighlightRange(topRange);
+  m_viewInternal->removeHighlightRange(topRange);
 }
 
 const QList< KTextEditor::SmartRange * > & KateView::externalHighlights( ) const
@@ -1966,16 +1968,6 @@ const QList< KTextEditor::SmartRange * > & KateView::externalHighlights( ) const
   return m_externalHighlights;
 }
 
-void KateView::addHighlightRange( KTextEditor::SmartRange * range )
-{
-  m_viewInternal->addHighlightRange(range);
-}
-
-void KateView::removeHighlightRange( KTextEditor::SmartRange * range )
-{
-  m_viewInternal->removeHighlightRange(range);
-}
-
 void KateView::addActions( KTextEditor::SmartRange * topRange )
 {
   if (m_actions.contains(topRange))
@@ -2501,7 +2493,12 @@ KateGotoBar *KateView::gotoBar ()
   if (m_gotoBar)
     return m_gotoBar;
 
-  return m_gotoBar = new KateGotoBar (m_viewBar);
+  m_gotoBar = new KateGotoBar (m_viewBar);
+#if QT_VERSION < KDE_MAKE_VERSION(4,4,0) // Remove this once Qt 4.4 becomes necessary
+  foreach (QWidget* widget, m_gotoBar->findChildren<QWidget*>())
+    actionCollection ()->addAssociatedWidget(widget);
+#endif
+  return m_gotoBar;
 }
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
