@@ -100,7 +100,6 @@ public:
       selectionDirection(0),
       tooltipColumn (0),
       selectionMode (Single),
-      contextMenuKey (KGlobalSettings::contextMenuKey()),
       showContextMenusOnPress (KGlobalSettings::showContextMenusOnPress()),
       mDropVisualizerWidth (4),
       paintAbove (0),
@@ -159,7 +158,6 @@ public:
   int tooltipColumn;
 
   SelectionModeExt selectionMode;
-  int contextMenuKey;
   bool showContextMenusOnPress;
 
   QRect mOldDropVisualizer;
@@ -552,7 +550,6 @@ void K3ListView::slotSettingsChanged(int category)
     break;
 
   case KGlobalSettings::SETTINGS_POPUPMENU:
-    d->contextMenuKey = KGlobalSettings::contextMenuKey ();
     d->showContextMenusOnPress = KGlobalSettings::showContextMenusOnPress ();
 
     if (d->showContextMenusOnPress)
@@ -1282,6 +1279,15 @@ void K3ListView::contentsDragEnterEvent(QDragEnterEvent *event)
     event->accept();
 }
 
+void K3ListView::contentsContextMenuEvent( QContextMenuEvent *event )
+{
+    Q3ListView::contentsContextMenuEvent(event);
+
+    if (event->reason() == QContextMenuEvent::Keyboard) {
+        emit menuShortCutPressed (this, currentItem());
+    }
+}
+
 void K3ListView::setDropVisualizerWidth (int w)
 {
   d->mDropVisualizerWidth = w > 0 ? w : 1;
@@ -1452,13 +1458,6 @@ bool K3ListView::below (Q3ListViewItem* i, const QPoint& p)
 
 void K3ListView::keyPressEvent (QKeyEvent* e)
 {
-  //don't we need a contextMenuModifier too ? (aleXXX)
-  if (e->key() == d->contextMenuKey)
-        {
-          emit menuShortCutPressed (this, currentItem());
-          return;
-        }
-
   if (d->selectionMode != FileManager)
         Q3ListView::keyPressEvent (e);
   else
