@@ -56,7 +56,7 @@ KConfigPrivate::KConfigPrivate(const KComponentData &componentData_, KConfig::Op
     : openFlags(flags), resourceType(resource), mBackend(0),
       bDynamicBackend(true),  bDirty(false), bReadDefaults(false),
       bFileImmutable(false), bForceGlobal(false), componentData(componentData_),
-      configState(KConfigBase::ReadWrite)
+      configState(KConfigBase::NoAccess)
 {
     sGlobalFileName = componentData.dirs()->saveLocation("config") +
                           QString::fromLatin1("kdeglobals");
@@ -141,7 +141,7 @@ KConfig::KConfig(KConfigPrivate &d)
 KConfig::~KConfig()
 {
     Q_D(KConfig);
-    if (d->mBackend.isUnique())
+    if (d->bDirty && d->mBackend.isUnique())
         sync();
     delete d;
 }
@@ -167,22 +167,24 @@ QStringList KConfig::groupList() const
 
 QStringList KConfigPrivate::groupList(const QByteArray& group) const
 {
-    QStringList groups;
+    QByteArray theGroup = group + '\x1d';
+    QSet<QString> groups;
 
     foreach (const KEntryKey& key, entryMap.keys())
-        if (key.mKey.isNull() && key.mGroup.startsWith(group) && key.mGroup != group)
-            groups << QString::fromUtf8(key.mGroup);
+        if (key.mKey.isNull() && key.mGroup.startsWith(theGroup))
+        {
+            QString groupname = QString::fromUtf8(key.mGroup.mid(theGroup.length()));
+            groups << groupname.left(groupname.indexOf('\x1d'));
+        }
 
-    return groups;
+    return groups.toList();
 }
 
 QStringList KConfig::keyList(const QString& aGroup) const
 {
     Q_D(const KConfig);
     QStringList keys;
-    const QByteArray theGroup(aGroup.isEmpty() ?
-            (aGroup.isNull()? d->currentGroup.toUtf8(): "<default>"):
-            aGroup.toUtf8());
+    const QByteArray theGroup(aGroup.isEmpty() ? "<default>" : aGroup.toUtf8());
 
     const KEntryMapConstIterator theEnd = d->entryMap.constEnd();
     KEntryMapConstIterator it = d->entryMap.findEntry(theGroup);
@@ -205,9 +207,7 @@ QMap<QString,QString> KConfig::entryMap(const QString& aGroup) const
 {
     Q_D(const KConfig);
     QMap<QString, QString> theMap;
-    const QByteArray theGroup(aGroup.isEmpty()?
-            (aGroup.isNull()? d->currentGroup.toUtf8(): "<default>"):
-            aGroup.toUtf8());
+    const QByteArray theGroup(aGroup.isEmpty() ? "<default>" : aGroup.toUtf8());
 
     const KEntryMapConstIterator theEnd = d->entryMap.constEnd();
     KEntryMapConstIterator it = d->entryMap.findEntry(theGroup, 0, 0);
@@ -216,9 +216,13 @@ QMap<QString,QString> KConfig::entryMap(const QString& aGroup) const
 
         for (; it != theEnd && it.key().mGroup == theGroup; ++it) {
             // leave the default values and deleted entries out
-            if (!it->bDeleted && !it.key().bDefault)
-                theMap.insert(QString::fromUtf8(it.key().mKey.constData()),
-                              QString::fromUtf8(it->mValue.constData()));
+            if (!it->bDeleted && !it.key().bDefault) {
+                const QString key = QString::fromUtf8(it.key().mKey.constData());
+                // the localized entry should come first, so don't overwrite it
+                // with the non-localized entry
+                if (!theMap.contains(key))
+                    theMap.insert(key,QString::fromUtf8(it->mValue.constData()));
+            }
         }
     }
 
@@ -228,10 +232,6 @@ QMap<QString,QString> KConfig::entryMap(const QString& aGroup) const
 void KConfig::sync()
 {
     Q_D(KConfig);
-    typedef KConfigBackend::ParseOptions ParseOptions;
-    typedef KConfigBackend::WriteOptions WriteOptions;
-    static const KConfigBackend::ParseOption ParseGlobal = KConfigBackend::ParseGlobal;
-    static const KConfigBackend::WriteOption WriteGlobal = KConfigBackend::WriteGlobal;
 
     Q_ASSERT(!isImmutable() && !name().isEmpty()); // can't write to an immutable or anonymous file.
 
@@ -245,35 +245,29 @@ void KConfig::sync()
         }
 
         // lock the local file
-        if (!d->lockLocal()) {
-            kWarning() << "couldn't lock local file";
+        if (d->configState == ReadWrite && !d->lockLocal()) {
+            qWarning() << "couldn't lock local file";
             return;
         }
 
-        KEntryMap toMerge;
         if (d->wantGlobals()) {
             KSharedPtr<KConfigBackend> tmp = KConfigBackend::create(componentData(), d->sGlobalFileName);
-            if (!tmp->lock(componentData())) {
+            if (d->configState == ReadWrite && !tmp->lock(componentData())) {
                 qWarning() << "couldn't lock global file";
                 return;
             }
-            if (d->wantMerge())
-                tmp->parseConfig(utf8Locale, toMerge, ParseGlobal);
-            tmp->writeConfig(utf8Locale, d->entryMap, toMerge, WriteGlobal, d->componentData);
-            toMerge.clear();
+            tmp->writeConfig(utf8Locale, d->entryMap, KConfigBackend::WriteGlobal, d->componentData);
             if (tmp->isLocked())
                 tmp->unlock();
         }
-        if (d->wantMerge())
-            d->mBackend->parseConfig(utf8Locale, toMerge, KConfigBackend::ParseExpansions);
-        if (d->mBackend->writeConfig(utf8Locale, d->entryMap, toMerge, WriteOptions(), d->componentData))
+        if (d->mBackend->writeConfig(utf8Locale, d->entryMap, KConfigBackend::WriteOptions(), d->componentData))
             d->bDirty = false;
         if (d->mBackend->isLocked())
             d->mBackend->unlock();
     }
 }
 
-void KConfig::clean()
+void KConfig::markAsClean()
 {
     Q_D(KConfig);
     d->bDirty = false;
@@ -354,6 +348,8 @@ void KConfigPrivate::changeFileName(const QString& name, const char* type)
         mBackend = KConfigBackend::create(componentData, file);
     else
         mBackend->setFilePath(file);
+
+    configState = mBackend->accessMode();
 }
 
 void KConfig::reparseConfiguration()
@@ -368,65 +364,32 @@ void KConfig::reparseConfiguration()
     d->bFileImmutable = false;
 
     // Parse all desired files from the least to the most specific.
-
-    // lock the local file
-    if (!d->lockLocal()) {
-        //what do we do now?
-    }
-
-    if (d->wantGlobals()) {
-        KSharedPtr<KConfigBackend> global = KConfigBackend::create(componentData(), d->sGlobalFileName);
-        // lock the global file
-        if (!global->lock(componentData())) {
-            //what do we do now?
-        }
-
+    if (d->wantGlobals())
         d->parseGlobalFiles();
 
-        // unlock the global file
-        if (global->isLocked())
-            global->unlock();
-    }
-
     d->parseConfigFiles();
-
-    // unlock local file
-    if (d->mBackend && d->mBackend->isLocked())
-        d->mBackend->unlock();
 }
 
 void KConfigPrivate::parseGlobalFiles()
 {
-    typedef KConfigBackend::ParseOptions ParseOptions;
-    static const KConfigBackend::ParseOption ParseDefaults = KConfigBackend::ParseDefaults;
-    static const KConfigBackend::ParseOption ParseGlobal = KConfigBackend::ParseGlobal;
-    static const KConfigBackend::ParseInfo ParseImmutable = KConfigBackend::ParseImmutable;
-
 //    qDebug() << "parsing global files" << globalFiles;
 
     // TODO: can we cache the values in etc_kderc / other global files
     //       on a per-application basis?
     const QByteArray utf8Locale = locale.toUtf8();
     foreach(const QString& file, globalFiles) {
-        ParseOptions parseOpts = ParseGlobal;
+        KConfigBackend::ParseOptions parseOpts = KConfigBackend::ParseGlobal|KConfigBackend::ParseExpansions;
         if (file != sGlobalFileName)
-            parseOpts |= ParseDefaults;
+            parseOpts |= KConfigBackend::ParseDefaults;
 
         KSharedPtr<KConfigBackend> backend = KConfigBackend::create(componentData, file);
-        if ( backend->parseConfig( utf8Locale, entryMap, parseOpts) == ParseImmutable)
+        if ( backend->parseConfig( utf8Locale, entryMap, parseOpts) == KConfigBackend::ParseImmutable)
             break;
     }
 }
 
 void KConfigPrivate::parseConfigFiles()
 {
-    typedef KConfigBackend::ParseOptions ParseOptions;
-    typedef KConfigBackend::ParseInfo ParseInfo;
-    static const KConfigBackend::ParseOption ParseExpansions = KConfigBackend::ParseExpansions;
-    static const KConfigBackend::ParseOption ParseDefaults = KConfigBackend::ParseDefaults;
-    static const KConfigBackend::ParseInfo ParseImmutable = KConfigBackend::ParseImmutable;
-    static const KConfigBackend::ParseInfo ParseOpenError = KConfigBackend::ParseOpenError;
-
     if (fileName == QLatin1String("kdeglobals") && wantGlobals())
         return; // already parsed in parseGlobalFiles()
 
@@ -453,21 +416,22 @@ void KConfigPrivate::parseConfigFiles()
 
         const QByteArray utf8Locale = locale.toUtf8();
         foreach(const QString& file, files) {
-            ParseOptions parseOpts = 0;
+            KConfigBackend::ParseOptions parseOpts;
             if (allowExecutableValues)
-                parseOpts |= ParseExpansions;
+                parseOpts |= KConfigBackend::ParseExpansions;
             if (file == mBackend->filePath()) {
-                ParseInfo info = mBackend->parseConfig(utf8Locale, entryMap, parseOpts);
-                if (info == ParseImmutable)
+                KConfigBackend::ParseInfo info = mBackend->parseConfig(utf8Locale, entryMap, parseOpts);
+                if (info == KConfigBackend::ParseImmutable)
                     bFileImmutable = true;
-                else if (info == ParseOpenError)
+                else if (info == KConfigBackend::ParseOpenError)
                     configState = KConfigBase::NoAccess;
                 else // some other error occurred
                     ; // FIXME what to do here?
             } else {
-                parseOpts |= ParseDefaults;
+                parseOpts |= KConfigBackend::ParseDefaults;
                 KSharedPtr<KConfigBackend> backend = KConfigBackend::create(componentData, file);
-                bFileImmutable = backend->parseConfig(utf8Locale, entryMap, parseOpts) == ParseImmutable;
+                bFileImmutable = (backend->parseConfig(utf8Locale, entryMap, parseOpts) ==
+                                  KConfigBackend::ParseImmutable);
             }
 
             if (bFileImmutable)
@@ -478,7 +442,7 @@ void KConfigPrivate::parseConfigFiles()
     }
 }
 
-KConfig::ConfigState KConfig::getConfigState() const
+KConfig::AccessMode KConfig::accessMode() const
 {
     Q_D(const KConfig);
     return d->configState;
@@ -540,7 +504,7 @@ bool KConfig::isImmutable() const
     return d->bFileImmutable;
 }
 
-bool KConfig::groupIsImmutableImpl(const QByteArray& aGroup) const
+bool KConfig::isGroupImmutableImpl(const QByteArray& aGroup) const
 {
     Q_D(const KConfig);
     return isImmutable()|d->entryMap.getEntryOption(aGroup, 0, 0, KEntryMap::EntryImmutable);
@@ -558,26 +522,14 @@ bool KConfig::forceGlobal() const
     return d->bForceGlobal;
 }
 
-void KConfig::setGroup(const QString& group)
+KConfigGroup KConfig::groupImpl(const QByteArray &group)
 {
-    Q_D(KConfig);
-    d->currentGroup = group;
+    return KConfigGroup(this, group.constData());
 }
 
-QString KConfig::group() const
+const KConfigGroup KConfig::groupImpl(const QByteArray &group) const
 {
-    Q_D(const KConfig);
-    return d->currentGroup;
-}
-
-KConfigGroup KConfig::groupImpl(const QByteArray &arr)
-{
-    return KConfigGroup(this, arr);
-}
-
-const KConfigGroup KConfig::groupImpl(const QByteArray &a) const
-{
-    return KConfigGroup(this, a);
+    return KConfigGroup(this, group.constData());
 }
 
 KEntryMap::EntryOptions convertToOptions(KConfig::WriteConfigFlags flags)
@@ -588,7 +540,7 @@ KEntryMap::EntryOptions convertToOptions(KConfig::WriteConfigFlags flags)
         options |= KEntryMap::EntryDirty;
     if (flags&KConfig::Global)
         options |= KEntryMap::EntryGlobal;
-    if (flags&KConfig::NLS)
+    if (flags&KConfig::Localized)
         options |= KEntryMap::EntryLocalized;
     return options;
 }
@@ -598,16 +550,30 @@ void KConfig::deleteGroupImpl(const QByteArray &aGroup, WriteConfigFlags flags)
     Q_D(KConfig);
     KEntryMap::EntryOptions options = convertToOptions(flags)|KEntryMap::EntryDeleted;
 
-    const QStringList keys = keyList(aGroup);
-    bool dirty = false;
-    foreach (const QString& key, keys) {
-        if (d->canWriteEntry(aGroup, key.toUtf8().constData())) {
-           d->entryMap.setEntry(aGroup, key.toUtf8(), QByteArray(), options);
-           dirty = true;
+    QByteArray theGroup = aGroup + '\x1d';
+    QSet<QByteArray> groups;
+    groups << aGroup;
+
+    foreach (const KEntryKey& key, d->entryMap.keys()) {
+        if (key.mKey.isNull() && key.mGroup.startsWith(theGroup)) {
+            groups << key.mGroup;
         }
     }
-    if (dirty)
+
+    bool dirty = false;
+    foreach (const QByteArray& group, groups) {
+        const QStringList keys = keyList(group);
+        foreach (const QString& key, keys) {
+            if (d->canWriteEntry(group, key.toUtf8().constData())) {
+                d->entryMap.setEntry(group, key.toUtf8(), QByteArray(), options);
+                dirty = true;
+            }
+        }
+    }
+
+    if (dirty) {
         d->setDirty(true);
+    }
 }
 
 bool KConfig::isConfigWritable(bool warnUser)
@@ -643,7 +609,7 @@ bool KConfig::hasGroupImpl(const QByteArray& aGroup) const
     return d->entryMap.hasEntry(aGroup);
 }
 
-bool KConfigPrivate::canWriteEntry(const QByteArray& group, const QByteArray& key, bool isDefault) const
+bool KConfigPrivate::canWriteEntry(const QByteArray& group, const char* key, bool isDefault) const
 {
     if (bFileImmutable ||
         entryMap.getEntryOption(group, key, KEntryMap::SearchLocalized, KEntryMap::EntryImmutable))
@@ -651,8 +617,8 @@ bool KConfigPrivate::canWriteEntry(const QByteArray& group, const QByteArray& ke
     return true;
 }
 
-void KConfigPrivate::putData( const QByteArray& group, const QByteArray& key,
-                      const QByteArray& value, KConfigBase::WriteConfigFlags flags, ExtendedWriteFlags extendedFlags)
+void KConfigPrivate::putData( const QByteArray& group, const char* key,
+                      const QByteArray& value, KConfigBase::WriteConfigFlags flags, bool expand)
 {
     // the KConfig object is dirty now
     // set this before any IO takes place so that if any derivative
@@ -662,33 +628,35 @@ void KConfigPrivate::putData( const QByteArray& group, const QByteArray& key,
     if( flags &  KConfigBase::Persistent )
         setDirty(true);
 
-    KEntryMap::EntryOptions options;
-    if (flags& KConfigBase::Global || bForceGlobal)
+    KEntryMap::EntryOptions options = convertToOptions(flags);
+    if (bForceGlobal)
         options |= KEntryMap::EntryGlobal;
-    if (flags& KConfigBase::NLS)
-        options |= KEntryMap::EntryLocalized;
-    if (flags& KConfigBase::Persistent)
-        options |= KEntryMap::EntryDirty;
-    if (extendedFlags & Expand)
-        options |=KEntryMap::EntryExpansion;
+    if (expand)
+        options |= KEntryMap::EntryExpansion;
 
-    if (extendedFlags & Delete) // deleting entry
+    if (value.isNull()) // deleting entry
         options |= KEntryMap::EntryDeleted;
 
     entryMap.setEntry(group, key, value, options);
 }
 
-QByteArray KConfigPrivate::lookupData(const QByteArray& group, const QByteArray& key, int flags) const
+QByteArray KConfigPrivate::lookupData(const QByteArray& group, const char* key,
+                                      KEntryMap::SearchFlags flags) const
 {
-    KEntryMapConstIterator it = entryMap.findEntry(group, key, KEntryMap::SearchFlags(flags));
+    if (bReadDefaults)
+        flags |= KEntryMap::SearchDefaults;
+    const KEntryMapConstIterator it = entryMap.findEntry(group, key, flags);
     if (it == entryMap.constEnd())
         return QByteArray();
     return it->mValue;
 }
 
-QString KConfigPrivate::lookupData(const QByteArray& group, const QByteArray& key, int flags, bool *expand) const
+QString KConfigPrivate::lookupData(const QByteArray& group, const char* key,
+                                   KEntryMap::SearchFlags flags, bool *expand) const
 {
-    return entryMap.getEntry(group, key, QString(), KEntryMap::SearchFlags(flags), expand);
+    if (bReadDefaults)
+        flags |= KEntryMap::SearchDefaults;
+    return entryMap.getEntry(group, key, QString(), flags, expand);
 }
 
 void KConfig::virtual_hook(int /*id*/, void* /*data*/)
