@@ -24,10 +24,13 @@
 #ifndef KCONFIGGROUP_H
 #define KCONFIGGROUP_H
 
+#include "kconfigbase.h"
+
 #include <kdecore_export.h>
-#include <kconfigbase.h>
+
 #include <QtCore/QExplicitlySharedDataPointer>
 #include <QtCore/QVariant>
+#include <QtCore/QStringList>
 
 class KConfig;
 class KConfigGroupPrivate;
@@ -42,7 +45,8 @@ typedef KSharedPtr<KSharedConfig> KSharedConfigPtr;
  * object, which are not associated with any group, use an
  * empty group name.
  * A KConfigGroup can be read-only if it is constructed from a const config object
- * or from a read-only group.
+ * or from another read-only group.
+ *
  */
 class KDECORE_EXPORT KConfigGroup : public KConfigBase
 {
@@ -56,22 +60,24 @@ public:
 
     /**
      * Construct a config group corresponding to @p group in @p master.
+     *
+     * This allows to create subgroups, by passing an existing group as @p master.
+     *
      * @p group is the group name encoded in UTF-8.
      */
-    KConfigGroup(KConfigBase *master, const QByteArray &group);
     KConfigGroup(KConfigBase *master, const QString &group);
     KConfigGroup(KConfigBase *master, const char *group);
 
     /**
      * Construct a read-only config group. A read-only group will silently ignore
      * any attempts to write to it.
+     *
+     * This allows to create subgroups, by passing an existing group as @p master.
      */
     KConfigGroup(const KConfigBase *master, const QString &group);
-    KConfigGroup(const KConfigBase *master, const QByteArray &group);
     KConfigGroup(const KConfigBase *master, const char *group);
 
     KConfigGroup(const KSharedConfigPtr &master, const QString& group);
-    KConfigGroup(const KSharedConfigPtr &master, const QByteArray& group);
     KConfigGroup(const KSharedConfigPtr &master, const char* group);
 
     KConfigGroup(const KConfigGroup &);
@@ -88,37 +94,25 @@ public:
     bool isValid() const;
 
     /**
-     * Delete all entries in the entire group
-     * @param pFlags flags passed to KConfig::deleteGroup
+     * The name of this group.
+     * The root group is named "<default>".
      */
-    void deleteGroup(WriteConfigFlags pFlags=Normal);
+    QString name() const;
 
     /**
-     * Returns a map (tree) of entries for all entries in this group.
-     *
-     * Only the actual entry string is returned, none of the
-     * other internal data should be included.
-     *
-     * @return A map of entries in this group, indexed by key.
+     * Check whether the containing KConfig object acutally contains a
+     * group with this name.
      */
-    QMap<QString, QString> entryMap() const;
+    bool exists() const;
 
-    /**
-     * Syncs the configuration object that this group belongs to.
-     */
+    /// @reimp
     void sync();
 
-    /**
-     * Changes the group of the object. This is a convenience function and should
-     * not be overused. Prefer another object for another group to avoid mixture of
-     * groups. A subgroup can only change to another subgroup of the parent.
-     */
-    void changeGroup( const QString &group );
-    void changeGroup( const QByteArray &group);
-    void changeGroup( const char *group);
+    /// @reimp
+    void markAsClean();
 
-    QString name() const;
-    bool exists() const;
+    /// @reimp
+    AccessMode accessMode() const;
 
     /**
      * Return the config object that this group belongs to.
@@ -127,22 +121,29 @@ public:
     const KConfig* config() const;
 
     /**
-     * Return the data in @p value converted to a QVariant.
-     * 
-     * @param pKey The name of the entry being converted, this is only used for error
-     * reporting.
-     * @param value The UTF-8 data to be converted.
-     * @param aDefault The default value if @p pKey is not found.
-     * @return @p value converted to QVariant, or @p aDefault if @p value is invalid or cannot be converted.
+     * Changes the group of the object. This is a convenience function and should
+     * not be overused. Prefer another object for another group to avoid mixture of
+     * groups. A subgroup can only change to another subgroup of the parent.
      */
-    static QVariant convertToQVariant(const char* pKey, const QByteArray& value, const QVariant& aDefault);
+    KDE_DEPRECATED void changeGroup( const QString &group );
+    KDE_DEPRECATED void changeGroup( const char *group);
 
-    QString readEntry(const char *key, const char * aDefault = 0) const
-    { return readEntry(QByteArray(key), aDefault); }
-    QString readEntry(const QByteArray &key, const char * aDefault = 0) const;
-    QString readEntry(const QString &key, const char * aDefault = 0) const
-    { return readEntry(key.toUtf8(), aDefault); }
+    /** 
+     * @reimp
+     */
+    QStringList groupList() const;
 
+    /**
+     * Returns a list of keys this group contains.
+     */
+    QStringList keyList() const;
+
+    /**
+     * Delete all entries in the entire group
+     * @param pFlags flags passed to KConfig::deleteGroup
+     */
+    void deleteGroup(WriteConfigFlags pFlags=Normal);
+    using KConfigBase::deleteGroup;
 
     /**
      * Reads the value of an entry specified by @p pKey in the current group.
@@ -164,29 +165,87 @@ public:
      * @return The value for this key, or @p aDefault.
      */
     template <typename T>
-    inline T readEntry(const QByteArray &key, const T &defaultValue) const
-    { return readListCheck(key, defaultValue); }
+        inline T readEntry(const QString& key, const T& aDefault) const
+            { return readCheck(key.toUtf8().constData(), aDefault); }
+    template <typename T>
+        inline T readEntry(const char *key, const T& aDefault) const
+            { return readCheck(key, aDefault); }
 
-    template <typename T>
-        T readEntry( const QString& pKey, const T& aDefault) const
-    { return readEntry(pKey.toUtf8(), aDefault); }
-    template <typename T>
-        T readEntry( const char *key, const T& aDefault) const
-    { return readEntry(QByteArray(key), aDefault); }
+    /**
+     * Reads the value of an entry specified by @p key in the current group.
+     *
+     * @param key The key to search for.
+     * @param aDefault A default value returned if the key was not found.
+     * @return The value for this key. Can be QVariant() if aDefault is null.
+     */
+    QVariant readEntry(const QString& key, const QVariant &aDefault) const;
+    QVariant readEntry(const char* key, const QVariant &aDefault) const;
+
+    /**
+     * Reads the value of an entry specified by @p key in the current group.
+     * If you want to read a path, please use readPathEntry().
+     *
+     * @param key The key to search for.
+     * @param aDefault A default value returned if the key was not found.
+     * @return The value for this key. Can be QString() if aDefault is null.
+     */
+    QString readEntry(const char* key, const QString& aDefault ) const;
+    QString readEntry(const QString& key, const QString& aDefault) const;
+
+    /**
+     * Reads the value of an entry specified by @p key in the current group.
+     *
+     * @param key The key to search for.
+     * @param aDefault A default value returned if the key was not found.
+     * @return The value for this key. Can be QString() if aDefault is null.
+     */
+    QString readEntry(const QString &key, const char * aDefault = 0) const;
+    QString readEntry(const char *key, const char *aDefault = 0 ) const;
+
+    /**
+     * Reads a list from the config object.
+     *
+     * @copydoc readEntry(const char*, const QList<T>&) const
+     *
+     * @warning This function doesn't convert the items returned
+     *          to any type. It's actually a list of QVariant::String's. If you
+     *          want the items converted to a specific type use
+     *          readEntry(const char*, const QList<T>&) const
+     */
+    QVariantList readEntry( const QString &key, const QVariantList& aDefault ) const;
+    QVariantList readEntry( const char* key, const QVariantList& aDefault ) const;
 
     /**
      * Reads a list of strings from the config object.
+     * @param key The key to search for.
+     * @param aDefault The default value to use if the key does not exist.
+     * @return The list. Contains @p aDefault if @p key does not exist.
+     */
+    QStringList readEntry(const QString &key, const QStringList& aDefault) const;
+    QStringList readEntry(const char* key, const QStringList& aDefault) const;
+
+    /**
+     * Reads a list of values from the config object.
+     * @param key The key to search for.
+     * @param aDefault The default value to use if the key does not exist.
+     * @return The list. Contains @p aDefault if @p key does not exist.
+     */
+    template<typename T>
+        inline QList<T> readEntry(const QString& key, const QList<T> &aDefault) const
+            { return readListCheck(key.toUtf8().constData(), aDefault); }
+    template<typename T>
+        inline QList<T> readEntry(const char* key, const QList<T> &aDefault) const
+            { return readListCheck(key, aDefault); }
+
+    /**
+     * Reads a list of strings from the config object, following XDG
+     * desktop entry spec separator semantics.
      * @param pKey The key to search for.
      * @param aDefault The default value to use if the key does not exist.
-     * @param sep The list separator.
      * @return The list. Contains @p aDefault if @p pKey does not exist.
      */
-    QStringList readEntry(const char* pKey, const QStringList& aDefault,
-                          char sep=',') const;
-    QStringList readEntry(const QByteArray& pKey, const QStringList& aDefault,
-                          char sep=',') const;
-    QStringList readEntry(const QString& pKey, const QStringList& aDefault,
-                          char sep=',') const;
+    QStringList readXdgListEntry(const QString& pKey, const QStringList& aDefault = QStringList()) const;
+    QStringList readXdgListEntry(const char* pKey, const QStringList& aDefault = QStringList()) const;
 
     /**
      * Reads a path.
@@ -197,11 +256,10 @@ public:
      *
      * @param pKey The key to search for.
      * @param aDefault A default value returned if the key was not found.
-     * @return The value for this key. Can be QString() if aDefault is null.
+     * @return The value for this key. Can be QString() if @p aDefault is null.
      */
-    QString readPathEntry( const QString& pKey, const QString & aDefault = QString() ) const;
-    QString readPathEntry( const QByteArray &key, const QString & aDefault = QString() ) const;
-    QString readPathEntry( const char *key, const QString & aDefault = QString() ) const;
+    QString readPathEntry( const QString& pKey, const QString & aDefault ) const;
+    QString readPathEntry( const char *key, const QString & aDefault ) const;
 
     /**
      * Reads a list of string paths.
@@ -211,12 +269,11 @@ public:
      * for this value, so that e.g. $HOME gets expanded.
      *
      * @param pKey The key to search for.
-     * @param sep  The list separator (default is ",").
+     * @param aDefault A default value returned if the key was not found.
      * @return The list. Empty if the entry does not exist.
      */
-    QStringList readPathListEntry( const QString& pKey, char sep = ',' ) const;
-    QStringList readPathListEntry( const QByteArray &key, char sep = ',' ) const;
-    QStringList readPathListEntry( const char *key, char sep = ',' ) const;
+    QStringList readPathEntry( const QString& pKey, const QStringList& aDefault ) const;
+    QStringList readPathEntry( const char *key, const QStringList& aDefault ) const;
 
     /**
      * Reads the value of an entry specified by @p pKey in the current group.
@@ -228,45 +285,132 @@ public:
      */
     QString readEntryUntranslated( const QString& pKey,
                                    const QString& aDefault = QString() ) const;
-    QString readEntryUntranslated( const QByteArray &key,
-                                   const QString& aDefault = QString() ) const;
     QString readEntryUntranslated( const char *key,
                                    const QString& aDefault = QString() ) const;
 
     /**
-     * @copydoc writeEntry( const char*, const QString&, WriteConfigFlags )
+     * Writes a value to the config object.
+     *
+     * @param key The key to write
+     * @param value The value to write
+     * @param pFlags The flags to use when writing this entry.
+     */
+    void writeEntry( const QString& key, const QVariant& value,
+                     WriteConfigFlags pFlags = Normal );
+    void writeEntry( const char *key, const QVariant& value,
+                     WriteConfigFlags pFlags = Normal );
+
+    /**
+     * Writes a value to the config object.
+     *
+     * @param key The key to write
+     * @param value The value to write
+     * @param pFlags The flags to use when writing this entry.
+     */
+    void writeEntry( const QString& key, const QString& value,
+                     WriteConfigFlags pFlags = Normal );
+    void writeEntry( const char *key, const QString& value,
+                     WriteConfigFlags pFlags = Normal );
+
+    /**
+     * Writes a value to the config object.
+     *
+     * @param key The key to write
+     * @param value The value to write
+     * @param pFlags The flags to use when writing this entry.
+     */
+    void writeEntry( const QString& key, const QByteArray& value,
+                     WriteConfigFlags pFlags = Normal );
+    void writeEntry( const char *key, const QByteArray& value,
+                     WriteConfigFlags pFlags = Normal );
+
+    /**
+     * Writes a value to the config object.
+     *
+     * @param key The key to write
+     * @param value The value to write
+     * @param pFlags The flags to use when writing this entry.
+     */
+    void writeEntry(const QString &key, const char *value, WriteConfigFlags pFlags = Normal);
+    void writeEntry(const char *key, const char *value, WriteConfigFlags pFlags = Normal);
+
+    /**
+     * Writes a value to the config object.
+     *
+     * @param key The key to write
+     * @param value The value to write
+     * @param pFlags The flags to use when writing this entry.
      */
     template <typename T>
-    void writeEntry(const QByteArray &key, const T& value, WriteConfigFlags pFlags = Normal )
-        { writeListCheck(key, value, pFlags); }
-    template <typename T>
-    void writeEntry( const QString& pKey, const T& value, WriteConfigFlags pFlags = Normal )
-        { writeEntry( pKey.toUtf8(), value, pFlags ); }
-    template <typename T>
-    void writeEntry( const char *key, const T& value, WriteConfigFlags pFlags = Normal )
-        { writeEntry( QByteArray(key), value, pFlags ); }
+        inline void writeEntry( const char *key, const T& value, WriteConfigFlags pFlags = Normal )
+            { writeCheck( key, value, pFlags ); }
 
+    template <typename T>
+        inline void writeEntry( const QString& key, const T& value, WriteConfigFlags pFlags = Normal )
+            { writeCheck( key.toUtf8().constData(), value, pFlags ); }
 
     /**
      * writeEntry() overridden to accept a list of strings.
      *
-     * @param pKey The key to write
+     * @param key The key to write
      * @param value The list to write
      * @param sep  The list separator (default is ",").
      * @param pFlags       The flags to use when writing this entry.
      *
      * @see  writeEntry()
      */
-    void writeEntry( const QString& pKey, const QStringList &value,
-                     char sep = ',',
+    void writeEntry( const QString& key, const QStringList &value,
                      WriteConfigFlags pFlags = Normal );
-    void writeEntry( const char *pKey, const QStringList &value,
-                     char sep = ',',
+
+    void writeEntry( const char* key, const QStringList &value,
                      WriteConfigFlags pFlags = Normal );
-    void writeEntry( const QByteArray &key, const QStringList &value,
-                     char sep = ',',
+
+    /**
+     * writeEntry() overridden to accept a list of QVariant values.
+     *
+     * @param key The key to write
+     * @param value The list to write
+     * @param sep  The list separator (default is ",").
+     * @param pFlags       The flags to use when writing this entry.
+     *
+     * @see  writeEntry()
+     */
+    void writeEntry( const QString& key, const QVariantList &value,
                      WriteConfigFlags pFlags = Normal );
 
+    void writeEntry( const char* key, const QVariantList &value,
+                     WriteConfigFlags pFlags = Normal );
+
+    /**
+     * Writes a list to the config object.
+     *
+     * @param key The key to write
+     * @param value The list to write
+     * @param pFlags The flags to use when writing this entry.
+     */
+    template <typename T>
+        inline void writeEntry(const QString& key, const QList<T> &value, WriteConfigFlags pFlags = Normal)
+            { writeListCheck( key.toUtf8().constData(), value, pFlags ); }
+
+    template <typename T>
+        inline void writeEntry(const char* key, const QList<T> &value, WriteConfigFlags pFlags = Normal)
+            { writeListCheck( key, value, pFlags ); }
+
+    /**
+     * Writes a list of strings to the config object, following XDG
+     * desktop entry spec separator semantics.
+     *
+     * @param pKey The key to write
+     * @param value The list to write
+     * @param pFlags       The flags to use when writing this entry.
+     *
+     * @see  writeEntry()
+     */
+    void writeXdgListEntry( const QString& pKey, const QStringList &value,
+                            WriteConfigFlags pFlags = Normal );
+    void writeXdgListEntry( const char *pKey, const QStringList &value,
+                            WriteConfigFlags pFlags = Normal );
+
     /**
      * Writes a file path.
      *
@@ -282,55 +426,46 @@ public:
                          WriteConfigFlags pFlags = Normal );
     void writePathEntry( const char *pKey, const QString & path,
                          WriteConfigFlags pFlags = Normal );
-    void writePathEntry( const QByteArray &key, const QString & path,
-                         WriteConfigFlags pFlags = Normal );
 
     /**
      * writePathEntry() overridden to accept a list of paths (strings).
      *
      * It is checked whether the paths are located under $HOME. If so each of
      * the paths are written out with the user's home-directory replaced with
-     * $HOME. The paths should be read back with readPathListEntry()
+     * $HOME. The paths should be read back with readPathEntry()
      *
      * @param pKey The key to write
      * @param value The list to write
-     * @param sep  The list separator (default is ",").
      * @param pFlags       The flags to use when writing this entry.
      *
-     * @see  writePathEntry()
-     * @see  readPathListEntry()
+     * @see  readPathEntry()
      */
     void writePathEntry( const QString& pKey, const QStringList &value,
-                         char sep = ',', WriteConfigFlags pFlags = Normal );
+                         WriteConfigFlags pFlags = Normal );
     void writePathEntry( const char *pKey, const QStringList &value,
-                         char sep = ',', WriteConfigFlags pFlags = Normal );
-    void writePathEntry( const QByteArray &key, const QStringList &value,
-                         char sep = ',', WriteConfigFlags pFlags = Normal );
+                         WriteConfigFlags pFlags = Normal );
 
     /**
      * Deletes the entry specified by @p pKey in the current group.
+     * This also hides system wide defaults.
      *
      * @param pKey The key to delete.
      * @param pFlags       The flags to use when deleting this entry.
      */
-    void deleteEntry( const QString& pKey, WriteConfigFlags pFlags = Normal );
-    void deleteEntry( const char *pKey, WriteConfigFlags pFlags = Normal );
-    void deleteEntry(const QByteArray &key, WriteConfigFlags flags = Normal);
+    void deleteEntry(const QString& pKey, WriteConfigFlags pFlags = Normal);
+    void deleteEntry(const char *pKey, WriteConfigFlags pFlags = Normal);
 
     /**
      * Checks whether the key has an entry in the currently active group.
      * Use this to determine whether a key is not specified for the current
-     * group (hasKey() returns false). Keys with null data are considered
-     * nonexistent.
+     * group (hasKey() returns false).
      *
      * @param key The key to search for.
      * @return If true, the key is available.
      */
-    bool hasKey(const QByteArray &key) const;
     bool hasKey(const QString &key) const;
     bool hasKey(const char *key) const;
 
-
     /**
      * Checks whether it is possible to change the given group.
      * @return whether changes may be made to this group.
@@ -343,16 +478,8 @@ public:
      * @return whether the entry @p key may be changed in the current group
      * in this configuration file.
      */
-    bool entryIsImmutable(const QString &key) const;
-    bool entryIsImmutable(const QByteArray &key) const;
-    bool entryIsImmutable(const char *key) const;
-
-    /**
-     * When set, all readEntry and readXXXEntry calls return the system
-     * wide (default) values instead of the user's preference.
-     * This is off by default.
-     */
-    void setReadDefaults(bool b);
+    bool isEntryImmutable(const QString &key) const;
+    bool isEntryImmutable(const char *key) const;
 
     /**
      * Reverts the entry with key @p key in the current group in the
@@ -366,7 +493,6 @@ public:
      */
     void revertToDefault(const QString &key);
     void revertToDefault(const char* key);
-    void revertToDefault(const QByteArray &key);
 
     /**
      * Returns whether a default is specified for an entry in either the
@@ -395,39 +521,52 @@ public:
      */
     bool hasDefault(const QString &key) const;
     bool hasDefault(const char* key) const;
-    bool hasDefault(const QByteArray &key) const;
 
-    QStringList groupList() const;
-    QStringList keyList() const;
-    void clean();
-    ConfigState getConfigState() const;
+    /**
+     * Returns a map (tree) of entries for all entries in this group.
+     *
+     * Only the actual entry string is returned, none of the
+     * other internal data should be included.
+     *
+     * @return A map of entries in this group, indexed by key.
+     */
+    QMap<QString, QString> entryMap() const;
 
-    inline void writeEntry(const char *key, const char *value, WriteConfigFlags pFlags = Normal);
-    inline void writeEntry(const QByteArray &key, const char *value, WriteConfigFlags pFlags = Normal);
-    inline void writeEntry(const QString &key, const char *value, WriteConfigFlags pFlags = Normal);
 protected:
     bool hasGroupImpl(const QByteArray &group) const;
     KConfigGroup groupImpl(const QByteArray &b);
     const KConfigGroup groupImpl(const QByteArray &b) const;
     void deleteGroupImpl(const QByteArray &group, WriteConfigFlags flags);
-    bool groupIsImmutableImpl(const QByteArray& aGroup) const;
+    bool isGroupImmutableImpl(const QByteArray& aGroup) const;
 
 private:
     QExplicitlySharedDataPointer<KConfigGroupPrivate> d;
 
-    template<class T>
-    inline T readListCheck(const QByteArray &key, const T &defaultValue) const;
+    template<typename T>
+    inline T readCheck(const char* key, const T &defaultValue) const;
 
-    template<class T>
-    inline QList<T> readListCheck(const QByteArray &key, const QList<T> &defaultValue) const;
+    template<typename T>
+    inline QList<T> readListCheck(const char* key, const QList<T> &defaultValue) const;
 
-    template<class T>
-    inline void writeListCheck(const QByteArray &key, const T &value, WriteConfigFlags pFlags);
+    template<typename T>
+    inline void writeCheck(const char* key, const T &value, WriteConfigFlags pFlags);
 
-    template<class T>
-    inline void writeListCheck(const QByteArray &key, const QList<T> &value, WriteConfigFlags pFlags);
+    template<typename T>
+    inline void writeListCheck(const char* key, const QList<T> &value, WriteConfigFlags pFlags);
 
     friend class KConfigGroupPrivate;
+
+    /**
+     * Return the data in @p value converted to a QVariant.
+     * 
+     * @param pKey The name of the entry being converted, this is only used for error
+     * reporting.
+     * @param value The UTF-8 data to be converted.
+     * @param aDefault The default value if @p pKey is not found.
+     * @return @p value converted to QVariant, or @p aDefault if @p value is invalid or cannot be converted.
+     */
+    static QVariant convertToQVariant(const char* pKey, const QByteArray& value, const QVariant& aDefault);
+    friend class KServicePrivate; // XXX yeah, ugly^5
 };
 
 #define KCONFIGGROUP_ENUMERATOR_ERROR(ENUM) \
@@ -435,128 +574,91 @@ private:
 "\" please use Q_ENUMS or Q_FLAGS to register it."
 
 #define KCONFIGGROUP_DECLARE_ENUM_QOBJECT(Class, Enum)                     \
-template <>                                                                \
-inline Class::Enum KConfigGroup::readEntry(const QByteArray &key, const Class::Enum& def) const\
+inline Class::Enum readEntry(const KConfigGroup& group, const char* key, const Class::Enum& def) \
 {                                                                          \
 const QMetaObject* M_obj = &Class::staticMetaObject;                       \
-int M_index = M_obj->indexOfEnumerator(#Enum);                             \
+const int M_index = M_obj->indexOfEnumerator(#Enum);                       \
 kFatal(M_index == -1) << KCONFIGGROUP_ENUMERATOR_ERROR(#Enum) << endl;     \
 const QMetaEnum M_enum = M_obj->enumerator(M_index);                       \
-const QByteArray str = readEntry(key, QByteArray(M_enum.valueToKey(def))); \
-return static_cast<Class::Enum>(M_enum.keyToValue(str.constData()));       \
+const QByteArray M_data = group.readEntry(key, QByteArray(M_enum.valueToKey(def)));\
+return static_cast<Class::Enum>(M_enum.keyToValue(M_data.constData()));    \
 }                                                                          \
-template <>                                                                \
-inline void KConfigGroup::writeEntry(const QByteArray &key, const Class::Enum& value, WriteConfigFlags flags)\
+inline void writeEntry(KConfigGroup& group, const char* key, const Class::Enum& value, KConfigBase::WriteConfigFlags flags = KConfigBase::Normal)\
 {                                                                          \
 const QMetaObject* M_obj = &Class::staticMetaObject;                       \
-int M_index = M_obj->indexOfEnumerator(#Enum);                             \
+const int M_index = M_obj->indexOfEnumerator(#Enum);                       \
 kFatal(M_index == -1) << KCONFIGGROUP_ENUMERATOR_ERROR(#Enum) << endl;     \
 const QMetaEnum M_enum = M_obj->enumerator(M_index);                       \
-writeEntry(key, QByteArray(M_enum.valueToKey(value)), flags);              \
+group.writeEntry(key, QByteArray(M_enum.valueToKey(value)), flags);              \
 }
 
 #define KCONFIGGROUP_DECLARE_FLAGS_QOBJECT(Class, Flags)                    \
-template <>                                                                 \
-inline Class::Flags KConfigGroup::readEntry(const QByteArray &key, const Class::Flags& def) const\
+inline Class::Flags readEntry(const KConfigGroup& group, const char* key, const Class::Flags& def) \
 {                                                                           \
 const QMetaObject* M_obj = &Class::staticMetaObject;                        \
-int M_index = M_obj->indexOfEnumerator(#Flags);                             \
+const int M_index = M_obj->indexOfEnumerator(#Flags);                       \
 kFatal(M_index == -1) << KCONFIGGROUP_ENUMERATOR_ERROR(#Flags) << endl;     \
 const QMetaEnum M_enum = M_obj->enumerator(M_index);                        \
-const QByteArray str = readEntry(key, QByteArray(M_enum.valueToKeys(def))); \
-return static_cast<Class::Flags>(M_enum.keysToValue(str.constData()));      \
+const QByteArray M_data = group.readEntry(key, QByteArray(M_enum.valueToKeys(def)));\
+return static_cast<Class::Flags>(M_enum.keysToValue(M_data.constData()));   \
 }                                                                           \
-template <>                                                                 \
-inline void KConfigGroup::writeEntry(const QByteArray &key, const Class::Flags& value, WriteConfigFlags flags)\
+inline void writeEntry(KConfigGroup& group, const char* key, const Class::Flags& value, KConfigBase::WriteConfigFlags flags = KConfigBase::Normal)\
 {                                                                           \
 const QMetaObject* M_obj = &Class::staticMetaObject;                        \
-int M_index = M_obj->indexOfEnumerator(#Flags);                             \
+const int M_index = M_obj->indexOfEnumerator(#Flags);                       \
 kFatal(M_index == -1) << KCONFIGGROUP_ENUMERATOR_ERROR(#Flags) << endl;     \
 const QMetaEnum M_enum = M_obj->enumerator(M_index);                        \
-writeEntry(key, QByteArray(M_enum.valueToKeys(value)), flags);              \
+group.writeEntry(key, QByteArray(M_enum.valueToKeys(value)), flags);              \
 }
 
-#include <conversion_check.h>
+#include "conversion_check.h"
 
-template<>
-KDECORE_EXPORT QVariantList KConfigGroup::readEntry<QVariantList>(const QByteArray &key, const QVariantList &defaultValue) const;
-template<>
-KDECORE_EXPORT QVariant KConfigGroup::readEntry<QVariant>(const QByteArray &key, const QVariant &defaultValue) const;
-template<>
-KDECORE_EXPORT QString KConfigGroup::readEntry<QString>(const QByteArray &key, const QString &defaultValue) const;
-template<class T>
-T KConfigGroup::readListCheck(const QByteArray &key, const T &defaultValue) const
+template <typename T>
+T KConfigGroup::readCheck(const char* key, const T &defaultValue) const
 {
-    ConversionCheck::to_QVariant<T>();
-    return qVariantValue<T>(readEntry(key, QVariant::fromValue(defaultValue)));
+  ConversionCheck::to_QVariant<T>();
+  return qvariant_cast<T>(readEntry(key, qVariantFromValue(defaultValue)));
 }
 
 template <typename T>
-QList<T> KConfigGroup::readListCheck(const QByteArray &key, const QList<T> &defaultValue) const
+QList<T> KConfigGroup::readListCheck(const char* key, const QList<T> &defaultValue) const
 {
   ConversionCheck::to_QVariant<T>();
   ConversionCheck::to_QString<T>();
 
-  if (!hasKey(key))
-    return defaultValue;
+  QVariantList data;
 
-  QVariantList vList;
-
-  if (!defaultValue.isEmpty()) {
-    Q_FOREACH (const T &aValue, defaultValue)
-      vList.append( aValue );
-  }
-  vList = readEntry( key, vList );
+  Q_FOREACH(const T& value, defaultValue)
+    data.append(qVariantFromValue(value));
 
   QList<T> list;
-  if (!vList.isEmpty()) {
-    Q_FOREACH (const QVariant &aValue, vList) {
-      Q_ASSERT(aValue.canConvert<T>());
-      list.append( qvariant_cast<T>(aValue) );
-    }
+  Q_FOREACH (const QVariant &value, readEntry<QVariantList>(key, data)) {
+    Q_ASSERT(qVariantCanConvert<T>(value));
+    list.append( qvariant_cast<T>(value) );
   }
 
   return list;
 }
 
-template<>
-KDECORE_EXPORT void KConfigGroup::writeEntry<QVariant>(const QByteArray &key, const QVariant &value, KConfigBase::WriteConfigFlags pFlags);
-template<>
-KDECORE_EXPORT void KConfigGroup::writeEntry<const char *>(const QByteArray &key, const char * const &value, KConfigBase::WriteConfigFlags pFlags);
-template<>
-KDECORE_EXPORT void KConfigGroup::writeEntry<QString>(const QByteArray &key, const QString &value, KConfigBase::WriteConfigFlags pFlags);
-template<>
-KDECORE_EXPORT void KConfigGroup::writeEntry<QByteArray>(const QByteArray &key, const QByteArray &value, KConfigBase::WriteConfigFlags pFlags);
-template<>
-KDECORE_EXPORT void KConfigGroup::writeEntry<QVariantList>(const QByteArray &key, const QVariantList &value, KConfigBase::WriteConfigFlags pFlags);
-
-inline void KConfigGroup::writeEntry(const char *key, const char *value, WriteConfigFlags pFlags)
-{ writeEntry<const char *>(QByteArray(key), value, pFlags); }
-inline void KConfigGroup::writeEntry(const QByteArray &key, const char *value, WriteConfigFlags pFlags)
-{ writeEntry<const char *>(key, value, pFlags); }
-inline void KConfigGroup::writeEntry(const QString &key, const char *value, WriteConfigFlags pFlags)
-{ writeEntry<const char *>(key.toUtf8(), value, pFlags); }
+template <typename T>
+void KConfigGroup::writeCheck( const char* key, const T& value,
+                               WriteConfigFlags pFlags )
+{
+    ConversionCheck::to_QVariant<T>();
+    writeEntry( key, qVariantFromValue(value), pFlags );
+}
 
 template <typename T>
-void KConfigGroup::writeListCheck( const QByteArray &key, const QList<T>& list,
-                              WriteConfigFlags pFlags )
+void KConfigGroup::writeListCheck( const char* key, const QList<T>& list,
+                                   WriteConfigFlags pFlags )
 {
   ConversionCheck::to_QVariant<T>();
   ConversionCheck::to_QString<T>();
-
-  QVariantList vList;
+  QVariantList data;
   Q_FOREACH(const T &value, list)
-    vList.append(value);
+    data.append(qVariantFromValue(value));
 
-  writeEntry( key, QVariant(vList), pFlags );
-}
-
-template <typename T>
-inline void KConfigGroup::writeListCheck( const QByteArray &key, const T& value,
-                              WriteConfigFlags pFlags )
-{
-  ConversionCheck::to_QVariant<T>();
-  writeEntry( key, qVariantFromValue(value), pFlags );
+  writeEntry( key, data, pFlags );
 }
 
 #endif // KCONFIGGROUP_H
