@@ -33,7 +33,6 @@
 #include <ksavefile.h>
 #include <kde_file.h>
 #include "kstandarddirs.h"
-#include <kdebug.h>
 
 #include <qdatetime.h>
 #include <qdir.h>
@@ -60,12 +59,18 @@ KConfigIniBackend::~KConfigIniBackend()
 {
 }
 
+KConfigBackend::ParseInfo
+        KConfigIniBackend::parseConfig(const QByteArray& currentLocale, KEntryMap& entryMap,
+                                       ParseOptions options)
+{
+    return parseConfig(currentLocale, entryMap, options, false);
+}
 
 KConfigBackend::ParseInfo
 KConfigIniBackend::parseConfig(const QByteArray& currentLocale, KEntryMap& entryMap,
-                               ParseOptions options)
+                               ParseOptions options, bool merging)
 {
-    if (filePath().isEmpty())
+    if (filePath().isEmpty() || !QFile::exists(filePath()))
         return ParseInfo();
 
     bool bDefault = options&ParseDefaults;
@@ -77,6 +82,8 @@ KConfigIniBackend::parseConfig(const QByteArray& currentLocale, KEntryMap& entry
     if (!file.open(QIODevice::ReadOnly|QIODevice::Text))
         return ParseOpenError;
 
+    QList<QByteArray> immutableGroups;
+
     bool fileOptionImmutable = false;
     bool groupOptionImmutable = false;
     bool groupSkip = false;
@@ -91,61 +98,63 @@ KConfigIniBackend::parseConfig(const QByteArray& currentLocale, KEntryMap& entry
             continue;
 
         if (line.at(0) == '[') { // found a group
-            int end = 0;
-            int opens=1;
-            do { // match brackets
-                end++;
-                if (line.at(end) == '[')
-                    opens++;
-                else if (line.at(end) == ']')
-                    opens--;
-            } while (end < line.length() && opens > 0);
-
-            if (end == line.length() && opens > 0) {
-                kWarning() << warningProlog(file, lineNo) << "Invalid group header.";
-                continue;
-            } else if (end == 3 && line.at(1) == '$' && line.at(2) == 'i') {
-                fileOptionImmutable = !kde_kiosk_exception;
-                continue;
-            }
-            currentGroup = printableToString(line.mid(1,end-1), file, lineNo);
-            line = line.remove(0, end+1);
-
             groupOptionImmutable = fileOptionImmutable;
-            if (!line.isEmpty() && line.at(0) == '[' && line.at(1) == '$') { // group option follows
-                if (line.at(2) == 'i')
-                    groupOptionImmutable = !kde_kiosk_exception;
-            }
+
+            QByteArray newGroup;
+            int start = 1, end;
+            do {
+                end = start;
+                for (;;) {
+                    if (end == line.length()) {
+                        qWarning() << warningProlog(file, lineNo) << "Invalid group header.";
+                        // XXX maybe reset the current group here?
+                        goto next_line;
+                    }
+                    if (line.at(end) == ']')
+                        break;
+                    end++;
+                }
+                if (end + 1 == line.length() && start + 2 == end &&
+                    line.at(start) == '$' && line.at(start + 1) == 'i')
+                {
+                    if (newGroup.isEmpty())
+                        fileOptionImmutable = !kde_kiosk_exception;
+                    else
+                        groupOptionImmutable = !kde_kiosk_exception;
+                }
+                else {
+                    if (!newGroup.isEmpty())
+                        newGroup += '\x1d';
+                    newGroup += printableToString(line.mid(start, end - start), file, lineNo);
+                }
+            } while ((start = end + 2) <= line.length() && line.at(end + 1) == '[');
+            currentGroup = newGroup;
 
             groupSkip = entryMap.getEntryOption(currentGroup, 0, 0, KEntryMap::EntryImmutable);
 
             if (groupSkip && !bDefault)
                 continue;
 
-            KEntryMap::EntryOptions opts=0;
             if (groupOptionImmutable)
-                opts = KEntryMap::EntryImmutable;
-            entryMap.setEntry(currentGroup, 0, QByteArray(), opts);
+                // Do not make the groups immutable until the entries from
+                // this file have been added.
+                immutableGroups.append(currentGroup);
         } else {
             if (groupSkip && !bDefault)
                 continue; // skip entry
 
-            int end = line.indexOf('=');
-
-            if (end == -1 && !line.contains("[$d]")) {
-                // no equals sign
-                kWarning() << warningProlog(file, lineNo) << "Invalid entry (missing '=')" << endl;
-                continue;
-            } else if (end == 1) {
-                // empty key
-                kWarning() << warningProlog(file, lineNo) << "Invalid entry (empty key)" << endl;
-                continue;
-            }
-            QByteArray aKey = line.left(end);
-            line = line.mid(end+1);
-            if (end != 1 && aKey.length() == 0) {
+            QByteArray aKey;
+            int eqpos = line.indexOf('=');
+            if (eqpos < 0) {
                 aKey = line;
-                line = "";
+                line.clear();
+            } else {
+                aKey = line.left(eqpos).trimmed();
+                line.remove(0, eqpos + 1);
+            }
+            if (aKey.isEmpty()) {
+                qWarning() << warningProlog(file, lineNo) << "Invalid entry (empty key)";
+                continue;
             }
 
             KEntryMap::EntryOptions entryOptions=0;
@@ -153,18 +162,15 @@ KConfigIniBackend::parseConfig(const QByteArray& currentLocale, KEntryMap& entry
                 entryOptions |= KEntryMap::EntryImmutable;
 
             QByteArray locale;
-            while (aKey.contains('[')) {
-                int start = aKey.indexOf('[');
-                end = aKey.indexOf(']', start);
-                if (start != -1 && end == -1) {
-                    kWarning() << warningProlog(file, lineNo)
-                            << "Invalid entry (missing ']')" << endl;
-                    continue;
-                } else if (end < start) {
-                    kWarning() << warningProlog(file, lineNo)
-                            << "Invalid entry (unmatched ']')" << endl;
-                    continue;
-                }else if (aKey.at(start+1) == '$') { // found option(s)
+            QByteArray rawKey;
+            int start;
+            while ((start = aKey.indexOf('[')) >= 0) {
+                int end = aKey.indexOf(']', start);
+                if (end < 0) {
+                    qWarning() << warningProlog(file, lineNo)
+                            << "Invalid entry (missing ']')";
+                    goto next_line;
+                } else if (end > start + 1 && aKey.at(start + 1) == '$') { // found option(s)
                     int i = start+2;
                     while (i < end) {
                         switch (aKey.at(i)) {
@@ -188,29 +194,40 @@ KConfigIniBackend::parseConfig(const QByteArray& currentLocale, KEntryMap& entry
                     }
                 } else { // found a locale
                     if (!locale.isNull()) {
-                        kWarning() << warningProlog(file, lineNo)
-                                << "Invalid entry (second locale!?)" << endl;
-                        continue;
+                        qWarning() << warningProlog(file, lineNo)
+                                << "Invalid entry (second locale!?)";
+                        goto next_line;
                     }
 
                     locale = aKey.mid(start+1,end-start-1);
+                    rawKey = aKey.left(end+1);
                 }
-                aKey = aKey.remove(start, end-start+1);
+                aKey.remove(start, end-start+1);
             }
 
+            if (eqpos < 0) { // Do this here after [$d] was checked
+                qWarning() << warningProlog(file, lineNo) << "Invalid entry (missing '=')";
+                continue;
+            }
             aKey = printableToString(aKey, file, lineNo);
             if (!locale.isEmpty()) {
                 if (locale != currentLocale) {
                     // backward compatibility. C == en_US
                     if (locale.at(0) != 'C' || currentLocale != "en_US") {
-                        aKey += '[' + locale + ']';
-                        locale = QByteArray();
+                        if (merging){
+                            entryOptions |= KEntryMap::EntryRawKey;
+                            aKey = rawKey; // store as unprocessed key
+                            locale = QByteArray();
+                        } else
+                            goto next_line; // skip this entry if we're not merging
                     }
                 }
             }
 
             if (options&ParseGlobal)
                 entryOptions |= KEntryMap::EntryGlobal;
+            if (bDefault)
+                entryOptions |= KEntryMap::EntryDefault;
             if (!locale.isNull())
                 entryOptions |= KEntryMap::EntryLocalized;
             entryMap.setEntry(currentGroup, aKey, printableToString(line, file, lineNo), entryOptions);
@@ -219,6 +236,11 @@ next_line:
         continue;
     }
 
+    // now make sure immutable groups are marked immutable
+    foreach(const QByteArray& group, immutableGroups) {
+        entryMap.setEntry(group, QByteArray(), QByteArray(), KEntryMap::EntryImmutable);
+    }
+
     return (fileOptionImmutable? ParseImmutable: ParseInfo());
 }
 
@@ -234,50 +256,59 @@ void KConfigIniBackend::writeEntries(const QByteArray& locale, QFile& file,
         if ((key.mGroup != "<default>") == defaultGroup)
             continue; // skip
 
-        // Skip default values and group headers
-        if (key.bDefault || key.mKey.isNull())
+        // Skip group headers
+        if (key.mKey.isNull())
             continue; // skip
 
         const KEntry& currentEntry = *it;
-        KEntryMapConstIterator testIt = it;
-        ++testIt;
-        if (testIt != end) { // might have a default
-            const KEntryKey& defaultKey = testIt.key();
-            if (defaultKey.bDefault ||
-                 defaultKey.mGroup != key.mGroup ||
-                 defaultKey.mKey != key.mKey ||
-                 defaultKey.bLocal != key.bLocal) {
-                if (currentEntry.bDeleted)
-                    continue; // Don't write deleted entries if there is no default
-            } else {
-                if (currentEntry.mValue == testIt->mValue &&
-                    currentEntry.bDeleted == testIt->bDeleted)
-                    continue; // same as default, don't write
-            }
-        }
-
         if (!defaultGroup && currentGroup != key.mGroup) {
             if (!firstEntry)
                 file.putChar('\n');
             currentGroup = key.mGroup;
-            file.putChar('[');
-            file.write(stringToPrintable(currentGroup));
-            file.write("]\n", 2);
+            for (int start = 0, end;; start = end + 1) {
+                file.putChar('[');
+                end = currentGroup.indexOf('\x1d', start);
+                if (end < 0) {
+                    int cgl = currentGroup.length();
+                    if (currentGroup.at(start) == '$' && cgl - start <= 10) {
+                        for (int i = start + 1; i < cgl; i++) {
+                            char c = currentGroup.at(i);
+                            if (c < 'a' || c > 'z')
+                                goto nope;
+                        }
+                        file.write("\\x24");
+                        start++;
+                    }
+                  nope:
+                    file.write(stringToPrintable(currentGroup.mid(start), GroupString));
+                    file.write("]\n", 2);
+                    break;
+                } else {
+                    file.write(stringToPrintable(currentGroup.mid(start, end - start), GroupString));
+                    file.putChar(']');
+                }
+            }
         }
 
         firstEntry = false;
         // it is data for a group
 
-        file.write(stringToPrintable(key.mKey)); // Key
-
-        if (currentEntry.bNLS) {
-            file.putChar('[');
-            file.write(locale); // Locale tag
-            file.putChar(']');
-        }
-        if (currentEntry.bDeleted)
-            file.write("[$d]", 4); // Deleted
+        if (key.bRaw) // unprocessed key with attached locale from merge
+            file.write(key.mKey);
         else {
+            file.write(stringToPrintable(key.mKey, KeyString)); // Key
+            if (key.bLocal && locale != "C") { // 'C' locale == untranslated
+                file.putChar('[');
+                file.write(locale); // locale tag
+                file.putChar(']');
+            }
+        }
+        if (currentEntry.bDeleted) {
+            if (currentEntry.bImmutable)
+                file.write("[$di]", 5); // Deleted + immutable
+            else
+                file.write("[$d]", 4); // Deleted
+        } else {
             if (currentEntry.bImmutable || currentEntry.bExpand) {
                 file.write("[$", 2);
                 if (currentEntry.bImmutable)
@@ -293,75 +324,6 @@ void KConfigIniBackend::writeEntries(const QByteArray& locale, QFile& file,
     }
 }
 
-bool KConfigIniBackend::writeConfig(const QByteArray& locale, KEntryMap& entryMap,
-                                    const KEntryMap& toMerge, WriteOptions options, const KComponentData &data)
-{
-    Q_ASSERT(!filePath().isEmpty());
-
-    KEntryMap tempMap = toMerge;
-    bool bGlobal = options & WriteGlobal;
-
-    int mergeCount = toMerge.count();
-    const KEntryMapIterator end = entryMap.end();
-    for (KEntryMapIterator it=entryMap.begin(); it != end; ++it) {
-        const KEntryKey& key = it.key();
-        if (key.bDefault) {
-            tempMap[key] = *it; // store default values in tempMap
-            continue;
-        }
-        if (mergeCount > 0 && !it->bDirty)
-            continue;
-
-        // only write back dirty entries that have the same
-        // "globality" as the file
-        if (it->bGlobal == bGlobal && it->bDirty) {
-            tempMap[key] = *it;
-            it->bDirty = false;
-        }
-    }
-
-    // now tempMap should contain all entries to be written
-    // so write it out to disk
-
-    // check if file exists
-    int fileMode = -1;
-    bool createNew = true;
-
-    KDE_struct_stat buf;
-    if (KDE_stat(QFile::encodeName(filePath()), &buf) == 0)
-    {
-        if (buf.st_uid == ::getuid() || buf.st_uid == uid_t(-2))
-        {
-            // Preserve file mode if file exists and is owned by user.
-            fileMode = buf.st_mode & 0777;
-        }
-        else
-        {
-            // File is not owned by user:
-            // Don't create new file but write to existing file instead.
-            createNew = false;
-        }
-    }
-
-    KSaveFile file( filePath(), data );
-    if (!file.open())
-      return false;
-
-    file.setTextModeEnabled(true); // to get eol translation
-
-    if (createNew)
-        file.setPermissions(QFile::ReadUser|QFile::WriteUser);
-
-    writeEntries(locale, file, tempMap);
-
-    if ( !file.size() && ((fileMode == -1) || (fileMode == 0600)) ) {
-        // File is empty and doesn't have special permissions: delete it.
-        file.abort();
-    }
-
-    return file.finalize();
-}
-
 void KConfigIniBackend::writeEntries(const QByteArray& locale, QFile& file, const KEntryMap& map)
 {
     bool firstEntry = true;
@@ -373,6 +335,127 @@ void KConfigIniBackend::writeEntries(const QByteArray& locale, QFile& file, cons
     writeEntries(locale, file, map, false, firstEntry);
 }
 
+bool KConfigIniBackend::writeConfig(const QByteArray& locale, KEntryMap& entryMap,
+                                    WriteOptions options, const KComponentData &data)
+{
+    Q_ASSERT(!filePath().isEmpty());
+
+    KEntryMap writeMap;
+    bool bGlobal = options & WriteGlobal;
+
+    {
+        ParseOptions opts = ParseExpansions;
+        if (bGlobal)
+            opts |= ParseGlobal;
+        ParseInfo info = parseConfig(locale, writeMap, opts, true);
+        if (info) // either there was an error or the file became immutable
+            return false;
+    }
+    const KEntryMapIterator end = entryMap.end();
+    for (KEntryMapIterator it=entryMap.begin(); it != end; ++it) {
+        if (!it->bDirty) // not dirty, doesn't overwrite entry in writeMap. skips default entries, too.
+            continue;
+
+        const KEntryKey& key = it.key();
+
+        // only write entries that have the same "globality" as the file
+        if (it->bGlobal == bGlobal) {
+            if (!it->bDeleted) {
+                writeMap[key] = *it;
+            } else {
+                KEntryKey defaultKey = key;
+                defaultKey.bDefault = true;
+                if (!entryMap.contains(defaultKey))
+                    writeMap.remove(key); // remove the deleted entry if there is no default
+                else
+                    writeMap[key] = *it; // otherwise write an explicitly deleted entry
+            }
+            it->bDirty = false;
+        }
+    }
+
+    // now writeMap should contain only entries to be written
+    // so write it out to disk
+
+    // check if file exists
+    QFile::Permissions fileMode = QFile::ReadUser | QFile::WriteUser;
+    bool createNew = true;
+
+    QFileInfo fi(filePath());
+    if (fi.exists())
+    {
+        if (fi.ownerId() == ::getuid())
+        {
+            // Preserve file mode if file exists and is owned by user.
+            fileMode = fi.permissions();
+        }
+        else
+        {
+            // File is not owned by user:
+            // Don't create new file but write to existing file instead.
+            createNew = false;
+        }
+    }
+
+    if (createNew) {
+        KSaveFile file( filePath(), data );
+        if (!file.open()) {
+            return false;
+        }
+
+        file.setPermissions(fileMode);
+
+        file.setTextModeEnabled(true); // to get eol translation
+        writeEntries(locale, file, writeMap);
+
+        if (!file.size() && (fileMode == (QFile::ReadUser | QFile::WriteUser))) {
+            // File is empty and doesn't have special permissions: delete it.
+            file.abort();
+
+            if (fi.exists()) {
+                // also remove the old file in case it existed. this can happen
+                // when we delete all the entries in an existing config file.
+                // if we don't do this, then deletions and revertToDefault's
+                // will mysteriously fail
+                QFile::remove(filePath());
+            }
+        } else {
+            // Normal case: Close the file
+            return file.finalize();
+        }
+    } else {
+        // Open existing file. *DON'T* create it if it suddenly does not exist!
+#ifdef Q_OS_UNIX
+        int fd = KDE_open(QFile::encodeName(filePath()), O_WRONLY | O_TRUNC);
+        if (fd < 0) {
+            return false;
+        }
+        FILE *fp = KDE_fdopen(fd, "w");
+        if (!fp) {
+            close(fd);
+            return false;
+        }
+        QFile f;
+        if (!f.open(fp, QIODevice::WriteOnly)) {
+            fclose(fp);
+            return false;
+        }
+        writeEntries(locale, f, writeMap);
+        f.close();
+        fclose(fp);
+#else
+        QFile f( filePath() );
+        // XXX This is broken - it DOES create the file if it is suddenly gone.
+        if (!f.open( QIODevice::WriteOnly | QIODevice::Truncate )) {
+            return false;
+        }
+        f.setTextModeEnabled(true);
+        writeEntries(locale, f, writeMap);
+#endif
+    }
+    return true;
+}
+
 bool KConfigIniBackend::isWritable() const
 {
     if (!filePath().isEmpty())
@@ -412,8 +495,21 @@ void KConfigIniBackend::setFilePath(const QString& file)
     }
 }
 
+KConfigBase::AccessMode KConfigIniBackend::accessMode() const
+{
+    if (filePath().isEmpty())
+        return KConfigBase::NoAccess;
+
+    if (KStandardDirs::checkAccess(filePath(), W_OK))
+        return KConfigBase::ReadWrite;
+
+    return KConfigBase::ReadOnly;
+}
+
 bool KConfigIniBackend::lock(const KComponentData& componentData)
 {
+    Q_ASSERT(!filePath().isEmpty());
+
     lockFile = new KLockFile(filePath() + QLatin1String(".lock"), componentData);
 
     if (lockFile->lock() == KLockFile::LockStale) // attempt to break the lock
@@ -450,7 +546,7 @@ QByteArray KConfigIniBackend::stringToPrintable(const QByteArray& aString, Strin
     int i = 0;
 
     // Protect leading space
-    if (s[0] == ' ') {
+    if (s[0] == ' ' && type != GroupString) {
         result.append("\\s");
         i++;
     }
@@ -475,9 +571,14 @@ QByteArray KConfigIniBackend::stringToPrintable(const QByteArray& aString, Strin
             case '\\':
                 result.append("\\\\");
                 break;
+            case '=':
+                if (type != KeyString) {
+                    result.append(s[i]);
+                    break;
+                }
+                goto doEscape;
             case '[':
             case ']':
-            case '=':
             // Above chars are OK to put in *value* strings as plaintext
                 if (type == ValueString) {
                     result.append(s[i]);
@@ -492,8 +593,8 @@ QByteArray KConfigIniBackend::stringToPrintable(const QByteArray& aString, Strin
     }
 
     // Protect trailing space
-    if (result.endsWith(' ')) {
-        result.replace(result.lastIndexOf(' '), 1, "\\s");
+    if (result.endsWith(' ') && type != GroupString) {
+        result.replace(result.length() - 1, 1, "\\s");
     }
     result.squeeze();
 
@@ -516,8 +617,8 @@ char KConfigIniBackend::charFromHex(const char *str, const QFile& file, int line
         } else {
             QByteArray e(str, 2);
             e.prepend("\\x");
-            kWarning() << warningProlog(file, line) << "Invalid hex character " << c
-                    << " in \\x<nn>-type escape sequence \"" << e.constData() << "\"." << endl;
+            qWarning() << warningProlog(file, line) << "Invalid hex character " << c
+                    << " in \\x<nn>-type escape sequence \"" << e.constData() << "\".";
             return 'x';
         }
     }
@@ -584,8 +685,8 @@ QByteArray KConfigIniBackend::printableToString(const QByteArray& aString, const
                     break;
                 default:
                     *r = '\\';
-                    kWarning() << warningProlog(file, line)
-                            << QString("Invalid escape sequence \"\\%1\".").arg(str[i]) << endl;
+                    qWarning() << warningProlog(file, line)
+                            << QString("Invalid escape sequence \"\\%1\".").arg(str[i]);
             }
         }
     }
