@@ -133,10 +133,15 @@ public:
       List(const KUrl &url);
       /**
        * Creates a list that contains the URLs from the given
-       * list.
+       * list of strings.
        * @param list the list containing the URLs as strings
        */
       List(const QStringList &list);
+      /**
+       * Creates a list that contains the URLs from the given QList<KUrl>.
+       * @param list the list containing the URLs
+       */
+      List(const QList<KUrl> &list);
       /**
        * Converts the URLs of this list to a list of strings.
        * @return the list of strings
@@ -984,6 +989,7 @@ public:
 
 private:
   void _setQuery( const QString& query );
+  void _setEncodedUrl(const QByteArray& url);
   QString toString() const; // forbidden, use url(), prettyUrl(), or pathOrUrl() instead.
   operator QString() const; // forbidden, use url(), prettyUrl(), or pathOrUrl() instead.
 private:
