@@ -67,9 +67,9 @@ public:
           checkRestrictions(true)
     { }
 
-    bool restrictionsActive;
-    bool dataRestrictionActive;
-    bool checkRestrictions;
+    bool restrictionsActive : 1;
+    bool dataRestrictionActive : 1;
+    bool checkRestrictions : 1;
     QMap<QByteArray, bool> restrictions;
     QStringList xdgdata_prefixes;
     QStringList xdgconf_prefixes;
@@ -408,10 +408,10 @@ bool KStandardDirs::addResourceDir( const char *type,
 }
 
 QString KStandardDirs::findResource( const char *type,
-                                     const QString& filename ) const
+                                     const QString& _filename ) const
 {
-    if (!QDir::isRelativePath(filename))
-        return filename; // absolute dirs are absolute dirs, right? :-/
+    if (!QDir::isRelativePath(_filename))
+        return _filename; // absolute dirs are absolute dirs, right? :-/
 
 #if 0
     kDebug(180) << "Find resource: " << type;
@@ -423,6 +423,13 @@ QString KStandardDirs::findResource( const char *type,
     }
 #endif
 
+    QString filename(_filename);
+#ifdef Q_OS_WIN
+    if(strcmp(type, "exe") == 0) {
+      if(!filename.endsWith(QLatin1String(".exe")))
+        filename += QLatin1String(".exe");
+    }
+#endif
     const QString dir = findResourceDir(type, filename);
     if (dir.isEmpty())
         return dir;
@@ -499,15 +506,22 @@ QStringList KStandardDirs::findDirs( const char *type,
 }
 
 QString KStandardDirs::findResourceDir( const char *type,
-                                        const QString& filename) const
+                                        const QString& _filename) const
 {
 #ifndef NDEBUG
-    if (filename.isEmpty()) {
+    if (_filename.isEmpty()) {
         kWarning() << "filename for type " << type << " in KStandardDirs::findResourceDir is not supposed to be empty!!";
         return QString();
     }
 #endif
 
+    QString filename(_filename);
+#ifdef Q_OS_WIN
+    if(strcmp(type, "exe") == 0) {
+      if(!filename.endsWith(QLatin1String(".exe")))
+        filename += QLatin1String(".exe");
+    }
+#endif
     if (d->restrictionsActive && (strcmp(type, "data")==0))
         applyDataRestrictions(filename);
     QStringList candidates = resourceDirs(type);
@@ -530,6 +544,14 @@ QString KStandardDirs::findResourceDir( const char *type,
 
 bool KStandardDirs::exists(const QString &fullPath)
 {
+#ifdef Q_OS_WIN
+    // access() and stat() give a stupid error message to the user
+    // if the path is not accessible at all (e.g. no disk in A:/ and
+    // we do stat("A:/.directory")
+    if(fullPath.at(fullPath.length() - 1) == '/')
+        return QDir(fullPath).exists();
+    return QFileInfo(fullPath).exists();
+#else
     KDE_struct_stat buff;
     if (access(QFile::encodeName(fullPath), R_OK) == 0 && KDE_stat( QFile::encodeName(fullPath), &buff ) == 0)
         if (fullPath.at(fullPath.length() - 1) != '/') {
@@ -539,6 +561,7 @@ bool KStandardDirs::exists(const QString &fullPath)
             if (S_ISDIR( buff.st_mode ))
                 return true;
     return false;
+#endif
 }
 
 static void lookupDirectory(const QString& path, const QString &relPart,
@@ -1078,7 +1101,8 @@ static QString checkExecutable( const QString& path, bool ignoreExecBit )
     if( info.exists() && ( ignoreExecBit || info.isExecutable() )
         && ( info.isFile() || info.isSymLink() ) ) {
         //kDebug(180) << "checkExecutable(): returning " << path;
-        return path;
+        info.makeAbsolute();
+        return info.filePath();
     }
     //kDebug(180) << "checkExecutable(): failed, returning empty string";
     return QString();
@@ -1097,11 +1121,12 @@ QString KStandardDirs::findExe( const QString& appname,
 #endif
     QFileInfo info;
 
-    // absolute path ?
-    if (!QDir::isRelativePath(real_appname))
+    // absolute or relative path?
+    if (real_appname.contains(QDir::separator()))
     {
         //kDebug(180) << "findExe(): absolute path given";
-        return checkExecutable(real_appname, options & IgnoreExecBit);
+        QString path = checkExecutable(real_appname, options & IgnoreExecBit);
+        return path;
     }
 
     //kDebug(180) << "findExe(): relative path given";
@@ -1181,6 +1206,19 @@ int KStandardDirs::findAllExe( QStringList& list, const QString& appname,
     return list.count();
 }
 
+static inline QString equalizePath(QString &str)
+{
+#ifdef Q_WS_WIN
+	// filter pathes through QFileInfo to have always 
+	// the same case for drive letters
+	QFileInfo f(str);
+	if (f.isAbsolute())
+		return f.absoluteFilePath();
+	else 
+#endif
+		return str;
+}
+
 static int tokenize( QStringList& tokens, const QString& str,
                      const QString& delim )
 {
@@ -1191,8 +1229,8 @@ static int tokenize( QStringList& tokens, const QString& str,
     {
         if ( delim.contains( str[ index ] ) )
         {
-            tokens.append( token );
-            token = "";
+			tokens.append( equalizePath(token) );
+			token = "";
         }
         else
         {
@@ -1201,9 +1239,9 @@ static int tokenize( QStringList& tokens, const QString& str,
     }
     if ( token.length() > 0 )
     {
-        tokens.append( token );
+        tokens.append( equalizePath(token) );
     }
-
+	
     return tokens.count();
 }
 
@@ -1346,7 +1384,7 @@ static QString readEnvPath(const char *env)
     QByteArray c_path = getenv(env);
     if (c_path.isEmpty())
         return QString();
-    return QFile::decodeName(c_path);
+	return QDir::fromNativeSeparators(QFile::decodeName(c_path));
 }
 
 #ifdef __linux__
