@@ -82,10 +82,10 @@ public:
     KEncodingDetector::EncodingChoiceSource m_source;
     KEncodingDetector::AutoDetectScript m_autoDetectLanguage;
 
-    bool m_visualRTL;
-    bool m_seenBody;
-    bool m_writtingHappened;
-    bool m_analyzeCalled; //for decode()
+    bool m_visualRTL : 1;
+    bool m_seenBody : 1;
+    bool m_writtingHappened : 1;
+    bool m_analyzeCalled : 1; //for decode()
     int m_multiByte;
 
     QByteArray m_bufferForDefferedEncDetection;
@@ -904,7 +904,9 @@ bool KEncodingDetector::analyze(const char *data, int len)
             //enc = d->m_codec->name();
             delete d->m_decoder;
             d->m_decoder = d->m_codec->makeDecoder();
+#ifdef DECODE_DEBUG
             kWarning() << "Detection by BOM";
+#endif
             if (is16Bit(d->m_codec) && c2==0x00)
             {
                 // utf16LE, we need to put the decoder in LE mode
