@@ -175,8 +175,8 @@ public:
   int fracDigits;
   KLocale::SignPosition positiveMonetarySignPosition;
   KLocale::SignPosition negativeMonetarySignPosition;
-  bool positivePrefixCurrencySymbol;
-  bool negativePrefixCurrencySymbol;
+  bool positivePrefixCurrencySymbol : 1;
+  bool negativePrefixCurrencySymbol : 1;
 
   // Date and time
   QString timeFormat;
@@ -265,10 +265,10 @@ void KLocalePrivate::initMainCatalogs(const QString & catalog)
 
     // catalogs from which each application can draw translations
     numberOfSysCatalogs = 4;
-    catalogNames.append(KCatalogName("kdeqt"));
-    catalogNames.append(KCatalogName("kdelibs4"));
-    catalogNames.append(KCatalogName("kio4"));
     catalogNames.append(KCatalogName("libphonon"));
+    catalogNames.append(KCatalogName("kio4"));
+    catalogNames.append(KCatalogName("kdelibs4"));
+    catalogNames.append(KCatalogName("kdeqt"));
 
     updateCatalogs(); // evaluate this for all languages
   }
@@ -301,7 +301,7 @@ void KLocalePrivate::initLanguageList(KConfig *config, bool useEnv)
     list += QFile::decodeName(getenv("KDE_LANG")).split(':');
 
   // Collect languages set by KDE config.
-  list += cg.readEntry("Language", QStringList(), ':');
+  list += cg.readEntry("Language", QString()).split(':');
 
   // Collect languages read from environment variables by gettext(3).
   if (useEnv) {
@@ -475,7 +475,7 @@ bool KLocalePrivate::setLanguage(const QStringList & languages)
   // 2) duplicate entries like in de:fr:de, where we have to keep the first occurrence of a language in order
   //    to preserve the order of precenence of the user
   // 3) languages into which the application is not translated. For those languages we should not even load kdelibs.mo or kio.po.
-  //    these langugage have to be dropped. Otherwise we get strange side effects, e.g. with Hebrew:
+  //    these languages have to be dropped. Otherwise we get strange side effects, e.g. with Hebrew:
   //    the right/left switch for languages that write from
   //    right to left (like Hebrew or Arabic) is set in kdelibs.mo. If you only have kdelibs.mo
   //    but nothing from appname.mo, you get a mostly English app with layout from right to left.
@@ -798,19 +798,33 @@ QString KLocale::translateQt(const char *context, const char *sourceText,
   if (comment && comment[0]) {
     // Comment given, go for context call.
     d->translate_priv(comment, sourceText, 0, 0, &language, &translation);
-    if (language != defaultLanguage())
-      return translation;
   }
   else {
     // Comment not given, go for try-fallback with context.
     if (context && context[0]) {
       d->translate_priv(context, sourceText, 0, 0, &language, &translation);
-      if (language != defaultLanguage())
-        return translation;
     }
-    d->translate_priv(0, sourceText, 0, 0, &language, &translation);
-    if (language != defaultLanguage())
-      return translation;
+    if (language.isEmpty() || language == defaultLanguage()) {
+      d->translate_priv(0, sourceText, 0, 0, &language, &translation);
+    }
+  }
+
+  if (language != defaultLanguage()) {
+    // Ignore any script that may be present; no arguments for placeholders
+    // supplied through tr().
+    // FIXME: Transcript fence literal duplicated here and in
+    // klocalizedstring.cpp.
+    int p = translation.indexOf("|/|");
+    if (p >= 0) {
+        translation = translation.left(p);
+    }
+    // Although placeholders are not expanded yet, resolve semantic tags
+    // for what is there.
+    // FIXME: Actually do this. Would need a new public method in
+    // KLocalizedString that just does semantic resolution. Probably good
+    // idea to have it, anyway.
+
+    return translation;
   }
 
   // No proper translation found, return empty according to Qt's expectation.
@@ -2201,7 +2215,7 @@ bool KLocalePrivate::setEncoding(int mibEnum)
 QStringList KLocale::allLanguagesList() const
 {
   if (!d->languages)
-    d->languages = new KConfig("all_languages", KConfig::CascadeConfig, "locale");
+    d->languages = new KConfig("all_languages", KConfig::NoGlobals, "locale");
 
   return d->languages->groupList();
 }
@@ -2209,7 +2223,7 @@ QStringList KLocale::allLanguagesList() const
 QString KLocale::languageCodeToName(const QString &language) const
 {
   if (!d->languages)
-    d->languages = new KConfig("all_languages", KConfig::CascadeConfig, "locale");
+    d->languages = new KConfig("all_languages", KConfig::NoGlobals, "locale");
 
   KConfigGroup cg(d->languages, language);
   return cg.readEntry("Name");
