@@ -94,7 +94,7 @@ namespace Kuit {
 
     namespace Numfmt { // number formats
         typedef enum {
-            Posix, US, Euro
+            Posix, US, Euro, Euro2, Euro2ct
         } Var;
     }
 
@@ -313,6 +313,7 @@ KuitSemanticsStaticData::KuitSemanticsStaticData ()
     SETUP_NUMFMT(Posix, "posix");
     SETUP_NUMFMT(US, "us");
     SETUP_NUMFMT(Euro, "euro");
+    SETUP_NUMFMT(Euro2ct, "euro2ct");
 
     // Known XML entities, direct/inverse mapping.
     xmlEntities["lt"] = '<';
@@ -639,11 +640,11 @@ void KuitSemanticsPrivate::setFormattingPatterns (const KCatalog &cat)
 
     // -------> Icode
     SET_PATTERN(Tag::Icode, Att::None, Fmt::Plain,
-                XXXX_NOOP2("@icode/plain",
+                I18N_NOOP2("@icode/plain",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
                            "“%1”"));
     SET_PATTERN(Tag::Icode, Att::None, Fmt::Rich,
-                XXXX_NOOP2("@icode/rich",
+                I18N_NOOP2("@icode/rich",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
                            "<tt>%1</tt>"));
 
@@ -651,7 +652,7 @@ void KuitSemanticsPrivate::setFormattingPatterns (const KCatalog &cat)
     SET_PATTERN(Tag::Bcode, Att::None, Fmt::Plain,
                 XXXX_NOOP2("@bcode/plain",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
-                           "%1"));
+                           "\n%1\n"));
     SET_PATTERN(Tag::Bcode, Att::None, Fmt::Rich,
                 XXXX_NOOP2("@bcode/rich",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
@@ -691,7 +692,7 @@ void KuitSemanticsPrivate::setFormattingPatterns (const KCatalog &cat)
     SET_PATTERN(Tag::Placeholder, Att::None, Fmt::Plain,
                 I18N_NOOP2("@placeholder/plain",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
-                           "<%1>"));
+                           "&lt;%1&gt;"));
     SET_PATTERN(Tag::Placeholder, Att::None, Fmt::Rich,
                 I18N_NOOP2("@placeholder/rich",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
@@ -701,7 +702,7 @@ void KuitSemanticsPrivate::setFormattingPatterns (const KCatalog &cat)
     SET_PATTERN(Tag::Email, Att::None, Fmt::Plain,
                 I18N_NOOP2("@email/plain",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
-                           "<%1>"));
+                           "&lt;%1&gt;"));
     SET_PATTERN(Tag::Email, Att::None, Fmt::Rich,
                 I18N_NOOP2("@email/rich",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
@@ -710,7 +711,7 @@ void KuitSemanticsPrivate::setFormattingPatterns (const KCatalog &cat)
                 I18N_NOOP2("@email-with-name/plain\n"
                            "%1 is name, %2 is address",
     // i18n: KUIT pattern, see the comment to the first of these entries above.
-                           "%1 <%2>"));
+                           "%1 &lt;%2&gt;"));
     SET_PATTERN(Tag::Email, Att::Address, Fmt::Rich,
                 I18N_NOOP2("@email-with-name/rich\n"
                            "%1 is name, %2 is address",
@@ -754,7 +755,14 @@ void KuitSemanticsPrivate::setTextTransformData (const KCatalog &cat)
 
     m_numfmtInt = Kuit::Numfmt::Posix;
     // i18n: Decide how integer-valued amounts will be formatted in your
-    // language. For possible formats, see translation documentation.
+    // language. Currently available number formats are:
+    //   posix   - decimal point
+    //   us      - thousands separation by comma, decimal point
+    //   euro    - thousands separation by point, decimal comma
+    //   euro2   - thousands separation by space, decimal comma
+    //   euro2ct - as euro2, except thousand not separated when <10000
+    // If none of the existing formats is appropriate for your language,
+    // write to kde-i18n-doc@kde.org to arrange for a new format.
     QString fmtnameInt = cat.translate("number-format:integer", "us").toLower();
     if (s->knownNumfmts.contains(fmtnameInt)) {
         m_numfmtInt = s->knownNumfmts[fmtnameInt];
@@ -767,7 +775,7 @@ void KuitSemanticsPrivate::setTextTransformData (const KCatalog &cat)
 
     m_numfmtReal = Kuit::Numfmt::Posix;
     // i18n: Decide how real-valued amounts will be formatted in your
-    // language. For possible formats, see translation documentation.
+    // language. See the comment to previous entry.
     QString fmtnameReal = cat.translate("number-format:real", "us").toLower();
     if (s->knownNumfmts.contains(fmtnameReal)) {
         m_numfmtReal = s->knownNumfmts[fmtnameReal];
@@ -807,20 +815,48 @@ void KuitSemanticsPrivate::setTextTransformData (const KCatalog &cat)
     #undef I18N_NOOP2
     #define I18N_NOOP2(ctxt, msg) msg
 
-    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Ctrl"));
     SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Alt"));
-    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Meta"));
-    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Shift"));
-
-    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Space"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "AltGr"));
     SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Backspace"));
-    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Insert"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "CapsLock"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Control"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Ctrl"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Del"));
     SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Delete"));
-    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Home"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Down"));
     SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "End"));
-    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "PageUp"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Enter"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Esc"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Escape"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Home"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Hyper"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Ins"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Insert"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Left"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Menu"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Meta"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "NumLock"));
     SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "PageDown"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "PageUp"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "PgDown"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "PgUp"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "PauseBreak"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "PrintScreen"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "PrtScr"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Return"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Right"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "ScrollLock"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Shift"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Space"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Super"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "SysReq"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Tab"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Up"));
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "Win"));
     // TODO: Add rest of the key names?
+
+    // i18n: Pattern for the function keys.
+    SET_KEYNAME(I18N_NOOP2("keyboard-key-name", "F%1"));
 }
 
 QString KuitSemanticsPrivate::format (const QString &text,
@@ -1317,40 +1353,30 @@ QString KuitSemanticsPrivate::modifyTagText (Kuit::TagVar tag,
                                              Kuit::FmtVar fmt) const
 {
     // numctx < 1 means that the number is not in numeric-id context.
-    if (tag == Kuit::Tag::NumIntg && numctx < 1) {
-        if (m_numfmtInt == Kuit::Numfmt::US) {
+    if (   (tag == Kuit::Tag::NumIntg || tag == Kuit::Tag::NumReal) \
+        && numctx < 1)
+    {
+        int numfmt = (tag == Kuit::Tag::NumIntg ? m_numfmtInt : m_numfmtReal);
+        switch (numfmt) {
+        case Kuit::Numfmt::US:
             return KuitFormats::toNumberUS(text);
-        }
-        else if (m_numfmtInt == Kuit::Numfmt::Euro) {
+        case Kuit::Numfmt::Euro:
             return KuitFormats::toNumberEuro(text);
-        }
-        else {
+        case Kuit::Numfmt::Euro2:
+            return KuitFormats::toNumberEuro2(text);
+        case Kuit::Numfmt::Euro2ct:
+            return KuitFormats::toNumberEuro2ct(text);
+        default:
             return text;
         }
     }
-
-    // numctx < 1 means that the number is not in numeric-id context.
-    if (tag == Kuit::Tag::NumReal && numctx < 1) {
-        if (m_numfmtReal == Kuit::Numfmt::US) {
-            return KuitFormats::toNumberUS(text);
-        }
-        else if (m_numfmtReal == Kuit::Numfmt::Euro) {
-            return KuitFormats::toNumberEuro(text);
-        }
-        else {
-            return text;
-        }
-    }
-
-    if (tag == Kuit::Tag::Filename) {
+    else if (tag == Kuit::Tag::Filename) {
         return QDir::toNativeSeparators(text);
     }
-
-    if (tag == Kuit::Tag::Shortcut) {
+    else if (tag == Kuit::Tag::Shortcut) {
         return KuitFormats::toKeyCombo(text, m_comboKeyDelim[fmt], m_keyNames);
     }
-
-    if (tag == Kuit::Tag::Interface) {
+    else if (tag == Kuit::Tag::Interface) {
         return KuitFormats::toInterfacePath(text, m_guiPathDelim[fmt]);
     }
 
@@ -1366,7 +1392,10 @@ QString KuitSemanticsPrivate::finalizeVisualText (const QString &final,
 
     QString text = final;
 
-    // Wrap with <qt> tag if rich text.
+    // Wrap with <html> tag if rich text.
+    // *Do not* go with <qt> instead: if the message is later concatenated
+    // to more rich text, <qt> would introduce line break, whereas a
+    // non-top <html> will be just ignored by Qt rich text engine.
     if (fmt == Kuit::Fmt::Rich) {
         QString rich = "<html>" + text + "</html>";
         return rich;
