@@ -48,7 +48,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     bool absPath = !QDir::isRelativePath(entryPath);
 
     // TODO: it makes sense to have a KConstConfigGroup I guess
-    KConfigGroup desktopGroup = const_cast<KDesktopFile*>(config)->desktopGroup();
+    const KConfigGroup desktopGroup = const_cast<KDesktopFile*>(config)->desktopGroup();
     QMap<QString, QString> entryMap = desktopGroup.entryMap();
 
     entryMap.remove("Encoding"); // reserved as part of Desktop Entry Standard
@@ -65,11 +65,6 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     entryMap.remove("Name");
     if ( m_strName.isEmpty() )
     {
-        if (desktopGroup.readEntry( "Exec" ).isEmpty())
-        {
-            m_bValid = false;
-            return;
-        }
         // Try to make up a name.
         m_strName = entryPath;
         int i = m_strName.lastIndexOf('/');
@@ -79,7 +74,7 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
             m_strName = m_strName.left(i);
     }
 
-    m_strType = desktopGroup.readEntry( "Type" );
+    m_strType = config->readType();
     entryMap.remove("Type");
     if ( m_strType.isEmpty() )
     {
@@ -98,6 +93,22 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
         return;
     }
 
+    // NOT readPathEntry, it is not XDG-compliant. Path entries written by
+    // KDE4 will be still treated as such, though.
+    m_strExec = desktopGroup.readEntry( "Exec", QString() );
+    entryMap.remove("Exec");
+
+    if ( m_strType == "Application" ) {
+        // It's an application? Should have an Exec line then, otherwise we can't run it
+        if (m_strExec.isEmpty()) {
+            kWarning(7012) << "The desktop entry file " << entryPath
+                           << " has Type=" << m_strType
+                           << " but no Exec line" << endl;
+            m_bValid = false;
+            return;
+        }
+    }
+
     // In case Try Exec is set, check if the application is available
     if (!config->tryExec()) {
         q->setDeleted( true );
@@ -139,29 +150,27 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     if (pos != -1)
         _name = _name.left(pos);
 
-    m_strExec = desktopGroup.readPathEntry( "Exec" );
-    entryMap.remove("Exec");
-
-    m_strIcon = desktopGroup.readEntry( "Icon" );
+    m_strIcon = config->readIcon();
     entryMap.remove("Icon");
     m_bTerminal = desktopGroup.readEntry( "Terminal", false); // should be a property IMHO
     entryMap.remove("Terminal");
     m_strTerminalOptions = desktopGroup.readEntry( "TerminalOptions" ); // should be a property IMHO
     entryMap.remove("TerminalOptions");
-    m_strPath = desktopGroup.readPathEntry( "Path" );
+    m_strPath = config->readPath();
     entryMap.remove("Path");
     m_strComment = config->readComment();
     entryMap.remove("Comment");
     m_strGenName = config->readGenericName();
     entryMap.remove("GenericName");
     QString _untranslatedGenericName = desktopGroup.readEntryUntranslated( "GenericName" );
-    entryMap.insert("UntranslatedGenericName", _untranslatedGenericName);
+    if (!_untranslatedGenericName.isEmpty())
+        entryMap.insert("UntranslatedGenericName", _untranslatedGenericName);
 
     m_lstKeywords = desktopGroup.readEntry("Keywords", QStringList());
     entryMap.remove("Keywords");
     m_lstKeywords += desktopGroup.readEntry("X-KDE-Keywords", QStringList());
     entryMap.remove("X-KDE-Keywords");
-    categories = desktopGroup.readEntry("Categories", QStringList(), ';');
+    categories = desktopGroup.readXdgListEntry("Categories");
     entryMap.remove("Categories");
     m_strLibrary = desktopGroup.readEntry( "X-KDE-Library" );
     entryMap.remove("X-KDE-Library");
@@ -170,13 +179,17 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     entryMap.remove("ServiceTypes");
     m_lstServiceTypes += desktopGroup.readEntry( "X-KDE-ServiceTypes", QStringList() );
     entryMap.remove("X-KDE-ServiceTypes");
-    m_lstServiceTypes += desktopGroup.readEntry( "MimeType", QStringList(), ';' ); // freedesktop.org standard
+    m_lstServiceTypes += desktopGroup.readXdgListEntry( "MimeType" );
     entryMap.remove("MimeType");
 
     if ( m_strType == "Application" && !m_lstServiceTypes.contains("Application") )
         // Applications implement the service type "Application" ;-)
         m_lstServiceTypes += "Application";
 
+    if (entryMap.contains("Actions")) {
+        parseActions(config, q);
+    }
+
     QString dbusStartupType = desktopGroup.readEntry("X-DBUS-StartupType").toLower();
     //Compatibility
     if( dbusStartupType.isEmpty() && desktopGroup.hasKey("X-DCOP-ServiceType"))
@@ -207,18 +220,52 @@ void KServicePrivate::init( const KDesktopFile *config, KService* q )
     // break BC, so we have to store it in m_mapProps.
 //  qDebug("Path = %s", entryPath.toLatin1().constData());
     QMap<QString,QString>::ConstIterator it = entryMap.begin();
-    for( ; it != entryMap.end();++it)
-    {
-//     qDebug("   Key = %s Data = %s", it.key().toLatin1().data(), it->toLatin1().data());
-        m_mapProps.insert( it.key(), QVariant( *it));
+    for( ; it != entryMap.end();++it) {
+        const QString key = it.key();
+        // do not store other translations like Name[fr]; kbuildsycoca will rerun if we change languages anyway
+        if (!key.contains('[')) {
+            //kDebug(7012) << "  Key =" << key << " Data =" << *it;
+            m_mapProps.insert(key, QVariant(*it));
+        }
+    }
+}
+
+void KServicePrivate::parseActions(const KDesktopFile *config, KService* q)
+{
+    const QStringList keys = config->readActions();
+    if (keys.isEmpty())
+        return;
+
+    QStringList::ConstIterator it = keys.begin();
+    const QStringList::ConstIterator end = keys.end();
+    for ( ; it != end; ++it ) {
+        const QString group = *it;
+        if (group == "_SEPARATOR_") {
+            m_actions.append(KServiceAction(group, QString(), QString(), QString(), false));
+            continue;
+        }
+
+        if (config->hasActionGroup(group)) {
+            const KConfigGroup cg = config->actionGroup(group);
+            if ( !cg.hasKey( "Name" ) || !cg.hasKey( "Exec" ) ) {
+                kWarning(7012) << "The action" << group << "in the desktop file" << q->entryPath()
+                               << "has no Name or no Exec key";
+            } else {
+                m_actions.append(KServiceAction(group,
+                                                cg.readEntry("Name"),
+                                                cg.readEntry("Icon"),
+                                                cg.readEntry("Exec"),
+                                                cg.readEntry("NoDisplay", false)));
+            }
+        } else {
+            kWarning(7012) << "The desktop file" << q->entryPath()
+                           << "references the action" << group << "but doesn't define it";
+        }
     }
 }
 
 void KServicePrivate::load(QDataStream& s)
 {
-    // dummies are here because of fields that were removed, to keep bin compat.
-    // Feel free to re-use, but fields for Applications only (not generic services)
-    // should rather be added to application.desktop
     qint8 def, term;
     qint8 dst, initpref;
 
@@ -234,10 +281,10 @@ void KServicePrivate::load(QDataStream& s)
       >> m_strDesktopEntryName
       >> initpref
       >> m_lstKeywords >> m_strGenName
-      >> categories >> menuId;
+      >> categories >> menuId >> m_actions;
 
-    m_bAllowAsDefault = def;
-    m_bTerminal = term;
+    m_bAllowAsDefault = (bool)def;
+    m_bTerminal = (bool)term;
     m_DBUSStartusType = (KService::DBusStartupType) dst;
     m_initialPreference = initpref;
 
@@ -263,7 +310,7 @@ void KServicePrivate::save(QDataStream& s)
       << m_strDesktopEntryName
       << initpref
       << m_lstKeywords << m_strGenName
-      << categories << menuId;
+      << categories << menuId << m_actions;
 }
 
 ////
@@ -444,6 +491,7 @@ QVariant KServicePrivate::property( const QString& _name, QVariant::Type t ) con
     default:
         // All others
         // For instance properties defined as StringList, like MimeTypes.
+        // XXX This API is accessible only through a friend declaration.
         return KConfigGroup::convertToQVariant(_name.toUtf8().constData(), it->toString().toUtf8(), t);
     }
 }
@@ -827,3 +875,9 @@ QStringList & KService::accessServiceTypes()
     Q_D(KService);
     return d->m_lstServiceTypes;
 }
+
+QList<KServiceAction> KService::actions() const
+{
+    Q_D(const KService);
+    return d->m_actions;
+}
