@@ -21,6 +21,7 @@
 #ifndef KSERVICE_H
 #define KSERVICE_H
 
+#include "kserviceaction.h"
 #include <QtCore/QStringList>
 #include <QtCore/QVariant>
 #include <klibloader.h>
@@ -304,6 +305,11 @@ public:
      */
     bool allowAsDefault() const;
 
+    /**
+     * Returns the actions defined in this desktop file
+     */
+    QList<KServiceAction> actions() const;
+
     /**
      * Checks whether this service can handle several files as
      * startup arguments.
