@@ -27,6 +27,8 @@
 #include <kconfiggroup.h>
 #include <kstandarddirs.h>
 
+#include <QtNetwork/QHostInfo>
+
 KCONFIGGROUP_DECLARE_ENUM_QOBJECT(KConfigTest,Testing)
 KCONFIGGROUP_DECLARE_FLAGS_QOBJECT(KConfigTest,Flags)
 
@@ -41,6 +43,7 @@ QTEST_KDEMAIN_CORE( KConfigTest )
 #define STRINGENTRY5 " "
 #define STRINGENTRY6 ""
 #define UTF8BITENTRY "Hello äöü"
+#define TRANSLATEDSTRINGENTRY1 "bonjour"
 #define BYTEARRAYENTRY QByteArray( "\x00\xff\x7f\x3c abc\x00\x00", 10 )
 #define ESCAPEKEY " []\0017[]==]"
 #define ESCAPEENTRY "[]\170[]]=3=]\\] "
@@ -50,7 +53,9 @@ QTEST_KDEMAIN_CORE( KConfigTest )
 #define RECTENTRY QRect( 10, 23, 5321, 13 )
 #define DATETIMEENTRY QDateTime( QDate( 2002, 06, 23 ), QTime( 12, 55, 40 ) )
 #define STRINGLISTENTRY (QStringList( "Hello," ) << " World")
-#define STRINGLISTEMPTYENTRY (QStringList())
+#define STRINGLISTEMPTYENTRY QStringList()
+#define STRINGLISTJUSTEMPTYELEMENT QStringList(QString())
+#define STRINGLISTEMPTYTRAINLINGELEMENT (QStringList( "Hi" ) << QString())
 #define STRINGLISTESCAPEODDENTRY (QStringList( "Hello\\\\\\" ) << "World")
 #define STRINGLISTESCAPEEVENENTRY (QStringList( "Hello\\\\\\\\" ) << "World")
 #define STRINGLISTESCAPECOMMAENTRY (QStringList( "Hel\\\\\\,\\\\,\\,\\\\\\\\,lo" ) << "World")
@@ -60,6 +65,10 @@ QTEST_KDEMAIN_CORE( KConfigTest )
 #define VARIANTLISTENTRY2 (QVariantList() << POINTENTRY << SIZEENTRY)
 #define HOMEPATH QDir::homePath()+"/foo"
 #define HOMEPATHESCAPE QDir::homePath()+"/foo/$HOME"
+#define SUBGROUPLIST (QStringList() << "SubGroup/3" << "SubGroup1" << "SubGroup2")
+#define PARENTGROUPKEYS (QStringList() << "parentgrpstring")
+#define SUBGROUP3KEYS (QStringList() << "sub3string")
+#define DOLLARGROUP "$i"
 
 void KConfigTest::initTestCase()
 {
@@ -105,11 +114,23 @@ void KConfigTest::initTestCase()
   cg.writeEntry( "dateTimeEntry", DATETIMEENTRY );
   cg.writeEntry( "dateEntry", DATETIMEENTRY.date() );
 
+  KConfigGroup ct = cg;
+  cg = KConfigGroup(&ct, "Nested Group 1");
+  cg.writeEntry("stringentry1", STRINGENTRY1);
+
+  cg = KConfigGroup(&ct, "Nested Group 2");
+  cg.writeEntry( "stringEntry2", STRINGENTRY2 );
+
+  cg = KConfigGroup(&cg, "Nested Group 2.1");
+  cg.writeEntry( "stringEntry3", STRINGENTRY3 );
+
   cg = KConfigGroup(&sc, "List Types" );
   cg.writeEntry( "listOfIntsEntry1", INTLISTENTRY1 );
   cg.writeEntry( "listOfByteArraysEntry1", BYTEARRAYLISTENTRY1 );
   cg.writeEntry( "stringListEntry", STRINGLISTENTRY );
   cg.writeEntry( "stringListEmptyEntry", STRINGLISTEMPTYENTRY );
+  cg.writeEntry( "stringListJustEmptyElement", STRINGLISTJUSTEMPTYELEMENT );
+  cg.writeEntry( "stringListEmptyTrailingElement", STRINGLISTEMPTYTRAINLINGELEMENT );
   cg.writeEntry( "stringListEscapeOddEntry", STRINGLISTESCAPEODDENTRY );
   cg.writeEntry( "stringListEscapeEvenEntry", STRINGLISTESCAPEEVENENTRY );
   cg.writeEntry( "stringListEscapeCommaEntry", STRINGLISTESCAPECOMMAENTRY );
@@ -120,21 +141,10 @@ void KConfigTest::initTestCase()
   cg.writePathEntry( "homepathescape", HOMEPATHESCAPE );
 
   cg = KConfigGroup(&sc, "Enum Types" );
-  cg.writeEntry( "enum-10", Tens );
-
-#ifndef Q_CC_MSVC
-  cg.writeEntry( "enum-100", Hundreds );
-#else
-  cg.writeEntry( "enum-100", Hundreds, KConfig::Normal );
-#endif
-
-  cg.writeEntry( "flags-bit0", Flags(bit0));
-
-#ifndef Q_CC_MSVC
-  cg.writeEntry( "flags-bit0-bit1", bit0|bit1);
-#else
-  cg.writeEntry( "flags-bit0-bit1", Flags(bit0|bit1), KConfig::Normal );
-#endif
+  writeEntry( cg, "enum-10", Tens );
+  writeEntry( cg, "enum-100", Hundreds );
+  writeEntry( cg, "flags-bit0", Flags(bit0));
+  writeEntry( cg, "flags-bit0-bit1", Flags(bit0|bit1) );
 
   cg = KConfigGroup(&sc, "ParentGroup" );
   KConfigGroup cg1(&cg, "SubGroup1" );
@@ -153,6 +163,27 @@ void KConfigTest::initTestCase()
   sg0.writeEntry("WarnOutput", 0);
   sg0.writeEntry("FatalOutput", 0);
   sc1.sync();
+
+  //Setup stuff to test KConfig::addConfigSources()
+  KConfig devcfg("specificrc");
+  KConfigGroup devonlygrp(&devcfg, "Specific Only Group");
+  devonlygrp.writeEntry("ExistingEntry", "DevValue");
+  KConfigGroup devandbasegrp(&devcfg, "Shared Group");
+  devandbasegrp.writeEntry("SomeSharedEntry", "DevValue");
+  devandbasegrp.writeEntry("SomeSpecificOnlyEntry", "DevValue");
+  devcfg.sync();
+  KConfig basecfg("baserc");
+  KConfigGroup basegrp(&basecfg, "Base Only Group");
+  basegrp.writeEntry("ExistingEntry", "BaseValue");
+  KConfigGroup baseanddevgrp(&basecfg, "Shared Group");
+  baseanddevgrp.writeEntry("SomeSharedEntry", "BaseValue");
+  baseanddevgrp.writeEntry("SomeBaseOnlyEntry", "BaseValue");
+  basecfg.sync();
+
+  KConfig gecfg("groupescapetest", KConfig::SimpleConfig);
+  cg = KConfigGroup(&gecfg, DOLLARGROUP);
+  cg.writeEntry( "entry", "doesntmatter" );
+
 }
 
 void KConfigTest::cleanupTestCase()
@@ -216,10 +247,14 @@ void KConfigTest::testSimple()
   KConfig sc2( "kconfigtest" );
   QCOMPARE(sc2.name(), QString("kconfigtest"));
 
+  // make sure groupList() isn't returning something it shouldn't
+  foreach(const QString& group, sc2.groupList())
+      QVERIFY(!group.isEmpty() && group != "<default>");
+
   KConfigGroup sc3( &sc2, "AAA");
 
   QVERIFY( sc3.hasKey( "stringEntry1" ) );
-  QVERIFY( !sc3.entryIsImmutable("stringEntry1") );
+  QVERIFY( !sc3.isEntryImmutable("stringEntry1") );
   QCOMPARE( sc3.readEntry( "stringEntry1" ), QString( STRINGENTRY1 ) );
 
   QVERIFY( !sc3.hasKey( "stringEntry2" ) );
@@ -250,6 +285,66 @@ void KConfigTest::testSimple()
   QCOMPARE( sc3.readEntry( "doubleEntry1", 0.0 ), DOUBLEENTRY );
 }
 
+void KConfigTest::testDefaults()
+{
+    KConfig config("defaulttest", KConfig::NoGlobals);
+#define DEFAULTS "defaulttest.defaults"
+    KConfig defaults(DEFAULTS, KConfig::SimpleConfig);
+
+    const QString Default("Default");
+    const QString NotDefault("Not Default");
+    const QString Value1(STRINGENTRY1);
+    const QString Value2(STRINGENTRY2);
+
+    KConfigGroup group = defaults.group("any group");
+    group.writeEntry("entry1", Default);
+    group.sync();
+
+    group = config.group("any group");
+    group.writeEntry("entry1", Value1);
+    group.writeEntry("entry2", Value2);
+    group.sync();
+
+    config.addConfigSources(QStringList() << KStandardDirs::locateLocal("config", DEFAULTS));
+    config.reparseConfiguration();
+
+    config.setReadDefaults(true);
+    QCOMPARE(group.readEntry("entry1", QString()), Default);
+    QCOMPARE(group.readEntry("entry2", NotDefault), NotDefault); // no default for entry2
+
+    config.setReadDefaults(false);
+    QCOMPARE(group.readEntry("entry1", Default), Value1);
+    QCOMPARE(group.readEntry("entry2", NotDefault), Value2);
+
+    group.revertToDefault("entry1");
+    QCOMPARE(group.readEntry("entry1", QString()), Default);
+    group.revertToDefault("entry2");
+    QCOMPARE(group.readEntry("entry2", QString()), QString());
+#undef DEFAULTS
+}
+
+void KConfigTest::testLocale()
+{
+    KConfig config("kconfigtest.locales", KConfig::SimpleConfig);
+    const QString Translated(TRANSLATEDSTRINGENTRY1);
+    const QString Untranslated(STRINGENTRY1);
+
+    KConfigGroup group = config.group("Hello");
+    group.writeEntry("stringEntry1", Untranslated);
+    config.setLocale("fr");
+    group.writeEntry("stringEntry1", Translated, KConfig::Localized|KConfig::Persistent);
+    config.sync();
+
+    QCOMPARE(group.readEntry("stringEntry1", QString()), Translated);
+    QCOMPARE(group.readEntryUntranslated("stringEntry1"), Untranslated);
+
+    config.setLocale("C"); // strings written in the "C" locale are written as nonlocalized
+    group.writeEntry("stringEntry1", Untranslated, KConfig::Localized|KConfig::Persistent);
+    config.sync();
+
+    QCOMPARE(group.readEntry("stringEntry1", QString()), Untranslated);
+}
+
 void KConfigTest::testLists()
 {
   KConfig sc2( "kconfigtest" );
@@ -258,7 +353,14 @@ void KConfigTest::testLists()
   QCOMPARE( sc3.readEntry( QString("stringListEntry"), QStringList()),
             STRINGLISTENTRY );
 
-  QVERIFY( sc3.hasKey( QString("stringListEmptyEntry") ) );
+  QCOMPARE( sc3.readEntry( QString("stringListEmptyEntry"), QStringList("wrong") ),
+            STRINGLISTEMPTYENTRY );
+
+  QCOMPARE( sc3.readEntry( QString("stringListJustEmptyElement"), QStringList() ),
+            STRINGLISTJUSTEMPTYELEMENT );
+
+  QCOMPARE( sc3.readEntry( QString("stringListEmptyTrailingElement"), QStringList() ),
+            STRINGLISTEMPTYTRAINLINGELEMENT );
 
   QCOMPARE( sc3.readEntry( QString("stringListEscapeOddEntry"), QStringList()),
             STRINGLISTESCAPEODDENTRY );
@@ -283,9 +385,55 @@ void KConfigTest::testPath()
 {
   KConfig sc2( "kconfigtest" );
   KConfigGroup sc3(&sc2, "Path Type");
-  QString p = sc3.readPathEntry("homepath");
+  QString p = sc3.readPathEntry("homepath", QString());
   QCOMPARE( sc3.readPathEntry( "homepath", QString() ), HOMEPATH );
   QCOMPARE( sc3.readPathEntry( "homepathescape", QString() ), HOMEPATHESCAPE );
+
+  {
+      QFile file(KStandardDirs::locateLocal("config", "pathtest"));
+      file.open(QIODevice::WriteOnly|QIODevice::Text);
+      QTextStream out(&file);
+      out.setCodec("UTF-8");
+      out << "[Test Group]" << endl
+              << "homePath=$HOME/foo" << endl
+              << "homePath2=file://$HOME/foo" << endl
+              << "URL[$e]=file://$HOME/foo" << endl
+              << "hostname[$e]=$(hostname)" << endl;
+  }
+  KConfig cf2("pathtest");
+  KConfigGroup group = cf2.group("Test Group");
+  QVERIFY(group.hasKey("homePath"));
+  QCOMPARE(group.readPathEntry("homePath", QString()), HOMEPATH);
+  QVERIFY(group.hasKey("homePath2"));
+  QCOMPARE(group.readPathEntry("homePath2", QString()), QString("file://") + HOMEPATH );
+  QVERIFY(group.hasKey("URL"));
+  QCOMPARE(group.readEntry("URL", QString()), QString("file://") + HOMEPATH );
+#if !defined(Q_OS_WIN32) && !defined(Q_OS_MAC)
+  // I don't know if this will work on windows
+  // This test hangs on OS X
+  QVERIFY(group.hasKey("hostname"));
+  QCOMPARE(group.readEntry("hostname", QString()), QHostInfo::localHostName());
+#endif
+}
+
+void KConfigTest::testPersistenceOfExpandFlagForPath()
+{
+  // This test checks that a path entry starting with $HOME is still flagged
+  // with the expand flag after the config was altered without rewriting the
+  // path entry.
+
+  // 1st step: Open the config, add a new dummy entry and then sync the config
+  // back to the storage.
+  {
+  KConfig sc2( "kconfigtest" );
+  KConfigGroup sc3(&sc2, "Path Type");
+  sc3.writeEntry( "dummy", "dummy" );
+  sc2.sync();
+  }
+
+  // 2nd step: Call testPath() again. Rewriting the config must not break
+  // the testPath() test.
+  testPath();
 }
 
 void KConfigTest::testComplex()
@@ -309,11 +457,11 @@ void KConfigTest::testEnums()
   KConfigGroup sc3(&sc, "Enum Types" );
 
   QCOMPARE( sc3.readEntry( "enum-10" ), QString("Tens"));
-  QVERIFY( sc3.readEntry( "enum-100", Ones) != Ones);
-  QVERIFY( sc3.readEntry( "enum-100", Ones) != Tens);
+  QVERIFY( readEntry( sc3, "enum-100", Ones) != Ones);
+  QVERIFY( readEntry( sc3, "enum-100", Ones) != Tens);
 
   QCOMPARE( sc3.readEntry( "flags-bit0" ), QString("bit0"));
-  QVERIFY( sc3.readEntry( "flags-bit0", Flags() ) == bit0 );
+  QVERIFY( readEntry( sc3, "flags-bit0", Flags() ) == bit0 );
 
   int eid = staticMetaObject.indexOfEnumerator( "Flags" );
   QVERIFY( eid != -1 );
@@ -321,7 +469,30 @@ void KConfigTest::testEnums()
   Flags bitfield = bit0|bit1;
 
   QCOMPARE( sc3.readEntry( "flags-bit0-bit1" ), QString( me.valueToKeys(bitfield) ) );
-  QVERIFY( sc3.readEntry( "flags-bit0-bit1", Flags() ) == bitfield );
+  QVERIFY( readEntry( sc3, "flags-bit0-bit1", Flags() ) == bitfield );
+}
+
+void KConfigTest::testEntryMap()
+{
+    KConfig sc("kconfigtest");
+    KConfigGroup cg(&sc, "Hello");
+    QMap<QString, QString> entryMap = cg.entryMap();
+    qDebug() << entryMap.keys();
+    QCOMPARE(entryMap.value("stringEntry1"), QString(STRINGENTRY1));
+    QCOMPARE(entryMap.value("stringEntry2"), QString(STRINGENTRY2));
+    QCOMPARE(entryMap.value("stringEntry3"), QString(STRINGENTRY3));
+    QCOMPARE(entryMap.value("stringEntry4"), QString(STRINGENTRY4));
+    QVERIFY(!entryMap.contains("stringEntry5"));
+    QVERIFY(!entryMap.contains("stringEntry6"));
+    QCOMPARE(entryMap.value("Test"), QString::fromUtf8(UTF8BITENTRY));
+    QCOMPARE(entryMap.value("bytearrayEntry"), QString::fromUtf8(BYTEARRAYENTRY));
+    QCOMPARE(entryMap.value("emptyEntry"), QString());
+    QVERIFY(entryMap.contains("emptyEntry"));
+    QCOMPARE(entryMap.value("boolEntry1"), QString(BOOLENTRY1?"true":"false"));
+    QCOMPARE(entryMap.value("boolEntry2"), QString(BOOLENTRY2?"true":"false"));
+    QCOMPARE(entryMap.value("keywith=equalsign"), QString(STRINGENTRY1));
+    QCOMPARE(entryMap.value("byteArrayEntry1"), QString(STRINGENTRY1));
+    QCOMPARE(entryMap.value("doubleEntry1"), QString::number(DOUBLEENTRY, 'g', 15));
 }
 
 void KConfigTest::testInvalid()
@@ -397,7 +568,7 @@ void KConfigTest::testChangeGroup()
     KConfigGroup sc3(&sc, "Hello");
     QCOMPARE(sc3.name(), QString("Hello"));
     KConfigGroup newGroup(sc3);
-    newGroup.changeGroup("FooBar");
+    newGroup.changeGroup("FooBar"); // deprecated!
     QCOMPARE(newGroup.name(), QString("FooBar"));
     QCOMPARE(sc3.name(), QString("Hello")); // unchanged
 }
@@ -410,7 +581,14 @@ void KConfigTest::testDelete()
   sc3.deleteEntry("Test");
   QCOMPARE( sc3.readEntry("Test", QString("Fietsbel")), QString("Fietsbel") );
 
+  KConfigGroup ct(&sc, "Complex Types");
+  KConfigGroup ng(&ct, "Nested Group 2");
   sc.deleteGroup("Complex Types");
+  QCOMPARE(sc.group("Complex Types").keyList().count(), 0);
+  QVERIFY(sc.group("Complex Types").exists()); // yep, we deleted it, but it still "exists"...
+  QCOMPARE(ct.group("Nested Group 1").keyList().count(), 0);
+  QCOMPARE(ct.group("Nested Group 2").keyList().count(), 0);
+  QCOMPARE(ng.group("Nested Group 2.1").keyList().count(), 0);
 
   KConfigGroup cg(&sc , "AAA" );
   cg.deleteGroup();
@@ -418,6 +596,23 @@ void KConfigTest::testDelete()
   QVERIFY( sc.entryMap("AAA").isEmpty() );
   QVERIFY( !sc.entryMap("Hello").isEmpty() ); //not deleted group
   QVERIFY( sc.entryMap("FooBar").isEmpty() ); //inexistant group
+
+  // test for entries that are marked as deleted when there is no default
+  KConfig cf("kconfigtest", KConfig::SimpleConfig); // make sure there are no defaults
+  cg = cf.group("Portable Devices");
+  cg.writeEntry("devices|manual|(null)", "whatever");
+  cg.writeEntry("devices|manual|/mnt/ipod", "/mnt/ipod");
+  cf.sync();
+
+  int count=0;
+  foreach(const QByteArray& item, readLines())
+      if (item.startsWith("devices|"))
+          count++;
+  QVERIFY(count == 2);
+  cg.deleteEntry("devices|manual|/mnt/ipod");
+  cf.sync();
+  foreach(const QByteArray& item, readLines())
+      QVERIFY(!item.contains("ipod"));
 }
 
 void KConfigTest::testDefaultGroup()
@@ -508,19 +703,101 @@ void KConfigTest::testEmptyGroup()
 #endif
 }
 
+void KConfigTest::testMerge()
+{
+    KConfig config("mergetest", KConfig::SimpleConfig);
+
+    KConfigGroup cg = config.group("some group");
+    cg.writeEntry("entry", " random entry");
+    cg.writeEntry("another entry", "blah blah blah");
+
+    { // simulate writing by another process
+        QFile file(KStandardDirs::locateLocal("config", "mergetest"));
+        file.open(QIODevice::WriteOnly|QIODevice::Text);
+        QTextStream out(&file);
+        out.setCodec("UTF-8");
+        out << "[Merged Group]" << endl
+                << "entry1=Testing" << endl
+                << "entry2=More Testing" << endl
+                << "[some group]" << endl
+                << "entry[fr]=French" << endl
+                << "entry[es]=Spanish" << endl
+                << "entry[de]=German" << endl;
+    }
+    config.sync();
+
+    {
+        QList<QByteArray> lines;
+        // this is what the file should look like
+        lines << "[Merged Group]\n"
+                << "entry1=Testing\n"
+                << "entry2=More Testing\n"
+                << "\n"
+                << "[some group]\n"
+                << "another entry=blah blah blah\n"
+                << "entry=\\srandom entry\n"
+                << "entry[de]=German\n"
+                << "entry[es]=Spanish\n"
+                << "entry[fr]=French\n";
+        QFile file(KStandardDirs::locateLocal("config", "mergetest"));
+        file.open(QIODevice::ReadOnly|QIODevice::Text);
+        foreach (const QByteArray& line, lines) {
+            QCOMPARE(line, file.readLine());
+        }
+    }
+}
+
+void KConfigTest::testImmutable()
+{
+    {
+        QFile file(KStandardDirs::locateLocal("config", "immutabletest"));
+        file.open(QIODevice::WriteOnly|QIODevice::Text);
+        QTextStream out(&file);
+        out.setCodec("UTF-8");
+        out << "[$i]" << endl
+                << "entry1=Testing" << endl
+            << "[group][$i]" << endl
+            << "[group][subgroup][$i]" << endl;
+    }
+
+    KConfig config("immutabletest", KConfig::SimpleConfig);
+    QVERIFY(config.isGroupImmutable(QByteArray()));
+    KConfigGroup cg = config.group(QByteArray());
+    QVERIFY(cg.isEntryImmutable("entry1"));
+    KConfigGroup cg1 = config.group("group");
+    QVERIFY(cg1.isImmutable());
+    KConfigGroup cg2 = cg1.group("subgroup");
+    QVERIFY(cg2.isImmutable());
+}
+
+void KConfigTest::testGroupEscape()
+{
+    KConfig config("groupescapetest", KConfig::SimpleConfig);
+    QVERIFY( config.group(DOLLARGROUP).exists() );
+}
+
 void KConfigTest::testSubGroup()
 {
     KConfig sc( "kconfigtest" );
     KConfigGroup cg( &sc, "ParentGroup" );
-    QVERIFY(cg.readEntry( "parentgrpstring", "") == QString("somevalue") );
+    QCOMPARE(cg.readEntry( "parentgrpstring", ""), QString("somevalue") );
     KConfigGroup subcg1( &cg, "SubGroup1");
-    QVERIFY(subcg1.readEntry( "somestring", "") == QString("somevalue") );
+    QCOMPARE(subcg1.name(), QString("SubGroup1"));
+    QCOMPARE(subcg1.readEntry( "somestring", ""), QString("somevalue") );
     KConfigGroup subcg2( &cg, "SubGroup2");
-    QVERIFY(subcg2.readEntry( "substring", "") == QString("somevalue") );
+    QCOMPARE(subcg2.name(), QString("SubGroup2"));
+    QCOMPARE(subcg2.readEntry( "substring", ""), QString("somevalue") );
     KConfigGroup subcg3( &cg, "SubGroup/3");
-    QVERIFY(subcg3.readEntry( "sub3string", "") == QString("somevalue") );
-    KConfigGroup subcg4( &subcg3, "3");
-    QVERIFY(!subcg4.exists());
+    QCOMPARE(subcg3.readEntry( "sub3string", ""), QString("somevalue") );
+    QCOMPARE(subcg3.name(), QString("SubGroup/3"));
+
+    QCOMPARE(cg.groupList(), SUBGROUPLIST );
+
+    QCOMPARE(subcg3.keyList(), SUBGROUP3KEYS);
+    QCOMPARE(cg.keyList(), PARENTGROUPKEYS);
+
+    QCOMPARE(QStringList(cg.entryMap().keys()), PARENTGROUPKEYS);
+    QCOMPARE(QStringList(subcg3.entryMap().keys()), SUBGROUP3KEYS);
 }
 
 void KConfigTest::testKdeglobals()
@@ -530,7 +807,7 @@ void KConfigTest::testKdeglobals()
     QVERIFY(sc.forceGlobal());
     QVERIFY(sc.name() == QLatin1String("kdeglobals"));
 
-    KConfig sc2(QString(), KConfig::IncludeGlobals);
+    KConfig sc2("", KConfig::IncludeGlobals);
     QVERIFY(sc2.forceGlobal());
     QVERIFY(sc2.name() == QLatin1String("kdeglobals"));
 
@@ -539,6 +816,39 @@ void KConfigTest::testKdeglobals()
     QVERIFY(sc3.name() == QLatin1String("kdeglobals"));
 }
 
+void KConfigTest::testAddConfigSources()
+{
+    KConfig cf("specificrc");
+
+    cf.addConfigSources(QStringList() << KStandardDirs::locateLocal("config", "baserc"));
+    cf.reparseConfiguration();
+
+    KConfigGroup specificgrp(&cf, "Specific Only Group");
+    QCOMPARE(specificgrp.readEntry("ExistingEntry", ""), QString("DevValue"));
+
+    KConfigGroup sharedgrp(&cf, "Shared Group");
+    QCOMPARE(sharedgrp.readEntry("SomeSpecificOnlyEntry",""), QString("DevValue"));
+    QCOMPARE(sharedgrp.readEntry("SomeBaseOnlyEntry",""), QString("BaseValue"));
+    QCOMPARE(sharedgrp.readEntry("SomeSharedEntry",""), QString("DevValue"));
+
+    KConfigGroup basegrp(&cf, "Base Only Group");
+    QCOMPARE(basegrp.readEntry("ExistingEntry", ""), QString("BaseValue"));
+    basegrp.writeEntry("New Entry Base Only", "SomeValue");
+
+    KConfigGroup newgrp(&cf, "New Group");
+    newgrp.writeEntry("New Entry", "SomeValue");
+
+    cf.sync();
+
+    KConfig plaincfg("specificrc");
+
+    KConfigGroup newgrp2(&plaincfg, "New Group");
+    QCOMPARE(newgrp2.readEntry("New Entry", ""), QString("SomeValue"));
+
+    KConfigGroup basegrp2(&plaincfg, "Base Only Group");
+    QCOMPARE(basegrp2.readEntry("New Entry Base Only", ""), QString("SomeValue"));
+}
+
 void KConfigTest::testKAboutDataOrganizationDomain()
 {
     KAboutData data( "app", 0, ki18n("program"), "version",
