@@ -52,7 +52,7 @@ void KMimeTypeTest::initTestCase()
         group.writeEntry("Type", "Service");
         group.writeEntry("X-KDE-Library", "faketextpart");
         group.writeEntry("ServiceTypes", "KParts/ReadOnlyPart");
-        group.writeEntry("MimeType", "text/plain");
+        group.writeEntry("MimeType", "text/plain;");
         group.writeEntry("InitialPreference",100);
     }
 
@@ -67,7 +67,7 @@ void KMimeTypeTest::initTestCase()
         group.writeEntry("Type", "Service");
         group.writeEntry("X-KDE-Library", "faketextplugin");
         group.writeEntry("ServiceTypes", "KTextEditor/Plugin");
-        group.writeEntry("MimeType", "text/plain");
+        group.writeEntry("MimeType", "text/plain;");
     }
 
     if ( mustUpdateKSycoca ) {
@@ -289,6 +289,7 @@ void KMimeTypeTest::testFindByNameAndContent()
     QByteArray mswordData = "\320\317\021\340\241\261\032\341";
     mime = KMimeType::findByNameAndContent("mswordfile.doc", mswordData);
     QVERIFY( mime );
+    // If you get powerpoint instead, then you're hit by https://bugs.freedesktop.org/show_bug.cgi?id=435
     QCOMPARE( mime->name(), QString::fromLatin1("application/msword") );
 
     // excelfile.xls -> application/vnd.ms-excel. Found by extension.
@@ -320,34 +321,41 @@ void KMimeTypeTest::testFindByNameAndContent()
     QCOMPARE( mime->name(), QString::fromLatin1("application/x-php") );
 }
 
-void KMimeTypeTest::testFindByContent()
+void KMimeTypeTest::testFindByContent_data()
 {
-    KMimeType::Ptr mime;
-
-    QByteArray textData = "Hello world";
-    mime = KMimeType::findByContent(textData);
-    QVERIFY( mime );
-    QCOMPARE( mime->name(), QString::fromLatin1("text/plain") );
-
-#if 0 // https://bugs.freedesktop.org/show_bug.cgi?id=11259
-    QByteArray htmlData = "<script>foo</script>";
-    mime = KMimeType::findByContent(htmlData);
-    QVERIFY( mime );
-    QCOMPARE( mime->name(), QString::fromLatin1("text/html") );
+    QTest::addColumn<QByteArray>("data");
+    QTest::addColumn<QString>("expectedMimeType");
+    QTest::newRow("simple text") << QByteArray("Hello world") << "text/plain";
+    QTest::newRow("html: <html>") << QByteArray("<html>foo</html>") << "text/html";
+#if 0 // currently buggy, xml and html have conflicting magic rules, discussed on xdg list.
+    QTest::newRow("html: comment+<html>") << QByteArray("<!--foo--><html>foo</html>") << "text/html";
 #endif
-
-    QByteArray pdfData = "%PDF-";
-    mime = KMimeType::findByContent(pdfData);
-    QVERIFY( mime );
-    QCOMPARE( mime->name(), QString::fromLatin1("application/pdf") );
+#if 0 // https://bugs.freedesktop.org/show_bug.cgi?id=11259
+    QTest::newRow("html: <script>") << QByteArray("<script>foo</script>") << "text/html";
+#endif
+    QTest::newRow("pdf") << QByteArray("%PDF-") << "application/pdf";
 
     QByteArray mswordData = "\320\317\021\340\241\261\032\341";
+    // same as \xD0\xCF\x11\xE0 \xA1\xB1\x1A\xE1
     QVERIFY(KMimeType::isBufferBinaryData(mswordData));
-    mime = KMimeType::findByContent(mswordData);
-    QVERIFY( mime );
     // We have no magic specific to msword data, so finding x-ole-storage is correct.
-    QCOMPARE( mime->name(), QString::fromLatin1("application/x-ole-storage") );
+    // If you get powerpoint instead, then you're hit by https://bugs.freedesktop.org/show_bug.cgi?id=435
+    QTest::newRow("msword") << mswordData << "application/x-ole-storage";
+}
 
+void KMimeTypeTest::testFindByContent()
+{
+    QFETCH(QByteArray, data);
+    QFETCH(QString, expectedMimeType);
+
+    KMimeType::Ptr mime = KMimeType::findByContent(data);
+    QVERIFY( mime );
+    QCOMPARE( mime->name(), expectedMimeType );
+}
+
+void KMimeTypeTest::testFindByFileContent()
+{
+    KMimeType::Ptr mime;
     // Calling findByContent on a directory
     mime = KMimeType::findByFileContent("/");
     QVERIFY( mime );
@@ -555,6 +563,7 @@ void KMimeTypeTest::testPatterns_data()
     QTest::addColumn<QString>("patterns");
     QTest::newRow("mimetype with a single pattern") << "application/pdf" << "*.pdf";
     QTest::newRow("mimetype with multiple patterns") << "application/x-kpresenter" << "*.kpr,*.kpt";
+    QTest::newRow("mimetype with multiple patterns, *.doc added by kde") << "text/plain" << "*.asc,*.txt,*.doc";
     QTest::newRow("mimetype with no patterns") << "application/pkcs7-mime" << QString();
 }
 
