@@ -18,6 +18,7 @@
  **/
 
 #include "kded.h"
+#include "kdedadaptor.h"
 #include "kdedmodule.h"
 
 #include "kresourcelist.h"
@@ -256,7 +257,6 @@ KDEDModule *Kded::loadModule(const QString &obj, bool onDemand)
 
 KDEDModule *Kded::loadModule(const KService::Ptr& s, bool onDemand)
 {
-    KDEDModule *module = 0;
     if (s && !s->library().isEmpty())
     {
         QString obj = s->desktopEntryName();
@@ -273,60 +273,42 @@ KDEDModule *Kded::loadModule(const KService::Ptr& s, bool onDemand)
                 return 0;
             }
         }
-        // get the library loader instance
 
-        KLibLoader *loader = KLibLoader::self();
-
-        QVariant v = s->property("X-KDE-FactoryName", QVariant::String);
-        QString factory = v.isValid() ? v.toString() : QString();
-        if (factory.isEmpty())
-        {
-            // Stay bugward compatible
-            v = s->property("X-KDE-Factory", QVariant::String);
-            factory = v.isValid() ? v.toString() : QString();
-        }
-        if (factory.isEmpty())
-            factory = s->library();
-
-        factory = "create_" + factory;
+        KDEDModule *module = 0;
         QString libname = "kded_"+s->library();
+        KPluginLoader loader(libname);
 
-        KLibrary *lib = loader->library(libname);
-        if (!lib)
-        {
-            kWarning() << "Could not load library. [ "
-                       << loader->lastErrorMessage() << " ]" << endl;
-            libname.prepend("lib");
-            lib = loader->library(libname);
-        }
-        if (lib)
-        {
-            // get the create_ function
+        KPluginFactory *factory = loader.factory();
+        if (!factory) {
+            // kde3 compat
+            QString factoryName = s->property("X-KDE-FactoryName", QVariant::String).toString();
+            if (factoryName.isEmpty())
+                factoryName = s->library();
+            factoryName = "create_" + factoryName;
+            KLibrary* lib = KLibLoader::self()->library(libname);
             KDEDModule* (*create)();
-            create = (KDEDModule* (*)())lib->resolveFunction(QFile::encodeName(factory));
-
+            create = (KDEDModule* (*)())lib->resolveFunction(QFile::encodeName(factoryName));
             if (create)
-            {
-                // create the module
                 module = create();
-                if (module)
-                {
-                    module->setModuleName(obj);
-                    m_modules.insert(obj, module);
-                    m_libs.insert(obj, lib);
-                    connect(module, SIGNAL(moduleDeleted(KDEDModule *)), SLOT(slotKDEDModuleRemoved(KDEDModule *)));
-                    kDebug(7020) << "Successfully loaded module '" << obj << "'\n";
-                    return module;
-                }
+            if (!module) {
+                kWarning() << "Could not load library. [ "
+                           << loader.errorString() << " ]";
             }
-            loader->unloadLibrary(libname);
+        } else {
+            // create the module
+            module = factory->create<KDEDModule>(this);
         }
-        else
-        {
-            kWarning() << "Could not load library. [ "
-                       << loader->lastErrorMessage() << " ]" << endl;
+        if (module) {
+            module->setModuleName(obj);
+            m_modules.insert(obj, module);
+            //m_libs.insert(obj, lib);
+            connect(module, SIGNAL(moduleDeleted(KDEDModule *)), SLOT(slotKDEDModuleRemoved(KDEDModule *)));
+            kDebug(7020) << "Successfully loaded module" << obj;
+            return module;
+        } else {
+            kDebug(7020) << "Could not load module" << obj;
+            //loader.unload();
         }
-        kDebug(7020) << "Could not load module '" << obj << "'\n";
     }
     return 0;
 }
@@ -336,7 +318,7 @@ bool Kded::unloadModule(const QString &obj)
   KDEDModule *module = m_modules.value(obj, 0);
   if (!module)
      return false;
-  kDebug(7020) << "Unloading module '" << obj << "'\n";
+  kDebug(7020) << "Unloading module" << obj;
   m_modules.remove(obj);
   delete module;
   return true;
@@ -350,9 +332,9 @@ QStringList Kded::loadedModules()
 void Kded::slotKDEDModuleRemoved(KDEDModule *module)
 {
   m_modules.remove(module->moduleName());
-  KLibrary *lib = m_libs.take(module->moduleName());
-  if (lib)
-     lib->unload();
+  //KLibrary *lib = m_libs.take(module->moduleName());
+  //if (lib)
+  //   lib->unload();
 }
 
 void Kded::slotApplicationRemoved(const QString &name, const QString &oldOwner,
@@ -507,7 +489,7 @@ void Kded::update(const QString& )
 {
   if (!m_recreateBusy)
   {
-    m_pTimer->start( 2000 );
+    m_pTimer->start( 10000 );
   }
 }
 
@@ -537,6 +519,9 @@ void Kded::readDirectory( const QString& _path )
   if ( m_pDirWatch->contains( path ) ) // Already seen this one?
      return;
 
+  m_pDirWatch->addDir(path,KDirWatch::WatchFiles|KDirWatch::WatchSubDirs);          // add watch on this dir
+  return; // KDirWatch now claims to also support recursive watching
+#if 0
   QDir d( _path, QString(), QDir::Unsorted, QDir::Readable | QDir::Executable | QDir::Dirs | QDir::Hidden );
   // set QDir ...
 
@@ -545,11 +530,9 @@ void Kded::readDirectory( const QString& _path )
   //                           Setting dirs
   //************************************************************************
 
-  m_pDirWatch->addDir(path);          // add watch on this dir
-
   if ( !d.exists() )                            // exists&isdir?
   {
-    kDebug(7020) << QString("Does not exist! (%1)").arg(_path);
+    kDebug(7020) << "Does not exist:" << _path;
     return;                             // return false
   }
 
@@ -571,6 +554,7 @@ void Kded::readDirectory( const QString& _path )
 
      readDirectory( file );      // yes, dive into it.
   }
+#endif
 }
 
 /*
@@ -639,7 +623,7 @@ KUpdateD::KUpdateD()
           path += '/';
 
        if (!m_pDirWatch->contains(path))
-          m_pDirWatch->addDir(path);
+          m_pDirWatch->addDir(path,KDirWatch::WatchFiles|KDirWatch::WatchSubDirs);
     }
 }
 
@@ -735,52 +719,6 @@ void KBuildsycocaAdaptor::recreate(const QDBusMessage &msg)
    Kded::self()->recreate(msg);
 }
 
-KdedAdaptor::KdedAdaptor(QObject *parent)
-   : QDBusAbstractAdaptor(parent)
-{
-}
-
-bool KdedAdaptor::loadModule(const QString &module)
-{
-   return Kded::self()->loadModule(module, false) != 0;
-}
-
-bool KdedAdaptor::unloadModule(const QString &module)
-{
-   return Kded::self()->unloadModule(module);
-}
-
-void KdedAdaptor::registerWindowId(qlonglong windowId, const QDBusMessage &msg)
-{
-   Kded::self()->registerWindowId(windowId, msg.service());
-}
-
-void KdedAdaptor::unregisterWindowId(qlonglong windowId, const QDBusMessage &msg)
-{
-   Kded::self()->unregisterWindowId(windowId, msg.service());
-}
-
-QStringList KdedAdaptor::loadedModules()
-{
-   return Kded::self()->loadedModules();
-}
-
-void KdedAdaptor::reconfigure()
-{
-   KGlobal::config()->reparseConfiguration();
-   Kded::self()->initModules();
-}
-
-void KdedAdaptor::loadSecondPhase()
-{
-   Kded::self()->loadSecondPhase();
-}
-
-void KdedAdaptor::quit()
-{
-   QCoreApplication::instance()->quit();
-}
-
 class KDEDApplication : public KUniqueApplication
 {
 public:
@@ -805,9 +743,9 @@ public:
 
 extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 {
-     KAboutData aboutData( "kded" /*don't change this one to kded4! dbus registration should be org.kde.kded etc.*/, 
+     KAboutData aboutData( "kded" /*don't change this one to kded4! dbus registration should be org.kde.kded etc.*/,
         "kdelibs4", ki18n("KDE Daemon"),
-        "$Id: kded.cpp 702581 2007-08-20 21:33:16Z thiago $",
+        "$Id: kded.cpp 736326 2007-11-13 23:24:13Z lunakl $",
         ki18n("KDE Daemon - triggers Sycoca database updates when needed"));
 
      KCmdLineOptions options;
@@ -836,13 +774,13 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
         checkStamps = cg.readEntry("CheckFileStamps", true);
         runBuildSycoca();
         runKonfUpdate();
-        exit(0);
+        return 0;
      }
 
      if (!KUniqueApplication::start())
      {
         fprintf(stderr, "KDE Daemon (kded) already running.\n");
-        exit(0);
+        return 0;
      }
 
      // Thiago: reenable if such a thing exists in QtDBus in the future
