@@ -135,8 +135,8 @@ namespace KStandardAction
     Zoom, Redisplay,
 
     // Go Menu
-    Up, Back, Forward, Home, Prior, Next, Goto, GotoPage, GotoLine,
-    FirstPage, LastPage,
+    Up, Back, Forward, Home /*Home page*/, Prior, Next, Goto, GotoPage, GotoLine,
+    FirstPage, LastPage, DocumentBack, DocumentForward,
 
     // Bookmarks Menu
     AddBookmark, EditBookmarks,
@@ -153,7 +153,7 @@ namespace KStandardAction
     Help, HelpContents, WhatsThis, ReportBug, AboutApp, AboutKDE,
     TipofDay,
 
-    // Another settings menu item
+    // Other standard actions
     ConfigureNotifications,
     FullScreen,
     Clear,
@@ -452,6 +452,16 @@ namespace KStandardAction
    */
   KDEUI_EXPORT KAction *lastPage(const QObject *recvr, const char *slot, QObject *parent);
 
+  /**
+   * Move back (document style menu).
+   */
+  KDEUI_EXPORT KAction *documentBack(const QObject *recvr, const char *slot, QObject *parent);
+
+  /**
+   * Move forward (document style menu).
+   */
+  KDEUI_EXPORT KAction *documentForward(const QObject *recvr, const char *slot, QObject *parent);
+
   /**
    * Add the current page to the bookmarks tree.
    */
