@@ -107,6 +107,7 @@ void KDialogPrivate::appendButton(KDialog::ButtonCode key, const KGuiItem &item)
       role = QDialogButtonBox::HelpRole;
       break;
     case KDialog::Default:
+    case KDialog::Reset:
       role = QDialogButtonBox::ResetRole;
       break;
     case KDialog::Ok:
@@ -116,10 +117,10 @@ void KDialogPrivate::appendButton(KDialog::ButtonCode key, const KGuiItem &item)
       role = QDialogButtonBox::ApplyRole;
       break;
     case KDialog::Try:
-    case KDialog::Close:
     case KDialog::Yes:
       role = QDialogButtonBox::YesRole;
       break;
+    case KDialog::Close:
     case KDialog::Cancel:
       role = QDialogButtonBox::RejectRole;
       break;
@@ -210,6 +211,8 @@ void KDialog::setButtons( ButtonCodes buttonMask )
     d->appendButton( Help, KStandardGuiItem::help() );
   if ( buttonMask & Default )
     d->appendButton( Default, KStandardGuiItem::defaults() );
+  if ( buttonMask & Reset )
+    d->appendButton( Reset, KStandardGuiItem::reset() );
   if ( buttonMask & User3 )
     d->appendButton( User3, KGuiItem() );
   if ( buttonMask & User2 )
@@ -292,6 +295,11 @@ void KDialog::setMainWidget( QWidget *widget )
 {
     Q_D(KDialog);
   d->mMainWidget = widget;
+  QLayout* layout = d->mMainWidget->layout();
+  if (layout) {
+    // Avoid double-margin problem
+    layout->setMargin(0);
+  }
   d->setupLayout();
 }
 
@@ -528,7 +536,7 @@ void KDialog::resizeLayout( QLayout *layout, int margin, int spacing ) //static
 static QRect screenRect( QWidget *widget, int screen )
 {
   QDesktopWidget *desktop = QApplication::desktop();
-  KConfig gc( "kdeglobals", KConfig::CascadeConfig );
+  KConfig gc( "kdeglobals", KConfig::NoGlobals );
   KConfigGroup cg(&gc, "Windows" );
   if ( desktop->isVirtualDesktop() &&
        cg.readEntry( "XineramaEnabled", true ) &&
@@ -838,10 +846,10 @@ void KDialog::setDetailsWidgetVisible( bool visible )
       if ( layout() )
         layout()->setEnabled( false );
 
-      adjustSize();
-
       d->mDetailsWidget->show();
 
+      adjustSize();
+
       if ( layout() ) {
         layout()->activate();
         layout()->setEnabled( true );
@@ -919,6 +927,9 @@ void KDialog::slotButtonClicked( int button )
     case Default:
       emit defaultClicked();
       break;
+    case Reset:
+      emit resetClicked();
+      break;
     case Details:
       setDetailsWidgetVisible( !d->mDetailsVisible );
       break;
