@@ -83,10 +83,15 @@ enum ColumnDesignation {
 };
 
 
-enum myRoles {
+enum MyRoles {
 	ShortcutRole = Qt::UserRole,
-	DefaultShortcutRole,
-	ItemPointerRole = Qt::UserRole + 2342
+	DefaultShortcutRole
+};
+
+
+enum ItemTypes {
+	NonActionItem = 0,
+	ActionItem = 1
 };
 
 
@@ -134,13 +139,11 @@ public:
 
 	void initGUI( KShortcutsEditor::ActionTypes actionTypes, KShortcutsEditor::LetterShortcuts allowLetterShortcuts );
 	void appendToView( uint nList, const QString &title = QString() );
-
-	static void readGlobalKeys( QMap< QString, KShortcut >& map );
-
-	KShortcutsEditorItem *itemFromIndex(const QModelIndex &index);
-
+	//used in appendToView
 	QTreeWidgetItem *findOrMakeItem(QTreeWidgetItem *parent, const QString &name);
 
+	static KShortcutsEditorItem *itemFromIndex(QTreeWidget *const w, const QModelIndex &index);
+
 	//helper functions for conflict resolution
 	bool stealShortcut(KShortcutsEditorItem *item, unsigned int column, const QKeySequence &seq);
 	void wontStealStandardShortcut(KStandardShortcut::StandardShortcut sa, const QKeySequence &seq);
@@ -168,7 +171,16 @@ public:
 };
 
 
-KShortcutsEditorDelegate::KShortcutsEditorDelegate(QAbstractItemView *parent, bool allowLetterShortcuts)
+//a gross hack to make a protected method public
+class QTreeWidgetHack : public QTreeWidget
+{
+public:
+	QTreeWidgetItem *itemFromIndex(const QModelIndex &index) const
+		{ return QTreeWidget::itemFromIndex(index); }
+};
+
+
+KShortcutsEditorDelegate::KShortcutsEditorDelegate(QTreeWidget *parent, bool allowLetterShortcuts)
  : KExtendableItemDelegate(parent),
    m_allowLetterShortcuts(allowLetterShortcuts),
    m_editor(0)
@@ -189,7 +201,7 @@ KShortcutsEditorDelegate::KShortcutsEditorDelegate(QAbstractItemView *parent, bo
 	QApplication::style()->drawPrimitive( QStyle::PE_IndicatorArrowDown, &option, &p );
 	setContractIcon( pixmap );
 
-	connect(parent, SIGNAL(clicked(QModelIndex)), this, SLOT(itemActivated(QModelIndex)));
+	connect(parent, SIGNAL(activated(QModelIndex)), this, SLOT(itemActivated(QModelIndex)));
 }
 
 
@@ -208,11 +220,12 @@ void KShortcutsEditorDelegate::itemActivated(QModelIndex index)
 	const QAbstractItemModel *model = index.model();
 	if (!model)
 		return;
-	KShortcutsEditorItem *item = model->data(index, ItemPointerRole)
-	                                    .value<KShortcutsEditorItem *>();
+	//As per our constructor our parent *is* a QTreeWidget
+	QTreeWidget *view = static_cast<QTreeWidget *>(parent());
 
+	KShortcutsEditorItem *item = KShortcutsEditorPrivate::itemFromIndex(view, index);
 	if (!item) {
-		//a non-leaf item
+		//that probably was a non-leaf (type() !=ActionItem) item
 		return;
 	}
 
@@ -220,8 +233,6 @@ void KShortcutsEditorDelegate::itemActivated(QModelIndex index)
 	if (column == Name) {
 		index = model->index(index.row(), LocalPrimary, index.parent());
 		column = LocalPrimary;
-		//As per KExtendableItemDelegate's constructor our parent *is* an abstract itemview
-		QAbstractItemView *view = static_cast<QAbstractItemView *>(parent());
 		view->selectionModel()->select(index, QItemSelectionModel::ClearAndSelect);
 	}
 
@@ -229,9 +240,9 @@ void KShortcutsEditorDelegate::itemActivated(QModelIndex index)
 		//we only want maximum ONE extender open at any time.
 		if (m_editingIndex.isValid()) {
 			QModelIndex idx = model->index(m_editingIndex.row(), Name, m_editingIndex.parent());
-			KShortcutsEditorItem *oldItem = model->data(idx, ItemPointerRole)
-			                                       .value<KShortcutsEditorItem *>();
-			Q_ASSERT(oldItem);
+			KShortcutsEditorItem *oldItem = KShortcutsEditorPrivate::itemFromIndex(view, idx);
+			Q_ASSERT(oldItem); //here we really expect nothing but a real KShortcutsEditorItem
+
 			oldItem->m_isNameBold = false;
 			contractItem(m_editingIndex);
 		}
@@ -262,14 +273,30 @@ void KShortcutsEditorDelegate::itemActivated(QModelIndex index)
 		extendItem(m_editor, index);
 
 	} else {
+		//the item is extended, and clicking on it again closes it
 		item->m_isNameBold = false;
 		contractItem(index);
+		view->selectionModel()->select(index, QItemSelectionModel::Clear);
 		m_editingIndex = QModelIndex();
 		m_editor = 0;
 	}
 }
 
 
+//slot
+void KShortcutsEditorDelegate::hiddenBySearchLine(QTreeWidgetItem *item, bool hidden)
+{
+	if (!hidden || !item) {
+		return;
+	}
+	QTreeWidget *view = static_cast<QTreeWidget *>(parent());
+	QTreeWidgetItem *editingItem = KShortcutsEditorPrivate::itemFromIndex(view, m_editingIndex);
+	if (editingItem == item) {
+		itemActivated(m_editingIndex); //this will *close* the item's editor because it's already open
+	}
+}
+
+
 //Prevent clicks in the empty part of the editor widget from closing the editor
 //because they would propagate to the itemview and be interpreted as a click in
 //an item's rect. That in turn would lead to an itemActivated() call, closing
@@ -426,6 +453,13 @@ KShortcutsEditor::~KShortcutsEditor()
 	delete d;
 }
 
+void KShortcutsEditor::clearCollections()
+{
+	d->ui.list->clear();
+	d->actionCollections.clear();
+	QTimer::singleShot(0, this, SLOT(resizeColumns()));
+}
+
 void KShortcutsEditor::addCollection(KActionCollection *collection, const QString &title)
 {
 	d->actionCollections.append(collection);
@@ -463,6 +497,7 @@ void KShortcutsEditor::addCollection(KActionCollection *collection, const QStrin
 			delete hier[l];
 	}
 
+	d->ui.list->sortItems(0, Qt::AscendingOrder);
 	QTimer::singleShot(0, this, SLOT(resizeColumns()));
 }
 
@@ -502,9 +537,7 @@ void KShortcutsEditorPrivate::initGUI( KShortcutsEditor::ActionTypes types, KSho
 
 	ui.setupUi(q);
 	ui.searchFilter->searchLine()->setTreeWidget(ui.list); // Plug into search line
-	ui.list->header()->setStretchLastSection(false);
 	ui.list->header()->setResizeMode(QHeaderView::ResizeToContents);
-	ui.list->header()->setResizeMode(0, QHeaderView::Stretch);
 	ui.list->header()->hideSection(GlobalAlternate);  //not expected to be very useful
 	ui.list->header()->hideSection(ShapeGesture);  //mouse gestures didn't make it in time...
 	ui.list->header()->hideSection(RockerGesture);
@@ -527,13 +560,22 @@ void KShortcutsEditorPrivate::initGUI( KShortcutsEditor::ActionTypes types, KSho
 	//TODO listen to changes to global shortcuts
 	QObject::connect(delegate, SIGNAL(shortcutChanged(QVariant, const QModelIndex &)),
 	                 q, SLOT(capturedShortcut(QVariant, const QModelIndex &)));
+	//hide the editor widget chen its item becomes hidden
+	QObject::connect(ui.searchFilter->searchLine(), SIGNAL(hiddenChanged(QTreeWidgetItem *, bool)),
+	                 delegate, SLOT(hiddenBySearchLine(QTreeWidgetItem *, bool)));
+	ui.searchFilter->setFocus();
 }
 
 
-//QTreeWidget::itemFromIndex is protected because... uhm... well, it just is.
-KShortcutsEditorItem *KShortcutsEditorPrivate::itemFromIndex(const QModelIndex &index)
+//static
+KShortcutsEditorItem *KShortcutsEditorPrivate::itemFromIndex(QTreeWidget *const w,
+                                                             const QModelIndex &index)
 {
-	return ui.list->model()->data(index, ItemPointerRole).value<KShortcutsEditorItem *>();
+	QTreeWidgetItem *item = static_cast<QTreeWidgetHack *>(w)->itemFromIndex(index);
+	if (item && item->type() == ActionItem) {
+		return static_cast<KShortcutsEditorItem *>(item);
+	}
+	return 0;
 }
 
 
@@ -544,7 +586,7 @@ QTreeWidgetItem *KShortcutsEditorPrivate::findOrMakeItem(QTreeWidgetItem *parent
 		if (child->text(0) == name)
 			return child;
 	}
-	QTreeWidgetItem *ret = new QTreeWidgetItem(parent);
+	QTreeWidgetItem *ret = new QTreeWidgetItem(parent, NonActionItem);
 	ret->setText(0, name);
 	ui.list->expandItem(ret);
 	ret->setFlags(ret->flags() & ~Qt::ItemIsSelectable);
@@ -559,7 +601,8 @@ void KShortcutsEditorPrivate::capturedShortcut(const QVariant &newShortcut, cons
 	if (!index.isValid())
 		return;
 	int column = index.column();
-	KShortcutsEditorItem *item = itemFromIndex(index);
+	KShortcutsEditorItem *item = itemFromIndex(ui.list, index);
+	Q_ASSERT(item);
 
 	if (column >= LocalPrimary && column <= GlobalAlternate)
 		changeKeyShortcut(item, column, newShortcut.value<QKeySequence>());
@@ -591,7 +634,9 @@ void KShortcutsEditorPrivate::changeKeyShortcut(KShortcutsEditorItem *item, uint
         bool conflict = false;
 		KShortcutsEditorItem *otherItem = 0;
 		for (QTreeWidgetItemIterator it(ui.list); (*it) && !conflict; ++it) {
-			if ((*it)->childCount())
+			//when moving around a shortcut in the same action item the intent is clear
+			//and we don't need to ask about conflicts
+			if ((*it)->childCount() || (*it == item))
 				continue;
 
 			otherItem = static_cast<KShortcutsEditorItem *>(*it);
@@ -635,7 +680,7 @@ void KShortcutsEditorPrivate::changeShapeGesture(KShortcutsEditorItem *item, con
 
 		//search for conflicts
 		for (QTreeWidgetItemIterator it(ui.list); (*it); ++it) {
-			if (!(*it)->parent())
+			if (!(*it)->parent() || (*it == item))
 				continue;
 
 			otherItem = static_cast<KShortcutsEditorItem *>(*it);
@@ -668,7 +713,7 @@ void KShortcutsEditorPrivate::changeRockerGesture(KShortcutsEditorItem *item, co
 		KShortcutsEditorItem *otherItem;
 
 		for (QTreeWidgetItemIterator it(ui.list); (*it); ++it) {
-			if (!(*it)->parent())
+			if (!(*it)->parent() || (*it == item))
 				continue;
 
 			otherItem = static_cast<KShortcutsEditorItem *>(*it);
@@ -790,7 +835,7 @@ void KShortcutsEditor::allDefault()
 
 //---------------------------------------------------
 KShortcutsEditorItem::KShortcutsEditorItem(QTreeWidgetItem *parent, KAction *action)
-	: QTreeWidgetItem(parent)
+	: QTreeWidgetItem(parent, ActionItem)
 	, m_action(action)
 	, m_isNameBold(false)
 	, m_oldLocalShortcut(0)
@@ -816,7 +861,7 @@ QVariant KShortcutsEditorItem::data(int column, int role) const
 	case Qt::DisplayRole:
 		switch(column) {
 		case Name:
-			return m_action->text().remove('&');
+			return i18nc("@item:intable Action name in shortcuts configuration", "%1", m_action->text().remove('&'));
 		case LocalPrimary:
 		case LocalAlternate:
 		case GlobalPrimary:
@@ -839,7 +884,7 @@ QVariant KShortcutsEditorItem::data(int column, int role) const
 	case Qt::WhatsThisRole:
 		return m_action->whatsThis();
 	case Qt::ToolTipRole:
-		return m_action->toolTip();
+		return i18nc("@info:tooltip Action tooltip in shortcuts configuration", "%1", m_action->toolTip());
 	case Qt::FontRole:
 		if (column == Name && m_isNameBold) {
 			QFont modifiedFont = treeWidget()->font();
@@ -853,9 +898,6 @@ QVariant KShortcutsEditorItem::data(int column, int role) const
 		else
 			return true;
 //the following are custom roles, defined in this source file only
-	case ItemPointerRole:
-		return QVariant::fromValue(const_cast<KShortcutsEditorItem *>(this));
-
 	case ShortcutRole:
 		switch(column) {
 		case LocalPrimary:
@@ -1025,7 +1067,8 @@ void KShortcutsEditorItem::undoChanges()
 		m_action->setShortcut(*m_oldLocalShortcut);
 
 	if (m_oldGlobalShortcut)
-		m_action->setGlobalShortcut(*m_oldGlobalShortcut);
+		m_action->setGlobalShortcut(*m_oldGlobalShortcut, KAction::ActiveShortcut,
+		                            KAction::NoAutoloading);
 
 	if (m_oldShapeGesture)
 		m_action->setShapeGesture(*m_oldShapeGesture);
