@@ -56,6 +56,9 @@
 #include <assert.h>
 #include <kconfiggroup.h>
 
+// The following define exists because the Qt SVG renderer needs
+// to be improved. This will be removed soon. (ereslibre)
+#undef KDE_QT_SVG_RENDERER_FIXED
 
 //#define NO_LAZYLOAD_ICONTHEME
 
@@ -194,10 +197,12 @@ public:
     int lastIconType; // see KIconLoader::type
     int lastIconThreshold; // see KIconLoader::threshold
     QList<KIconThemeNode *> links;
-    bool extraDesktopIconsLoaded :1;
     QHash<QString, KSvgRenderer*> svgRenderers;
     KIconCache* mIconCache;
-    bool mIconThemeInited;
+    bool extraDesktopIconsLoaded :1;
+    // lazy loading: initIconThemes() is only needed when the "links" list is needed
+    // mIconThemeInited is used inside initIconThemes() to init only once
+    bool mIconThemeInited :1;
     QString appname;
 
     KIconLoader *q;
@@ -227,11 +232,18 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
 
     int count = 0;
     foreach (const QString& overlay, overlays) {
+        // Ensure empty strings fill up a emblem spot
+        // Needed when you have several emblems to ensure they're always painted
+        // at the same place, even if one is not here
+        if (overlay.isEmpty()) {
+            ++count;
+            continue;
+        }
+
         //TODO: should we pass in the kstate? it results in a slower
         //      path, and perhaps emblems should remain in the default state
         //      anyways?
-        QPixmap pixmap = iconLoader->loadIcon("emblem-" + overlay,
-                                              group, overlaySize, state, QStringList(), 0, true);
+        QPixmap pixmap = iconLoader->loadIcon(overlay, group, overlaySize, state, QStringList(), 0, true);
 
         if (pixmap.isNull()) {
             continue;
@@ -242,16 +254,20 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
         case 0:
             // bottom left corner
             startPoint = QPoint(2, iconSize - overlaySize - 2);
+            break;
         case 1:
             // bottom right corner
             startPoint = QPoint(iconSize - overlaySize - 2,
                                 iconSize - overlaySize - 2);
+            break;
         case 2:
             // top right corner
-            startPoint = QPoint(2, iconSize - overlaySize - 2);
+            startPoint = QPoint(iconSize - overlaySize - 2, 2);
+            break;
         case 3:
             // top left corner
             startPoint = QPoint(2, 2);
+            break;
         }
 
         painter.drawPixmap(startPoint, pixmap);
@@ -331,6 +347,7 @@ KIconLoader::KIconLoader(const KComponentData &componentData, QObject* parent)
 void KIconLoader::reconfigure( const QString& _appname, KStandardDirs *_dirs )
 {
     delete d;
+    d = new KIconLoaderPrivate(this);
     d->init( _appname, _dirs );
 }
 
@@ -394,7 +411,7 @@ bool KIconLoaderPrivate::initIconThemes()
         // If mpThemeRoot isn't 0 then initing has succeeded
         return (mpThemeRoot != 0);
     }
-    kDebug(264) ;
+    kDebug(264);
     mIconThemeInited = true;
 
     // Add the default theme and its base themes to the theme tree
@@ -470,7 +487,7 @@ KIconLoader::~KIconLoader()
 
 void KIconLoader::addAppDir(const QString& appname)
 {
-    const_cast<KIconLoader*>(this)->d->initIconThemes();
+    d->initIconThemes();
 
     d->mpDirs->addResourceType("appicon", "data", appname + "/pics/");
     // ################## KDE4: consider removing the toolbar directory
@@ -526,7 +543,7 @@ void KIconLoader::addExtraDesktopThemes()
 {
     if ( d->extraDesktopIconsLoaded ) return;
 
-    const_cast<KIconLoader*>(this)->d->initIconThemes();
+    d->initIconThemes();
 
     QStringList list;
     QStringList icnlibs = KGlobal::dirs()->resourceDirs("icon");
@@ -582,30 +599,12 @@ bool KIconLoader::extraDesktopThemesAdded() const
 
 QString KIconLoaderPrivate::removeIconExtension(const QString &name) const
 {
-    int extensionLength=0;
-
-    QString ext = name.right(4);
-
-    if (ext == ".png" || ext == ".xpm" )
-      extensionLength=4;
-    else
-    {
-        static const QString &svgz_ext = KGlobal::staticQString(".svgz");
-        if (name.endsWith(svgz_ext))
-            extensionLength=5;
-        else if (ext == ".svg")
-            extensionLength=4;
+    if (name.endsWith(".png") || name.endsWith(".xpm") || name.endsWith(".svg")) {
+        return name.left(name.length() - 4);
+    } else if (name.endsWith(".svgz")) {
+        return name.left(name.length() - 5);
     }
 
-    if ( extensionLength > 0 )
-    {
-#ifndef NDEBUG
-        kDebug(264) << "Application " << KGlobal::mainComponent().componentName()
-                     << " loads icon " << name << " with extension." << endl;
-#endif
-
-        return name.left(name.length() - extensionLength);
-    }
     return name;
 }
 
@@ -615,9 +614,39 @@ K3Icon KIconLoaderPrivate::findMatchingIcon(const QString& name, int size) const
     const_cast<KIconLoaderPrivate*>(this)->initIconThemes();
 
     K3Icon icon;
-    const char * const ext[4] = { ".png", ".svgz", ".svg", ".xpm" };
 
-    /* To follow the XDG icon theme and icon naming specifications,
+// The following code has been commented out because the Qt SVG renderer needs
+// to be improved. If you are going to change/remove some code from this part,
+// please contact me before (ereslibre@kde.org), or kde-core-devel@kde.org. (ereslibre)
+#ifdef KDE_QT_SVG_RENDERER_FIXED
+    const char * ext1[4] = { ".png", ".svgz", ".svg", ".xpm" };
+    const char * ext2[4] = { ".svgz", ".svg", ".png", ".xpm" };
+    const char ** ext;
+
+    if (size == KIconLoader::SizeSmall ||
+        size == KIconLoader::SizeSmallMedium ||
+        size == KIconLoader::SizeMedium ||
+        size == KIconLoader::SizeLarge ||
+        size == KIconLoader::SizeHuge ||
+        size == KIconLoader::SizeEnormous)
+    {
+        ext = ext1; // size is standard, give preference to PNG over SVG when searching
+    }
+    else
+    {
+        ext = ext2; // size is non-standard, give preference to SVG over PNG when searching
+    }
+
+    /* If size parameter is a standard one, that means:
+
+           - KIconLoader::SizeSmall
+           - KIconLoader::SizeSmallMedium
+           - KIconLoader::SizeMedium
+           - KIconLoader::SizeLarge
+           - KIconLoader::SizeHuge
+           - KIconLoader::SizeEnormous
+
+       To follow the XDG icon theme and icon naming specifications,
        the order in which we look for an icon is:
 
        png, svgz, svg, xpm exact match
@@ -637,8 +666,36 @@ K3Icon KIconLoaderPrivate::findMatchingIcon(const QString& name, int size) const
        less specific fallbacks in this next theme
        (...)
 
+       and so on.
+
+       If size parameter is a non-standard one, then we give more preference to
+       SVG format since drawing SVG's gives better quality and despite being
+       slower than resizing a PNG image, the cases where non-standard sizes are
+       asked are very rare. For non-standard sizes what we have is:
+
+       svgz, svg, png, xpm exact match
+       svgz, svg, png, xpm best match
+       less specific fallback in this theme: svgz, svg, png, xpm exact match
+                                             svgz, svg, png, xpm best match
+       even less specific fallback in this theme: [same order]
+       (...)
+
+       next theme in inheritance tree: svgz, svg, png, xpm exact match
+                                       svgz, svg, png, xpm best match
+       less specific fallbacks in this next theme
+       (...)
+
+       next theme in inheritance tree: svgz, svg, png, xpm exact match
+                                       svgz, svg, png, xpm best match
+       less specific fallbacks in this next theme
+       (...)
+
        and so on.
        */
+#else
+    const char * const ext[4] = { ".png", ".svgz", ".svg", ".xpm" };
+#endif
+
     foreach(KIconThemeNode *themeNode, links)
     {
         QStringList nameParts = name.split("-");
@@ -646,6 +703,10 @@ K3Icon KIconLoaderPrivate::findMatchingIcon(const QString& name, int size) const
 
         while (!nameParts.isEmpty())
         {
+// The following code has been commented out because the Qt SVG renderer needs
+// to be improved. If you are going to change/remove some code from this part,
+// please contact me before (ereslibre@kde.org), or kde-core-devel@kde.org. (ereslibre)
+#ifdef KDE_QT_SVG_RENDERER_FIXED
             for (int i = 0 ; i < 4 ; i++)
             {
                 icon = themeNode->theme->iconPath(currentName + ext[i], size, KIconLoader::MatchExact);
@@ -659,6 +720,18 @@ K3Icon KIconLoaderPrivate::findMatchingIcon(const QString& name, int size) const
                 if (icon.isValid())
                     return icon;
             }
+#else
+            for (int i = 0 ; i < 4 ; i++)
+            {
+                icon = themeNode->theme->iconPath(currentName + ext[i], size, KIconLoader::MatchExact);
+                if (icon.isValid())
+                    return icon;
+
+                icon = themeNode->theme->iconPath(currentName + ext[i], size, KIconLoader::MatchBest);
+                if (icon.isValid())
+                    return icon;
+            }
+#endif
 
             nameParts.removeLast();
             currentName = nameParts.join("-");
@@ -686,12 +759,23 @@ inline QString KIconLoaderPrivate::unknownIconPath( int size ) const
 QString KIconLoader::iconPath(const QString& _name, int group_or_size,
                               bool canReturnNull) const
 {
-    if (!const_cast<KIconLoader*>(this)->d->initIconThemes()) {
+    if (!d->initIconThemes()) {
         return QString();
     }
 
-    if (!QDir::isRelativePath(_name))
+    if (_name.isEmpty()
+#ifdef Q_OS_WIN
+        || (_name.length() > 1 &&
+            (_name[0].isLetter() && _name[1] == QLatin1Char(':') ||
+             _name[0] == '/'     && _name[1] == '/' ||
+             _name[0] == '\\'    && _name[1] == '\\')))
+#else
+        || _name[0] == '/')
+#endif
+    {
+        // we have either an absolute path or nothing to work with
         return _name;
+    }
 
     QString name = d->removeIconExtension( _name );
 
@@ -765,8 +849,9 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
     QString name = _name;
     QString path;
     QPixmap pix;
-    QString key;
-    bool absolutePath=false, favIconOverlay=false;
+    bool unknownIcon = false;
+    bool absolutePath = false;
+    bool favIconOverlay = false;
 
     // Special case for absolute path icons.
     if (name.startsWith("favicons/"))
@@ -774,20 +859,32 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
        favIconOverlay = true;
        name = KStandardDirs::locateLocal("cache", name+".png");
     }
-    if (!QDir::isRelativePath(name)) absolutePath=true;
+
+    if (!name.isEmpty()
+#ifdef Q_WS_WIN
+        && !QDir::isRelativePath(name))
+#else
+        && name[0] == '/')
+#endif
+    {
+        absolutePath = true;
+    }
 
     static const QString &str_unknown = KGlobal::staticQString("unknown");
 
     // Special case for "User" icons.
     if (group == KIconLoader::User)
     {
-        key = "$kicou_";
-        key += QString::number(size); key += '_';
-        key += name;
+        QString key;
+        key.reserve(200);
+        key.append("$kicou_");
+        key.append(name).append('_').append(QString::number(size));
+        key.append(overlays.join("_")); // krazy:exclude=doublequote_chars
+
         if (d->mIconCache->find(key, pix, path_store)) {
             //kDebug(264) << "KIL: " << "found the icon from KIC";
             return pix;
-        } else if (!const_cast<KIconLoader*>(this)->d->initIconThemes()) {
+        } else if (!d->initIconThemes()) {
             return pix;  // null pixmap
         }
 
@@ -795,6 +892,10 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
                         iconPath(name, KIconLoader::User, canReturnNull);
         if (path.isEmpty())
         {
+            if (!canReturnNull) {
+                kWarning(264) << "No such icon" << _name;
+                unknownIcon = true;
+            }
             if (canReturnNull)
                 return pix;
             // We don't know the desired size: use small
@@ -815,6 +916,12 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
             img=img.scaled(size,size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
 
         pix = QPixmap::fromImage(img);
+        d->drawOverlays(this, KIconLoader::Desktop, state, pix, overlays);
+#ifndef NDEBUG
+        // do not insert 'unknown' icons into cache in debug mode;
+        // thus warnings are displayed every time a broken icon has been requested
+        if (!unknownIcon)
+#endif
         d->mIconCache->insert(key, pix, path);
         return pix;
     }
@@ -856,27 +963,29 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
 
     // Generate a unique cache key for the icon.
 
-    key = "$kico_";
-    key += name; key += '_';
-    key += QString::number(size); key += '_';
+    QString key;
+    key.reserve(100);
+    key.append("$kico_");
+    key.append(name).append('_').append(QString::number(size));
 
     QString overlayKey = overlays.join("_"); // krazy:exclude=doublequote_chars
     QString noEffectKey = key + overlayKey;
 
     if (group >= 0)
     {
-        key += d->mpEffect.fingerprint(group, state);
+        key.append(d->mpEffect.fingerprint(group, state));
         if (d->mpGroups[group].dblPixels)
-            key += QLatin1String(":dblsize");
-    } else
-        key += QLatin1String("noeffect");
+            key.append(QLatin1String(":dblsize"));
+    } else {
+        key.append(QLatin1String("noeffect"));
+    }
     key.append(overlayKey);
 
     // Is the icon in the cache?
     if (d->mIconCache->find(key, pix, path_store)) {
         //kDebug() << "KIL: " << "found icon from KIC";
         return pix;
-    } else if (!const_cast<KIconLoader*>(this)->d->initIconThemes()) {
+    } else if (!d->initIconThemes()) {
         return pix; // null pixmap
     }
 
@@ -898,7 +1007,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
         else
         {
             if (!name.isEmpty())
-                icon = d->findMatchingIcon(favIconOverlay ? QString("www") : name, size);
+                icon = d->findMatchingIcon(favIconOverlay ? QString("text-html") : name, size);
 
             if (!icon.isValid())
             {
@@ -906,9 +1015,13 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
                 if (!name.isEmpty()) {
                     pix = loadIcon(name, KIconLoader::User, size, state, overlays, path_store, true);
                 }
-                if (!pix.isNull() || canReturnNull)
+                if (!pix.isNull() || canReturnNull) {
+                    d->mIconCache->insert(key, pix, path);
                     return pix;
+                }
 
+                kWarning(264) << "No such icon" << _name;
+                unknownIcon = true;
                 icon = d->findMatchingIcon(str_unknown, size);
                 if (!icon.isValid())
                 {
@@ -981,10 +1094,15 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
     }
 
     // Scale the icon and apply effects if necessary
+#ifndef KDE_QT_SVG_RENDERER_FIXED
+    // The following code needs to be removed after the SVG rendering has been
+    // fixed (please take a look at the comment above). Please do not remove the
+    // #if condition as it marks what needs to be removed (ereslibre)
     if (iconType == KIconLoader::Scalable && size != img->width())
     {
         *img = img->scaled(size, size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
     }
+#endif
     if (iconType == KIconLoader::Threshold && size != img->width())
     {
         if ( abs(size-img->width())>iconThreshold )
@@ -999,26 +1117,39 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
         *img = d->mpEffect.apply(*img, group, state);
     }
 
+    bool addToCache = true;
     if (favIconOverlay)
     {
         QImage favIcon(name, "PNG");
-        int x = img->width() - favIcon.width() - 1,
-            y = img->height() - favIcon.height() - 1;
-        favIcon = favIcon.convertToFormat(QImage::Format_ARGB32);
-        *img = img->convertToFormat(QImage::Format_ARGB32);
-        for( int line = 0;
-             line < favIcon.height();
-             ++line )
+        if (favIcon.isNull())
         {
-            QRgb* fpos = reinterpret_cast< QRgb* >( favIcon.scanLine( line ));
-            QRgb* ipos = reinterpret_cast< QRgb* >( img->scanLine( line + y )) + x;
-            for( int i = 0;
-                 i < favIcon.width();
-                 ++i, ++fpos, ++ipos )
-                *ipos = qRgba( ( qRed( *ipos ) * ( 255 - qAlpha( *fpos )) + qRed( *fpos ) * qAlpha( *fpos )) / 255,
-                               ( qGreen( *ipos ) * ( 255 - qAlpha( *fpos )) + qGreen( *fpos ) * qAlpha( *fpos )) / 255,
-                               ( qBlue( *ipos ) * ( 255 - qAlpha( *fpos )) + qBlue( *fpos ) * qAlpha( *fpos )) / 255,
-                               ( qAlpha( *ipos ) * ( 255 - qAlpha( *fpos )) + qAlpha( *fpos ) * qAlpha( *fpos )) / 255 );
+            // favIcon not there yet, don't try to blend it and don't cache the
+            // result
+            addToCache = false;
+        }
+        else
+        {
+            // Blend favIcon over img.
+            // FIXME: This code should be updated to use modern QPainter
+            // features.
+            int x = img->width() - favIcon.width() - 1,
+                y = img->height() - favIcon.height() - 1;
+            favIcon = favIcon.convertToFormat(QImage::Format_ARGB32);
+            *img = img->convertToFormat(QImage::Format_ARGB32);
+            for( int line = 0;
+                 line < favIcon.height();
+                 ++line )
+            {
+                QRgb* fpos = reinterpret_cast< QRgb* >( favIcon.scanLine( line ));
+                QRgb* ipos = reinterpret_cast< QRgb* >( img->scanLine( line + y )) + x;
+                for( int i = 0;
+                     i < favIcon.width();
+                     ++i, ++fpos, ++ipos )
+                    *ipos = qRgba( ( qRed( *ipos ) * ( 255 - qAlpha( *fpos )) + qRed( *fpos ) * qAlpha( *fpos )) / 255,
+                                   ( qGreen( *ipos ) * ( 255 - qAlpha( *fpos )) + qGreen( *fpos ) * qAlpha( *fpos )) / 255,
+                                   ( qBlue( *ipos ) * ( 255 - qAlpha( *fpos )) + qBlue( *fpos ) * qAlpha( *fpos )) / 255,
+                                   ( qAlpha( *ipos ) * ( 255 - qAlpha( *fpos )) + qAlpha( *fpos ) * qAlpha( *fpos )) / 255 );
+            }
         }
     }
 
@@ -1028,7 +1159,16 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
 
     delete img;
 
-    d->mIconCache->insert(key, pix, path);
+#ifndef NDEBUG
+    if (unknownIcon)
+    {
+        addToCache = false;
+    }
+#endif
+    if (addToCache)
+    {
+        d->mIconCache->insert(key, pix, path);
+    }
     return pix;
 }
 
@@ -1054,7 +1194,7 @@ QString KIconLoader::moviePath(const QString& name, KIconLoader::Group group, in
 {
     if (!d->mpGroups) return QString();
 
-    const_cast<KIconLoader*>(this)->d->initIconThemes();
+    d->initIconThemes();
 
     if ( (group < -1 || group >= KIconLoader::LastGroup) && group != KIconLoader::User )
     {
@@ -1108,7 +1248,7 @@ QStringList KIconLoader::loadAnimated(const QString& name, KIconLoader::Group gr
 
     if (!d->mpGroups) return lst;
 
-    const_cast<KIconLoader*>(this)->d->initIconThemes();
+    d->initIconThemes();
 
     if ((group < -1) || (group >= KIconLoader::LastGroup))
     {
@@ -1157,6 +1297,7 @@ QStringList KIconLoader::loadAnimated(const QString& name, KIconLoader::Group gr
 
 KIconTheme *KIconLoader::theme() const
 {
+    d->initIconThemes();
     if (d->mpThemeRoot) return d->mpThemeRoot->theme;
     return 0L;
 }
@@ -1177,8 +1318,8 @@ QStringList KIconLoader::queryIconsByDir( const QString& iconsDir ) const
 {
     QDir dir(iconsDir);
     QStringList formats;
-    formats << "*.png" << "*.xpm";
-    QStringList lst = dir.entryList(formats, QDir::Files);
+    formats << "*.png" << "*.xpm" << "*.svg" << "*.svgz";
+    const QStringList lst = dir.entryList(formats, QDir::Files);
     QStringList result;
     QStringList::ConstIterator it;
     for (it=lst.begin(); it!=lst.end(); ++it)
@@ -1189,6 +1330,8 @@ QStringList KIconLoader::queryIconsByDir( const QString& iconsDir ) const
 QStringList KIconLoader::queryIconsByContext(int group_or_size,
                                              KIconLoader::Context context) const
 {
+    d->initIconThemes();
+
     QStringList result;
     if (group_or_size >= KIconLoader::LastGroup)
     {
@@ -1228,6 +1371,8 @@ QStringList KIconLoader::queryIconsByContext(int group_or_size,
 
 QStringList KIconLoader::queryIcons(int group_or_size, KIconLoader::Context context) const
 {
+    d->initIconThemes();
+
     QStringList result;
     if (group_or_size >= KIconLoader::LastGroup)
     {
