@@ -48,6 +48,10 @@
 
 #include <kconfiggroup.h>
 
+// The following define exists because the Qt SVG renderer needs
+// to be improved. This will be removed soon. (ereslibre)
+#undef KDE_QT_SVG_RENDERER_FIXED
+
 class KIconTheme::KIconThemePrivate
 {
 public:
@@ -194,10 +198,10 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
     }
 
     d->hidden = cfg.readEntry("Hidden", false);
-    d->example = cfg.readPathEntry("Example");
-    d->screenshot = cfg.readPathEntry("ScreenShot");
+    d->example = cfg.readPathEntry("Example", QString());
+    d->screenshot = cfg.readPathEntry("ScreenShot", QString());
 
-    QStringList dirs = cfg.readPathListEntry("Directories");
+    const QStringList dirs = cfg.readPathEntry("Directories", QStringList());
     for (it=dirs.begin(); it!=dirs.end(); ++it)
     {
         KConfigGroup cg(d->sharedConfig, *it);
@@ -206,9 +210,7 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
 	    if (KStandardDirs::exists(*itDir + *it + '/'))
 	    {
 	        KIconThemeDir *dir = new KIconThemeDir(*itDir + *it, cg);
-	        if (!dir->isValid())
-	        {
-	            kDebug(264) << "Icon directory " << *itDir << " group " << *it << " not valid.\n";
+	        if (!dir->isValid()) {
 	            delete dir;
 	        }
 	        else
@@ -442,7 +444,7 @@ K3Icon KIconTheme::iconPath(const QString& name, int size, KIconLoader::MatchTyp
                 ((size < dir->minSize()) || (size > dir->maxSize())))
               continue;
             if ((dir->type() == KIconLoader::Threshold) &&
-		(abs(dir->size()-size) > dir->threshold()))
+                (abs(dir->size()-size) > dir->threshold()))
                 continue;
         } else
         {
@@ -471,8 +473,7 @@ K3Icon KIconTheme::iconPath(const QString& name, int size, KIconLoader::MatchTyp
              it's a downscale, and we only had upscales befores.
              This is to avoid scaling up unless we have to,
              since that looks very ugly */
-          if ((abs(dw) >= abs(delta)) ||
-              (delta > 0 && dw < 0))
+          if ((abs(dw) >= abs(delta)) || (delta > 0 && dw < 0))
             continue;
         }
 
@@ -480,18 +481,25 @@ K3Icon KIconTheme::iconPath(const QString& name, int size, KIconLoader::MatchTyp
         if (path.isEmpty())
             continue;
         icon.path = path;
+// The following code has been commented out because the Qt SVG renderer needs
+// to be improved. If you are going to change/remove some code from this part,
+// please contact me before (ereslibre@kde.org), or kde-core-devel@kde.org. (ereslibre)
+#ifdef KDE_QT_SVG_RENDERER_FIXED
+        icon.size = size;
+#else
         icon.size = dir->size();
+#endif
         icon.type = dir->type();
-	icon.threshold = dir->threshold();
+        icon.threshold = dir->threshold();
         icon.context = dir->context();
 
         // if we got in MatchExact that far, we find no better
         if (match == KIconLoader::MatchExact)
             return icon;
-	else
+        else
         {
-	    delta = dw;
-	    if (delta==0) return icon; // We won't find a better match anyway
+            delta = dw;
+            if (delta==0) return icon; // We won't find a better match anyway
         }
     }
     return icon;
@@ -637,7 +645,7 @@ KIconThemeDir::KIconThemeDir(const QString& dir, const KConfigGroup &config)
     else if (tmp == "Status")
         mContext = KIconLoader::StatusIcon;
     else {
-        kDebug(264) << "Invalid Context= line for icon theme: " << mDir << "\n";
+        kDebug(264) << "Invalid Context=" << tmp << "line for icon theme: " << mDir << "\n";
         return;
     }
     tmp = config.readEntry("Type");
@@ -648,7 +656,7 @@ KIconThemeDir::KIconThemeDir(const QString& dir, const KConfigGroup &config)
     else if (tmp == "Threshold")
         mType = KIconLoader::Threshold;
     else {
-        kDebug(264) << "Invalid Type= line for icon theme: " << mDir << "\n";
+        kDebug(264) << "Invalid Type=" << tmp << "line for icon theme: " << mDir << "\n";
         return;
     }
     if (mType == KIconLoader::Scalable)
