@@ -39,20 +39,18 @@
 class KTreeWidgetSearchLine::Private
 {
   public:
-    Private( KTreeWidgetSearchLine *_parent )
-      : parent( _parent ),
+    Private( KTreeWidgetSearchLine *_q )
+      : q( _q ),
         caseSensitive( Qt::CaseInsensitive ),
-        activeSearch( false ),
         keepParentsVisible( true ),
         canChooseColumns( true ),
         queuedSearches( 0 )
     {
     }
 
-    KTreeWidgetSearchLine *parent;
+    KTreeWidgetSearchLine *q;
     QList<QTreeWidget *> treeWidgets;
     Qt::CaseSensitivity caseSensitive;
-    bool activeSearch;
     bool keepParentsVisible;
     bool canChooseColumns;
     QString search;
@@ -87,7 +85,7 @@ class QTreeWidgetWorkaround : public QTreeWidget
 
 void KTreeWidgetSearchLine::Private::_k_rowsInserted( const QModelIndex & parentIndex, int start, int end ) const
 {
-  QAbstractItemModel* model = qobject_cast<QAbstractItemModel*>( parent->sender() );
+  QAbstractItemModel* model = qobject_cast<QAbstractItemModel*>( q->sender() );
   if ( !model )
     return;
 
@@ -101,17 +99,22 @@ void KTreeWidgetSearchLine::Private::_k_rowsInserted( const QModelIndex & parent
   if ( !widget )
     return;
 
-  QTreeWidgetWorkaround* widgetWorkaround = static_cast<QTreeWidgetWorkaround *>( widget );
-  for ( int i = start; i <= end; ++i ) {
-    if ( QTreeWidgetItem* item = widgetWorkaround->itemFromIndex( model->index( i, 0, parentIndex ) ) )
-      item->treeWidget()->setItemHidden( item, !parent->itemMatches( item, parent->text() ) );
+  QTreeWidgetWorkaround* widgetW = static_cast<QTreeWidgetWorkaround *>(widget);
+  for (int i = start; i <= end; ++i) {
+      if (QTreeWidgetItem *item = widgetW->itemFromIndex(model->index(i, 0, parentIndex))) {
+          bool newHidden = !q->itemMatches(item, q->text());
+          if (item->isHidden() != newHidden) {
+              item->setHidden(newHidden);
+              emit q->hiddenChanged(item, newHidden);
+          }
+      }
   }
 }
 
 void KTreeWidgetSearchLine::Private::_k_treeWidgetDeleted( QObject *object )
 {
   treeWidgets.removeAll( static_cast<QTreeWidget *>( object ) );
-  parent->setEnabled( treeWidgets.isEmpty() );
+  q->setEnabled( treeWidgets.isEmpty() );
 }
 
 void KTreeWidgetSearchLine::Private::_k_slotColumnActivated( QAction *action )
@@ -150,7 +153,7 @@ void KTreeWidgetSearchLine::Private::_k_slotColumnActivated( QAction *action )
     }
   }
 
-  parent->updateSearch();
+  q->updateSearch();
 }
 
 void KTreeWidgetSearchLine::Private::_k_slotAllVisibleColumns()
@@ -160,7 +163,7 @@ void KTreeWidgetSearchLine::Private::_k_slotAllVisibleColumns()
   else
     searchColumns.clear();
 
-  parent->updateSearch();
+  q->updateSearch();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -170,17 +173,19 @@ void KTreeWidgetSearchLine::Private::_k_slotAllVisibleColumns()
 
 void KTreeWidgetSearchLine::Private::checkColumns()
 {
-  canChooseColumns = parent->canChooseColumnsCheck();
+  canChooseColumns = q->canChooseColumnsCheck();
 }
 
-void KTreeWidgetSearchLine::Private::checkItemParentsNotVisible( QTreeWidget *treeWidget )
+void KTreeWidgetSearchLine::Private::checkItemParentsNotVisible(QTreeWidget *treeWidget)
 {
-  QTreeWidgetItemIterator it( treeWidget );
-
-  for ( ; *it; ++it ) {
-    QTreeWidgetItem *item = *it;
-    item->treeWidget()->setItemHidden( item, !parent->itemMatches( item, search ) );
-  }
+    for (QTreeWidgetItemIterator it(treeWidget); *it; ++it) {
+        QTreeWidgetItem *item = *it;
+        bool newHidden = !q->itemMatches(item, search);
+        if (item->isHidden() != newHidden) {
+            item->setHidden(newHidden);
+            emit q->hiddenChanged(item, newHidden);
+        }
+    }
 }
 
 #include <kvbox.h>
@@ -192,23 +197,21 @@ void KTreeWidgetSearchLine::Private::checkItemParentsNotVisible( QTreeWidget *tr
  *  \return \c true if an item which should be visible is found, \c false if all items found should be hidden. If this function
  *             returns true and \p highestHiddenParent was not 0, highestHiddenParent will have been shown.
  */
-bool KTreeWidgetSearchLine::Private::checkItemParentsVisible( QTreeWidgetItem* item )
+bool KTreeWidgetSearchLine::Private::checkItemParentsVisible(QTreeWidgetItem *item)
 {
-  QTreeWidget* treeWidget = item->treeWidget();
+    bool childMatch = false;
+    for (int i = 0; i < item->childCount(); ++i) {
+        childMatch |= checkItemParentsVisible(item->child(i));
+    }
 
-  bool childMatch = false;
-  for ( int i = 0; i < item->childCount(); ++i )
-    childMatch |= checkItemParentsVisible( item->child( i ) );
+    // Should this item be shown? It should if any children should be, or if it matches.
+    bool newHidden = !childMatch && !q->itemMatches(item, search);
+    if (item->isHidden() != newHidden) {
+        item->setHidden(newHidden);
+        emit q->hiddenChanged(item, newHidden);
+    }
 
-  // Should this item be shown? It should if any children should be, or if it matches.
-  if ( childMatch || parent->itemMatches( item, search ) ) {
-    treeWidget->setItemHidden( item, false );
-    return true;
-  }
-
-  treeWidget->setItemHidden( item, true );
-
-  return false;
+    return !newHidden;
 }
 
 
@@ -216,8 +219,8 @@ bool KTreeWidgetSearchLine::Private::checkItemParentsVisible( QTreeWidgetItem* i
 // public methods
 ////////////////////////////////////////////////////////////////////////////////
 
-KTreeWidgetSearchLine::KTreeWidgetSearchLine( QWidget *parent, QTreeWidget *treeWidget )
-  : KLineEdit( parent ), d( new Private( this ) )
+KTreeWidgetSearchLine::KTreeWidgetSearchLine( QWidget *q, QTreeWidget *treeWidget )
+  : KLineEdit( q ), d( new Private( this ) )
 {
   connect( this, SIGNAL( textChanged( const QString& ) ),
            this, SLOT( _k_queueSearch( const QString& ) ) );
@@ -230,9 +233,9 @@ KTreeWidgetSearchLine::KTreeWidgetSearchLine( QWidget *parent, QTreeWidget *tree
   }
 }
 
-KTreeWidgetSearchLine::KTreeWidgetSearchLine( QWidget *parent,
+KTreeWidgetSearchLine::KTreeWidgetSearchLine( QWidget *q,
                                               const QList<QTreeWidget *> &treeWidgets )
-  : KLineEdit( parent ), d( new Private( this ) )
+  : KLineEdit( q ), d( new Private( this ) )
 {
   connect( this, SIGNAL( textChanged( const QString& ) ),
            this, SLOT( _k_queueSearch( const QString& ) ) );
@@ -520,6 +523,26 @@ bool KTreeWidgetSearchLine::canChooseColumnsCheck()
   return true;
 }
 
+bool KTreeWidgetSearchLine::event(QEvent *event) {
+
+	if (event->type() == QEvent::KeyPress) {
+		QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
+		if(keyEvent->matches(QKeySequence::MoveToNextLine) || keyEvent->matches(QKeySequence::SelectNextLine) ||
+		   keyEvent->matches(QKeySequence::MoveToPreviousLine) || keyEvent->matches(QKeySequence::SelectPreviousLine) ||
+		   keyEvent->matches(QKeySequence::MoveToNextPage) ||  keyEvent->matches(QKeySequence::SelectNextPage) ||
+		   keyEvent->matches(QKeySequence::MoveToPreviousPage) ||  keyEvent->matches(QKeySequence::SelectPreviousPage) ||
+		   keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return)
+		{
+			QTreeWidget *first = d->treeWidgets.first();
+			if(first) {
+				QApplication::sendEvent(first, event);
+				return true;
+			}
+		}
+	}
+	return KLineEdit::event(event);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // protected slots
 ////////////////////////////////////////////////////////////////////////////////
@@ -529,7 +552,7 @@ void KTreeWidgetSearchLine::Private::_k_queueSearch( const QString &_search )
   queuedSearches++;
   search = _search;
 
-  QTimer::singleShot( 200, parent, SLOT( _k_activateSearch() ) );
+  QTimer::singleShot( 200, q, SLOT( _k_activateSearch() ) );
 }
 
 void KTreeWidgetSearchLine::Private::_k_activateSearch()
@@ -537,7 +560,7 @@ void KTreeWidgetSearchLine::Private::_k_activateSearch()
   --queuedSearches;
 
   if ( queuedSearches == 0 )
-    parent->updateSearch( search );
+    q->updateSearch( search );
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -562,7 +585,8 @@ KTreeWidgetSearchLineWidget::KTreeWidgetSearchLineWidget( QWidget *parent, QTree
 {
   d->treeWidget = treeWidget;
 
-  QTimer::singleShot( 0, this, SLOT( createWidgets() ) );
+  createWidgets();
+
 }
 
 KTreeWidgetSearchLineWidget::~KTreeWidgetSearchLineWidget()
@@ -589,6 +613,7 @@ void KTreeWidgetSearchLineWidget::createWidgets()
   layout->setMargin( 0 );
   layout->addWidget( label );
   layout->addWidget( d->searchLine );
+  setFocusProxy( searchLine() );
 }
 
 KTreeWidgetSearchLine *KTreeWidgetSearchLineWidget::searchLine() const
