@@ -27,6 +27,7 @@
 
 #include "klineedit.h"
 #include "klineedit_p.h"
+#include "kdeuiwidgetsproxystyle_p.h"
 
 #include <kconfig.h>
 #include <QtGui/QToolTip>
@@ -42,6 +43,8 @@
 #include <kapplication.h>
 #include <kauthorized.h>
 #include <kicon.h>
+#include <kaction.h>
+#include <kstandardaction.h>
 
 #include <QtCore/QTimer>
 #include <QtGui/QClipboard>
@@ -120,6 +123,32 @@ public:
     QMap<KGlobalSettings::Completion, bool> disableCompletionMap;
 };
 
+// FIXME: Go back to using StyleSheets instead of a proxy style
+// once Qt has been fixed not to mess with widget font when
+// using StyleSheets
+class KLineEditStyle : public KdeUiProxyStyle
+{
+public:
+  KLineEditStyle(KLineEdit *parent) : KdeUiProxyStyle(parent), overlap(0) {}
+ 
+  QRect subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const;
+  
+  int overlap;
+};
+
+QRect KLineEditStyle::subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
+{
+  if (element == SE_LineEditContents)
+  {
+    QRect rect = style()->subElementRect(SE_LineEditContents, option, widget);
+
+    if (option->direction == Qt::LeftToRight) return rect.adjusted(0, 0, -overlap, 0);
+    else return rect.adjusted(overlap, 0, 0, 0);
+  }
+
+  return KdeUiProxyStyle::subElementRect(element, option, widget);
+}
+
 bool KLineEdit::KLineEditPrivate::backspacePerformsCompletion = false;
 bool KLineEdit::KLineEditPrivate::initialized = false;
 
@@ -162,6 +191,10 @@ void KLineEdit::init()
       d->previousHighlightedTextColor=p.color(QPalette::Normal,QPalette::HighlightedText);
     if ( !d->previousHighlightColor.isValid() )
       d->previousHighlightColor=p.color(QPalette::Normal,QPalette::Highlight);
+    
+    QStyle *lineEditStyle = new KLineEditStyle(this);
+    lineEditStyle->setParent(this);
+    setStyle(lineEditStyle);
 }
 
 QString KLineEdit::clickMessage() const
@@ -182,14 +215,14 @@ void KLineEdit::setClearButtonShown(bool show)
 
         updateClearButtonIcon(text());
         updateClearButton();
-        d->clearButton->animateVisible(false);
         connect(this, SIGNAL(textChanged(QString)), this, SLOT(updateClearButtonIcon(QString)));
     } else {
         disconnect(this, SIGNAL(textChanged(QString)), this, SLOT(updateClearButtonIcon(QString)));
         delete d->clearButton;
         d->clearButton = 0;
         d->clickInClear = false;
-        setStyleSheet(QString());
+        KLineEditStyle *lestyle = dynamic_cast<KLineEditStyle *>(style());
+        if (lestyle) lestyle->overlap = 0;
     }
 }
 
@@ -222,9 +255,9 @@ void KLineEdit::updateClearButtonIcon(const QString& text)
     }
 
     if (qApp->isLeftToRight()) {
-        d->clearButton->setPixmap(SmallIcon("clear-left", 0, clearButtonState));
+        d->clearButton->setPixmap(SmallIcon("edit-clear-locationbar-rtl", 0, clearButtonState));
     } else {
-        d->clearButton->setPixmap(SmallIcon("locationbar-erase", 0, clearButtonState));
+        d->clearButton->setPixmap(SmallIcon("edit-clear-locationbar-ltr", 0, clearButtonState));
     }
 }
 
@@ -241,12 +274,9 @@ void KLineEdit::updateClearButton()
 
     if (newButtonSize != d->clearButton->size()) {
         d->clearButton->resize(newButtonSize);
-
-        // we don't need to check for RTL here as Qt does that for us
-        // when it comes to style sheets
-        kDebug() << "setting padding right to " << buttonWidth;
-        setStyleSheet( QString( "QLineEdit { padding-right: %1; }" )
-                       .arg( buttonWidth ) );
+        
+        KLineEditStyle *lestyle = dynamic_cast<KLineEditStyle *>(style());
+        if (lestyle) lestyle->overlap = buttonWidth + frameWidth;
     }
 
     if (qApp->isLeftToRight()) {
@@ -397,7 +427,8 @@ void KLineEdit::setReadOnly(bool readOnly)
 
         if (d->clearButton) {
             d->clearButton->animateVisible(false);
-            setStyleSheet(QString());
+            KLineEditStyle *lestyle = dynamic_cast<KLineEditStyle *>(style());
+            if (lestyle) lestyle->overlap = 0;
         }
     }
     else
@@ -411,8 +442,9 @@ void KLineEdit::setReadOnly(bool readOnly)
 
         if (d->clearButton && !text().isEmpty()) {
             int buttonWidth = d->clearButton->sizeHint().width();
-            setStyleSheet(QString("QLineEdit { padding-right: %1; }").arg(buttonWidth));
             d->clearButton->animateVisible(true);
+            KLineEditStyle *lestyle = dynamic_cast<KLineEditStyle *>(style());
+            if (lestyle) lestyle->overlap = buttonWidth;
         }
     }
 }
@@ -1013,6 +1045,24 @@ void KLineEdit::tripleClickTimeout()
 QMenu* KLineEdit::createStandardContextMenu()
 {
     QMenu *popup = QLineEdit::createStandardContextMenu();
+    
+    if( !isReadOnly() )
+    {
+        QList<QAction *> actionList = popup->actions();
+        enum { UndoAct, RedoAct, CutAct, CopyAct, PasteAct, ClearAct, SelectAllAct, NCountActs }; 
+        QAction *separatorAction = 0L;
+        int idx = actionList.indexOf( actionList[SelectAllAct] ) + 1;
+        if ( idx < actionList.count() )
+            separatorAction = actionList.at( idx );         
+        if ( separatorAction )
+        {
+            KAction *clearAllAction = KStandardAction::clear( this, SLOT( clear() ), this) ;
+            if ( text().isEmpty() )
+                clearAllAction->setEnabled( false );    
+            popup->insertAction( separatorAction, clearAllAction );
+        }
+    }
+
     KIconTheme::assignIconsToContextMenu( KIconTheme::TextEditor, popup->actions () );
 
     // If a completion object is present and the input
