@@ -43,16 +43,13 @@
 #include <QtGui/QSessionManager>
 #include <QtGui/QStyle>
 #include <QtGui/QWidget>
-#if 0
-#include <qdom.h>
-#endif
 
 #include <kaction.h>
 #include <kapplication.h>
 #include <kauthorized.h>
 #include <kconfig.h>
 #include <kdebug.h>
-#include <kedittoolbar.h>
+#include <kdialog.h>
 #include <khelpmenu.h>
 #include <klocale.h>
 #include <kmenubar.h>
@@ -589,17 +586,21 @@ void KMainWindow::saveMainWindowSettings(const KConfigGroup &_cg)
     cg.writeEntry("State", state.toBase64());
     // One day will need to save the version number, but for now, assume 0
 
+    if ( !autoSaveSettings() || configGroup == autoSaveGroup() ) {
+        if(!cg.hasDefault("ToolBarsMovable") && !KToolBar::toolBarsLocked())
+            cg.revertToDefault("ToolBarsMovable");
+        else
+            cg.writeEntry("ToolBarsMovable", KToolBar::toolBarsLocked() ? "Disabled" : "Enabled");    
+    }
+
     int n = 1; // Toolbar counter. toolbars are counted from 1,
     foreach (KToolBar* toolbar, toolBars()) {
         QString group;
-        if (!configGroup.isEmpty())
-        {
-           // Give a number to the toolbar, but prefer a name if there is one,
-           // because there's no real guarantee on the ordering of toolbars
-           group = (toolbar->objectName().isEmpty() ? QString::number(n) : QString(" ")+toolbar->objectName());
-           group.prepend(" Toolbar");
-           group.prepend(configGroup);
-        }
+        // Give a number to the toolbar, but prefer a name if there is one,
+        // because there's no real guarantee on the ordering of toolbars
+        group = (toolbar->objectName().isEmpty() ? QString::number(n) : QString(" ")+toolbar->objectName());
+        group.prepend(" Toolbar");
+        group.prepend(configGroup);
         KConfigGroup groupGrp(cg.config(), group);
         toolbar->saveSettings(groupGrp);
         n++;
@@ -676,6 +677,14 @@ void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg, bool force)
 
     QString configGroup = cg.name();
 
+    if ( !autoSaveSettings() || configGroup == autoSaveGroup() ) {
+        QString entry = cg.readEntry ("ToolBarsMovable", "Enabled");
+        if ( entry == "Disabled" )
+            KToolBar::setToolBarsLocked(true);
+        else
+            KToolBar::setToolBarsLocked(false);
+    }
+
     int n = 1; // Toolbar counter. toolbars are counted from 1,
     foreach (KToolBar* toolbar, toolBars()) {
         QString group;
