@@ -3,9 +3,7 @@
  * Initial implementation:
  *     Copyright (c) 1997 Patrick Dowler <dowler@morgul.fsh.uvic.ca>
  * Rewritten and maintained by:
- *     Copyright (c) 2000 Dirk A. Mueller <mueller@kde.org>
- * KDoubleSpinBox:
- *     Copyright (c) 2002 Marc Mutz <mutz@kde.org>
+ *     Copyright (c) 2000 Dirk Mueller <mueller@kde.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -299,9 +297,9 @@ public:
     QSize        m_sizeSpin;
     
     KIntNumInputPrivate( KIntNumInput *q, int r )
-	: q(q),
+        : q(q),
           referencePoint( r ),
-	  blockRelative( 0 ) {}
+          blockRelative( 0 ) {}
 };
 
 
@@ -341,12 +339,12 @@ void KIntNumInput::init(int val, int _base)
 #ifdef __GNUC__
 #warning KDE4 we NEED to fix the validation of values here
 #endif
-//	if (_base != 10)
+//        if (_base != 10)
 //        m_spin->setValidator(new KIntValidator(this, _base, "KNumInput::KIntValidtr"));
 
     connect(d->m_spin, SIGNAL(valueChanged(int)), SLOT(spinValueChanged(int)));
     connect(this, SIGNAL(valueChanged(int)),
-	    SLOT(slotEmitRelativeValueChanged(int)));
+            SLOT(slotEmitRelativeValueChanged(int)));
 
     setFocusProxy(d->m_spin);
     layout(true);
@@ -377,37 +375,25 @@ void KIntNumInput::slotEmitRelativeValueChanged( int value ) {
     emit relativeValueChanged( double( value ) / double( d->referencePoint ) );
 }
 
-void KIntNumInput::setRange(int lower, int upper, int step, bool slider)
+void KIntNumInput::setSliderEnabled(bool slider)
 {
     K_USING_KNUMINPUT_P(priv);
-  
-    upper = qMax(upper, lower);
-    lower = qMin(upper, lower);
-    d->m_spin->setMinimum(lower);
-    d->m_spin->setMaximum(upper);
-    d->m_spin->setSingleStep(step);
-
-    step = d->m_spin->singleStep(); // maybe QRangeControl didn't like out lineStep?
-
     if(slider) {
-	if (priv->m_slider)
-	    priv->m_slider->setRange(lower, upper);
-	else {
-	    priv->m_slider = new QSlider(Qt::Horizontal, this);
-	    priv->m_slider->setMinimum(lower);
-	    priv->m_slider->setMaximum(upper);
-	    priv->m_slider->setPageStep(step);
-	    priv->m_slider->setValue(d->m_spin->value());
-	    priv->m_slider->setTickPosition(QSlider::TicksBelow);
-	    connect(priv->m_slider, SIGNAL(valueChanged(int)),
-		    d->m_spin, SLOT(setValue(int)));
-	}
+        if (!priv->m_slider) {
+            priv->m_slider = new QSlider(Qt::Horizontal, this);
+            connect(priv->m_slider, SIGNAL(valueChanged(int)),
+                    d->m_spin, SLOT(setValue(int)));
+            priv->m_slider->setTickPosition(QSlider::TicksBelow);
+        }
 
-	// calculate (upper-lower)/10 without overflowing int's:
-        int major = calcDiffByTen( upper, lower );
-	if ( major==0 ) major = step; // #### workaround Qt bug in 2.1-beta4
+        priv->m_slider->setRange(d->m_spin->minimum(), d->m_spin->maximum());
+        priv->m_slider->setPageStep(d->m_spin->singleStep());
+        priv->m_slider->setValue(d->m_spin->value());
 
-        priv->m_slider->setSingleStep(step);
+        // calculate (upper-lower)/10 without overflowing int's:
+        int major = calcDiffByTen( d->m_spin->maximum(), d->m_spin->minimum() );
+
+        priv->m_slider->setSingleStep(d->m_spin->singleStep());
         priv->m_slider->setPageStep(major);
         priv->m_slider->setTickInterval(major);
     }
@@ -415,6 +401,18 @@ void KIntNumInput::setRange(int lower, int upper, int step, bool slider)
         delete priv->m_slider;
         priv->m_slider = 0;
     }
+}
+
+void KIntNumInput::setRange(int lower, int upper, int step)
+{
+    upper = qMax(upper, lower);
+    lower = qMin(upper, lower);
+    d->m_spin->setMinimum(lower);
+    d->m_spin->setMaximum(upper);
+    d->m_spin->setSingleStep(step);
+
+    step = d->m_spin->singleStep(); // maybe QRangeControl didn't like out lineStep?
+    setSliderEnabled(true);
 
     // check that reference point is still inside valid range:
     setReferencePoint( referencePoint() );
@@ -422,10 +420,15 @@ void KIntNumInput::setRange(int lower, int upper, int step, bool slider)
     layout(true);
 }
 
+void KIntNumInput::setRange(int lower, int upper, int step, bool slider)
+{
+    setRange(lower, upper, step);
+    setSliderEnabled(slider);
+}
+
 void KIntNumInput::setMinimum(int min)
 {
-    K_USING_KNUMINPUT_P(priv);
-    setRange(min, d->m_spin->maximum(), d->m_spin->singleStep(), priv->m_slider);
+    setRange(min, d->m_spin->maximum(), d->m_spin->singleStep());
 }
 
 int KIntNumInput::minimum() const
@@ -435,8 +438,7 @@ int KIntNumInput::minimum() const
 
 void KIntNumInput::setMaximum(int max)
 {
-    K_USING_KNUMINPUT_P(priv);
-    setRange(d->m_spin->minimum(), max, d->m_spin->singleStep(), priv->m_slider);
+    setRange(d->m_spin->minimum(), max, d->m_spin->singleStep());
 }
 
 int KIntNumInput::maximum() const
@@ -531,10 +533,10 @@ void KIntNumInput::resizeEvent(QResizeEvent* e)
         w += priv->m_colw2 + 8;
 
         if(priv->m_slider)
-            priv->m_slider->setGeometry(w, h, e->size().width() - w, d->m_sizeSpin.height());
+            priv->m_slider->setGeometry(w, h, e->size().width() - w, d->m_sizeSpin.height() + KDialog::spacingHint());
     }
     else if(priv->m_slider) {
-        priv->m_slider->setGeometry(w, h, e->size().width() - (w + priv->m_colw2 + KDialog::spacingHint()), d->m_sizeSpin.height());
+        priv->m_slider->setGeometry(w, h, e->size().width() - (w + priv->m_colw2 + KDialog::spacingHint()), d->m_sizeSpin.height() + KDialog::spacingHint());
         d->m_spin->setGeometry(w + priv->m_slider->size().width() + KDialog::spacingHint(), h, priv->m_colw2, d->m_sizeSpin.height());
     }
     else {
@@ -601,10 +603,10 @@ void KIntNumInput::setLabel(const QString & label, Qt::Alignment a)
 class KDoubleNumInput::KDoubleNumInputPrivate {
 public:
     KDoubleNumInputPrivate( double r )
-	: spin( 0 ),
-	  referencePoint( r ),
-	  blockRelative ( 0 ) {}
-    KDoubleSpinBox * spin;
+        : spin( 0 ),
+          referencePoint( r ),
+          blockRelative ( 0 ) {}
+    QDoubleSpinBox * spin;
     double referencePoint;
     short blockRelative;
     QSize m_sizeEdit;
@@ -620,7 +622,7 @@ KDoubleNumInput::KDoubleNumInput(QWidget *parent)
 }
 
 KDoubleNumInput::KDoubleNumInput(double lower, double upper, double value, QWidget *parent,
-				 double step, int precision)
+                                 double step, int precision)
     : KNumInput(parent)
     , d( new KDoubleNumInputPrivate( value ) )
 {
@@ -628,8 +630,8 @@ KDoubleNumInput::KDoubleNumInput(double lower, double upper, double value, QWidg
 }
 
 KDoubleNumInput::KDoubleNumInput(KNumInput *below,
-				 double lower, double upper, double value, QWidget *parent,
-				 double step, int precision)
+                                 double lower, double upper, double value, QWidget *parent,
+                                 double step, int precision)
     : KNumInput(parent,below)
     , d( new KDoubleNumInputPrivate( value ) )
 {
@@ -648,15 +650,20 @@ QString KDoubleNumInput::specialValueText() const
 
 
 void KDoubleNumInput::init(double value, double lower, double upper,
-			   double step, int precision )
+                           double step, int precision )
 {
-    d->spin = new KDoubleSpinBox(lower, upper, step, value, this,precision);
+    d->spin = new QDoubleSpinBox(this);
+    d->spin->setRange(lower, upper);
+    d->spin->setSingleStep(step);
+    d->spin->setValue(value);
+    d->spin->setDecimals(precision);
+
     d->spin->setObjectName("KDoubleNumInput::d->spin" );
     setFocusProxy(d->spin);
     connect( d->spin, SIGNAL(valueChanged(double)),
-	     this, SIGNAL(valueChanged(double)) );
+             this, SIGNAL(valueChanged(double)) );
     connect( this, SIGNAL(valueChanged(double)),
-	     this, SLOT(slotEmitRelativeValueChanged(double)) );
+             this, SLOT(slotEmitRelativeValueChanged(double)) );
 
     updateLegacyMembers();
 
@@ -686,6 +693,24 @@ void KDoubleNumInput::sliderMoved(int val)
     d->spin->setValue( mapSliderToSpin( val ) );
 }
 
+void KDoubleNumInput::spinBoxChanged(double val)
+{
+    K_USING_KNUMINPUT_P(priv);
+
+    double spinmin = d->spin->minimum();
+    double spinmax = d->spin->maximum();
+    double slidemin = priv->m_slider->minimum(); // cast int to double to avoid
+    double slidemax = priv->m_slider->maximum(); // overflow in rel denominator
+
+    double rel = ( val - spinmin) / (spinmax - spinmin);
+
+    if (priv->m_slider) {
+        priv->m_slider->blockSignals(true);
+        priv->m_slider->setValue(int(slidemin + rel * (slidemax - slidemin)));
+        priv->m_slider->blockSignals(false);
+    }
+}
+
 void KDoubleNumInput::slotEmitRelativeValueChanged( double value )
 {
     if ( !d->referencePoint ) return;
@@ -708,7 +733,7 @@ QSize KDoubleNumInput::minimumSizeHint() const
         h += 4 + priv->m_sizeLabel.height();
     else
         // label is in the same row as the other widgets
-	h = qMax(h, priv->m_sizeLabel.height() + 2);
+        h = qMax(h, priv->m_sizeLabel.height() + 2);
 
     w = priv->m_slider ? priv->m_slider->sizeHint().width() + 8 : 0;
     w += priv->m_colw1 + priv->m_colw2;
@@ -741,12 +766,12 @@ void KDoubleNumInput::resizeEvent(QResizeEvent* e)
         w += priv->m_colw2 + KDialog::spacingHint();
 
         if(priv->m_slider)
-            priv->m_slider->setGeometry(w, h, e->size().width() - w, d->m_sizeEdit.height());
+            priv->m_slider->setGeometry(w, h, e->size().width() - w, d->m_sizeEdit.height() + KDialog::spacingHint());
     }
     else if(priv->m_slider) {
         priv->m_slider->setGeometry(w, h, e->size().width() -
                                     (priv->m_colw1 + priv->m_colw2 + KDialog::spacingHint()),
-                              d->m_sizeEdit.height());
+                              d->m_sizeEdit.height() + KDialog::spacingHint());
         d->spin->setGeometry(w + priv->m_slider->width() + KDialog::spacingHint(), h,
                              priv->m_colw2, d->m_sizeEdit.height());
     }
@@ -792,42 +817,42 @@ void KDoubleNumInput::setRange(double lower, double upper, double step,
                                                            bool slider)
 {
     K_USING_KNUMINPUT_P(priv);
-  
+
     if( priv->m_slider ) {
-	// don't update the slider to avoid an endless recursion
-	QSpinBox * spin = d->spin;
-	disconnect(spin, SIGNAL(valueChanged(int)),
-		priv->m_slider, SLOT(setValue(int)) );
+        // don't update the slider to avoid an endless recursion
+        QDoubleSpinBox * spin = d->spin;
+        disconnect(spin, SIGNAL(valueChanged(int)),
+                priv->m_slider, SLOT(setValue(int)) );
     }
-    d->spin->setRange( lower, upper, step, d->spin->precision() );
+    d->spin->setRange( lower, upper);
+    d->spin->setSingleStep(step);
 
     if(slider) {
-	// upcast to base type to get the minimum/maximum in int form:
-	QSpinBox * spin = d->spin;
+        // upcast to base type to get the minimum/maximum in int form:
+        QDoubleSpinBox * spin = d->spin;
         int slmax = spin->maximum();
-	int slmin = spin->minimum();
+        int slmin = spin->minimum();
         int slvalue = spin->value();
-	int slstep = spin->singleStep();
+        int slstep = spin->singleStep();
         if (priv->m_slider) {
             priv->m_slider->setRange(slmin, slmax);
-	    priv->m_slider->setSingleStep(slstep);
+            priv->m_slider->setSingleStep(slstep);
             priv->m_slider->setValue(slvalue);
         } else {
             priv->m_slider = new QSlider(Qt::Horizontal, this);
             priv->m_slider->setMinimum(slmin);
             priv->m_slider->setMaximum(slmax);
-            priv->m_slider->setPageStep(slstep);
+            priv->m_slider->setSingleStep(slstep);
             priv->m_slider->setValue(slvalue);
             priv->m_slider->setTickPosition(QSlider::TicksBelow);
-	    // feedback line: when one moves, the other moves, too:
+            // feedback line: when one moves, the other moves, too:
             connect(priv->m_slider, SIGNAL(valueChanged(int)),
                     SLOT(sliderMoved(int)) );
         }
-	connect(spin, SIGNAL(valueChanged(int)),
-			priv->m_slider, SLOT(setValue(int)) );
-	// calculate ( slmax - slmin ) / 10 without overflowing ints:
-	int major = calcDiffByTen( slmax, slmin );
-	if ( !major ) major = slstep; // ### needed?
+        connect(spin, SIGNAL(valueChanged(double)), SLOT(spinBoxChanged(double)) );
+        // calculate ( slmax - slmin ) / 10 without overflowing ints:
+        int major = calcDiffByTen( slmax, slmin );
+        if ( !major ) major = slstep; // ### needed?
         priv->m_slider->setTickInterval(major);
     } else {
         delete priv->m_slider;
@@ -862,7 +887,7 @@ double KDoubleNumInput::maximum() const
     return d->spin->maximum();
 }
 
-double  KDoubleNumInput::value() const
+double KDoubleNumInput::value() const
 {
     return d->spin->value();
 }
@@ -902,16 +927,16 @@ void KDoubleNumInput::setPrefix(const QString &prefix)
     layout(true);
 }
 
-void KDoubleNumInput::setPrecision(int precision)
+void KDoubleNumInput::setDecimals(int decimals)
 {
-    d->spin->setPrecision( precision );
+    d->spin->setDecimals(decimals);
 
     layout(true);
 }
 
-int KDoubleNumInput::precision() const
+int KDoubleNumInput::decimals() const
 {
-    return d->spin->precision();
+    return d->spin->decimals();
 }
 
 void KDoubleNumInput::setSpecialValueText(const QString& text)
@@ -933,289 +958,4 @@ void KDoubleNumInput::setLabel(const QString & label, Qt::Alignment a)
 
 }
 
-// ----------------------------------------------------------------------------
-
-
-class KDoubleSpinBoxValidator : public KDoubleValidator
-{
-public:
-    KDoubleSpinBoxValidator( KDoubleSpinBox *sb,double bottom, double top, int decimals)
-        : KDoubleValidator( bottom, top, decimals, sb), spinBox( sb ) { }
-
-    virtual State validate( QString& str, int& pos ) const;
-
-private:
-    KDoubleSpinBox *spinBox;
-};
-
-QValidator::State KDoubleSpinBoxValidator::validate( QString& str, int& pos ) const
-{
-    QString pref = spinBox->prefix();
-    QString suff = spinBox->suffix();
-    QString suffStriped = suff.trimmed();
-    int overhead = pref.length() + suff.length();
-    State state = Invalid;
-
-    if ( overhead == 0 ) {
-        state = KDoubleValidator::validate( str, pos );
-    } else {
-        bool stripedVersion = false;
-        if ( str.length() >= overhead && str.startsWith(pref)
-             && (str.endsWith(suff)
-                 || (stripedVersion = str.endsWith(suffStriped))) ) {
-            if ( stripedVersion )
-                overhead = pref.length() + suffStriped.length();
-            QString core = str.mid( pref.length(), str.length() - overhead );
-            int corePos = pos - pref.length();
-            state = KDoubleValidator::validate( core, corePos );
-            pos = corePos + pref.length();
-            str.replace( pref.length(), str.length() - overhead, core );
-        } else {
-            state = KDoubleValidator::validate( str, pos );
-            if ( state == Invalid ) {
-                // trimmed(), cf. QSpinBox::interpretText()
-                QString special = spinBox->specialValueText().trimmed();
-                QString candidate = str.trimmed();
-
-                if ( special.startsWith(candidate) ) {
-                    if ( candidate.length() == special.length() ) {
-                        state = Acceptable;
-                    } else {
-                        state = Intermediate;
-                    }
-                }
-            }
-        }
-    }
-    return state;
-}
-
-// We use a kind of fixed-point arithmetic to represent the range of
-// doubles [mLower,mUpper] in steps of 10^(-mPrecision). Thus, the
-// following relations hold:
-//
-// 1. factor = 10^mPrecision
-// 2. basicStep = 1/factor = 10^(-mPrecision);
-// 3. lowerInt = lower * factor;
-// 4. upperInt = upper * factor;
-// 5. lower = lowerInt * basicStep;
-// 6. upper = upperInt * basicStep;
-class KDoubleSpinBox::Private {
-public:
-  Private( int precision=1 )
-    : mPrecision( precision ),
-      mValidator( 0 )
-  {
-  }
-
-  int factor() const {
-    int f = 1;
-    for ( int i = 0 ; i < mPrecision ; ++i ) f *= 10;
-    return f;
-  }
-
-  double basicStep() const {
-    return 1.0/double(factor());
-  }
-
-  int mapToInt( double value, bool * ok ) const {
-    assert( ok );
-    const double f = factor();
-    if ( value > double(INT_MAX) / f ) {
-      kWarning() << "KDoubleSpinBox: can't represent value " << value
-		  << "in terms of fixed-point numbers with precision "
-		  << mPrecision << endl;
-      *ok = false;
-      return INT_MAX;
-    } else if ( value < double(INT_MIN) / f ) {
-      kWarning() << "KDoubleSpinBox: can't represent value " << value
-		  << "in terms of fixed-point numbers with precision "
-		  << mPrecision << endl;
-      *ok = false;
-      return INT_MIN;
-    } else {
-      *ok = true;
-      return int( value * f + ( value < 0 ? -0.5 : 0.5 ) );
-    }
-  }
-
-  double mapToDouble( int value ) const {
-    return double(value) * basicStep();
-  }
-
-  int mPrecision;
-  KDoubleSpinBoxValidator * mValidator;
-};
-
-KDoubleSpinBox::KDoubleSpinBox( QWidget * parent)
-  : QSpinBox( parent), d(new Private())
-{
-  lineEdit()->setAlignment( Qt::AlignRight );
-  updateValidator();
-  connect( this, SIGNAL(valueChanged(int)), SLOT(slotValueChanged(int)) );
-}
-
-KDoubleSpinBox::KDoubleSpinBox( double lower, double upper, double step,
-				double value, QWidget *parent, int precision)
-  : QSpinBox( parent), d(new Private())
-{
-  lineEdit()->setAlignment( Qt::AlignRight );
-  setRange( lower, upper, step, precision );
-  setValue( value );
-  connect( this, SIGNAL(valueChanged(int)), SLOT(slotValueChanged(int)) );
-}
-
-KDoubleSpinBox::~KDoubleSpinBox() {
-  delete d;
-}
-
-bool KDoubleSpinBox::acceptLocalizedNumbers() const {
-  if ( !d->mValidator ) return true; // we'll set one that does;
-                                     // can't do it now, since we're const
-  return d->mValidator->acceptLocalizedNumbers();
-}
-
-void KDoubleSpinBox::setAcceptLocalizedNumbers( bool accept ) {
-  if ( !d->mValidator ) updateValidator();
-  d->mValidator->setAcceptLocalizedNumbers( accept );
-}
-
-void KDoubleSpinBox::setRange( double lower, double upper, double step,
-			       int precision ) {
-  lower = qMin(upper, lower);
-  upper = qMax(upper, lower);
-  setPrecision( precision, true ); // disable bounds checking, since
-  setMinimum( lower );            // it's done in set{Minimum,Maximum}
-  setMaximum( upper );            // anyway and we want lower, upper
-  setSingleStep( step );             // and step to have the right precision
-}
-
-int KDoubleSpinBox::precision() const {
-  return d->mPrecision;
-}
-
-void KDoubleSpinBox::setPrecision( int precision ) {
-    setPrecision( precision, false );
-}
-
-void KDoubleSpinBox::setPrecision( int precision, bool force ) {
-  if ( precision < 1 ) return;
-  if ( !force ) {
-    int maxPrec = maxPrecision();
-    if ( precision > maxPrec )
-      precision = maxPrec;
-  }
-  d->mPrecision = precision;
-  updateValidator();
-}
-
-int KDoubleSpinBox::maxPrecision() const {
-    // INT_MAX must be > maxAbsValue * 10^precision
-    // ==> 10^precision < INT_MAX / maxAbsValue
-    // ==> precision < log10 ( INT_MAX / maxAbsValue )
-    // ==> maxPrecision = floor( log10 ( INT_MAX / maxAbsValue ) );
-    double maxAbsValue = qMax( fabs(minimum()), fabs(maximum()) );
-    if ( maxAbsValue == 0 ) return 6; // return arbitrary value to avoid dbz...
-
-    return int( floor( log10( double(INT_MAX) / maxAbsValue ) ) );
-}
-
-double KDoubleSpinBox::value() const {
-  return d->mapToDouble( base::value() );
-}
-
-void KDoubleSpinBox::setValue( double value ) {
-    if ( value == this->value() ) return;
-    if ( value < minimum() )
-	base::setValue( base::minimum() );
-    else if ( value > maximum() )
-	base::setValue( base::maximum() );
-    else {
-	bool ok = false;
-	base::setValue( d->mapToInt( value, &ok ) );
-	assert( ok );
-    }
-}
-
-double KDoubleSpinBox::minimum() const {
-  return d->mapToDouble( base::minimum() );
-}
-
-void KDoubleSpinBox::setMinimum( double value ) {
-  bool ok = false;
-  int min = d->mapToInt( value, &ok );
-  if ( !ok ) return;
-  base::setMinimum( min );
-  updateValidator();
-}
-
-
-double KDoubleSpinBox::maximum() const {
-  return d->mapToDouble( base::maximum() );
-}
-
-void KDoubleSpinBox::setMaximum( double value ) {
-  bool ok = false;
-  int max = d->mapToInt( value, &ok );
-  if ( !ok ) return;
-  base::setMaximum( max );
-  updateValidator();
-}
-
-double KDoubleSpinBox::singleStep() const {
-  return d->mapToDouble( base::singleStep() );
-}
-
-void KDoubleSpinBox::setSingleStep( double step ) {
-  bool ok = false;
-  if ( step > maximum() - minimum() )
-    base::setSingleStep( 1 );
-  else
-    base::setSingleStep( qMax( d->mapToInt( step, &ok ), 1 ) );
-}
-
-QString KDoubleSpinBox::textFromValue( int value ) const{
-  if ( acceptLocalizedNumbers() )
-    return KGlobal::locale()
-      ->formatNumber( d->mapToDouble( value ), d->mPrecision );
-  else
-    return QString().setNum( d->mapToDouble( value ), 'f', d->mPrecision );
-}
-
-int KDoubleSpinBox::valueFromText( const QString &text ) const{
-  double value;
-  bool ok;
-  if ( acceptLocalizedNumbers() )
-    value = KGlobal::locale()->readNumber( text, &ok );
-  else
-    value = cleanText().toDouble( &ok );
-  if ( !ok ) return 0;
-  if ( value > maximum() )
-    value = maximum();
-  else if ( value < minimum() )
-    value = minimum();
-  return d->mapToInt( value, &ok );
-}
-
-void KDoubleSpinBox::setValidator( const QValidator * ) {
-  // silently discard the new validator. We don't want another one ;-)
-}
-
-void KDoubleSpinBox::slotValueChanged( int value ) {
-  emit valueChanged( d->mapToDouble( value ) );
-}
-
-void KDoubleSpinBox::updateValidator() {
-  if ( !d->mValidator ) {
-    d->mValidator =  new KDoubleSpinBoxValidator(this, minimum(), maximum(), precision());
-    d->mValidator->setObjectName("d->mValidator" );
-#ifdef __GNUC__
-#warning KDE4 we NEED to fix the validation of values here
-#endif
-//    base::setValidator( d->mValidator );
-  } else
-    d->mValidator->setRange( minimum(), maximum(), precision() );
-}
-
-
 #include "knuminput.moc"
