@@ -154,8 +154,8 @@ class KToolBar::Private
     // Default Values.
     int IconSizeDefault;
     Qt::ToolButtonStyle ToolButtonStyleDefault;
-    bool HiddenDefault;
-    bool NewLineDefault;
+    bool HiddenDefault : 1;
+    bool NewLineDefault : 1;
     int OffsetDefault;
     QString PositionDefault;
 
@@ -194,13 +194,13 @@ void KToolBar::Private::init( bool readConfig, bool _honorStyle )
 
   if ( !KAuthorized::authorize( "movable_toolbars" ) )
     parent->setMovable( false );
+  else
+    parent->setMovable( !KToolBar::toolBarsLocked() );
 
   connect( parent, SIGNAL( movableChanged( bool ) ),
            parent, SLOT( slotMovableChanged( bool ) ) );
 
   parent->setAcceptDrops( true );
-
-  parent->setFont( KGlobalSettings::toolBarFont() );
 }
 
 void KToolBar::Private::getAttributes( QString &position, Qt::ToolButtonStyle &toolButtonStyle, int &index ) const
@@ -273,7 +273,7 @@ KMenu *KToolBar::Private::contextMenu()
     contextSize = new KMenu( i18n( "Icon Size" ), context );
     context->addMenu( contextSize );
 
-    contextIconSizes.insert( contextSize->addAction( i18n( "Default" ), parent, SLOT( slotContextIconSize() ) ), 0 );
+    contextIconSizes.insert( contextSize->addAction( i18n( "Default" ), parent, SLOT( slotContextIconSize() ) ), IconSizeDefault );
 
     // Query the current theme for available sizes
     KIconTheme *theme = KIconLoader::global()->theme();
@@ -611,11 +611,6 @@ void KToolBar::Private::slotContextIconSize()
 void KToolBar::Private::slotLockToolBars( bool lock )
 {
   parent->setToolBarsLocked( lock );
-
-  if(lock)
-    contextLockAction->setText(i18n( "Unlock Toolbars" ));
-  else
-    contextLockAction->setText(i18n( "Lock Toolbars" ));
 }
 
 
@@ -664,27 +659,9 @@ bool KToolBar::contextMenuEnabled() const
   return d->enableContext;
 }
 
-QString KToolBar::settingsGroup() const
+void KToolBar::saveSettings( KConfigGroup &cg )
 {
-  QString configGroup;
-  if ( objectName().isEmpty() || d->isMainToolBar() )
-    configGroup = "Toolbar style";
-  else
-    configGroup = QString( objectName() ) + " Toolbar style";
-
-  if ( mainWindow() ) {
-    configGroup.prepend(" ");
-    configGroup.prepend( mainWindow()->objectName() );
-  }
-
-  return configGroup;
-}
-
-void KToolBar::saveSettings( KConfigGroup &_cg )
-{
-  KConfigGroup cg = _cg;
-    if ( cg.name().isEmpty() )
-        cg.changeGroup(settingsGroup());
+    Q_ASSERT(!cg.name().isEmpty());
 
   QString position;
   Qt::ToolButtonStyle ToolButtonStyle;
@@ -722,18 +699,14 @@ void KToolBar::saveSettings( KConfigGroup &_cg )
     // The whole set of indexes has to be saved.
     //kDebug(220) << name() << "                writing index " << index;
 
+    KMainWindow* kmw = mainWindow();
+    QList<KToolBar*> toolbarList = kmw->findChildren<KToolBar*>();
+
     // don't save if there's only one toolbar
-
-    // Don't use kmw->toolBarIterator() because you might
-    // mess up someone else's iterator.  Make the list on your own
-
-    /* FIXME KMainWindow port - no replacement
-    QList<KToolBar*> toolbarList = mainWindow()->findChildren<KToolBar*>();
-
     if ( !kmw || toolbarList.count() > 1 )
         cg.writeEntry("Index", index);
     else
-        cg.revertToDefault("Index");*/
+        cg.revertToDefault("Index");
 
     /* FIXME KMainWindow port - no replacement
     if(!cg.hasDefault("Offset") && offset() == d->OffsetDefault )
@@ -932,11 +905,9 @@ void KToolBar::saveState( QDomElement &current ) const
   current.setAttribute( "toolButtonStyleDefault", d->toolButtonStyleToString( d->ToolButtonStyleDefault ) );
 }
 
-void KToolBar::applySettings( const KConfigGroup &_cg, bool force )
+void KToolBar::applySettings( const KConfigGroup &cg, bool force )
 {
-    KConfigGroup cg = _cg;
-    if ( cg.name().isEmpty() )
-        cg.changeGroup(settingsGroup());
+    Q_ASSERT( !cg.name().isEmpty() );
 
   /*
     Let's explain this a bit more in details.
@@ -983,11 +954,9 @@ void KToolBar::applySettings( const KConfigGroup &_cg, bool force )
   }
 }
 
-void KToolBar::applyAppearanceSettings( KConfigGroup &_cg, bool forceGlobal )
+void KToolBar::applyAppearanceSettings( const KConfigGroup &cg, bool forceGlobal )
 {
-    KConfigGroup cg = _cg;
-    if ( cg.name().isEmpty() )
-        cg.changeGroup(settingsGroup());
+    Q_ASSERT(! cg.name().isEmpty() );
 
   // If we have application-specific settings in the XML file,
   // and nothing in the application's config file, then
@@ -1287,18 +1256,32 @@ bool KToolBar::eventFilter( QObject * watched, QEvent * event )
     QAction* act = tb->actions().first();
     if ( event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease ) {
       QMouseEvent* me = static_cast<QMouseEvent*>( event );
-      if ( me->button() == Qt::MidButton /*&& 
+      if ( me->button() == Qt::MidButton /*&&
            act->receivers(SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)))*/ ) {
-        if ( me->type() == QEvent::MouseButtonPress )  
-          tb->setDown(true); 
+        if ( me->type() == QEvent::MouseButtonPress )
+          tb->setDown(true);
         else {
           tb->setDown(false);
           QMetaObject::invokeMethod(act, "triggered", Qt::DirectConnection,
-               Q_ARG(Qt::MouseButtons, me->button()), 
+               Q_ARG(Qt::MouseButtons, me->button()),
                Q_ARG(Qt::KeyboardModifiers, QApplication::keyboardModifiers() ));
         }
       }
     }
+
+    // CJK languages use more verbose accelerator marker: they add a Latin
+    // letter in parenthesis, and put accelerator on that. Hence, the default
+    // removal of ampersand only may not be enough there, instead the whole
+    // parenthesis construct should be removed. Provide these filtering i18n
+    // messages so that translators can use Transcript for custom removal.
+    if ( event->type() == QEvent::Show || event->type() == QEvent::Paint || event->type() == QEvent::EnabledChange ) {
+      act = tb->defaultAction();
+      if (act) {
+        QString text = act->iconText().isEmpty() ? act->text() : act->iconText();
+        tb->setText(i18nc("@action:intoolbar Text label of toolbar button", "%1", text));
+        tb->setToolTip(i18nc("@info:tooltip Tooltip of toolbar button", "%1", act->toolTip()));
+      }
+    }
   }
 
   found:
@@ -1381,8 +1364,12 @@ void KToolBar::setToolBarsLocked( bool locked )
     KToolBar::Private::s_locked = locked;
 
     foreach ( KMainWindow* mw, KMainWindow::memberList() )
-      foreach ( KToolBar* toolbar, mw->findChildren<KToolBar*>() )
+      foreach ( KToolBar* toolbar, mw->findChildren<KToolBar*>() ) {
         toolbar->d->setLocked( locked );
+        if (toolbar->d->contextLockAction)
+            toolbar->d->contextLockAction->setText(locked ? i18n( "Unlock Toolbars" ) : i18n( "Lock Toolbars" ));
+      }
+
   }
 }
 
