@@ -526,20 +526,10 @@ WId KWindowSystem::transientFor( WId win )
 
 void KWindowSystem::setMainWindow( QWidget* subwindow, WId mainwindow )
 {
+    // This will be Qt::WA_X11BypassTransientForHint in Qt4.4, but the code is already there now.
+    subwindow->setAttribute( Qt::WidgetAttribute( 99 ));
     if( mainwindow != 0 )
-    {
-        /*
-         Grmbl. See QDialog::show(). That should get fixed in Qt somehow.
-        */
-        if( qobject_cast< QDialog* >( subwindow ) != NULL
-            && subwindow->parentWidget() == NULL
-            && qApp->activeWindow() != NULL )
-        {
-            kWarning() << "KWindowSystem::setMainWindow(): There either mustn't be qApp->mainWidget(),"
-                " or the dialog must have a non-NULL parent, otherwise Qt will reset the change. Bummer." << endl;
-        }
         XSetTransientForHint( QX11Info::display(), subwindow->winId(), mainwindow );
-    }
     else
         XDeleteProperty( QX11Info::display(), subwindow->winId(), XA_WM_TRANSIENT_FOR );
 }
@@ -667,7 +657,7 @@ QPixmap KWindowSystem::icon( WId win, int width, int height, bool scale, int fla
         // If the icon is still a null pixmap, load the icon for X applications
         // as a last resort:
         if ( result.isNull() ) {
-            QPixmap pm = KIconLoader::global()->loadIcon( "x", KIconLoader::Small, iconWidth,
+            QPixmap pm = KIconLoader::global()->loadIcon( "xorg", KIconLoader::Small, iconWidth,
                                                           KIconLoader::DefaultState, QStringList(), 0, true );
 	    if( scale && !pm.isNull() )
 		result = QPixmap::fromImage( pm.toImage().scaled( width, height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
