@@ -22,7 +22,6 @@
 #include "kdirmodel.h"
 #include "kdiroperatordetailview_p.h"
 #include "kdirsortfilterproxymodel.h"
-#include "kfileview.h"
 #include "kfileitem.h"
 #include "kfilemetapreview.h"
 #include "kpreviewwidgetbase.h"
@@ -74,6 +73,7 @@
 #include <ktoggleaction.h>
 #include <kactionmenu.h>
 #include <kconfiggroup.h>
+#include <kdeversion.h>
 
 
 template class QHash<QString, KFileItem>;
@@ -92,13 +92,10 @@ protected:
     virtual QStyleOptionViewItem viewOptions() const;
     virtual void dragEnterEvent(QDragEnterEvent* event);
     virtual void mousePressEvent(QMouseEvent *event);
-
-private:
-    QStyleOptionViewItem m_viewOptions;
 };
 
 KDirOperatorIconView::KDirOperatorIconView(QWidget *parent) :
-        QListView(parent)
+    QListView(parent)
 {
     setViewMode(QListView::IconMode);
     setFlow(QListView::TopToBottom);
@@ -109,13 +106,7 @@ KDirOperatorIconView::KDirOperatorIconView(QWidget *parent) :
     setVerticalScrollMode(QListView::ScrollPerPixel);
     setHorizontalScrollMode(QListView::ScrollPerPixel);
 
-    m_viewOptions = QListView::viewOptions();
-    m_viewOptions.showDecorationSelected = true;
-    m_viewOptions.decorationPosition = QStyleOptionViewItem::Left;
-    m_viewOptions.displayAlignment = Qt::AlignLeft | Qt::AlignVCenter;
-    m_viewOptions.decorationSize = QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
-
-    QFontMetrics metrics(m_viewOptions.font);
+    QFontMetrics metrics(viewOptions().font);
     const int fontHeight = metrics.height();
     setGridSize(QSize(fontHeight * 10, fontHeight + 4));
 }
@@ -126,7 +117,13 @@ KDirOperatorIconView::~KDirOperatorIconView()
 
 QStyleOptionViewItem KDirOperatorIconView::viewOptions() const
 {
-    return m_viewOptions;
+    QStyleOptionViewItem viewOptions = QListView::viewOptions();
+    viewOptions.showDecorationSelected = true;
+    viewOptions.decorationPosition = QStyleOptionViewItem::Left;
+    viewOptions.displayAlignment = Qt::AlignLeft | Qt::AlignVCenter;
+    viewOptions.decorationSize = QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
+
+    return viewOptions;
 }
 
 void KDirOperatorIconView::dragEnterEvent(QDragEnterEvent* event)
@@ -139,8 +136,8 @@ void KDirOperatorIconView::dragEnterEvent(QDragEnterEvent* event)
 void KDirOperatorIconView::mousePressEvent(QMouseEvent *event)
 {
     if (!indexAt(event->pos()).isValid()) {
-        const Qt::KeyboardModifiers modifier = QApplication::keyboardModifiers();
-        if (!(modifier & Qt::ShiftModifier) && !(modifier & Qt::ControlModifier)) {
+        const Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+        if (!(modifiers & Qt::ShiftModifier) && !(modifiers & Qt::ControlModifier)) {
             clearSelection();
         }
     }
@@ -185,7 +182,9 @@ public:
     void _k_slotProperties();
     void _k_slotPressed(const QModelIndex&);
     void _k_slotClicked(const QModelIndex&);
+    void _k_slotActivated(const QModelIndex&);
     void _k_slotDoubleClicked(const QModelIndex&);
+    void _k_slotSelectionChanged();
     void _k_openContextMenu(const QPoint&);
     void _k_triggerPreview(const QModelIndex&);
     void _k_cancelPreview();
@@ -266,6 +265,7 @@ KDirOperator::Private::Private(KDirOperator *_parent) :
 KDirOperator::Private::~Private()
 {
     delete itemView;
+    itemView = 0;
 
     // TODO:
     // if (configGroup) {
@@ -275,11 +275,18 @@ KDirOperator::Private::~Private()
     qDeleteAll(backStack);
     qDeleteAll(forwardStack);
     delete preview;
+    preview = 0;
 
-    delete dirLister;
+    delete proxyModel;
+    proxyModel = 0;
+    delete dirModel;
+    dirModel = 0;
+    dirLister = 0; // deleted by KDirModel
     delete configGroup;
+    configGroup = 0;
 
     delete progressDelayTimer;
+    progressDelayTimer = 0;
 }
 
 KDirOperator::KDirOperator(const KUrl& _url, QWidget *parent) :
@@ -370,7 +377,8 @@ KDirLister *KDirOperator::dirLister() const
 
 void KDirOperator::resetCursor()
 {
-    QApplication::restoreOverrideCursor();
+    if (qApp)
+        QApplication::restoreOverrideCursor();
     d->progressBar->hide();
 }
 
@@ -435,6 +443,10 @@ void KDirOperator::setPreviewWidget(KPreviewWidgetBase *w)
     delete d->preview;
     d->preview = w;
 
+    if (w) {
+        d->splitter->addWidget(w);
+    }
+
     KToggleAction *previewAction = static_cast<KToggleAction*>(d->actionCollection->action("preview"));
     previewAction->setEnabled(showPreview);
     previewAction->setChecked(showPreview);
@@ -750,14 +762,14 @@ KIO::CopyJob * KDirOperator::trash(const KFileItemList& items,
                     i18n("<qt>Do you really want to trash\n <b>'%1'</b>?</qt>" ,
                          files.first()),
                     i18n("Trash File"),
-                    KGuiItem(i18nc("to trash", "&Trash"), "edit-trash"),
+                    KGuiItem(i18nc("to trash", "&Trash"), "user-trash"),
                     KStandardGuiItem::cancel(), "AskForTrash");
         } else
             ret = KMessageBox::warningContinueCancelList(parent,
                     i18np("translators: not called for n == 1", "Do you really want to trash these %1 items?", items.count()),
                     files,
                     i18n("Trash Files"),
-                    KGuiItem(i18nc("to trash", "&Trash"), "edit-trash"),
+                    KGuiItem(i18nc("to trash", "&Trash"), "user-trash"),
                     KStandardGuiItem::cancel(), "AskForTrash");
         doIt = (ret == KMessageBox::Continue);
     }
@@ -1108,6 +1120,24 @@ void KDirOperator::activatedMenu(const KFileItem &item, const QPoint &pos)
     d->actionMenu->menu()->exec(pos);
 }
 
+void KDirOperator::changeEvent(QEvent *event)
+{
+    QWidget::changeEvent(event);
+
+    if (event->type() == QEvent::FontChange) {
+        // Only cast when necessary, speeding up things
+        KDirOperatorIconView *iconView = dynamic_cast<KDirOperatorIconView*>(d->itemView);
+
+        if (!iconView) {
+            return;
+        }
+
+        QFontMetrics metrics(font());
+        const int fontHeight = metrics.height();
+        iconView->setGridSize(QSize(fontHeight * 10, fontHeight + 4));
+    }
+}
+
 bool KDirOperator::Private::checkPreviewInternal() const
 {
     QStringList supported = KIO::PreviewJob::supportedMimeTypes();
@@ -1281,6 +1311,8 @@ void KDirOperator::setView(QAbstractItemView *view)
     d->itemView->setMouseTracking(true);
     //d->itemView->setDropOptions(d->dropOptions);
 
+    connect(d->itemView, SIGNAL(activated(const QModelIndex&)),
+            this, SLOT(_k_slotActivated(const QModelIndex&)));
     connect(d->itemView, SIGNAL(pressed(const QModelIndex&)),
             this, SLOT(_k_slotPressed(const QModelIndex&)));
     connect(d->itemView, SIGNAL(clicked(const QModelIndex&)),
@@ -1293,7 +1325,6 @@ void KDirOperator::setView(QAbstractItemView *view)
             this, SLOT(_k_triggerPreview(const QModelIndex&)));
     connect(d->itemView, SIGNAL(viewportEntered()),
             this, SLOT(_k_cancelPreview()));
-
     // assure that the sorting state d->sorting matches with the current action
     const bool descending = d->actionCollection->action("descending")->isChecked();
     if (!descending && d->sorting & QDir::Reversed) {
@@ -1318,6 +1349,12 @@ void KDirOperator::setView(QAbstractItemView *view)
         d->itemView->setSelectionModel(selectionModel);
         QMetaObject::invokeMethod(this, "_k_assureVisibleSelection", Qt::QueuedConnection);
     }
+    connect(d->itemView->selectionModel(),
+            SIGNAL(currentChanged(const QModelIndex&,const QModelIndex&)),
+            this, SLOT(_k_triggerPreview(const QModelIndex&)));
+    connect(d->itemView->selectionModel(),
+            SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)),
+            this, SLOT(_k_slotSelectionChanged()));
 
     emit viewChanged(view);
 }
@@ -1333,7 +1370,7 @@ void KDirOperator::setDirLister(KDirLister *lister)
     delete d->proxyModel;
     d->proxyModel = 0;
 
-    delete d->dirLister;
+    //delete d->dirLister; // deleted by KDirModel already, which took ownership
     d->dirLister = lister;
 
     d->dirModel = new KDirModel();
@@ -1373,8 +1410,7 @@ void KDirOperator::selectFile(const KFileItem &item)
 
 void KDirOperator::highlightFile(const KFileItem &item)
 {
-    Q_ASSERT(!item.isNull());
-    if (d->preview != 0) {
+    if ((d->preview != 0) && !item.isNull()) {
         d->preview->showPreview(item.url());
     }
 
@@ -1383,23 +1419,29 @@ void KDirOperator::highlightFile(const KFileItem &item)
 
 void KDirOperator::setCurrentItem(const QString& filename)
 {
-    if (d->itemView == 0) {
-        return;
-    }
-
     KFileItem item;
     if ( !filename.isNull() ) {
         item = d->dirLister->findByName(filename);
     }
+    setCurrentItem(item);
+}
+
+void KDirOperator::setCurrentItem(const KFileItem& item)
+{
+    if (d->itemView == 0) {
+        return;
+    }
 
     QItemSelectionModel *selModel = d->itemView->selectionModel();
-    selModel->clear();
-    if (!item.isNull()) {
-        const QModelIndex dirIndex = d->dirModel->indexForItem(item);
-        const QModelIndex proxyIndex = d->proxyModel->mapFromSource(dirIndex);
-        selModel->setCurrentIndex(proxyIndex, QItemSelectionModel::Select);
-        selModel->select(proxyIndex, QItemSelectionModel::Select);
-        d->_k_assureVisibleSelection();
+    if (selModel) {
+        selModel->clear();
+        if (!item.isNull()) {
+            const QModelIndex dirIndex = d->dirModel->indexForItem(item);
+            const QModelIndex proxyIndex = d->proxyModel->mapFromSource(dirIndex);
+            selModel->setCurrentIndex(proxyIndex, QItemSelectionModel::Select);
+            selModel->select(proxyIndex, QItemSelectionModel::Select);
+            d->_k_assureVisibleSelection();
+        }
     }
 }
 
@@ -1453,7 +1495,6 @@ void KDirOperator::setupActions()
 {
     d->actionCollection = new KActionCollection(this);
     d->actionCollection->setObjectName("KDirOperator::actionCollection");
-    d->actionCollection->setAssociatedWidget(topLevelWidget());
 
     d->actionMenu = new KActionMenu(i18n("Menu"), this);
     d->actionCollection->addAction("popupMenu", d->actionMenu);
@@ -1468,18 +1509,18 @@ void KDirOperator::setupActions()
     QAction* homeAction = d->actionCollection->addAction(KStandardAction::Home, "home", this, SLOT(home()));
     homeAction->setText(i18n("Home Folder"));
 
-    QAction* reloadAction = d->actionCollection->addAction(KStandardAction::Redisplay, "reload", this, SLOT(rereadDir()));
+    KAction* reloadAction = d->actionCollection->addAction(KStandardAction::Redisplay, "reload", this, SLOT(rereadDir()));
     reloadAction->setText(i18n("Reload"));
     reloadAction->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::Reload));
 
-    QAction* mkdirAction = new KAction(i18n("New Folder..."), this);
+    KAction* mkdirAction = new KAction(i18n("New Folder..."), this);
     d->actionCollection->addAction("mkdir", mkdirAction);
     mkdirAction->setIcon(KIcon(QLatin1String("folder-new")));
     connect(mkdirAction, SIGNAL(triggered(bool)), this, SLOT(mkdir()));
 
     KAction* trash = new KAction(i18n("Move to Trash"), this);
     d->actionCollection->addAction("trash", trash);
-    trash->setIcon(KIcon("edit-trash"));
+    trash->setIcon(KIcon("user-trash"));
     trash->setShortcuts(KShortcut(Qt::Key_Delete));
     connect(trash, SIGNAL(triggered(bool)), SLOT(trashSelected()));
 
@@ -1521,12 +1562,12 @@ void KDirOperator::setupActions()
 
     KToggleAction *shortAction = new KToggleAction(i18n("Short View"), this);
     d->actionCollection->addAction("short view",  shortAction);
-    shortAction->setIcon(KIcon(QLatin1String("fileview-multicolumn")));
+    shortAction->setIcon(KIcon(QLatin1String("view-list-icons")));
     connect(shortAction, SIGNAL(activated()), SLOT(_k_slotSimpleView()));
 
     KToggleAction *detailedAction = new KToggleAction(i18n("Detailed View"), this);
     d->actionCollection->addAction("detailed view", detailedAction);
-    detailedAction->setIcon(KIcon(QLatin1String("fileview-detailed")));
+    detailedAction->setIcon(KIcon(QLatin1String("view-list-details")));
     connect(detailedAction, SIGNAL(activated()), SLOT(_k_slotDetailedView()));
 
     QActionGroup* viewGroup = new QActionGroup(this);
@@ -1539,12 +1580,13 @@ void KDirOperator::setupActions()
 
     KToggleAction *previewAction = new KToggleAction(i18n("Show Preview"), this);
     d->actionCollection->addAction("preview", previewAction);
-    previewAction->setIcon(KIcon("thumbnail-show"));
+    previewAction->setIcon(KIcon("view-preview"));
     connect(previewAction, SIGNAL(toggled(bool)),
             SLOT(_k_togglePreview(bool)));
 
     action = new KAction(i18n("Properties"), this);
     d->actionCollection->addAction("properties", action);
+    action->setIcon(KIcon("document-properties"));
     action->setShortcut(KShortcut(Qt::ALT + Qt::Key_Return));
     connect(action, SIGNAL(triggered(bool)), this, SLOT(_k_slotProperties()));
 
@@ -1555,6 +1597,12 @@ void KDirOperator::setupActions()
     viewMenu->addAction(detailedAction);
     // TODO: QAbstractItemView does not offer an action collection. Provide
     // an interface to add a custom action collection.
+
+#if (QT_VERSION >= QT_VERSION_CHECK(4, 4, 0))
+    d->actionCollection->addAssociatedWidget(topLevelWidget());
+    foreach (QAction* action, d->actionCollection->actions())
+      action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+#endif
 }
 
 void KDirOperator::setupMenu()
@@ -1819,6 +1867,10 @@ void KDirOperator::Private::_k_slotIOFinished()
     progressBar->hide();
     emit parent->finishedLoading();
     parent->resetCursor();
+
+    if (preview) {
+        preview->clearPreview();
+    }
 }
 
 void KDirOperator::Private::_k_slotCanceled()
@@ -1887,18 +1939,31 @@ void KDirOperator::Private::_k_slotPressed(const QModelIndex&)
 
 void KDirOperator::Private::_k_slotClicked(const QModelIndex& index)
 {
-    if (!leftButtonPressed) {
+    if (!leftButtonPressed || (index.column() != KDirModel::Name)) {
         return;
     }
 
+    _k_slotActivated(index);
+}
+
+void KDirOperator::Private::_k_slotActivated(const QModelIndex& index)
+{
     const QModelIndex dirIndex = proxyModel->mapToSource(index);
     KFileItem item = dirModel->itemForIndex(dirIndex);
+    bool selectDir = false;
+
+    if (item.isNull())
+        return;
+
     if (item.isDir()) {
-        if (KGlobalSettings::singleClick()) {
-            parent->selectDir(item);
-        } else {
-            parent->highlightFile(item);
-        }
+        const Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+        selectDir = KGlobalSettings::singleClick() &&
+                    !(modifiers & Qt::ShiftModifier) &&
+                    !(modifiers & Qt::ControlModifier);
+    }
+
+    if (selectDir) {
+        parent->selectDir(item);
     } else {
         parent->highlightFile(item);
     }
@@ -1912,6 +1977,10 @@ void KDirOperator::Private::_k_slotDoubleClicked(const QModelIndex& index)
 
     const QModelIndex dirIndex = proxyModel->mapToSource(index);
     KFileItem item = dirModel->itemForIndex(dirIndex);
+
+    if (item.isNull())
+        return;
+
     if (item.isDir()) {
         parent->selectDir(item);
     } else {
@@ -1919,6 +1988,23 @@ void KDirOperator::Private::_k_slotDoubleClicked(const QModelIndex& index)
     }
 }
 
+void KDirOperator::Private::_k_slotSelectionChanged()
+{
+    if (itemView == 0) {
+        return;
+    }
+
+    // In the multiselection mode each selection change is indicated by
+    // emitting a null item. Also when the selection has been cleared, a
+    // null item must be emitted (see _k_slotClicked()).
+    const bool multiSelectionMode = (itemView->selectionMode() == QAbstractItemView::ExtendedSelection);
+    const bool hasSelection = itemView->selectionModel()->hasSelection();
+    if (multiSelectionMode || !hasSelection) {
+        KFileItem nullItem;
+        parent->highlightFile(nullItem);
+    }
+}
+
 void KDirOperator::Private::_k_openContextMenu(const QPoint& pos)
 {
     leftButtonPressed = false;
@@ -1926,6 +2012,10 @@ void KDirOperator::Private::_k_openContextMenu(const QPoint& pos)
     const QModelIndex proxyIndex = itemView->indexAt(pos);
     const QModelIndex dirIndex = proxyModel->mapToSource(proxyIndex);
     KFileItem item = dirModel->itemForIndex(dirIndex);
+
+    if (item.isNull())
+        return;
+
     parent->activatedMenu(item, QCursor::pos());
 }
 
@@ -1934,6 +2024,10 @@ void KDirOperator::Private::_k_triggerPreview(const QModelIndex& index)
     if ((preview != 0) && index.isValid() && (index.column() == KDirModel::Name)) {
         const QModelIndex dirIndex = proxyModel->mapToSource(index);
         const KFileItem item = dirModel->itemForIndex(dirIndex);
+
+        if (item.isNull())
+            return;
+
         if (!item.isDir()) {
             previewUrl = item.url();
             previewTimer->start(300);
