@@ -22,6 +22,7 @@
 #include <QtCore/QTimeLine>
 #include <QtCore/QTimer>
 #include <QtGui/QMenu>
+#include <QtGui/QPainter>
 #include <QtGui/QItemDelegate>
 #include <QtGui/QKeyEvent>
 
@@ -55,7 +56,9 @@ public:
     void setIconSize(int newSize);
 
     void addAppearingItem(const QModelIndex &index);
-    void setAppearingIconSize(int newSize);
+    void setAppearingItemProgress(qreal value);
+    void addDisappearingItem(const QModelIndex &index);
+    void setDisappearingItemProgress(qreal value);
 
 private:
     KFilePlacesView *m_view;
@@ -63,6 +66,11 @@ private:
 
     QList<QPersistentModelIndex> m_appearingItems;
     int m_appearingIconSize;
+    qreal m_appearingOpacity;
+
+    QList<QPersistentModelIndex> m_disappearingItems;
+    int m_disappearingIconSize;
+    qreal m_disappearingOpacity;
 };
 
 KFilePlacesViewDelegate::KFilePlacesViewDelegate(KFilePlacesView *parent) :
@@ -81,6 +89,8 @@ QSize KFilePlacesViewDelegate::sizeHint(const QStyleOptionViewItem &option,
     int iconSize = m_iconSize;
     if (m_appearingItems.contains(index)) {
         iconSize = m_appearingIconSize;
+    } else if (m_disappearingItems.contains(index)) {
+        iconSize = m_disappearingIconSize;
     }
 
     size.setHeight(iconSize + KDialog::marginHint());
@@ -89,14 +99,19 @@ QSize KFilePlacesViewDelegate::sizeHint(const QStyleOptionViewItem &option,
 
 void KFilePlacesViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-    int iconSize = m_iconSize;
+    painter->save();
+
     if (m_appearingItems.contains(index)) {
-        iconSize = m_appearingIconSize;
+        painter->setOpacity(m_appearingOpacity);
+    } else if (m_disappearingItems.contains(index)) {
+        painter->setOpacity(m_disappearingOpacity);
     }
 
     QStyleOptionViewItem opt = option;
-    opt.decorationSize = QSize(iconSize, iconSize);
+    opt.decorationSize = QSize(m_iconSize, m_iconSize);
     QItemDelegate::paint(painter, opt, index);
+
+    painter->restore();
 }
 
 int KFilePlacesViewDelegate::iconSize() const
@@ -114,13 +129,48 @@ void KFilePlacesViewDelegate::addAppearingItem(const QModelIndex &index)
     m_appearingItems << index;
 }
 
-void KFilePlacesViewDelegate::setAppearingIconSize(int newSize)
+void KFilePlacesViewDelegate::setAppearingItemProgress(qreal value)
 {
-    m_appearingIconSize = newSize;
+    if (value<=0.25) {
+        m_appearingOpacity = 0.0;
+        m_appearingIconSize = iconSize()*value*4;
 
-    if (m_appearingIconSize>=m_iconSize) {
-        m_appearingItems.clear();
+        if (m_appearingIconSize>=m_iconSize) {
+            m_appearingIconSize = m_iconSize;
+        }
+    } else {
         m_appearingIconSize = m_iconSize;
+        m_appearingOpacity = (value-0.25)*4/3;
+
+        if (value>=1.0) {
+            m_appearingItems.clear();
+        }
+    }
+}
+
+void KFilePlacesViewDelegate::addDisappearingItem(const QModelIndex &index)
+{
+    m_disappearingItems << index;
+}
+
+void KFilePlacesViewDelegate::setDisappearingItemProgress(qreal value)
+{
+    value = 1.0 - value;
+
+    if (value<=0.25) {
+        m_disappearingOpacity = 0.0;
+        m_disappearingIconSize = iconSize()*value*4;
+
+        if (m_disappearingIconSize>=m_iconSize) {
+            m_disappearingIconSize = m_iconSize;
+        }
+
+        if (value<=0.0) {
+            m_disappearingItems.clear();
+        }
+    } else {
+        m_disappearingIconSize = m_iconSize;
+        m_disappearingOpacity = (value-0.25)*4/3;
     }
 }
 
@@ -146,12 +196,14 @@ public:
     void _k_storageSetupDone(const QModelIndex &index, bool success);
     void _k_adaptItemsUpdate(qreal value);
     void _k_itemAppearUpdate(qreal value);
+    void _k_itemDisappearUpdate(qreal value);
     void _k_enableSmoothItemResizing();
 
     QTimeLine adaptItemsTimeline;
     int oldSize, endSize;
 
     QTimeLine itemAppearTimeline;
+    QTimeLine itemDisappearTimeline;
 };
 
 KFilePlacesView::KFilePlacesView(QWidget *parent)
@@ -185,15 +237,21 @@ KFilePlacesView::KFilePlacesView(QWidget *parent)
 
     connect(&d->adaptItemsTimeline, SIGNAL(valueChanged(qreal)),
             this, SLOT(_k_adaptItemsUpdate(qreal)));
-    d->adaptItemsTimeline.setDuration(150);
-    d->adaptItemsTimeline.setUpdateInterval(30);
+    d->adaptItemsTimeline.setDuration(500);
+    d->adaptItemsTimeline.setUpdateInterval(5);
     d->adaptItemsTimeline.setCurveShape(QTimeLine::EaseInOutCurve);
 
     connect(&d->itemAppearTimeline, SIGNAL(valueChanged(qreal)),
             this, SLOT(_k_itemAppearUpdate(qreal)));
-    d->itemAppearTimeline.setDuration(150);
-    d->itemAppearTimeline.setUpdateInterval(30);
-    d->itemAppearTimeline.setCurveShape(QTimeLine::EaseOutCurve);
+    d->itemAppearTimeline.setDuration(500);
+    d->itemAppearTimeline.setUpdateInterval(5);
+    d->itemAppearTimeline.setCurveShape(QTimeLine::EaseInOutCurve);
+
+    connect(&d->itemDisappearTimeline, SIGNAL(valueChanged(qreal)),
+            this, SLOT(_k_itemDisappearUpdate(qreal)));
+    d->itemDisappearTimeline.setDuration(500);
+    d->itemDisappearTimeline.setUpdateInterval(5);
+    d->itemDisappearTimeline.setCurveShape(QTimeLine::EaseInOutCurve);
 }
 
 KFilePlacesView::~KFilePlacesView()
@@ -203,21 +261,35 @@ KFilePlacesView::~KFilePlacesView()
 
 void KFilePlacesView::setUrl(const KUrl &url)
 {
+    KUrl oldUrl = d->currentUrl;
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel*>(model());
 
     if (placesModel==0) return;
 
     QModelIndex index = placesModel->closestItem(url);
+    QModelIndex current = selectionModel()->currentIndex();
 
     if (index.isValid()) {
-        if (d->currentUrl!=url && placesModel->isHidden(index)) {
+        if (current!=index && placesModel->isHidden(current)) {
+            KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate*>(itemDelegate());
+            delegate->addDisappearingItem(current);
+
+            if (d->itemDisappearTimeline.state()!=QTimeLine::Running) {
+                delegate->setDisappearingItemProgress(0.0);
+                d->itemDisappearTimeline.start();
+            }
+        }
+
+        if (current!=index && placesModel->isHidden(index)) {
             KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate*>(itemDelegate());
             delegate->addAppearingItem(index);
 
             if (d->itemAppearTimeline.state()!=QTimeLine::Running) {
-                delegate->setAppearingIconSize(1);
+                delegate->setAppearingItemProgress(0.0);
                 d->itemAppearTimeline.start();
             }
+
+            setRowHidden(index.row(), false);
         }
 
         d->currentUrl = url;
@@ -226,7 +298,10 @@ void KFilePlacesView::setUrl(const KUrl &url)
         d->currentUrl = KUrl();
         selectionModel()->clear();
     }
-    d->updateHiddenRows();
+
+    if (!current.isValid()) {
+        d->updateHiddenRows();
+    }
 }
 
 void KFilePlacesView::setShowAll(bool showAll)
@@ -236,7 +311,6 @@ void KFilePlacesView::setShowAll(bool showAll)
     if (placesModel==0) return;
 
     d->showAll = showAll;
-    d->updateHiddenRows();
 
     KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate*>(itemDelegate());
 
@@ -244,6 +318,8 @@ void KFilePlacesView::setShowAll(bool showAll)
     QModelIndex current = placesModel->closestItem(d->currentUrl);
 
     if (showAll) {
+        d->updateHiddenRows();
+
         for (int i=0; i<rowCount; ++i) {
             QModelIndex index = placesModel->index(i, 0);
             if (index!=current && placesModel->isHidden(index)) {
@@ -252,21 +328,33 @@ void KFilePlacesView::setShowAll(bool showAll)
         }
 
         if (d->itemAppearTimeline.state()!=QTimeLine::Running) {
-            delegate->setAppearingIconSize(1);
+            delegate->setAppearingItemProgress(0.0);
             d->itemAppearTimeline.start();
         }
-    }
+    } else {
+        for (int i=0; i<rowCount; ++i) {
+            QModelIndex index = placesModel->index(i, 0);
+            if (index!=current && placesModel->isHidden(index)) {
+                delegate->addDisappearingItem(index);
+            }
+        }
 
+        if (d->itemDisappearTimeline.state()!=QTimeLine::Running) {
+            delegate->setDisappearingItemProgress(0.0);
+            d->itemDisappearTimeline.start();
+        }
+    }
 }
 
 void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
 {
     KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel*>(model());
+    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate*>(itemDelegate());
 
     if (placesModel==0) return;
 
     QModelIndex index = indexAt(event->pos());
-    QString label = placesModel->text(index);
+    QString label = placesModel->text(index).replace('&',"&&");
 
     QMenu menu;
 
@@ -274,7 +362,7 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
     QAction *hide = 0;
     if (index.isValid()) {
         if (!placesModel->isDevice(index)) {
-            edit = menu.addAction(KIcon("edit"), i18n("&Edit '%1'...", label));
+            edit = menu.addAction(KIcon("document-properties"), i18n("&Edit '%1'...", label));
         }
 
         hide = menu.addAction(i18n("&Hide '%1'", label));
@@ -327,13 +415,24 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
         placesModel->removePlace(index);
     } else if (hide != 0 && result == hide) {
         placesModel->setPlaceHidden(index, hide->isChecked());
+        QModelIndex current = placesModel->closestItem(d->currentUrl);
+
+        if (index!=current && !d->showAll && hide->isChecked()) {
+            delegate->addDisappearingItem(index);
+
+            if (d->itemDisappearTimeline.state()!=QTimeLine::Running) {
+                delegate->setDisappearingItemProgress(0.0);
+                d->itemDisappearTimeline.start();
+            }
+        }
     } else if (showAll != 0 && result == showAll) {
         setShowAll(showAll->isChecked());
     } else if (teardown != 0 && result == teardown) {
         placesModel->requestTeardown(index);
     }
 
-    setUrl(d->currentUrl);
+    index = placesModel->closestItem(d->currentUrl);
+    selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect);
 }
 
 void KFilePlacesView::resizeEvent(QResizeEvent *event)
@@ -354,24 +453,33 @@ void KFilePlacesView::hideEvent(QHideEvent *event)
     d->smoothItemResizing = false;
 }
 
+void KFilePlacesView::setModel(QAbstractItemModel *model)
+{
+    QListView::setModel(model);
+    d->updateHiddenRows();
+}
+
 void KFilePlacesView::rowsInserted(const QModelIndex &parent, int start, int end)
 {
     QListView::rowsInserted(parent, start, end);
     setUrl(d->currentUrl);
 
-#if 0 // Disable until the model is fixed and sane again signals wise
     KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate*>(itemDelegate());
+    KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel*>(model());
 
     for (int i=start; i<=end; ++i) {
-        QModelIndex index = model()->index(i, 0, parent);
-        delegate->addAppearingItem(index);
+        QModelIndex index = placesModel->index(i, 0, parent);
+        if (d->showAll || !placesModel->isHidden(index)) {
+            delegate->addAppearingItem(index);
+        } else {
+            setRowHidden(i, true);
+        }
     }
 
     if (d->itemAppearTimeline.state()!=QTimeLine::Running) {
-        delegate->setAppearingIconSize(1);
+        delegate->setAppearingItemProgress(0.0);
         d->itemAppearTimeline.start();
     }
-#endif
 }
 
 QSize KFilePlacesView::sizeHint() const
@@ -493,22 +601,7 @@ void KFilePlacesView::Private::updateHiddenRows()
 
 void KFilePlacesView::Private::_k_placeClicked(const QModelIndex &index)
 {
-    KFilePlacesModel *placesModel = qobject_cast<KFilePlacesModel*>(q->model());
-
-    if (placesModel==0) return;
-
-    lastClickedIndex = QPersistentModelIndex();
-
-    if (placesModel->setupNeeded(index)) {
-        QObject::connect(placesModel, SIGNAL(setupDone(const QModelIndex &, bool)),
-                         q, SLOT(_k_storageSetupDone(const QModelIndex &, bool)));
-
-        lastClickedIndex = index;
-        placesModel->requestSetup(index);
-        return;
-    }
-
-    setCurrentIndex(index);
+    _k_placeActivated(index);
 }
 
 void KFilePlacesView::Private::_k_placeActivated(const QModelIndex &index)
@@ -519,7 +612,7 @@ void KFilePlacesView::Private::_k_placeActivated(const QModelIndex &index)
 
     lastClickedIndex = QPersistentModelIndex();
 
-    if (placesModel->isDevice(index)) {
+    if (placesModel->setupNeeded(index)) {
         QObject::connect(placesModel, SIGNAL(setupDone(const QModelIndex &, bool)),
                          q, SLOT(_k_storageSetupDone(const QModelIndex &, bool)));
 
@@ -565,9 +658,21 @@ void KFilePlacesView::Private::_k_adaptItemsUpdate(qreal value)
 void KFilePlacesView::Private::_k_itemAppearUpdate(qreal value)
 {
     KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate*>(q->itemDelegate());
-    int size = delegate->iconSize()*value;
 
-    delegate->setAppearingIconSize(size);
+    delegate->setAppearingItemProgress(value);
+    q->scheduleDelayedItemsLayout();
+}
+
+void KFilePlacesView::Private::_k_itemDisappearUpdate(qreal value)
+{
+    KFilePlacesViewDelegate *delegate = dynamic_cast<KFilePlacesViewDelegate*>(q->itemDelegate());
+
+    delegate->setDisappearingItemProgress(value);
+
+    if (value>=1.0) {
+        updateHiddenRows();
+    }
+
     q->scheduleDelayedItemsLayout();
 }
 
@@ -576,9 +681,9 @@ void KFilePlacesView::Private::_k_enableSmoothItemResizing()
     smoothItemResizing = true;
 }
 
-void KFilePlacesView::dataChanged(const QModelIndex &/*topLeft*/, const QModelIndex &/*bottomRight*/)
+void KFilePlacesView::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight)
 {
-    d->updateHiddenRows();
+    QListView::dataChanged(topLeft, bottomRight);
 }
 
 #include "kfileplacesview.moc"
