@@ -36,7 +36,6 @@
 #include <kdiroperator.h>
 #include <kdirselectdialog.h>
 #include <kfilefiltercombo.h>
-#include <kfileview.h>
 #include <kimagefilepreview.h>
 #include <kmenu.h>
 #include <kmimetype.h>
@@ -110,7 +109,7 @@ public:
      * Returns the URL which represents the directory of \a url. If
      * \a url already is a directory, then just \a url is returned.
      */
-    KUrl directoryUrl(const KUrl& url) const;
+    static KUrl directoryUrl(const KUrl& url);
 
     /**
      * Returns the absolute version of the URL specified in locationEdit.
@@ -238,46 +237,40 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     d->urlNavigator->setPlacesSelectorVisible(false);
 
     KUrl u;
-    QString text;
     KUrlComboBox *pathCombo = d->urlNavigator->editor();
 #ifdef Q_WS_WIN
     foreach( const QFileInfo &drive,QFSFileEngine::drives() )
     {
         u.setPath( drive.filePath() );
-        text = i18n("Drive: %1",  u.toLocalFile() );
-        pathCombo->addDefaultUrl( u,
-                                  KIO::pixmapForUrl( u, 0, KIconLoader::Small ),
-                                  text );
+        pathCombo->addDefaultUrl(u,
+                                 KIO::pixmapForUrl( u, 0, KIconLoader::Small ),
+                                 i18n("Drive: %1",  u.toLocalFile()));
     }
 #else
-    u.setPath( QDir::rootPath() );
-    text = i18n("Root Folder: %1",  u.toLocalFile() );
-    pathCombo->addDefaultUrl( u,
-                              KIO::pixmapForUrl( u, 0, KIconLoader::Small ),
-                              text );
+    u.setPath(QDir::rootPath());
+    pathCombo->addDefaultUrl(u,
+                             KIO::pixmapForUrl(u, 0, KIconLoader::Small),
+                             u.toLocalFile());
 #endif
 
-    u.setPath( QDir::homePath() );
-    text = i18n("Home Folder: %1",  u.path( KUrl::AddTrailingSlash ) );
-    pathCombo->addDefaultUrl( u, KIO::pixmapForUrl( u, 0, KIconLoader::Small ),
-                              text );
+    u.setPath(QDir::homePath());
+    pathCombo->addDefaultUrl(u, KIO::pixmapForUrl(u, 0, KIconLoader::Small),
+                             u.path(KUrl::AddTrailingSlash));
 
     KUrl docPath;
     docPath.setPath( KGlobalSettings::documentPath() );
     if ( (u.path(KUrl::AddTrailingSlash) != docPath.path(KUrl::AddTrailingSlash)) &&
           QDir(docPath.path(KUrl::AddTrailingSlash)).exists() )
     {
-      text = i18n("Documents: %1",  docPath.path( KUrl::AddTrailingSlash ) );
         pathCombo->addDefaultUrl( docPath,
                                   KIO::pixmapForUrl( docPath, 0, KIconLoader::Small ),
-                                  text );
+                                  docPath.path(KUrl::AddTrailingSlash));
     }
 
     u.setPath( KGlobalSettings::desktopPath() );
-    text = i18n("Desktop: %1",  u.path( KUrl::AddTrailingSlash ) );
-    pathCombo->addDefaultUrl( u,
-                              KIO::pixmapForUrl( u, 0, KIconLoader::Small ),
-                              text );
+    pathCombo->addDefaultUrl(u,
+                             KIO::pixmapForUrl(u, 0, KIconLoader::Small),
+                             u.path(KUrl::AddTrailingSlash));
 
     d->url = getStartUrl( startDir, d->fileClass );
     d->selection = d->url.url();
@@ -405,7 +398,6 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     d->updateLocationWhatsThis();
     d->locationLabel->setBuddy(d->locationEdit);
 
-    d->locationEdit->setFocus();
     KUrlCompletion *fileCompletionObj = new KUrlCompletion( KUrlCompletion::FileCompletion );
     QString dir = d->url.url(KUrl::AddTrailingSlash);
 
@@ -437,7 +429,7 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     // the Automatically Select Extension checkbox
     // (the text, visibility etc. is set in updateAutoSelectExtension(), which is called by readConfig())
     d->autoSelectExtCheckBox = new QCheckBox (this);
-    d->autoSelectExtCheckBox->setStyleSheet(QString("QCheckBox { spacing-top: %1px; }").arg(KDialog::spacingHint()));
+    d->autoSelectExtCheckBox->setStyleSheet(QString("QCheckBox { padding-top: %1px; }").arg(KDialog::spacingHint()));
     connect(d->autoSelectExtCheckBox, SIGNAL(clicked()), SLOT(_k_slotAutoSelectExtClicked()));
 
     d->initGUI(); // activate GM
@@ -449,6 +441,7 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     d->ops->setViewConfig(*d->viewConfigGroup);
     d->readConfig(* d->viewConfigGroup);
     setSelection(d->selection);
+    d->locationEdit->setFocus();
 }
 
 KFileWidget::~KFileWidget()
@@ -982,8 +975,10 @@ void KFileWidgetPrivate::multiSelectionChanged()
     setLocationText( text.trimmed() );
 }
 
-KUrl KFileWidgetPrivate::directoryUrl(const KUrl& url) const
+KUrl KFileWidgetPrivate::directoryUrl(const KUrl& url)
 {
+    if (!url.isValid())
+        return url;
     KFileItem item(S_IFDIR, KFileItem::Unknown, url);
     item.refresh();
     return item.isDir() ? url : url.upUrl();
@@ -1036,7 +1031,6 @@ void KFileWidgetPrivate::initSpeedbar()
     placesView = new KFilePlacesView( q );
     placesView->setModel(new KFilePlacesModel(placesView));
     placesView->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
-    placesView->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum ) );
 
     placesView->setObjectName( QLatin1String( "url bar" ) );
     QObject::connect( placesView, SIGNAL( urlChanged( const KUrl& )),
@@ -1157,7 +1151,10 @@ void KFileWidgetPrivate::_k_urlEntered(const KUrl& url)
     urlNavigator->setUrl( directoryUrl( url ) );
 
     QString dir = url.url(KUrl::AddTrailingSlash);
-    static_cast<KUrlCompletion*>( locationEdit->completionObject() )->setDir( dir );
+    // is trigged in ctor before completion object is set
+    KUrlCompletion *completion = dynamic_cast<KUrlCompletion*>( locationEdit->completionObject() );
+    if( completion )
+        completion->setDir( dir );
 
     if ( placesView )
         placesView->setUrl( url );
@@ -1457,7 +1454,7 @@ void KFileWidgetPrivate::readConfig( const KConfigGroup &configGroup)
     ops->readConfig(configGroup);
 
     KUrlComboBox *combo = urlNavigator->editor();
-    combo->setUrls( configGroup.readPathListEntry( RecentURLs ), KUrlComboBox::RemoveTop );
+    combo->setUrls( configGroup.readPathEntry( RecentURLs, QStringList() ), KUrlComboBox::RemoveTop );
     combo->setMaxItems( configGroup.readEntry( RecentURLsNumber,
                                        DefaultRecentURLsNumber ) );
     combo->setUrl( ops->url() );
@@ -1514,6 +1511,8 @@ void KFileWidgetPrivate::writeConfig(KConfigGroup &configGroup)
     configGroup.writeEntry( ShowBookmarks, bookmarkHandler != 0 );
     configGroup.writeEntry( AutoSelectExtChecked, autoSelectExtChecked );
     configGroup.writeEntry( BreadcrumbNavigation, !urlNavigator->isUrlEditable() );
+
+    ops->writeConfig(configGroup);
 }
 
 
@@ -1523,7 +1522,7 @@ void KFileWidgetPrivate::readRecentFiles( KConfig *kc )
 
     locationEdit->setMaxItems( cg.readEntry( RecentFilesNumber,
                                              DefaultRecentURLsNumber ) );
-    locationEdit->setUrls( cg.readPathListEntry( RecentFiles ),
+    locationEdit->setUrls( cg.readPathEntry( RecentFiles, QStringList() ),
                            KUrlComboBox::RemoveBottom );
     locationEdit->insertItem(0, QString()); // dummy item without pixmap
     locationEdit->setCurrentIndex( 0 );
@@ -2005,7 +2004,7 @@ void KFileWidgetPrivate::_k_toggleBookmarks(bool show)
         q->connect( bookmarkHandler, SIGNAL( openUrl( const QString& )),
                     SLOT( _k_enterUrl( const QString& )));
 
-        bookmarkButton = new KActionMenu(KIcon("bookmark"),i18n("Bookmarks"), q);
+        bookmarkButton = new KActionMenu(KIcon("bookmarks"),i18n("Bookmarks"), q);
         bookmarkButton->setDelayed(false);
         q->actionCollection()->addAction("bookmark", bookmarkButton);
         bookmarkButton->setMenu(bookmarkHandler->menu());
