@@ -28,6 +28,7 @@
 #include "kurltogglebutton_p.h"
 
 #include <kfileitem.h>
+#include <kglobalsettings.h>
 #include <kicon.h>
 #include <klineedit.h>
 #include <klocale.h>
@@ -641,6 +642,7 @@ void KUrlNavigator::Private::updateButtons(const QString& path, int startIndex)
 
             if (isFirstButton) {
                 button->setText(text);
+                button->updateMinimumWidth();
             }
 
             if (createButton) {
@@ -762,12 +764,12 @@ void KUrlNavigator::Private::deleteButtons()
 
 QString KUrlNavigator::Private::retrievePlacePath(const QString& path) const
 {
-    int idx = path.indexOf(QString("///"));
+    int idx = path.indexOf(QLatin1String("///"));
     if (idx >= 0) {
         idx += 3;
     } else {
-        idx = path.indexOf(QString("//"));
-        idx = path.indexOf("/", (idx < 0) ? 0 : idx + 2);
+        idx = path.indexOf(QLatin1String("//"));
+        idx = path.indexOf(QLatin1Char('/'), (idx < 0) ? 0 : idx + 2);
     }
     return (idx < 0) ? path : path.left(idx);
 }
@@ -777,6 +779,7 @@ bool KUrlNavigator::Private::isCompressedPath(const KUrl& url) const
     const KMimeType::Ptr mime = KMimeType::findByPath(url.path(KUrl::RemoveTrailingSlash));
     // Note: this list of MIME types depends on the protocols implemented by kio_archive
     return  mime->is("application/x-compressed-tar") ||
+            mime->is("application/x-bzip-compressed-tar") ||
             mime->is("application/x-tar") ||
             mime->is("application/x-tarz") ||
             mime->is("application/x-tzo") || // (not sure KTar supports those?)
@@ -810,8 +813,11 @@ KUrlNavigator::KUrlNavigator(KFilePlacesModel* placesModel,
 {
     d->m_history.prepend(HistoryElem(url));
 
-    QFontMetrics fontMetrics(font());
-    setMinimumHeight(fontMetrics.height() + 10);
+    const QFont font = KGlobalSettings::generalFont();
+    setFont(font);
+
+    const int minHeight = d->m_pathBox->sizeHint().height();
+    setMinimumHeight(minHeight);
 
     setLayout(d->m_layout);
     setMinimumWidth(100);
@@ -855,7 +861,11 @@ KUrl KUrlNavigator::url(int index) const
         if (index == 0) {
             // prevent the last "/" from being stripped
             // or we end up with an empty path
-            pathOrUrl = "/";
+#ifdef Q_OS_WIN
+            pathOrUrl = pathOrUrl.length() > 2 ? pathOrUrl.left(3) : QDir::rootPath();
+#else
+            pathOrUrl = QLatin1String("/");
+#endif
         } else {
             pathOrUrl = pathOrUrl.section('/', 0, index);
         }
