@@ -27,6 +27,8 @@
 #include "cssproperties.h"
 #include "cssvalues.h"
 
+#include <dom/dom_exception.h>
+
 using namespace DOM;
 using namespace khtml;
 
@@ -1011,7 +1013,7 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
         return new CSSPrimitiveValueImpl(style->wordSpacing(), CSSPrimitiveValue::CSS_PX);
     case CSS_PROP_Z_INDEX:
         if (style->hasAutoZIndex())
-            return new CSSPrimitiveValueImpl(CSS_VAL_NORMAL);
+            return new CSSPrimitiveValueImpl(CSS_VAL_AUTO);
         return new CSSPrimitiveValueImpl(style->zIndex(), CSSPrimitiveValue::CSS_NUMBER);
     case CSS_PROP_BACKGROUND:
         break;
@@ -1093,27 +1095,36 @@ bool RenderStyleDeclarationImpl::getPropertyPriority( int ) const
     return false;
 }
 
-DOM::DOMString RenderStyleDeclarationImpl::removeProperty( int, bool )
+DOM::DOMString RenderStyleDeclarationImpl::removeProperty( int )
 {
     // ### emit error since we're read-only
     return DOMString();
 }
 
-bool RenderStyleDeclarationImpl::setProperty ( int, const DOM::DOMString&, bool,
-                                               bool )
+void RenderStyleDeclarationImpl::removePropertiesInSet(const int* set, unsigned length)
+{
+     // ### emit error since we're read-only
+}
+
+bool RenderStyleDeclarationImpl::setProperty ( int, const DOM::DOMString &, bool, int &ec)
+{
+    ec = DOMException::NO_MODIFICATION_ALLOWED_ERR;
+    return false;
+}
+
+bool RenderStyleDeclarationImpl::setProperty ( int, const DOM::DOMString&, bool )
 {
     // ### emit error since we're read-only
     return false;
 }
 
-void RenderStyleDeclarationImpl::setProperty ( int, int, bool,
-                                               bool )
+void RenderStyleDeclarationImpl::setProperty ( int, int, bool )
 {
     // ### emit error since we're read-only
 }
 
 void RenderStyleDeclarationImpl::setLengthProperty( int, const DOM::DOMString&, bool,
-                                                    bool, bool )
+                                                    bool )
 {
     // ### emit error since we're read-only
 }
@@ -1141,7 +1152,6 @@ CSSProperty RenderStyleDeclarationImpl::property( int id ) const
     CSSProperty prop;
     prop.m_id = id;
     prop.m_important = false;
-    prop.nonCSSHint = false;
 
     CSSValueImpl* v = getPropertyCSSValue( id );
     if ( !v )
