@@ -105,7 +105,6 @@ CSSParser::CSSParser( bool strictParsing )
     rule = 0;
     id = 0;
     important = false;
-    nonCSSHint = false;
 
     m_inParseShorthand = 0;
     m_currentShorthand = 0;
@@ -204,12 +203,24 @@ CSSRuleImpl *CSSParser::parseRule( DOM::CSSStyleSheetImpl *sheet, const DOM::DOM
     return result;
 }
 
+static void addParsedProperties( DOM::CSSStyleDeclarationImpl *declaration, CSSProperty** parsedProperties,
+                                 int numProperties)
+{
+    for ( int i = 0; i < numProperties; i++ ) {
+        // Only add properties that have no !important counterpart present
+        if (!declaration->getPropertyPriority(parsedProperties[i]->id()) || parsedProperties[i]->isImportant()) {
+            declaration->removeProperty(parsedProperties[i]->m_id);
+            declaration->values()->append( parsedProperties[i] );
+        }
+    }
+}
+
 bool CSSParser::parseValue( DOM::CSSStyleDeclarationImpl *declaration, int _id, const DOM::DOMString &string,
-                            bool _important, bool _nonCSSHint )
+                            bool _important)
 {
 #ifdef CSS_DEBUG
     kDebug( 6080 ) << "CSSParser::parseValue: id=" << _id << " important=" << _important
-                    << " nonCSSHint=" << _nonCSSHint << " value='" << string.string() << "'" << endl;
+                   << " value='" << string.string() << "'" << endl;
 #endif
 
     styleElement = declaration->stylesheet();
@@ -226,7 +237,6 @@ bool CSSParser::parseValue( DOM::CSSStyleDeclarationImpl *declaration, int _id,
 
     id = _id;
     important = _important;
-    nonCSSHint = _nonCSSHint;
 
     runParser(length);
 
@@ -236,21 +246,17 @@ bool CSSParser::parseValue( DOM::CSSStyleDeclarationImpl *declaration, int _id,
     bool ok = false;
     if ( numParsedProperties ) {
         ok = true;
-        for ( int i = 0; i < numParsedProperties; i++ ) {
-            declaration->removeProperty(parsedProperties[i]->m_id, nonCSSHint);
-            declaration->values()->append( parsedProperties[i] );
-        }
+        addParsedProperties(declaration, parsedProperties, numParsedProperties);
         numParsedProperties = 0;
     }
 
     return ok;
 }
 
-bool CSSParser::parseDeclaration( DOM::CSSStyleDeclarationImpl *declaration, const DOM::DOMString &string,
-                                  bool _nonCSSHint )
+bool CSSParser::parseDeclaration( DOM::CSSStyleDeclarationImpl *declaration, const DOM::DOMString &string)
 {
 #ifdef CSS_DEBUG
-    kDebug( 6080 ) << "CSSParser::parseDeclaration: nonCSSHint=" << nonCSSHint
+    kDebug( 6080 ) << "CSSParser::parseDeclaration:"
                     << " value='" << string.string() << "'" << endl;
 #endif
 
@@ -265,8 +271,6 @@ bool CSSParser::parseDeclaration( DOM::CSSStyleDeclarationImpl *declaration, con
     memcpy( data + strlen( khtml_decls ), string.unicode(), string.length()*sizeof( unsigned short) );
     data[length-4] = '}';
 
-    nonCSSHint = _nonCSSHint;
-
     runParser(length);
 
     delete rule;
@@ -275,10 +279,7 @@ bool CSSParser::parseDeclaration( DOM::CSSStyleDeclarationImpl *declaration, con
     bool ok = false;
     if ( numParsedProperties ) {
         ok = true;
-        for ( int i = 0; i < numParsedProperties; i++ ) {
-            declaration->removeProperty(parsedProperties[i]->m_id, false);
-            declaration->values()->append( parsedProperties[i] );
-        }
+        addParsedProperties(declaration, parsedProperties, numParsedProperties);
         numParsedProperties = 0;
     }
 
@@ -291,7 +292,6 @@ void CSSParser::addProperty( int propId, CSSValueImpl *value, bool important )
     prop->m_id = propId;
     prop->setValue( value );
     prop->m_important = important;
-    prop->nonCSSHint = nonCSSHint;
 
     if ( numParsedProperties >= maxParsedProperties ) {
         maxParsedProperties += 32;
@@ -303,8 +303,7 @@ void CSSParser::addProperty( int propId, CSSValueImpl *value, bool important )
 
 CSSStyleDeclarationImpl *CSSParser::createStyleDeclaration( CSSStyleRuleImpl *rule )
 {
-    Q3PtrList<CSSProperty> *propList = new Q3PtrList<CSSProperty>;
-    propList->setAutoDelete( true );
+    QList<CSSProperty*> *propList = new QList<CSSProperty*>;
     for ( int i = 0; i < numParsedProperties; i++ )
         propList->append( parsedProperties[i] );
 
@@ -397,7 +396,7 @@ bool CSSParser::parseValue( int propId, bool important )
     } else if (id == CSS_VAL_INITIAL ) {
         if (num != 1)
             return false;
-        addProperty(propId, new CSSInitialValueImpl(), important);
+        addProperty(propId, new CSSInitialValueImpl(false/*implicit initial*/), important);
         return true;
     }
 
@@ -641,7 +640,7 @@ bool CSSParser::parseValue( int propId, bool important )
     }
     case CSS_PROP__KHTML_BORDER_HORIZONTAL_SPACING:
     case CSS_PROP__KHTML_BORDER_VERTICAL_SPACING:
-        valid_primitive = validUnit(value, FLength|FNonNeg, strict&(!nonCSSHint));
+        valid_primitive = validUnit(value, FLength|FNonNeg, strict);
         break;
 
     case CSS_PROP_SCROLLBAR_FACE_COLOR:         // IE5.5
@@ -671,7 +670,7 @@ bool CSSParser::parseValue( int propId, bool important )
         if ( id == CSS_VAL__KHTML_TEXT || id == CSS_VAL_MENU ||
              (id >= CSS_VAL_AQUA && id <= CSS_VAL_WINDOWTEXT ) ||
              id == CSS_VAL_TRANSPARENT ||
-             (id >= CSS_VAL_GREY && id < CSS_VAL__KHTML_TEXT && (nonCSSHint|!strict) ) ) {
+             (id >= CSS_VAL_GREY && id < CSS_VAL__KHTML_TEXT && !strict ) ) {
             valid_primitive = true;
         } else {
             parsedValue = parseColor();
@@ -736,7 +735,7 @@ bool CSSParser::parseValue( int propId, bool important )
         if (id == CSS_VAL_THIN || id == CSS_VAL_MEDIUM || id == CSS_VAL_THICK)
             valid_primitive = true;
         else
-            valid_primitive = ( validUnit( value, FLength, strict&(!nonCSSHint) ) );
+            valid_primitive = ( validUnit( value, FLength, strict ) );
         break;
 
     case CSS_PROP_LETTER_SPACING:       // normal | <length> | inherit
@@ -744,11 +743,11 @@ bool CSSParser::parseValue( int propId, bool important )
         if ( id == CSS_VAL_NORMAL )
             valid_primitive = true;
         else
-            valid_primitive = validUnit( value, FLength, strict&(!nonCSSHint) );
+            valid_primitive = validUnit( value, FLength, strict );
         break;
 
     case CSS_PROP_TEXT_INDENT:          //  <length> | <percentage> | inherit
-        valid_primitive = ( !id && validUnit( value, FLength|FPercent, strict&(!nonCSSHint) ) );
+        valid_primitive = ( !id && validUnit( value, FLength|FPercent, strict ) );
         break;
 
     case CSS_PROP_PADDING_TOP:          //  <length> | <percentage> | inherit
@@ -756,7 +755,7 @@ bool CSSParser::parseValue( int propId, bool important )
     case CSS_PROP_PADDING_BOTTOM:       //   Which is defined as
     case CSS_PROP_PADDING_LEFT:         //   <length> | <percentage>
     case CSS_PROP__KHTML_PADDING_START:
-        valid_primitive = ( !id && validUnit( value, FLength|FPercent|FNonNeg, strict&(!nonCSSHint) ) );
+        valid_primitive = ( !id && validUnit( value, FLength|FPercent|FNonNeg, strict ) );
         break;
 
     case CSS_PROP_MAX_HEIGHT:           // <length> | <percentage> | none | inherit
@@ -768,7 +767,7 @@ bool CSSParser::parseValue( int propId, bool important )
         /* nobreak */
     case CSS_PROP_MIN_HEIGHT:           // <length> | <percentage> | inherit
     case CSS_PROP_MIN_WIDTH:            // <length> | <percentage> | inherit
-            valid_primitive = ( !id && validUnit( value, FLength|FPercent|FNonNeg, strict&(!nonCSSHint) ) );
+            valid_primitive = ( !id && validUnit( value, FLength|FPercent|FNonNeg, strict ) );
         break;
 
     case CSS_PROP_FONT_SIZE:
@@ -776,7 +775,7 @@ bool CSSParser::parseValue( int propId, bool important )
         if (id >= CSS_VAL_XX_SMALL && id <= CSS_VAL_LARGER)
             valid_primitive = true;
         else
-            valid_primitive = ( validUnit( value, FLength|FPercent, strict&(!nonCSSHint) ) );
+            valid_primitive = ( validUnit( value, FLength|FPercent, strict ) );
         break;
 
     case CSS_PROP_FONT_STYLE:           // normal | italic | oblique | inherit
@@ -796,7 +795,7 @@ bool CSSParser::parseValue( int propId, bool important )
         if ( id >= CSS_VAL_BASELINE && id <= CSS_VAL__KHTML_BASELINE_MIDDLE )
             valid_primitive = true;
         else
-            valid_primitive = ( !id && validUnit( value, FLength|FPercent, strict&(!nonCSSHint) ) );
+            valid_primitive = ( !id && validUnit( value, FLength|FPercent, strict ) );
         break;
 
     case CSS_PROP_HEIGHT:               // <length> | <percentage> | auto | inherit
@@ -805,7 +804,7 @@ bool CSSParser::parseValue( int propId, bool important )
             valid_primitive = true;
         else
             // ### handle multilength case where we allow relative units
-            valid_primitive = ( !id && validUnit( value, FLength|FPercent|FNonNeg, strict&(!nonCSSHint) ) );
+            valid_primitive = ( !id && validUnit( value, FLength|FPercent|FNonNeg, strict ) );
         break;
 
     case CSS_PROP_BOTTOM:               // <length> | <percentage> | auto | inherit
@@ -820,7 +819,7 @@ bool CSSParser::parseValue( int propId, bool important )
         if ( id == CSS_VAL_AUTO )
             valid_primitive = true;
         else
-            valid_primitive = ( !id && validUnit( value, FLength|FPercent, strict&(!nonCSSHint) ) );
+            valid_primitive = ( !id && validUnit( value, FLength|FPercent, strict ) );
         break;
 
     case CSS_PROP_Z_INDEX:              // auto | <integer> | inherit
@@ -840,7 +839,7 @@ bool CSSParser::parseValue( int propId, bool important )
         if ( id == CSS_VAL_NORMAL )
             valid_primitive = true;
         else
-            valid_primitive = ( !id && validUnit( value, FNumber|FLength|FPercent, strict&(!nonCSSHint) ) );
+            valid_primitive = ( !id && validUnit( value, FNumber|FLength|FPercent, strict ) );
         break;
     case CSS_PROP_COUNTER_INCREMENT:    // [ <identifier> <integer>? ]+ | none | inherit
         if ( id == CSS_VAL_NONE )
@@ -941,7 +940,7 @@ bool CSSParser::parseValue( int propId, bool important )
         if (id == CSS_VAL_SMALL || id == CSS_VAL_LARGE || id == CSS_VAL_MEDIUM)
             valid_primitive = true;
         else
-            valid_primitive = validUnit(value, FLength|FPercent, strict&(!nonCSSHint));
+            valid_primitive = validUnit(value, FLength|FPercent, strict);
         break;
     case CSS_PROP__KHTML_MARQUEE_STYLE:
         if (id == CSS_VAL_NONE || id == CSS_VAL_SLIDE || id == CSS_VAL_SCROLL || id == CSS_VAL_ALTERNATE ||
@@ -952,13 +951,13 @@ bool CSSParser::parseValue( int propId, bool important )
         if (id == CSS_VAL_INFINITE)
             valid_primitive = true;
         else
-            valid_primitive = validUnit(value, FInteger|FNonNeg, strict&(!nonCSSHint));
+            valid_primitive = validUnit(value, FInteger|FNonNeg, strict);
         break;
     case CSS_PROP__KHTML_MARQUEE_SPEED:
         if (id == CSS_VAL_NORMAL || id == CSS_VAL_SLOW || id == CSS_VAL_FAST)
             valid_primitive = true;
         else
-            valid_primitive = validUnit(value, FTime|FInteger|FNonNeg, strict&(!nonCSSHint));
+            valid_primitive = validUnit(value, FTime|FInteger|FNonNeg, strict);
         break;
     case CSS_PROP_TEXT_OVERFLOW: // clip | ellipsis
         if (id == CSS_VAL_CLIP || id == CSS_VAL_ELLIPSIS)
@@ -1141,9 +1140,9 @@ bool CSSParser::parseBackgroundShorthand(bool important)
                     goto fail;
 
                 if (!parsedProperty[i] && properties[i] != CSS_PROP_BACKGROUND_COLOR) {
-                    addBackgroundValue(values[i], new CSSInitialValueImpl());
+                    addBackgroundValue(values[i], new CSSInitialValueImpl(true/*implicit initial*/));
                     if (properties[i] == CSS_PROP_BACKGROUND_POSITION)
-                        addBackgroundValue(positionYValue, new CSSInitialValueImpl());
+                        addBackgroundValue(positionYValue, new CSSInitialValueImpl(true/*implicit initial*/));
                 }
                 parsedProperty[i] = false;
             }
@@ -1174,9 +1173,9 @@ bool CSSParser::parseBackgroundShorthand(bool important)
     // Fill in any remaining properties with the initial value.
     for (i = 0; i < numProperties; ++i) {
         if (!parsedProperty[i]) {
-            addBackgroundValue(values[i], new CSSInitialValueImpl());
+            addBackgroundValue(values[i], new CSSInitialValueImpl(true/*implicit initial*/));
             if (properties[i] == CSS_PROP_BACKGROUND_POSITION)
-                addBackgroundValue(positionYValue, new CSSInitialValueImpl());
+                addBackgroundValue(positionYValue, new CSSInitialValueImpl(true/*implicit initial*/));
         }
     }
 
@@ -1232,7 +1231,7 @@ bool CSSParser::parseShortHand(int propId, const int *properties, int numPropert
     m_implicitShorthand = true;
     for (int i = 0; i < numProperties; ++i) {
         if (!fnd[i])
-            addProperty(properties[i], new CSSInitialValueImpl(), important);
+            addProperty(properties[i], new CSSInitialValueImpl(true/*implicit initial*/), important);
     }
     m_implicitShorthand = false;
 
@@ -1416,11 +1415,15 @@ CSSValueImpl* CSSParser::parseBackgroundColor()
     return parseColor();
 }
 
-CSSValueImpl* CSSParser::parseBackgroundImage()
+CSSValueImpl* CSSParser::parseBackgroundImage(bool& didParse)
 {
-    if (valueList->current()->id == CSS_VAL_NONE)
+    didParse = false;
+    if (valueList->current()->id == CSS_VAL_NONE) {
+        didParse = true;
         return new CSSImageValueImpl();
+    }
     if (valueList->current()->unit == CSSPrimitiveValue::CSS_URI) {
+        didParse = true;
         DOMString uri = khtml::parseURL(domString(valueList->current()->string));
         if (!uri.isEmpty())
             return new CSSImageValueImpl(DOMString(KUrl(styleElement->baseURL(), uri.string()).url()),
@@ -1580,11 +1583,13 @@ bool CSSParser::parseBackgroundProperty(int propId, int& propId1, int& propId2,
                     if (currValue)
                         valueList->next();
                     break;
-                case CSS_PROP_BACKGROUND_IMAGE:
-                    currValue = parseBackgroundImage();
-                    if (currValue)
+                case CSS_PROP_BACKGROUND_IMAGE: {
+                    bool didParse=false;
+                    currValue = parseBackgroundImage(didParse);
+                    if (didParse)
                         valueList->next();
                     break;
+                }
                 case CSS_PROP__KHTML_BACKGROUND_CLIP:
                 case CSS_PROP__KHTML_BACKGROUND_ORIGIN:
                     if (val->id == CSS_VAL_BORDER || val->id == CSS_VAL_PADDING || val->id == CSS_VAL_CONTENT) {
