@@ -32,15 +32,18 @@
 #include <rendering/render_canvas.h>
 #include <khtml_part.h>
 #include <kjs/scriptfunction.h> // private API
+#include <kjs/function_object.h> // even more private API. or not. I dunno.
+#include "debugger/debugwindow.h"
 
 #include <kdebug.h>
 
 using namespace KJS;
+using namespace KJSDebugger;
 using namespace DOM;
 
 // -------------------------------------------------------------------------
 
-JSEventListener::JSEventListener(ObjectImp *_listener, ObjectImp *_compareListenerImp, ObjectImp *_win, bool _html)
+JSEventListener::JSEventListener(JSObject *_listener, JSObject *_compareListenerImp, JSObject *_win, bool _html)
   : listener( _listener ), compareListenerImp( _compareListenerImp ), html( _html ), win( _win )
 {
     //fprintf(stderr,"JSEventListener::JSEventListener this=%p listener=%p\n",this,listener.imp());
@@ -60,7 +63,9 @@ JSEventListener::~JSEventListener()
 void JSEventListener::handleEvent(DOM::Event &evt)
 {
 #ifdef KJS_DEBUGGER
-  if (KJSDebugWin::debugWindow() && KJSDebugWin::debugWindow()->inSession())
+  //### This is the wrong place to do this --- we need 
+  // a more global/general stategy to prevent unwanted event loop recursion issues.
+  if (DebugWindow::window() && DebugWindow::window()->inSession())
     return;
 #endif
   KHTMLPart *part = qobject_cast<KHTMLPart*>(static_cast<Window*>(win.get())->part());
@@ -78,7 +83,7 @@ void JSEventListener::handleEvent(DOM::Event &evt)
     args.append(getDOMEvent(exec,evt.handle()));
 
     // Set "this" to the event's current target
-    ObjectImp *thisObj = getDOMNode(exec,evt.currentTarget().handle())->getObject();
+    JSObject *thisObj = getDOMNode(exec,evt.currentTarget().handle())->getObject();
     if ( !thisObj ) {
       // Window events (window.onload/window.onresize etc.) must have 'this' set to the window.
       // DocumentImpl::defaultEventHandler sets currentTarget to 0 to mean 'window'.
@@ -92,7 +97,7 @@ void JSEventListener::handleEvent(DOM::Event &evt)
     interpreter->setCurrentEvent( &evt );
 
     interpreter->startCPUGuard();
-    ValueImp *retval = listener->call(exec, thisObj, args);
+    JSValue *retval = listener->call(exec, thisObj, args);
     interpreter->stopCPUGuard();
 
     window->setCurrentEvent( 0 );
@@ -118,13 +123,15 @@ DOM::DOMString JSEventListener::eventListenerType()
 	return "_khtml_JSEventListener";
 }
 
-ObjectImp *JSEventListener::listenerObj() const
+JSObject *JSEventListener::listenerObj() const
 {
   return listener;
 }
 
-JSLazyEventListener::JSLazyEventListener(const QString &_code, const QString &_name, ObjectImp *_win, DOM::NodeImpl* _originalNode)
-  : JSEventListener(0, 0, _win, true), code(_code), name(_name), parsed(false)
+JSLazyEventListener::JSLazyEventListener(const QString &_code, const QString &_url, int _lineNum,
+                               const QString &_name, JSObject *_win, DOM::NodeImpl* _originalNode)
+  : JSEventListener(0, 0, _win, true), code(_code), url(_url), lineNum(_lineNum), 
+    name(_name), parsed(false)
 {
   // We don't retain the original node, because we assume it
   // will stay alive as long as this handler object is around
@@ -151,7 +158,7 @@ void JSLazyEventListener::handleEvent(DOM::Event &evt)
 }
 
 
-ObjectImp *JSLazyEventListener::listenerObj() const
+JSObject *JSLazyEventListener::listenerObj() const
 {
   parseCode();
   return listener;
@@ -170,14 +177,15 @@ void JSLazyEventListener::parseCode() const
       ExecState *exec = interpreter->globalExec();
 
       //KJS::Constructor constr(KJS::Global::current().get("Function").imp());
-      KJS::ObjectImp *constr = interpreter->builtinFunction();
+      KJS::FunctionObjectImp *constr = static_cast<KJS::FunctionObjectImp*>(interpreter->builtinFunction());
       KJS::List args;
 
       static KJS::UString eventString("event");
-
-      args.append(KJS::String(eventString));
-      args.append(KJS::String(code));
-      listener = constr->construct(exec, args); // ### is globalExec ok ?
+      
+      args.append(jsString(eventString));
+      args.append(jsString(code));
+      listener = constr->construct(exec, args, 
+            Identifier(UString(name)), url, lineNum); // ### is globalExec ok ?
 
       if (exec->hadException()) {
         exec->clearException();
@@ -188,7 +196,6 @@ void JSLazyEventListener::parseCode() const
         listener = 0;// Error creating function
       } else {
         DeclaredFunctionImp *declFunc = static_cast<DeclaredFunctionImp*>(listener.get());
-        declFunc->setFunctionName(Identifier(UString(name)));
 
         if (originalNode)
         {
@@ -196,7 +203,7 @@ void JSLazyEventListener::parseCode() const
           // (and the document, and the form - see KJS::HTMLElement::eventHandlerScope)
           ScopeChain scope = declFunc->scope();
 
-          ObjectImp *thisObj = getDOMNode(exec, originalNode)->getObject();
+          JSObject *thisObj = getDOMNode(exec, originalNode)->getObject();
 
           if (thisObj) {
             static_cast<DOMNode*>(thisObj)->pushEventHandlerScope(exec, scope);
@@ -249,7 +256,7 @@ DOMEvent::DOMEvent(ExecState *exec, DOM::EventImpl* e)
   setPrototype(DOMEventProto::self(exec));
 }
 
-DOMEvent::DOMEvent(ObjectImp *proto, DOM::EventImpl* e):
+DOMEvent::DOMEvent(JSObject *proto, DOM::EventImpl* e):
   m_impl(e) {
   setPrototype(proto);
 }
@@ -269,53 +276,53 @@ bool DOMEvent::getOwnPropertySlot(ExecState *exec, const Identifier& propertyNam
   return getStaticValueSlot<DOMEvent, DOMObject>(exec,&DOMEventTable,this,propertyName,slot);
 }
 
-ValueImp *DOMEvent::getValueProperty(ExecState *exec, int token) const
+JSValue *DOMEvent::getValueProperty(ExecState *exec, int token) const
 {
   DOM::EventImpl& event = *impl();
   switch (token) {
   case Type:
-    return String(event.type());
+    return jsString(event.type());
   case Target:
   case SrcElement: /*MSIE extension - "the object that fired the event"*/
     return getDOMNode(exec,event.target());
   case CurrentTarget:
     return getDOMNode(exec,event.currentTarget());
   case EventPhase:
-    return Number((unsigned int)event.eventPhase());
+    return jsNumber((unsigned int)event.eventPhase());
   case Bubbles:
-    return Boolean(event.bubbles());
+    return jsBoolean(event.bubbles());
   case Cancelable:
-    return Boolean(event.cancelable());
+    return jsBoolean(event.cancelable());
   case TimeStamp:
-    return Number((long unsigned int)event.timeStamp()); // ### long long ?
+    return jsNumber((long unsigned int)event.timeStamp()); // ### long long ?
   case ReturnValue: // MSIE extension
     // return false == cancel, so this returns the -opposite- of defaultPrevented
-    return Boolean(!event.defaultPrevented());
+    return jsBoolean(!event.defaultPrevented());
   case CancelBubble: // MSIE extension
-    return Boolean(event.propagationStopped());
+    return jsBoolean(event.propagationStopped());
   default:
     kDebug(6070) << "WARNING: Unhandled token in DOMEvent::getValueProperty : " << token;
     return 0;
   }
 }
 
-ValueImp *DOMEvent::defaultValue(ExecState *exec, KJS::JSType hint) const
+JSValue *DOMEvent::defaultValue(ExecState *exec, KJS::JSType hint) const
 {
   if (m_impl->id() == EventImpl::ERROR_EVENT && !m_impl->message().isNull()) {
-    return String(m_impl->message());
+    return jsString(m_impl->message());
   }
   else
     return DOMObject::defaultValue(exec,hint);
 }
 
 void DOMEvent::put(ExecState *exec, const Identifier &propertyName,
-                      ValueImp *value, int attr)
+                      JSValue *value, int attr)
 {
   lookupPut<DOMEvent, DOMObject>(exec, propertyName, value, attr,
                                           &DOMEventTable, this);
 }
 
-void DOMEvent::putValueProperty(ExecState *exec, int token, ValueImp *value, int)
+void DOMEvent::putValueProperty(ExecState *exec, int token, JSValue *value, int)
 {
   switch (token) {
   case ReturnValue: // MSIE equivalent for "preventDefault" (but with a way to reset it)
@@ -331,28 +338,28 @@ void DOMEvent::putValueProperty(ExecState *exec, int token, ValueImp *value, int
   }
 }
 
-ValueImp *DOMEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMEventProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMEvent, thisObj );
   DOM::EventImpl& event = *static_cast<DOMEvent *>( thisObj )->impl();
   switch (id) {
     case DOMEvent::StopPropagation:
       event.stopPropagation(true);
-      return Undefined();
+      return jsUndefined();
     case DOMEvent::PreventDefault:
       event.preventDefault(true);
-      return Undefined();
+      return jsUndefined();
     case DOMEvent::InitEvent:
       event.initEvent(args[0]->toString(exec).domString(),args[1]->toBoolean(exec),args[2]->toBoolean(exec));
-      return Undefined();
+      return jsUndefined();
   };
-  return Undefined();
+  return jsUndefined();
 }
 
-ValueImp *KJS::getDOMEvent(ExecState *exec, DOM::EventImpl* ei)
+JSValue *KJS::getDOMEvent(ExecState *exec, DOM::EventImpl* ei)
 {
   if (!ei)
-    return Null();
+    return jsNull();
   ScriptInterpreter* interp = static_cast<ScriptInterpreter *>(exec->dynamicInterpreter());
   DOMObject *ret = interp->getDOMObject(ei);
   if (!ret) {
@@ -375,9 +382,9 @@ ValueImp *KJS::getDOMEvent(ExecState *exec, DOM::EventImpl* ei)
   return ret;
 }
 
-DOM::EventImpl* KJS::toEvent(ValueImp *val)
+DOM::EventImpl* KJS::toEvent(JSValue *val)
 {
-  ObjectImp *obj = val->getObject();
+  JSObject *obj = val->getObject();
   if (!obj || !obj->inherits(&DOMEvent::info))
     return 0;
 
@@ -433,13 +440,13 @@ bool EventExceptionConstructor::getOwnPropertySlot(ExecState *exec, const Identi
   return getStaticValueSlot<EventExceptionConstructor, DOMObject>(exec,&EventExceptionConstructorTable,this,propertyName,slot);
 }
 
-ValueImp *EventExceptionConstructor::getValueProperty(ExecState *, int token) const
+JSValue *EventExceptionConstructor::getValueProperty(ExecState *, int token) const
 {
   // We use the token as the value to return directly
-  return Number(token);
+  return jsNumber(token);
 }
 
-ValueImp *KJS::getEventExceptionConstructor(ExecState *exec)
+JSValue *KJS::getEventExceptionConstructor(ExecState *exec)
 {
   return cacheGlobalObject<EventExceptionConstructor>(exec, "[[eventException.constructor]]");
 }
@@ -470,7 +477,7 @@ KJS_IMPLEMENT_PROTOTYPE("DOMUIEvent",DOMUIEventProto,DOMUIEventProtoFunc)
 DOMUIEvent::DOMUIEvent(ExecState *exec, DOM::UIEventImpl* ue) :
   DOMEvent(DOMUIEventProto::self(exec), ue) {}
 
-DOMUIEvent::DOMUIEvent(ObjectImp *proto, DOM::UIEventImpl* ue) :
+DOMUIEvent::DOMUIEvent(JSObject *proto, DOM::UIEventImpl* ue) :
   DOMEvent(proto, ue) {}
 
 DOMUIEvent::~DOMUIEvent()
@@ -482,42 +489,42 @@ bool DOMUIEvent::getOwnPropertySlot(ExecState *exec, const Identifier& propertyN
   return getStaticValueSlot<DOMUIEvent, DOMEvent>(exec,&DOMUIEventTable,this,propertyName,slot);
 }
 
-ValueImp *DOMUIEvent::getValueProperty(ExecState *exec, int token) const
+JSValue *DOMUIEvent::getValueProperty(ExecState *exec, int token) const
 {
   DOM::UIEventImpl& event = *impl();
   switch (token) {
   case View:
     return getDOMAbstractView(exec,event.view());
   case Detail:
-    return Number(event.detail());
+    return jsNumber(event.detail());
   case KeyCode:
     // IE-compatibility
-    return Number(event.keyCode());
+    return jsNumber(event.keyCode());
   case CharCode:
     // IE-compatibility
-    return Number(event.charCode());
+    return jsNumber(event.charCode());
   case LayerX:
     // NS-compatibility
-    return Number(event.layerX());
+    return jsNumber(event.layerX());
   case LayerY:
     // NS-compatibility
-    return Number(event.layerY());
+    return jsNumber(event.layerY());
   case PageX:
     // NS-compatibility
-    return Number(event.pageX());
+    return jsNumber(event.pageX());
   case PageY:
     // NS-compatibility
-    return Number(event.pageY());
+    return jsNumber(event.pageY());
   case Which:
     // NS-compatibility
-    return Number(event.which());
+    return jsNumber(event.which());
   default:
     kDebug(6070) << "WARNING: Unhandled token in DOMUIEvent::getValueProperty : " << token;
-    return Undefined();
+    return jsUndefined();
   }
 }
 
-ValueImp *DOMUIEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMUIEventProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMUIEvent, thisObj );
   DOM::UIEventImpl& uiEvent = *static_cast<DOMUIEvent *>(thisObj)->impl();
@@ -530,9 +537,9 @@ ValueImp *DOMUIEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisOb
                                                      v,
                                                      args[4]->toInteger(exec));
       }
-      return Undefined();
+      return jsUndefined();
   }
-  return Undefined();
+  return jsUndefined();
 }
 
 // -------------------------------------------------------------------------
@@ -582,20 +589,20 @@ bool DOMMouseEvent::getOwnPropertySlot(ExecState *exec, const Identifier& proper
   return getStaticValueSlot<DOMMouseEvent, DOMUIEvent>(exec,&DOMMouseEventTable,this,propertyName,slot);
 }
 
-ValueImp *DOMMouseEvent::getValueProperty(ExecState *exec, int token) const
+JSValue *DOMMouseEvent::getValueProperty(ExecState *exec, int token) const
 {
   DOM::MouseEventImpl& event = *impl();
   switch (token) {
   case ScreenX:
-    return Number(event.screenX());
+    return jsNumber(event.screenX());
   case ScreenY:
-    return Number(event.screenY());
+    return jsNumber(event.screenY());
   case ClientX:
   case X:
-    return Number(event.clientX());
+    return jsNumber(event.clientX());
   case ClientY:
   case Y:
-    return Number(event.clientY());
+    return jsNumber(event.clientY());
   case OffsetX:
   case OffsetY: // MSIE extension
   {
@@ -621,26 +628,26 @@ ValueImp *DOMMouseEvent::getValueProperty(ExecState *exec, int token) const
         y += cYPos;
       }
     }
-    return Number( token == OffsetX ? x : y );
+    return jsNumber( token == OffsetX ? x : y );
   }
   case CtrlKey:
-    return Boolean(event.ctrlKey());
+    return jsBoolean(event.ctrlKey());
   case ShiftKey:
-    return Boolean(event.shiftKey());
+    return jsBoolean(event.shiftKey());
   case AltKey:
-    return Boolean(event.altKey());
+    return jsBoolean(event.altKey());
   case MetaKey:
-    return Boolean(event.metaKey());
+    return jsBoolean(event.metaKey());
   case Button:
   {
     if ( exec->dynamicInterpreter()->compatMode() == Interpreter::NetscapeCompat ) {
-        return Number(event.button());
+        return jsNumber(event.button());
     }
     // Tricky. The DOM (and khtml) use 0 for LMB, 1 for MMB and 2 for RMB
     // but MSIE uses 1=LMB, 2=RMB, 4=MMB, as a bitfield
     int domButton = event.button();
     int button = domButton==0 ? 1 : domButton==1 ? 4 : domButton==2 ? 2 : 0;
-    return Number( (unsigned int)button );
+    return jsNumber( (unsigned int)button );
   }
   case ToElement:
     // MSIE extension - "the object toward which the user is moving the mouse pointer"
@@ -661,7 +668,7 @@ ValueImp *DOMMouseEvent::getValueProperty(ExecState *exec, int token) const
   }
 }
 
-ValueImp *DOMMouseEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMMouseEventProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMMouseEvent, thisObj );
   DOM::MouseEventImpl& mouseEvent = *static_cast<DOMMouseEvent *>(thisObj)->impl();
@@ -682,9 +689,9 @@ ValueImp *DOMMouseEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thi
                                 args[12]->toBoolean(exec), // metaKeyArg
                                 args[13]->toInteger(exec), // buttonArg
                                 toNode(args[14])); // relatedTargetArg
-      return Undefined();
+      return jsUndefined();
   }
-  return Undefined();
+  return jsUndefined();
 }
 
 // -------------------------------------------------------------------------
@@ -702,7 +709,7 @@ const ClassInfo DOMKeyEventBase::info = { "KeyEventBase", &DOMUIEvent::info, &DO
 @end
 */
 
-DOMKeyEventBase::DOMKeyEventBase(ObjectImp* proto, DOM::KeyEventBaseImpl* ke) :
+DOMKeyEventBase::DOMKeyEventBase(JSObject* proto, DOM::KeyEventBaseImpl* ke) :
   DOMUIEvent(proto, ke) {}
 
 DOMKeyEventBase::~DOMKeyEventBase()
@@ -716,27 +723,27 @@ bool DOMKeyEventBase::getOwnPropertySlot(ExecState *exec, const Identifier& prop
   return getStaticValueSlot<DOMKeyEventBase, DOMUIEvent>(exec,&DOMKeyEventBaseTable,this,propertyName,slot);
 }
 
-ValueImp* DOMKeyEventBase::getValueProperty(ExecState *, int token) const
+JSValue* DOMKeyEventBase::getValueProperty(ExecState *, int token) const
 {
   DOM::KeyEventBaseImpl* tevent = impl();
   switch (token) {
   case Key:
-    return Number(tevent->keyVal());
+    return jsNumber(tevent->keyVal());
   case VirtKey:
-    return Number(tevent->virtKeyVal());
+    return jsNumber(tevent->virtKeyVal());
   // these modifier attributes actually belong into a KeyboardEvent interface,
   // but we want them on "keypress" as well.
   case CtrlKey:
-    return Boolean(tevent->ctrlKey());
+    return jsBoolean(tevent->ctrlKey());
   case ShiftKey:
-    return Boolean(tevent->shiftKey());
+    return jsBoolean(tevent->shiftKey());
   case AltKey:
-    return Boolean(tevent->altKey());
+    return jsBoolean(tevent->altKey());
   case MetaKey:
-    return Boolean(tevent->metaKey());
+    return jsBoolean(tevent->metaKey());
   default:
     kDebug(6070) << "WARNING: Unhandled token in DOMKeyEventBase::getValueProperty : " << token;
-    return KJS::Undefined();
+    return jsUndefined();
   }
 }
 
@@ -772,19 +779,19 @@ bool DOMTextEvent::getOwnPropertySlot(ExecState *exec, const Identifier& propert
 }
 
 
-ValueImp *DOMTextEvent::getValueProperty(ExecState *, int token) const
+JSValue *DOMTextEvent::getValueProperty(ExecState *, int token) const
 {
   DOM::TextEventImpl& tevent = *impl();
   switch (token) {
   case Data:
-    return String(tevent.data());
+    return jsString(tevent.data());
   default:
     kDebug(6070) << "WARNING: Unhandled token in DOMTextEvent::getValueProperty : " << token;
-    return KJS::Undefined();
+    return jsUndefined();
   }
 }
 
-ValueImp *DOMTextEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMTextEventProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMTextEvent, thisObj );
   DOM::TextEventImpl& keyEvent = *static_cast<DOMTextEvent *>(thisObj)->impl();
@@ -796,9 +803,9 @@ ValueImp *DOMTextEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *this
                             args[2]->toBoolean(exec), // cancelableArg
                             toAbstractView(args[3]), // viewArg
                             args[4]->toString(exec).domString()); // dataArg
-      return Undefined();
+      return jsUndefined();
   }
-  return Undefined();
+  return jsUndefined();
 }
 // -------------------------------------------------------------------------
 const ClassInfo DOMKeyboardEvent::info = { "KeyboardEvent", &DOMKeyEventBase::info, &DOMKeyboardEventTable, 0 };
@@ -832,21 +839,21 @@ bool DOMKeyboardEvent::getOwnPropertySlot(ExecState *exec, const Identifier& pro
   return getStaticValueSlot<DOMKeyboardEvent, DOMKeyEventBase>(exec,&DOMKeyboardEventTable,this,propertyName,slot);
 }
 
-ValueImp* DOMKeyboardEvent::getValueProperty(ExecState *, int token) const
+JSValue* DOMKeyboardEvent::getValueProperty(ExecState *, int token) const
 {
   DOM::KeyboardEventImpl* tevent = impl();
   switch (token) {
   case KeyIdentifier:
-    return String(tevent->keyIdentifier());
+    return jsString(tevent->keyIdentifier());
   case KeyLocation:
-    return Number(tevent->keyLocation());
+    return jsNumber(tevent->keyLocation());
   default:
     kDebug(6070) << "WARNING: Unhandled token in DOMKeyboardEvent::getValueProperty : " << token;
-    return KJS::Undefined();
+    return jsUndefined();
   }
 }
 
-ValueImp* DOMKeyboardEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp* thisObj, const List &args)
+JSValue* DOMKeyboardEventProtoFunc::callAsFunction(ExecState *exec, JSObject* thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMKeyboardEvent, thisObj );
   DOM::KeyboardEventImpl* keyEvent = static_cast<DOMKeyboardEvent *>(thisObj)->impl();
@@ -861,9 +868,9 @@ ValueImp* DOMKeyboardEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp*
                             args[6]->toString(exec).domString()); //modifiersList
       break;
     case DOMKeyboardEvent::GetModifierState:
-      return Boolean(keyEvent->getModifierState(args[0]->toString(exec).domString()));
+      return jsBoolean(keyEvent->getModifierState(args[0]->toString(exec).domString()));
   }
-  return Undefined();
+  return jsUndefined();
 }
 
 // -------------------------------------------------------------------------
@@ -889,13 +896,13 @@ bool KeyboardEventConstructor::getOwnPropertySlot(ExecState *exec, const Identif
   return getStaticValueSlot<KeyboardEventConstructor, DOMObject>(exec,&KeyboardEventConstructorTable,this,propertyName,slot);
 }
 
-ValueImp* KeyboardEventConstructor::getValueProperty(ExecState *, int token) const
+JSValue* KeyboardEventConstructor::getValueProperty(ExecState *, int token) const
 {
   // We use the token as the value to return directly
-  return Number(token);
+  return jsNumber(token);
 }
 
-ValueImp* KJS::getKeyboardEventConstructor(ExecState *exec)
+JSValue* KJS::getKeyboardEventConstructor(ExecState *exec)
 {
   return cacheGlobalObject<KeyboardEventConstructor>(exec, "[[keyboardEvent.constructor]]");
 }
@@ -920,13 +927,13 @@ bool MutationEventConstructor::getOwnPropertySlot(ExecState *exec, const Identif
   return getStaticValueSlot<MutationEventConstructor, DOMObject>(exec,&MutationEventConstructorTable,this,propertyName,slot);
 }
 
-ValueImp *MutationEventConstructor::getValueProperty(ExecState *, int token) const
+JSValue *MutationEventConstructor::getValueProperty(ExecState *, int token) const
 {
   // We use the token as the value to return directly
-  return Number(token);
+  return jsNumber(token);
 }
 
-ValueImp *KJS::getMutationEventConstructor(ExecState *exec)
+JSValue *KJS::getMutationEventConstructor(ExecState *exec)
 {
   return cacheGlobalObject<MutationEventConstructor>(exec, "[[mutationEvent.constructor]]");
 }
@@ -962,7 +969,7 @@ bool DOMMutationEvent::getOwnPropertySlot(ExecState *exec, const Identifier& pro
   return getStaticValueSlot<DOMMutationEvent, DOMEvent>(exec,&DOMMutationEventTable,this,propertyName,slot);
 }
 
-ValueImp *DOMMutationEvent::getValueProperty(ExecState *exec, int token) const
+JSValue *DOMMutationEvent::getValueProperty(ExecState *exec, int token) const
 {
   DOM::MutationEventImpl& event = *impl();
   switch (token) {
@@ -971,20 +978,20 @@ ValueImp *DOMMutationEvent::getValueProperty(ExecState *exec, int token) const
     return getDOMNode(exec,relatedNode.handle());
   }
   case PrevValue:
-    return String(event.prevValue());
+    return jsString(event.prevValue());
   case NewValue:
-    return String(event.newValue());
+    return jsString(event.newValue());
   case AttrName:
-    return String(event.attrName());
+    return jsString(event.attrName());
   case AttrChange:
-    return Number((unsigned int)event.attrChange());
+    return jsNumber((unsigned int)event.attrChange());
   default:
     kDebug(6070) << "WARNING: Unhandled token in DOMMutationEvent::getValueProperty : " << token;
     return 0;
   }
 }
 
-ValueImp *DOMMutationEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMMutationEventProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMMutationEvent, thisObj );
   DOM::MutationEventImpl& mutationEvent = *static_cast<DOMMutationEvent *>(thisObj)->impl();
@@ -998,7 +1005,7 @@ ValueImp *DOMMutationEventProtoFunc::callAsFunction(ExecState *exec, ObjectImp *
                                       args[5]->toString(exec).domString(), // newValueArg
                                       args[6]->toString(exec).domString(), // attrNameArg
                                       args[7]->toInteger(exec)); // attrChangeArg
-      return Undefined();
+      return jsUndefined();
   }
-  return Undefined();
+  return jsUndefined();
 }
