@@ -99,7 +99,7 @@ void XMLHttpRequestQObject::slotRedirection( KIO::Job* job, const KUrl& url)
 }
 
 XMLHttpRequestConstructorImp::XMLHttpRequestConstructorImp(ExecState *, DOM::DocumentImpl* d)
-    : ObjectImp(), doc(d)
+    : JSObject(), doc(d)
 {
 }
 
@@ -108,7 +108,7 @@ bool XMLHttpRequestConstructorImp::implementsConstruct() const
   return true;
 }
 
-ObjectImp *XMLHttpRequestConstructorImp::construct(ExecState *exec, const List &)
+JSObject *XMLHttpRequestConstructorImp::construct(ExecState *exec, const List &)
 {
   return new XMLHttpRequest(exec, doc.get());
 }
@@ -133,11 +133,11 @@ bool XMLHttpRequest::getOwnPropertySlot(ExecState *exec, const Identifier& prope
   return getStaticValueSlot<XMLHttpRequest, DOMObject>(exec, &XMLHttpRequestTable, this, propertyName, slot);
 }
 
-ValueImp *XMLHttpRequest::getValueProperty(ExecState *exec, int token) const
+JSValue *XMLHttpRequest::getValueProperty(ExecState *exec, int token) const
 {
   switch (token) {
   case ReadyState:
-    return Number(m_state);
+    return jsNumber(m_state);
   case ResponseText:
     return ::getStringOrNull(DOM::DOMString(response));
   case ResponseXML:
@@ -150,7 +150,8 @@ ValueImp *XMLHttpRequest::getValueProperty(ExecState *exec, int token) const
       if (!m_mimeTypeOverride.isEmpty()) {
 	  mimeType = m_mimeTypeOverride;
       } else {
-	  ValueImp *header = getResponseHeader("Content-Type");
+          int dummy;
+	  JSValue *header = getResponseHeader("Content-Type", dummy);
 	  if (header->type() != UndefinedType)
 	      mimeType = header->toString(exec).qstring().split(";")[0].trimmed();
       }
@@ -172,7 +173,7 @@ ValueImp *XMLHttpRequest::getValueProperty(ExecState *exec, int token) const
     }
 
     if (!typeIsXML) {
-      return Null();
+      return jsNull();
     }
 
     return getDOMNode(exec,responseXML.get());
@@ -184,13 +185,13 @@ ValueImp *XMLHttpRequest::getValueProperty(ExecState *exec, int token) const
    if (onReadyStateChangeListener && onReadyStateChangeListener->listenerObj()) {
      return onReadyStateChangeListener->listenerObj();
    } else {
-     return Null();
+     return jsNull();
    }
   case Onload:
    if (onLoadListener && onLoadListener->listenerObj()) {
      return onLoadListener->listenerObj();
    } else {
-     return Null();
+     return jsNull();
    }
   default:
     kWarning() << "XMLHttpRequest::getValueProperty unhandled token " << token;
@@ -198,12 +199,12 @@ ValueImp *XMLHttpRequest::getValueProperty(ExecState *exec, int token) const
   }
 }
 
-void XMLHttpRequest::put(ExecState *exec, const Identifier &propertyName, ValueImp *value, int attr)
+void XMLHttpRequest::put(ExecState *exec, const Identifier &propertyName, JSValue *value, int attr)
 {
   lookupPut<XMLHttpRequest,DOMObject>(exec, propertyName, value, attr, &XMLHttpRequestTable, this );
 }
 
-void XMLHttpRequest::putValueProperty(ExecState *exec, int token, ValueImp *value, int /*attr*/)
+void XMLHttpRequest::putValueProperty(ExecState *exec, int token, JSValue *value, int /*attr*/)
 {
   switch(token) {
   case Onreadystatechange:
@@ -246,7 +247,7 @@ static bool isValidFieldName(const QString& name)
     return true;
 }
 
-static bool isValidFieldValue(const QString& name)
+static bool isValidFieldValue(const QString& /*name*/)
 {
     // ### what is invalid?
     return true;
@@ -327,7 +328,7 @@ void XMLHttpRequest::changeState(XMLHttpRequestState newState)
 {
   if (m_state != newState) {
     m_state = newState;
-    ProtectedPtr<ObjectImp> ref(this);
+    ProtectedPtr<JSObject> ref(this);
 
     if (onReadyStateChangeListener != 0 && doc->view() && doc->view()->part()) {
       DOM::Event ev = doc->view()->part()->document().createEvent("HTMLEvents");
@@ -559,25 +560,43 @@ void XMLHttpRequest::setRequestHeader(const QString& _name, const QString& _valu
   m_requestHeaders[_name] = value;
 }
 
-ValueImp *XMLHttpRequest::getAllResponseHeaders() const
+JSValue *XMLHttpRequest::getAllResponseHeaders(int& ec) const
 {
+  if (m_state < XHRS_Receiving) {
+      ec = DOMException::INVALID_STATE_ERR;
+      return jsString("");
+  }
+  
+  // ### test error flag, return jsNull
+
   if (responseHeaders.isEmpty()) {
-    return Undefined();
+    return jsUndefined();
   }
 
   int endOfLine = responseHeaders.indexOf("\n");
 
   if (endOfLine == -1) {
-    return Undefined();
+    return jsUndefined();
   }
 
-  return String(responseHeaders.mid(endOfLine + 1) + '\n');
+  return jsString(responseHeaders.mid(endOfLine + 1) + '\n');
 }
 
-ValueImp *XMLHttpRequest::getResponseHeader(const QString& name) const
+JSValue *XMLHttpRequest::getResponseHeader(const QString& name, int& ec) const
 {
+  if (m_state < XHRS_Receiving) {
+      ec = DOMException::INVALID_STATE_ERR;
+      return jsString("");
+  }
+
+  if (!isValidFieldName(name)) {
+    return jsString("");
+  }
+
+  // ### test error flag, return jsNull
+
   if (responseHeaders.isEmpty()) {
-    return Undefined();
+    return jsUndefined();
   }
 
   QRegExp headerLinePattern(name + ':', Qt::CaseInsensitive);
@@ -596,18 +615,18 @@ ValueImp *XMLHttpRequest::getResponseHeader(const QString& name) const
 
 
   if (headerLinePos == -1) {
-    return Null();
+    return jsNull();
   }
 
   int endOfLine = responseHeaders.indexOf("\n", headerLinePos + matchLength);
 
-  return String(responseHeaders.mid(headerLinePos + matchLength, endOfLine - (headerLinePos + matchLength)).trimmed());
+  return jsString(responseHeaders.mid(headerLinePos + matchLength, endOfLine - (headerLinePos + matchLength)).trimmed());
 }
 
-static ValueImp *httpStatus(const QString& response, bool textStatus = false)
+static JSValue *httpStatus(const QString& response, bool textStatus = false)
 {
   if (response.isEmpty()) {
-    return Undefined();
+    return jsUndefined();
   }
 
   int endOfLine = response.indexOf("\n");
@@ -616,12 +635,12 @@ static ValueImp *httpStatus(const QString& response, bool textStatus = false)
   int codeEnd = firstLine.indexOf(" ", codeStart + 1);
 
   if (codeStart == -1 || codeEnd == -1) {
-    return Undefined();
+    return jsUndefined();
   }
 
   if (textStatus) {
     QString statusText = firstLine.mid(codeEnd + 1, endOfLine - (codeEnd + 1)).trimmed();
-    return String(statusText);
+    return jsString(statusText);
   }
 
   QString number = firstLine.mid(codeStart + 1, codeEnd - (codeStart + 1));
@@ -629,18 +648,18 @@ static ValueImp *httpStatus(const QString& response, bool textStatus = false)
   bool ok = false;
   int code = number.toInt(&ok);
   if (!ok) {
-    return Undefined();
+    return jsUndefined();
   }
 
-  return Number(code);
+  return jsNumber(code);
 }
 
-ValueImp *XMLHttpRequest::getStatus() const
+JSValue *XMLHttpRequest::getStatus() const
 {
   return httpStatus(responseHeaders);
 }
 
-ValueImp *XMLHttpRequest::getStatusText() const
+JSValue *XMLHttpRequest::getStatusText() const
 {
   return httpStatus(responseHeaders, true);
 }
@@ -756,7 +775,7 @@ void XMLHttpRequest::slotData(KIO::Job*, const QByteArray &_data)
   }
 }
 
-ValueImp *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   if (!thisObj->inherits(&XMLHttpRequest::info)) {
     return throwError(exec, TypeError);
@@ -769,14 +788,21 @@ ValueImp *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, ObjectImp *th
   switch (id) {
   case XMLHttpRequest::Abort:
     request->abort();
-    return Undefined();
+    return jsUndefined();
   case XMLHttpRequest::GetAllResponseHeaders:
-    return request->getAllResponseHeaders();
-  case XMLHttpRequest::GetResponseHeader:
-    if (args.size() < 1)
-        return throwError(exec, SyntaxError, "Not enough arguments");
-
-    return request->getResponseHeader(args[0]->toString(exec).qstring());
+    {
+      JSValue *ret = request->getAllResponseHeaders(ec);
+      setDOMException(exec, ec);
+      return ret;
+    }
+  case XMLHttpRequest::GetResponseHeader: 
+    {
+      if (args.size() < 1)
+          return throwError(exec, SyntaxError, "Not enough arguments");
+      JSValue *ret = request->getResponseHeader(args[0]->toString(exec).qstring(), ec);
+      setDOMException(exec, ec);
+      return ret;
+    }
   case XMLHttpRequest::Open:
     {
       if (args.size() < 2)
@@ -785,7 +811,7 @@ ValueImp *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, ObjectImp *th
       QString method = args[0]->toString(exec).qstring();
       KHTMLPart *part = qobject_cast<KHTMLPart*>(Window::retrieveActive(exec)->part());
       if (!part)
-        return Undefined();
+        return jsUndefined();
       KUrl url = KUrl(part->document().completeURL(args[1]->toString(exec).qstring()).string());
 
       bool async = true;
@@ -803,7 +829,7 @@ ValueImp *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, ObjectImp *th
 
       request->open(method, url, async);
 
-      return Undefined();
+      return jsUndefined();
     }
   case XMLHttpRequest::Send:
     {
@@ -817,7 +843,7 @@ ValueImp *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, ObjectImp *th
             body = doc->toString().string();
             // FIXME: also need to set content type, including encoding!
   
-          } catch(DOM::DOMException& e) {
+          } catch(DOM::DOMException&) {
             return throwError(exec, GeneralError, "Exception serializing document");
           }
         } else {
@@ -853,7 +879,7 @@ ValueImp *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, ObjectImp *th
       return jsUndefined();
   }
 
-  return Undefined();
+  return jsUndefined();
 }
 
 } // end namespace
