@@ -121,7 +121,7 @@ void HTMLBodyElementImpl::parseAttribute(AttributeImpl *attr)
     case ATTR_LINK:
     {
         if(!m_styleSheet) {
-            m_styleSheet = new CSSStyleSheetImpl(this,DOMString(),true);
+            m_styleSheet = new CSSStyleSheetImpl(this,DOMString(),true /*implicit*/);
             m_styleSheet->ref();
         }
         QString aStr;
@@ -132,8 +132,7 @@ void HTMLBodyElementImpl::parseAttribute(AttributeImpl *attr)
 	else if ( attr->id() == ATTR_ALINK )
 	    aStr = "a:active";
 	aStr += " { color: " + attr->value().string() + "; }";
-        m_styleSheet->parseString(aStr, !getDocument()->inCompatMode());
-        m_styleSheet->setNonCSSHints();
+        m_styleSheet->parseString(aStr, false);
         if (attached())
             getDocument()->updateStyleSelector();
         break;
@@ -663,6 +662,33 @@ NodeImpl::Id HTMLIFrameElementImpl::id() const
     return ID_IFRAME;
 }
 
+void HTMLIFrameElementImpl::defaultEventHandler(EventImpl *e)
+{
+     // ### duplicated in HTMLObjectBaseElementImpl
+     if ( e->target() == this && m_render && m_render->isWidget() 
+                                    && static_cast<RenderWidget*>(m_render)->isRedirectedWidget() 
+                                    && qobject_cast<KHTMLView*>(static_cast<RenderWidget*>(m_render)->widget())) {
+        switch(e->id())  {
+        case EventImpl::MOUSEDOWN_EVENT:
+        case EventImpl::MOUSEUP_EVENT:
+        case EventImpl::MOUSEMOVE_EVENT:
+        case EventImpl::MOUSEOUT_EVENT:
+        case EventImpl::MOUSEOVER_EVENT:
+        case EventImpl::KHTML_MOUSEWHEEL_EVENT:
+        case EventImpl::KEYDOWN_EVENT:
+        case EventImpl::KEYUP_EVENT:
+        case EventImpl::KEYPRESS_EVENT:
+        case EventImpl::DOMFOCUSIN_EVENT:
+        case EventImpl::DOMFOCUSOUT_EVENT:
+            if (static_cast<RenderWidget*>(m_render)->handleEvent(*e))
+                e->setDefaultHandled();
+        default:
+            break;
+        }
+    }
+    HTMLFrameElementImpl::defaultEventHandler(e);
+}
+
 void HTMLIFrameElementImpl::parseAttribute(AttributeImpl *attr )
 {
     switch (  attr->id() )
