@@ -94,12 +94,12 @@ HTMLFormElementImpl::~HTMLFormElementImpl()
     if (getDocument() && getDocument()->view() && getDocument()->view()->part()) {
         getDocument()->view()->part()->dequeueWallet(this);
     }
-    Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements);
-    for (; it.current(); ++it)
-        it.current()->m_form = 0;
-    Q3PtrListIterator<HTMLImageElementImpl> it2(imgElements);
-    for (; it2.current(); ++it2)
-        it2.current()->m_form = 0;
+    QListIterator<HTMLGenericFormElementImpl*> it(formElements);
+    while (it.hasNext())
+        it.next()->m_form = 0;
+    QListIterator<HTMLImageElementImpl*> it2(imgElements);
+    while (it2.hasNext())
+        it2.next()->m_form = 0;
 }
 
 NodeImpl::Id HTMLFormElementImpl::id() const
@@ -126,9 +126,9 @@ DOMString HTMLFormElementImpl::action() const
 long HTMLFormElementImpl::length() const
 {
     int len = 0;
-    Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements);
-    for (; it.current(); ++it)
-	if (it.current()->isEnumeratable())
+    QListIterator<HTMLGenericFormElementImpl*> it(formElements);
+    while (it.hasNext())
+	if (it.next()->isEnumeratable())
 	    ++len;
 
     return len;
@@ -274,8 +274,8 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
 
     QStringList fileUploads, fileNotUploads;
 
-    for (Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
-        HTMLGenericFormElementImpl* const current = it.current();
+    for (QListIterator<HTMLGenericFormElementImpl*> it(formElements); it.hasNext();) {
+        HTMLGenericFormElementImpl* const current = it.next();
         khtml::encodingList lst;
 
         if (!current->disabled() && current->encoding(codec, lst, m_multipart))
@@ -345,13 +345,10 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
                     ++it;
 
                     // append body
-                    const unsigned int old_size = form_data.size();
-		    form_data.resize( old_size + hstr.length() + (*it).size() + 1);
-                    memcpy(form_data.data() + old_size, hstr.data(), hstr.length());
-		    memcpy(form_data.data() + old_size + hstr.length(), *it, (*it).size());
-                    form_data[form_data.size()-2] = '\r';
-                    form_data[form_data.size()-1] = '\n';
-
+                    form_data.append(hstr);
+                    form_data.append(*it);
+                    form_data.append("\r\n");
+                    
 		    // reset unsubmittedFormChange flag
                     if (current->id() == ID_INPUT &&
                         static_cast<HTMLInputElementImpl*>(current)->inputType() == HTMLInputElementImpl::TEXT)
@@ -468,9 +465,10 @@ void HTMLFormElementImpl::walletOpened(KWallet::Wallet *w) {
     if (w->readMap(key, map))
         return; // failed, abort
 
-    for (Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
-        if (it.current()->id() == ID_INPUT) {
-            HTMLInputElementImpl* const current = static_cast<HTMLInputElementImpl*>(it.current());
+    for (QListIterator<HTMLGenericFormElementImpl*> it(formElements); it.hasNext();) {
+        HTMLGenericFormElementImpl* const cur = it.next();
+        if (cur->id() == ID_INPUT) {
+            HTMLInputElementImpl* const current = static_cast<HTMLInputElementImpl*>(cur);
             if ((current->inputType() == HTMLInputElementImpl::PASSWORD ||
                     current->inputType() == HTMLInputElementImpl::TEXT) &&
                     !current->readOnly() &&
@@ -489,15 +487,16 @@ void HTMLFormElementImpl::submitFromKeyboard()
     // if there is none, do a submit anyway if not more
     // than one <input type=text> or <input type=password>
     unsigned int inputtext = 0;
-    for (Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
-        if (it.current()->id() == ID_BUTTON) {
-            HTMLButtonElementImpl* const current = static_cast<HTMLButtonElementImpl *>(it.current());
+    for (QListIterator<HTMLGenericFormElementImpl*> it(formElements); it.hasNext();) {
+        HTMLGenericFormElementImpl* const cur = it.next();
+        if (cur->id() == ID_BUTTON) {
+            HTMLButtonElementImpl* const current = static_cast<HTMLButtonElementImpl *>(cur);
             if (current->buttonType() == HTMLButtonElementImpl::SUBMIT && !current->disabled()) {
                 current->click();
                 return;
             }
-        } else if (it.current()->id() == ID_INPUT) {
-            HTMLInputElementImpl* const current = static_cast<HTMLInputElementImpl *>(it.current());
+        } else if (cur->id() == ID_INPUT) {
+            HTMLInputElementImpl* const current = static_cast<HTMLInputElementImpl *>(cur);
             switch(current->inputType())  {
             case HTMLInputElementImpl::SUBMIT:
             case HTMLInputElementImpl::IMAGE:
@@ -530,9 +529,10 @@ void HTMLFormElementImpl::gatherWalletData()
     m_haveTextarea = false;
     const KUrl formUrl(getDocument()->URL());
     if (view && !view->nonPasswordStorableSite(formUrl.host())) {
-        for (Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
-            if (it.current()->id() == ID_INPUT)  {
-                HTMLInputElementImpl* const c = static_cast<HTMLInputElementImpl*> (it.current());
+        for (QListIterator<HTMLGenericFormElementImpl*> it(formElements); it.hasNext();) {
+            HTMLGenericFormElementImpl* const cur = it.next();
+            if (cur->id() == ID_INPUT)  {
+                HTMLInputElementImpl* const c = static_cast<HTMLInputElementImpl*> (cur);
                 if ((c->inputType() == HTMLInputElementImpl::TEXT ||
                         c->inputType() == HTMLInputElementImpl::PASSWORD) &&
                         !c->readOnly())  {
@@ -542,7 +542,7 @@ void HTMLFormElementImpl::gatherWalletData()
                         m_havePassword = true;
                 }
             }
-            else if (it.current()->id() == ID_TEXTAREA)
+            else if (cur->id() == ID_TEXTAREA)
                 m_haveTextarea = true;
         }
     }
@@ -701,8 +701,8 @@ void HTMLFormElementImpl::reset(  )
         return;
     }
 
-    for (Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it)
-        it.current()->reset();
+    for (QListIterator<HTMLGenericFormElementImpl*> it(formElements); it.hasNext();)
+        it.next()->reset();
 
     m_inreset = false;
 }
@@ -780,8 +780,8 @@ void HTMLFormElementImpl::addId   (const QString& id)
 
 void HTMLFormElementImpl::radioClicked( HTMLGenericFormElementImpl *caller )
 {
-    for (Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
-        HTMLGenericFormElementImpl* const current = it.current();
+    for (QListIterator<HTMLGenericFormElementImpl*> it(formElements); it.hasNext();) {
+        HTMLGenericFormElementImpl* const current = it.next();
         if (current->id() == ID_INPUT &&
             static_cast<HTMLInputElementImpl*>(current)->inputType() == HTMLInputElementImpl::RADIO &&
             current != caller && current->form() == caller->form() && current->name() == caller->name())
@@ -796,7 +796,9 @@ void HTMLFormElementImpl::registerFormElement(HTMLGenericFormElementImpl *e)
 
 void HTMLFormElementImpl::removeFormElement(HTMLGenericFormElementImpl *e)
 {
-    formElements.remove(e);
+    int i = formElements.indexOf(e); 
+    if (i != -1)
+        formElements.removeAt(i);
 }
 
 void HTMLFormElementImpl::registerImgElement(HTMLImageElementImpl *e)
@@ -806,7 +808,9 @@ void HTMLFormElementImpl::registerImgElement(HTMLImageElementImpl *e)
 
 void HTMLFormElementImpl::removeImgElement(HTMLImageElementImpl *e)
 {
-    imgElements.remove(e);
+    int i = imgElements.indexOf(e); 
+    if (i != -1)
+        imgElements.removeAt(i);
 }
 
 // -------------------------------------------------------------------------
@@ -986,7 +990,7 @@ bool HTMLGenericFormElementImpl::isFocusable() const
     if (id() == ID_BUTTON)
 	return true;
 
-    if (!m_render)
+    if (!m_render || !m_render->isWidget())
 	return false;
 
     QWidget* widget = static_cast<RenderWidget*>(m_render)->widget();
@@ -1056,7 +1060,7 @@ void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
 	    // handle tabbing out, either from a single or repeated key event.
 	    if ( evt->id() == EventImpl::KEYPRESS_EVENT && evt->isKeyRelatedEvent() ) {
 	        QKeyEvent* const k = static_cast<KeyEventBaseImpl *>(evt)->qKeyEvent();
-	        if ( k && (k->key() == Qt::Key_Tab || k->key() == Qt::Key_BackTab) ) {
+	        if ( k && (k->key() == Qt::Key_Tab || k->key() == Qt::Key_Backtab) ) {
 		    QWidget* const widget = static_cast<RenderWidget*>(m_render)->widget();
 		    if (widget)
 			{
@@ -1079,7 +1083,7 @@ void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
 	    // combobox shouldn't emit onblur)
 	}
     }
-    if (evt->target() == this && evt->isMouseEvent() && renderer())
+    if (evt->target() == this && evt->isMouseEvent() && evt->id() != EventImpl::KHTML_MOUSEWHEEL_EVENT && renderer())
         evt->setDefaultHandled();
 
     HTMLElementImpl::defaultEventHandler(evt);
@@ -1652,17 +1656,14 @@ bool HTMLInputElementImpl::encoding(const QTextCodec* codec, khtml::encodingList
             // can't submit file in www-url-form encoded
             QWidget* const toplevel = static_cast<RenderSubmitButton*>(m_render)->widget()->topLevelWidget();
             if (multipart) {
-                QByteArray filearray( "" );
+                QByteArray filearray;
                 if ( KIO::NetAccess::stat(fileurl, filestat, toplevel)) {
                     const KFileItem fileitem(filestat, fileurl, true, false);
                     if ( fileitem.isFile() &&
                          KIO::NetAccess::download(fileurl, local, toplevel) ) {
                         QFile file(local);
-                        filearray.resize(file.size()+1);
                         if ( file.open( QIODevice::ReadOnly ) ) {
-                            const int readbytes = file.read( filearray.data(), file.size());
-                            if ( readbytes >= 0 )
-                                filearray[readbytes] = '\0';
+                            filearray = file.read( file.size() );
                             file.close();
                         }
                         KIO::NetAccess::removeTempFile( local );
@@ -2547,7 +2548,9 @@ bool HTMLKeygenElementImpl::encoding(const QTextCodec* codec, khtml::encodingLis
     encoded_values += enc_name;
 
     // pop up the fancy certificate creation dialog here
-    KSSLKeyGen* const kg = new KSSLKeyGen(static_cast<RenderWidget *>(m_render)->widget(), "Key Generator", true);
+    KSSLKeyGen* const kg = new KSSLKeyGen(static_cast<RenderWidget *>(m_render)->widget());
+    kg->setWindowTitle(i18n("Key Generator"));
+    kg->setModal(true);
 
     kg->setKeySize(0);
     successful = (QDialog::Accepted == kg->exec());
@@ -2910,15 +2913,15 @@ DOMString HTMLTextAreaElementImpl::defaultValue()
 void HTMLTextAreaElementImpl::setDefaultValue(DOMString _defaultValue)
 {
     // there may be comments - remove all the text nodes and replace them with one
-    Q3PtrList<NodeImpl> toRemove;
+    QList<NodeImpl*> toRemove;
     NodeImpl *n;
     for (n = firstChild(); n; n = n->nextSibling())
         if (n->isTextNode())
             toRemove.append(n);
-    Q3PtrListIterator<NodeImpl> it(toRemove);
+    QListIterator<NodeImpl*> it(toRemove);
     int exceptioncode = 0;
-    for (; it.current(); ++it) {
-        removeChild(it.current(), exceptioncode);
+    while (it.hasNext()) {
+        removeChild(it.next(), exceptioncode);
     }
     insertBefore(getDocument()->createTextNode(_defaultValue.implementation()),firstChild(), exceptioncode);
     setValue(_defaultValue);
