@@ -116,7 +116,7 @@ public:
     {
         QRect frameGeom = frameInfo[frame].geom;
 
-        // Take the passed paint rectangle in gif screen coordinates, and 
+        // Take the passed paint rectangle in gif screen coordinates, and
         // clip it to the frame's geometry
         QRect screenPaintRect = QRect(sx, sy, width, height) & frameGeom;
 
@@ -125,13 +125,13 @@ public:
 
         curFrame->paint(dx + screenPaintRect.x() - sx, dy + screenPaintRect.y() - sy, p,
                 framePaintRect.x(), framePaintRect.y(),
-                framePaintRect.width(), framePaintRect.height(), false /* don't get back to us!*/);
+                framePaintRect.width(), framePaintRect.height());
     }
 
     // Renders current gif screen state on the painter
     void renderCurScreen(int dx, int dy, QPainter* p, int sx, int sy, int width, int height)
     {
-        // Depending on the previous frame's mode, we make have to cut out a hole when 
+        // Depending on the previous frame's mode, we may have to cut out a hole when
         // painting the canvas, since if previous frame had BG disposal, we have to do OpSrc.
         if (previousWasBG)
         {
@@ -206,7 +206,7 @@ public:
 
         // Move over to next frame if need be, incorporating 
         // the change effect of current one onto the screen.
-        if (shouldSwitchFrame) 
+        if (shouldSwitchFrame)
         {
             updateScreenAfterDispose();
 
@@ -336,10 +336,28 @@ public:
             DGifCloseFile(file);
             return Error;
         }
+        
 
         //We use canvas size only for animations
-        if (file->ImageCount > 1)
+        if (file->ImageCount > 1) {
+            // Verify it..
+            if (!ImageManager::isAcceptableSize(file->SWidth, file->SHeight)) {
+                DGifCloseFile(file);
+                return Error;
+            }
             notifyImageInfo(file->SWidth, file->SHeight);
+        }
+        
+        // Check each frame to be within the size limit policy
+        for (int frame = 0; frame < file->ImageCount; ++frame)
+        {
+            //Extract colormap, geometry, so that we can create the frame
+            SavedImage* curFrame = &file->SavedImages[frame];
+            if (!ImageManager::isAcceptableSize(curFrame->ImageDesc.Width, curFrame->ImageDesc.Height)) {
+                DGifCloseFile(file);
+                return Error;
+            }
+        }
         
         QVector<GIFFrameInfo> frameProps;
         
@@ -405,7 +423,7 @@ public:
             // If we have transparency, we need to go an RGBA mode.
             ImageFormat format;
             if (trans != -1)
-                format.type = ImageFormat::Image_RGBA_32;
+                format.type = ImageFormat::Image_ARGB_32; // Premultiply always OK, since we only have 0/1 alpha
             else
                 format.type = ImageFormat::Image_Palette_8;
 
@@ -519,6 +537,8 @@ public:
             PixmapPlane* frame0  = requestFrame0();
             frame0->animProvider = new GIFAnimProvider(frame0, image, frameProps, bgColor);
         }
+        
+        DGifCloseFile(file);
 
         return Done;
     }
