@@ -34,6 +34,7 @@
 class QPainter;
 class QRect;
 template< typename T > class QVector;
+template <class T> class QStack;
 
 namespace DOM {
     class HTMLDocumentImpl;
@@ -59,6 +60,8 @@ namespace khtml {
         class RenderLineEdit;
     class RenderPartObject;
     class RenderWidget;
+    class RenderLayer;
+    class RenderBox;
     class CSSStyleSelector;
     class LineEditWidget;
     class CaretBox;
@@ -103,6 +106,8 @@ class KHTML_EXPORT KHTMLView : public QScrollArea, public khtml::KHTMLWidget
     friend class khtml::RenderLineEdit;
     friend class khtml::RenderPartObject;
     friend class khtml::RenderWidget;
+    friend class khtml::RenderLayer;
+    friend class khtml::RenderBox;
     friend class khtml::CSSStyleSelector;
     friend class khtml::LineEditWidget;
     friend class KJS::WindowFunc;
@@ -150,12 +155,20 @@ public:
     int marginHeight() { return _marginHeight; }
 
     /**
-     * Sets verticals scrollbar mode. Reimplemented for internal reasons.
+     * Sets vertical scrollbar mode. 
+     *
+     * WARNING: do not call this method on a base class pointer unless you
+     *          specifically want QAbstractScrollArea's variant (not recommended).
+     *          QAbstractScrollArea::setVerticalScrollBarPolicy is *not* virtual.
      */
     virtual void setVerticalScrollBarPolicy( Qt::ScrollBarPolicy policy );
 
     /**
-     * Sets horizontal scrollbar mode. Reimplemented for internal reasons.
+     * Sets horizontal scrollbar mode.
+     *
+     * WARNING: do not call this method on a base class pointer unless you
+     *          specifically want QAbstractScrollArea's variant (not recommended).
+     *          QAbstractScrollArea::setHorizontalScrollBarPolicy is *not* virtual.
      */
     virtual void setHorizontalScrollBarPolicy( Qt::ScrollBarPolicy policy );
 
@@ -163,7 +176,7 @@ public:
      * Prints the HTML document.
      * @param quick if true, fully automated printing, without print dialog
      */
-    void print( bool quick = false ); 
+    void print( bool quick = false );
 
     /**
      * Display all accesskeys in small tooltips
@@ -201,7 +214,7 @@ public:
      * Returns the height of the viewport
      */
     int visibleHeight() const;
-    
+
     /**
      *  Place the contents area point x/y
      *  at the top left of the viewport
@@ -226,7 +239,7 @@ public:
      * Returns a point translated to contents area coordinates
      * @param x x coordinate of viewport point to translate
      * @param y y coordinate of viewport point to translate
-     * @param cx resulting x coordinate 
+     * @param cx resulting x coordinate
      * @param cy resulting y coordinate
      *
      */
@@ -236,7 +249,7 @@ public:
      * Returns a point translated to viewport coordinates
      * @param x x coordinate of contents area point to translate
      * @param y y coordinate of contents area point to translate
-     * @param cx resulting x coordinate 
+     * @param cx resulting x coordinate
      * @param cy resulting y coordinate
      *
      */
@@ -248,14 +261,14 @@ public:
      * @param y y offset
      */
     void scrollBy(int x, int y);
-    
+
     /**
      * Requests an update of the content area
      * @param r the content area rectangle to update
      */
     void updateContents( const QRect& r );
     void updateContents(int x, int y, int w, int h);
-    
+
     void addChild(QWidget *child, int dx, int dy);
 
     /**
@@ -264,13 +277,13 @@ public:
      */
     void repaintContents( const QRect& r );
     void repaintContents(int x, int y, int w, int h);
-    
+
     /**
      * Apply a zoom level to the content area
      * @param percent a zoom level expressed as a percentage
      */
     void setZoomLevel( int percent );
-    
+
     /**
      * Retrieve the current zoom level
      *
@@ -365,6 +378,11 @@ private:
     void setMouseEventsTarget( QWidget* w );
     QWidget* mouseEventsTarget() const;
 
+    QStack<QRegion>* clipHolder() const;
+    void setClipHolder( QStack<QRegion>* ch );
+    
+    void setPart(KHTMLPart *part);
+
     /**
      * Paints the HTML document to a QPainter.
      * The document will be scaled to match the width of
@@ -375,13 +393,7 @@ private:
      **/
     void paint(QPainter *p, const QRect &rc, int yOff = 0, bool *more = 0);
 
-#if 0
-    /**
-     * Paints the HTML document to a QPainter.
-     * The document will be scaled to match the width.
-     **/
-    void paint(QPainter *p, int width);
-#endif
+    void render(QPainter *p, const QRect& r, const QPoint& off);
 
     /**
      * Get/set the CSS Media Type.
@@ -405,7 +417,7 @@ private:
     bool focusNodeWithAccessKey(QChar c, KHTMLView* caller = NULL);
     QMap< DOM::ElementImpl*, QChar > buildFallbackAccessKeys() const;
     void displayAccessKeys( KHTMLView* caller, KHTMLView* origview, QVector< QChar >& taken, bool use_fallbacks );
-
+    bool isScrollingFromMouseWheel() const;
     void setHasStaticBackground();
     void applyTransforms( int& x, int& y, int& w, int& h) const;
     void revertTransforms( int& x, int& y, int& w, int& h) const;
