@@ -69,8 +69,9 @@
 #include <kiconloader.h>
 #include <scheduler.h>
 #include <kdebug.h>
-#include "khtml_factory.h"
-#include "khtml_part.h"
+
+#include <khtml_global.h>
+#include <khtml_part.h>
 
 #ifdef IMAGE_TITLES
 #include <qfile.h>
@@ -139,7 +140,8 @@ void CachedObject::ref(CachedObjectClient *c)
     // unfortunately we can be ref'ed multiple times from the
     // same object,  because it uses e.g. the same foreground
     // and the same background picture. so deal with it.
-    m_clients.insert(c,c);
+    // Hence the use of a QHash rather than of a QSet.
+    m_clients.insertMulti(c,c);
     Cache::removeFromLRUList(this);
     m_accessCount++;
 }
@@ -149,11 +151,11 @@ void CachedObject::deref(CachedObjectClient *c)
     assert( c );
     assert( m_clients.count() );
     assert( !canDelete() );
-    assert( m_clients.find( c ) );
+    assert( m_clients.contains( c ) );
 
     Cache::flush();
 
-    m_clients.remove(c);
+    m_clients.take(c);
 
     if (allowInLRUList())
         Cache::insertInLRUList(this);
@@ -282,10 +284,8 @@ void CachedCSSStyleSheet::checkNotify()
 
     CDEBUG << "CachedCSSStyleSheet:: finishedLoading " << m_url.string() << endl;
 
-    // it() first increments, then returnes the current item.
-    // this avoids skipping an item when setStyleSheet deletes the "current" one.
-    for (Q3PtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
-        it()->setStyleSheet( m_url, m_sheet, m_charset );
+    for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
+        it.next().value()->setStyleSheet( m_url, m_sheet, m_charset );
 }
 
 
@@ -296,10 +296,8 @@ void CachedCSSStyleSheet::error( int err, const char* text )
     m_errText = text;
     m_loading = false;
 
-    // it() first increments, then returnes the current item.
-    // this avoids skipping an item when setStyleSheet deletes the "current" one.
-    for (Q3PtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
-        it()->error( m_err, m_errText );
+    for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
+        it.next().value()->error( m_err, m_errText );
 }
 
 #if 0
@@ -370,8 +368,8 @@ void CachedScript::checkNotify()
 {
     if(m_loading) return;
 
-    for (Q3PtrDictIterator<CachedObjectClient> it( m_clients); it.current();)
-        it()->notifyFinished(this);
+    for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
+        it.next().value()->notifyFinished(this);
 }
 
 void CachedScript::error( int /*err*/, const char* /*text*/ )
@@ -399,7 +397,7 @@ CachedImage::CachedImage(DocLoader* dl, const DOMString &url, KIO::CacheControl
     //pixPart = 0;
     bg = 0;
     scaled = 0;
-    bgColor = qRgba( 0, 0, 0, 0xFF );
+    bgColor = qRgba( 0, 0, 0, 0 );
     typeChecked = false;
     isFullyTransparent = false;
     monochrome = false;
@@ -409,7 +407,7 @@ CachedImage::CachedImage(DocLoader* dl, const DOMString &url, KIO::CacheControl
     setAccept( acceptHeader );
     m_showAnimations = dl->showAnimations();
 
-    if ( KHTMLFactory::defaultHTMLSettings()->isAdFiltered( url.string() ) ) {
+    if ( KHTMLGlobal::defaultHTMLSettings()->isAdFiltered( url.string() ) ) {
         m_wasBlocked = true;
         CachedObject::finish();
     }
@@ -466,7 +464,7 @@ QPixmap CachedImage::tiled_pixmap(const QColor& newc, int xWidth, int xHeight)
 #warning "Needs some additional performance work"
 #endif
 
-    static QRgb bgTransparent = qRgba( 0, 0, 0, 0xFF );
+    static QRgb bgTransparent = qRgba( 0, 0, 0, 0 );
 
     QSize s(pixmap_size());
     int w = xWidth;
@@ -520,7 +518,7 @@ QPixmap CachedImage::tiled_pixmap(const QColor& newc, int xWidth, int xHeight)
         if ( r.height() < BGMINHEIGHT )
             h = ((BGMINHEIGHT-1) / xHeight + 1) * xHeight;
     }
-    
+
     if ( w != xWidth  || h != xHeight )
     {
         // kDebug() << "pre-tiling " << s.width() << "," << s.height() << " to " << w << "," << h;
@@ -532,7 +530,7 @@ QPixmap CachedImage::tiled_pixmap(const QColor& newc, int xWidth, int xHeight)
             else
                 bg->fill( Qt::transparent );
         }
-        
+
         QPainter p(bg);
         p.drawTiledPixmap(0, 0, w, h, *src);
         p.end();
@@ -543,7 +541,7 @@ QPixmap CachedImage::tiled_pixmap(const QColor& newc, int xWidth, int xHeight)
         // we were asked for the entire pixmap. Cache it.
         // ### goes against imload stuff, but it's far too expensive
         //     to recreate the full pixmap each time it's requested as
-        //     we don't know what portion of it will be used eventually 
+        //     we don't know what portion of it will be used eventually
         //     (by e.g. paintBackgroundExtended). It could be a few pixels of
         //     a huge image. See #140248/#1 for an obvious example.
         //     Imload probably needs to handle all painting in paintBackgroundExtended.
@@ -572,8 +570,8 @@ QPixmap* CachedImage::scaled_pixmap( int xWidth, int xHeight )
         delete scaled;
     }
 
-    //### this is quite awful performance-wise. It should avoid 
-    // alpha if not needed, and go to pixmap, etc. 
+    //### this is quite awful performance-wise. It should avoid
+    // alpha if not needed, and go to pixmap, etc.
     QImage im(xWidth, xHeight, QImage::Format_ARGB32_Premultiplied);
 
     QPainter paint(&im);
@@ -597,14 +595,24 @@ QPixmap CachedImage::pixmap( ) const
 
     int w = i->size().width();
     int h = i->size().height();
-    QImage im(w, h, QImage::Format_ARGB32_Premultiplied);
 
-    QPainter paint(&im);
-    paint.setCompositionMode(QPainter::CompositionMode_Source);
-    ImagePainter pi(i);
-    pi.paint(0, 0, &paint);
-    paint.end();
-    return QPixmap::fromImage( im );
+    if (i->hasAlpha()) {
+        QImage im(w, h, QImage::Format_ARGB32_Premultiplied);
+
+        QPainter paint(&im);
+        paint.setCompositionMode(QPainter::CompositionMode_Source);
+        ImagePainter pi(i);
+        pi.paint(0, 0, &paint);
+        paint.end();
+        return QPixmap::fromImage( im );
+    } else {
+        QPixmap pm(w, h);
+        QPainter paint(&pm);
+        ImagePainter pi(i);
+        pi.paint(0, 0, &paint);
+        paint.end();
+        return pm;
+    }
 }
 
 
@@ -641,9 +649,9 @@ void CachedImage::imageChange     (khtmlImLoad::Image* /*img*/, QRect region)
 
 void CachedImage::doNotifyFinished()
 {
-    for (Q3PtrDictIterator<CachedObjectClient> it( m_clients); it.current();)
+    for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
     {
-        it()->notifyFinished(this);
+        it.next().value()->notifyFinished(this);
     }
 }
 
@@ -671,12 +679,12 @@ void CachedImage::imageDone(khtmlImLoad::Image* /*img*/)
 
 void CachedImage::do_notify(const QRect& r)
 {
-    for (Q3PtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
+    for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
     {
 #ifdef LOADER_DEBUG
-        kDebug(6060) << " image "<<this<<" notify of geom client " << it.current() << "\n";
+        kDebug(6060) << " image "<<this<<" notify of geom client " << it.peekNext() << "\n";
 #endif
-        it()->updatePixmap( r, this);
+        it.next().value()->updatePixmap( r, this);
     }
 }
 
@@ -756,8 +764,8 @@ void CachedImage::movieStatus(int status)
                 monochrome = false;
             }
         }
-        for (Q3PtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
-            it()->notifyFinished( this );
+        for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
+            it.next().value()->notifyFinished( this );
 	m_status = Cached; //all done
     }
 
@@ -913,8 +921,8 @@ void CachedImage::data ( QBuffer &_buffer, bool eof )
             else
                 do_notify(*p, p->rect());
 
-            for (Q3PtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
-                it()->notifyFinished( this );
+            for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
+                it.next().value()->notifyFinished( this );
             m_status = Cached; //all done
         }
 #endif
@@ -941,8 +949,8 @@ void CachedImage::error( int /*err*/, const char* /*text*/ )
     m_hadError = true;
     m_loading = false;
     do_notify(QRect(0, 0, 16, 16));
-    for (Q3PtrDictIterator<CachedObjectClient> it( m_clients ); it.current();)
-        it()->notifyFinished(this);
+    for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
+        it.next().value()->notifyFinished(this);
 }
 
 // -------------------------------------------------------------------------------------------
@@ -977,8 +985,8 @@ void CachedSound::checkNotify()
 {
     if(m_loading) return;
 
-    for (Q3PtrDictIterator<CachedObjectClient> it( m_clients); it.current();)
-        it()->notifyFinished(this);
+    for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
+        it.next().value()->notifyFinished(this);
 }
 
 void CachedSound::error( int /*err*/, const char* /*text*/ )
@@ -1044,11 +1052,7 @@ void DocLoader::setExpireDate(time_t _expireDate, bool relative)
 
 void DocLoader::insertCachedObject( CachedObject* o ) const
 {
-    if ( m_docObjects.find(o) )
-        return;
-    m_docObjects.insert( o, o );
-    if ( m_docObjects.count() > 3 * m_docObjects.size() )
-        m_docObjects.resize(khtml::nextSeed( m_docObjects.size() ) );
+    m_docObjects.insert( o );
 }
 
 bool DocLoader::needReload(CachedObject *existing, const QString& fullURL)
@@ -1115,8 +1119,8 @@ CachedCSSStyleSheet *DocLoader::requestStyleSheet( const DOM::DOMString &url, co
 CachedScript *DocLoader::requestScript( const DOM::DOMString &url, const QString& charset)
 {
     DOCLOADER_SECCHECK(true);
-    if ( ! KHTMLFactory::defaultHTMLSettings()->isJavaScriptEnabled(fullURL.host()) ||
-           KHTMLFactory::defaultHTMLSettings()->isAdFiltered(fullURL.url()))
+    if ( ! KHTMLGlobal::defaultHTMLSettings()->isJavaScriptEnabled(fullURL.host()) ||
+           KHTMLGlobal::defaultHTMLSettings()->isAdFiltered(fullURL.url()))
 	return 0L;
 
     CachedScript* s = Cache::requestObject<CachedScript, CachedObject::Script>( this, fullURL, 0 );
@@ -1143,10 +1147,12 @@ void DocLoader::setAutoloadImages( bool enable )
 
     if ( !m_bautoloadImages ) return;
 
-    for ( Q3PtrDictIterator<CachedObject> it( m_docObjects ); it.current(); ++it )
-        if ( it.current()->type() == CachedObject::Image )
+    for ( QSetIterator<CachedObject*> it( m_docObjects ); it.hasNext(); )
+    {
+        CachedObject* cur = it.next();
+        if ( cur->type() == CachedObject::Image )
         {
-            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( it.current()) );
+            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>(cur) );
 
             CachedObject::Status status = img->status();
             if ( status != CachedObject::Unknown )
@@ -1154,6 +1160,7 @@ void DocLoader::setAutoloadImages( bool enable )
 
             Cache::loader()->load(this, img, true);
         }
+    }
 }
 
 void DocLoader::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimations )
@@ -1161,48 +1168,57 @@ void DocLoader::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimation
     if ( showAnimations == m_showAnimations ) return;
     m_showAnimations = showAnimations;
 
-    for ( Q3PtrDictIterator<CachedObject> it( m_docObjects ); it.current(); ++it )
-        if ( it.current()->type() == CachedObject::Image )
+    for ( QSetIterator<CachedObject*> it( m_docObjects ); it.hasNext(); )
+    {
+        CachedObject* cur = it.next();
+        if ( cur->type() == CachedObject::Image )
         {
-            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( it.current() ) );
+            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( cur ) );
 
             img->setShowAnimations( m_showAnimations );
         }
+    }
 }
 
 void DocLoader::pauseAnimations()
 {
-    for ( Q3PtrDictIterator<CachedObject> it( m_docObjects ); it.current(); ++it )
-        if ( it.current()->type() == CachedObject::Image )
+    for ( QSetIterator<CachedObject*> it( m_docObjects ); it.hasNext(); )
+    {
+        CachedObject* cur = it.next();
+        if ( cur->type() == CachedObject::Image )
         {
-            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( it.current() ) );
+            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( cur ) );
 
             img->pauseAnimations();
         }
+    }
 }
 
 void DocLoader::resumeAnimations()
 {
-    for ( Q3PtrDictIterator<CachedObject> it( m_docObjects ); it.current(); ++it )
-        if ( it.current()->type() == CachedObject::Image )
+    for ( QSetIterator<CachedObject*> it( m_docObjects ); it.hasNext(); )
+    {
+        CachedObject* cur = it.next();
+        if ( cur->type() == CachedObject::Image )
         {
-            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( it.current() ) );
+            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( cur ) );
 
             img->resumeAnimations();
         }
+    }
 }
 
 // ------------------------------------------------------------------------------------------
 
 Loader::Loader() : QObject()
 {
-    m_requestsLoading.setAutoDelete( true );
     connect(&m_timer, SIGNAL(timeout()), this, SLOT( servePendingRequests() ) );
 }
 
 Loader::~Loader()
 {
     qDeleteAll(m_requestsPending);
+    qDeleteAll(m_requestsLoading);
 }
 
 void Loader::load(DocLoader* dl, CachedObject *object, bool incremental)
@@ -1253,14 +1269,14 @@ void Loader::servePendingRequests()
         if ( req->object->schedule() )
             KIO::Scheduler::scheduleJob( job );
 
-        m_requestsLoading.insert(job, req);
+        m_requestsLoading.insertMulti(job, req);
     }
 }
 
 void Loader::slotFinished( KJob* job )
 {
-  Request *r = m_requestsLoading.take( job );
   KIO::TransferJob* j = static_cast<KIO::TransferJob*>(job);
+  Request *r = m_requestsLoading.take( j );
 
   if ( !r )
     return;
@@ -1349,9 +1365,8 @@ int Loader::numRequests( DocLoader* dl ) const
         if ( req->m_docLoader == dl )
             res++;
 
-    Q3PtrDictIterator<Request> lIt( m_requestsLoading );
-    for (; lIt.current(); ++lIt )
-        if ( lIt.current()->m_docLoader == dl )
+    foreach( Request* req, m_requestsLoading)
+        if ( req->m_docLoader == dl )
             res++;
 
     return res;
@@ -1373,33 +1388,30 @@ void Loader::cancelRequests( DocLoader* dl )
 
     //kDebug( 6060 ) << "got " << m_requestsLoading.count() << "loading requests";
 
-    Q3PtrDictIterator<Request> lIt( m_requestsLoading );
-    while ( lIt.current() )
+    QMutableHashIterator<KIO::Job*,Request*> lIt( m_requestsLoading );
+    while ( lIt.hasNext() )
     {
-        if ( lIt.current()->m_docLoader == dl )
+        lIt.next();
+        if ( lIt.value()->m_docLoader == dl )
         {
             //kDebug( 6060 ) << "canceling loading request for " << lIt.current()->object->url().string();
-            KIO::Job *job = static_cast<KIO::Job *>( lIt.currentKey() );
-            Cache::removeCacheEntry( lIt.current()->object );
-            m_requestsLoading.remove( lIt.currentKey() );
+            KIO::Job *job = static_cast<KIO::Job *>( lIt.key() );
+            Cache::removeCacheEntry( lIt.value()->object );
+            delete lIt.value();
+            lIt.remove();
             job->kill();
-            //emit requestFailed( dl, pIt.current()->object );
         }
-        else
-            ++lIt;
     }
 }
 
 KIO::Job *Loader::jobForRequest( const DOM::DOMString &url ) const
 {
-    Q3PtrDictIterator<Request> it( m_requestsLoading );
-
-    for (; it.current(); ++it )
+    QHashIterator<KIO::Job*,Request*> it( m_requestsLoading );
+    while (it.hasNext())
     {
-        CachedObject *obj = it.current()->object;
-
-        if ( obj && obj->url() == url )
-            return static_cast<KIO::Job *>( it.currentKey() );
+        it.next();
+        if ( it.value()->object && it.value()->object->url() == url )
+            return static_cast<KIO::Job *>( it.key() );
     }
 
     return 0;
@@ -1408,7 +1420,7 @@ KIO::Job *Loader::jobForRequest( const DOM::DOMString &url ) const
 // ----------------------------------------------------------------------------
 
 
-Q3Dict<CachedObject> *Cache::cache;
+QHash<QString,CachedObject*> *Cache::cache;
 QLinkedList<DocLoader*>    *Cache::docloader;
 QLinkedList<CachedObject*> *Cache::freeList;
 Loader *Cache::m_loader;
@@ -1423,7 +1435,7 @@ QPixmap *Cache::blockedPixmap;
 void Cache::init()
 {
     if ( !cache )
-        cache = new Q3Dict<CachedObject>(401, true);
+        cache = new QHash<QString,CachedObject*>();
 
     if ( !docloader )
         docloader = new QLinkedList<DocLoader*>;
@@ -1432,7 +1444,7 @@ void Cache::init()
         nullPixmap = new QPixmap;
 
     if ( !brokenPixmap )
-        brokenPixmap = new QPixmap(KHTMLFactory::iconLoader()->loadIcon("file-broken", KIconLoader::Desktop, 16, KIconLoader::DisabledState));
+        brokenPixmap = new QPixmap(KHTMLGlobal::iconLoader()->loadIcon("image-missing", KIconLoader::Desktop, 16, KIconLoader::DisabledState));
 
     if ( !blockedPixmap ) {
         blockedPixmap = new QPixmap();
@@ -1453,17 +1465,16 @@ void Cache::clear()
     kDebug( 6060 ) << "Cache: CLEAR!";
     statistics();
 #endif
-    cache->setAutoDelete( true );
 
 #ifndef NDEBUG
     bool crash = false;
-    for (Q3DictIterator<CachedObject> it(*cache); it.current(); ++it) {
-        if (!it.current()->canDelete()) {
+    foreach (CachedObject* co, *cache) {
+        if (!co->canDelete()) {
             kDebug( 6060 ) << " Object in cache still linked to";
-            kDebug( 6060 ) << " -> URL: " << it.current()->url();
-            kDebug( 6060 ) << " -> #clients: " << it.current()->count();
+            kDebug( 6060 ) << " -> URL: " << co->url();
+            kDebug( 6060 ) << " -> #clients: " << co->count();
             crash = true;
-//         assert(it.current()->canDelete());
+//         assert(co->canDelete());
         }
     }
     foreach (CachedObject* co, *freeList) {
@@ -1473,9 +1484,9 @@ void Cache::clear()
             kDebug( 6060 ) << " -> #clients: " << co->count();
             crash = true;
             /*
-            QPtrDictIterator<CachedObjectClient> it(freeList->current()->m_clients);
-            for(;it.current(); ++it) {
-                if (dynamic_cast<RenderObject*>(it.current())) {
+            foreach (CachedObjectClient* cur, (*co->m_clients)))
+            {
+                if (dynamic_cast<RenderObject*>(cur)) {
                     kDebug( 6060 ) << " --> RenderObject";
                 } else
                     kDebug( 6060 ) << " --> Something else";
@@ -1485,7 +1496,7 @@ void Cache::clear()
     }
     assert(!crash);
 #endif
-
+    qDeleteAll(*cache);
     delete cache; cache = 0;
     delete nullPixmap; nullPixmap = 0;
     delete brokenPixmap; brokenPixmap = 0;
@@ -1502,7 +1513,7 @@ CachedObjectType* Cache::requestObject( DocLoader* dl, const KUrl& kurl, const c
     KIO::CacheControl cachePolicy = dl->cachePolicy();
 
     QString url = kurl.url();
-    CachedObject* o = cache->find(url);
+    CachedObject* o = cache->value(url);
 
     if ( o && o->type() != CachedType ) {
         removeCacheEntry( o );
@@ -1511,7 +1522,7 @@ CachedObjectType* Cache::requestObject( DocLoader* dl, const KUrl& kurl, const c
 
     if ( o && dl->needReload( o, url ) ) {
         o = 0;
-        assert( cache->find( url ) == 0 );
+        assert( !cache->contains( url ) );
     }
 
     if(!o)
@@ -1539,9 +1550,8 @@ CachedObjectType* Cache::requestObject( DocLoader* dl, const KUrl& kurl, const c
 
 void Cache::preloadStyleSheet( const QString &url, const QString &stylesheet_data)
 {
-    CachedObject *o = cache->find(url);
-    if(o)
-        removeCacheEntry(o);
+    if (cache->contains(url))
+        removeCacheEntry(cache->value(url));
 
     CachedCSSStyleSheet *stylesheet = new CachedCSSStyleSheet(url, stylesheet_data);
     cache->insert( url, stylesheet );
@@ -1549,9 +1559,8 @@ void Cache::preloadStyleSheet( const QString &url, const QString &stylesheet_dat
 
 void Cache::preloadScript( const QString &url, const QString &script_data)
 {
-    CachedObject *o = cache->find(url);
-    if(o)
-        removeCacheEntry(o);
+    if (cache->contains(url))
+        removeCacheEntry(cache->value(url));
 
     CachedScript *script = new CachedScript(url, script_data);
     cache->insert( url, script );
@@ -1589,7 +1598,6 @@ void Cache::setSize( int bytes )
 
 void Cache::statistics()
 {
-    CachedObject *o;
     // this function is for debugging purposes only
     init();
 
@@ -1600,10 +1608,8 @@ void Cache::statistics()
     int scripts = 0;
     int stylesheets = 0;
     int sound = 0;
-    Q3DictIterator<CachedObject> it(*cache);
-    for(it.toFirst(); it.current(); ++it)
+    foreach (CachedObject* o, *cache)
     {
-        o = it.current();
         switch(o->type()) {
         case CachedObject::Image:
         {
