@@ -115,6 +115,7 @@ public:
     BidiIterator findNextLineBreak(BidiIterator &start, BidiState &info );
     RootInlineBox* constructLine(const BidiIterator& start, const BidiIterator& end);
     InlineFlowBox* createLineBoxes(RenderObject* obj);
+    bool inlineChildNeedsLineBox(RenderObject* obj);
     void computeHorizontalPositionsForLine(InlineFlowBox* lineBox, BidiState &bidi);
     void computeVerticalPositionsForLine(RootInlineBox* lineBox);
     bool clearLineOfPageBreaks(InlineFlowBox* lineBox);
@@ -222,7 +223,7 @@ protected:
         int endY;
         short left;
         short width;
-        Type type : 1; // left or right aligned
+        KDE_BF_ENUM(Type) type : 1; // left or right aligned
         bool noPaint : 1;
         bool crossedLayer : 1; // lock noPaint flag
     };
@@ -366,7 +367,7 @@ private:
 
     bool m_childrenInline : 1;
     bool m_firstLine      : 1; // used in inline layouting
-    EClear m_clearStatus  : 2; // used during layuting of paragraphs
+    KDE_BF_ENUM(EClear) m_clearStatus  : 2; // used during layuting of paragraphs
     bool m_avoidPageBreak : 1; // anonymous avoid page-break block
     bool m_topMarginQuirk : 1;
     bool m_bottomMarginQuirk : 1;
