@@ -215,13 +215,6 @@ void RenderBox::detach()
     RenderArena* arena = renderArena();
 
     detachRemainingChildren();
-    
-    InlineBox* ph = placeHolderBox();
-    if (ph) {
-        ph->detach(arena);
-        setPlaceHolderBox( 0 );
-    }
-
     RenderContainer::detach();
 
     if (layer)
@@ -259,7 +252,7 @@ InlineBox* RenderBox::createInlineBox(bool /*makePlaceHolderBox*/, bool /*isRoot
 {
     if (m_placeHolderBox)
         m_placeHolderBox->detach(renderArena(), true/*noRemove*/);
-    return (m_placeHolderBox = new (renderArena()) InlineBox(this));
+    return (m_placeHolderBox = new (renderArena()) PlaceHolderBox(this));
 }
 
 void RenderBox::deleteInlineBoxes(RenderArena* /*arena*/)
@@ -536,6 +529,18 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, const Back
     bool shouldPaintBackgroundImage = bg && bg->isComplete() && !bg->isTransparent() && !bg->isErrorImage();
     QColor bgColor = c;
 
+    // the "bottom" of the page can't be transparent
+    // unless this is a subframe
+    if (!bgLayer->next() && isRoot())
+    {
+        KHTMLView* v = canvas()->view();
+        if (!v || !v->m_kwp->isRedirected()) {
+            if (bgColor.alpha() == 0)
+                bgColor = p->background().color();
+            bgColor.setAlpha(255);
+        }
+    }
+
     // Paint the color first underneath all images.
     if (!bgLayer->next() && bgColor.isValid() && qAlpha(bgColor.rgba()) > 0)
         p->fillRect(clipr.x(), clipr.y(), clipr.width(), clipr.height(), bgColor);
@@ -700,7 +705,7 @@ void RenderBox::outlineBox(QPainter *p, int _tx, int _ty, const char *color)
     p->drawRect(_tx, _ty, m_width, m_height);
 }
 
-QRect RenderBox::getOverflowClipRect(int tx, int ty)
+QRect RenderBox::overflowClipRect(int tx, int ty)
 {
     // XXX When overflow-clip (CSS3) is implemented, we'll obtain the property
     // here.
@@ -719,7 +724,7 @@ QRect RenderBox::getOverflowClipRect(int tx, int ty)
     return QRect(clipx,clipy,clipw,cliph);
 }
 
-QRect RenderBox::getClipRect(int tx, int ty)
+QRect RenderBox::clipRect(int tx, int ty)
 {
     int bl=borderLeft(),bt=borderTop(),bb=borderBottom(),br=borderRight();
     // ### what about paddings?
@@ -896,9 +901,12 @@ void RenderBox::repaintRectangle(int x, int y, int w, int h, Priority p, bool f)
 
 void RenderBox::relativePositionOffset(int &tx, int &ty) const
 {
-    if(!style()->left().isVariable())
-        tx += style()->left().width(containingBlockWidth());
-    else if(!style()->right().isVariable())
+    if (!style()->left().isVariable()) {
+        if (!style()->right().isVariable() && containingBlock()->style()->direction() == RTL)
+            tx -= style()->right().width(containingBlockWidth());
+        else
+            tx +=  style()->left().width(containingBlockWidth());
+    } else if (!style()->right().isVariable())
         tx -= style()->right().width(containingBlockWidth());
     if(!style()->top().isVariable())
     {
@@ -974,6 +982,13 @@ void RenderBox::calcWidth()
                     m_width = minW;
                     widthType = minWidthType;
                 }
+                if (short iw = intrinsicWidth()) {
+                    // some elements (e.g. Fieldset) have pseudo-replaced behaviour in quirk mode
+                    if (m_width < iw) {
+                        m_width = iw;
+                        widthType = Fixed;
+                    }
+                }
             }
 
             if (widthType == Variable) {
@@ -1526,6 +1541,15 @@ void RenderBox::calcAbsoluteHorizontal()
             m_x = minXPos;
         }
     }
+ 
+    if (short iw = intrinsicWidth()) {
+        // some elements (e.g. Fieldset) have pseudo-replaced behaviour in quirk mode
+        if (m_width < iw - bordersPlusPadding)
+            calcAbsoluteHorizontalValues(Length(iw - bordersPlusPadding, Fixed), containerBlock, containerDirection, 
+                                     containerWidth, bordersPlusPadding,
+                                     left, right, marginLeft, marginRight,
+                                     m_width, m_marginLeft, m_marginRight, m_x);
+    }
 
     // Put m_width into correct form.
     m_width += bordersPlusPadding;
@@ -2224,16 +2248,38 @@ int RenderBox::highestPosition(bool /*includeOverflowInterior*/, bool includeSel
 int RenderBox::lowestPosition(bool /*includeOverflowInterior*/, bool includeSelf) const
 {
     return includeSelf ? m_height : 0;
+    if (!includeSelf || !m_width)
+        return 0;
+    int bottom = m_height;
+    if (isRelPositioned()) {
+        int x;
+        relativePositionOffset(x, bottom);
+    }
+    return bottom;
 }
 
 int RenderBox::rightmostPosition(bool /*includeOverflowInterior*/, bool includeSelf) const
 {
-    return includeSelf ? m_width : 0;
+    if (!includeSelf || !m_height)
+        return 0;
+    int right = m_width;
+    if (isRelPositioned()) {
+        int y;
+        relativePositionOffset(right, y);
+    }
+    return right;
 }
 
 int RenderBox::leftmostPosition(bool /*includeOverflowInterior*/, bool includeSelf) const
 {
-    return includeSelf ? 0 : m_width;
+    if (!includeSelf || !m_height)
+        return m_width;
+    int left = 0;
+    if (isRelPositioned()) {
+        int y;
+        relativePositionOffset(left, y);
+    }
+    return left;
 }
 
 int RenderBox::pageTopAfter(int y) const
@@ -2295,15 +2341,29 @@ bool RenderBox::handleEvent(const DOM::EventImpl& e)
 
         QWheelEvent we(p, -me.detail()*40, buttons, state, orient);
         KHTMLAssert(layer());
+        KHTMLView* v = element()->getDocument()->view();
+        if ( ((orient == Qt::Vertical && (v->contentsHeight() > v->visibleHeight()))  ||
+              (orient == Qt::Horizontal && (v->contentsWidth() > v->visibleWidth()))) &&
+               v->isScrollingFromMouseWheel() ) {
+            // don't propagate wheel events to overflows if heuristics say the view is being scrolled by mouse wheel
+            accepted = false;
+            break;
+        }
+
+        bool d = (we.delta() < 0);
         if (orient == Qt::Vertical) {
-            if (QWidget* w = layer()->verticalScrollbar())
-                QApplication::sendEvent( w, &we);
+            if (QScrollBar* vsb = layer()->verticalScrollbar()) {
+                if (d && vsb->value() != vsb->maximum() || !d && vsb->value() != vsb->minimum())
+                    QApplication::sendEvent( vsb, &we);
+                accepted = true;
+            }
         } else {
-            if (QWidget* w = layer()->horizontalScrollbar())
-                QApplication::sendEvent( w, &we);
+            if (QScrollBar* hsb = layer()->horizontalScrollbar()) {
+                if (d && hsb->value() != hsb->maximum() || !d && hsb->value() != hsb->minimum())
+                    QApplication::sendEvent( hsb, &we);
+                accepted = true;
+            }
         }
-        if (we.isAccepted())
-            accepted = true;
         break;
       }
       case EventImpl::KEYDOWN_EVENT:
