@@ -33,6 +33,7 @@
 #include "rendering/render_canvas.h"
 #include "rendering/render_generated.h"
 #include "rendering/render_inline.h"
+#include "rendering/render_layer.h"
 #include "xml/dom_docimpl.h"
 #include "css/css_valueimpl.h"
 
@@ -48,29 +49,6 @@ RenderContainer::RenderContainer(DOM::NodeImpl* node)
     m_last = 0;
 }
 
-void RenderContainer::detach()
-{
-    // We simulate removeNode calls for all our children
-    // and set parent to 0 to avoid removeNode from being called.
-    // First call removeLayers and removeFromObjectLists since they assume
-    // a valid render-tree
-    for(RenderObject* n = m_first; n; n = n->nextSibling() ) {
-        n->removeLayers(enclosingLayer());
-        n->removeFromObjectLists();
-    }
-
-    RenderObject* next;
-    for(RenderObject* n = m_first; n; n = next ) {
-        n->setParent(0);
-        next = n->nextSibling();
-        n->detach();
-    }
-    m_first = 0;
-    m_last = 0;
-
-    RenderObject::detach();
-}
-
 void RenderContainer::addChild(RenderObject *newChild, RenderObject *beforeChild)
 {
 #ifdef DEBUG_LAYOUT
@@ -162,20 +140,44 @@ void RenderContainer::addChild(RenderObject *newChild, RenderObject *beforeChild
 RenderObject* RenderContainer::removeChildNode(RenderObject* oldChild)
 {
     KHTMLAssert(oldChild->parent() == this);
+    bool inCleanup = !document()->renderer();
 
-    // So that we'll get the appropriate dirty bit set (either that a normal flow child got yanked or
-    // that a positioned child got yanked).  We also repaint, so that the area exposed when the child
-    // disappears gets repainted properly.
-    if ( document()->renderer() ) {
-        oldChild->setNeedsLayoutAndMinMaxRecalc();
+    if ( !inCleanup ) {
+        oldChild->setNeedsLayoutAndMinMaxRecalc(); // Dirty the containing block chain
+        oldChild->setNeedsLayout( false ); // The child itself does not need to layout - it's going away.
+
+        // Repaint, so that the area exposed when the child
+        // disappears gets repainted properly.
         oldChild->repaint();
+    }
+    
+    // detach the place holder box
+    if (oldChild->isBox()) {
+        RenderBox* rb = static_cast<RenderBox*>(oldChild);
+        InlineBox* ph = rb->placeHolderBox();
+        if (ph) {
+            ph->detach(rb->renderArena(), inCleanup /*NoRemove*/);
+            rb->setPlaceHolderBox( 0 );
+        }
+    }
 
-        // Keep our layer hierarchy updated.
-        oldChild->removeLayers(enclosingLayer());
+    if ( !inCleanup ) {
+        // if we remove visible child from an invisible parent, we don't know the layer visibility any more
+        RenderLayer* layer = 0;
+        if (m_style->visibility() != VISIBLE && oldChild->style()->visibility() == VISIBLE && !oldChild->layer()) {
+            layer = enclosingLayer();
+            layer->dirtyVisibleContentStatus();
+        }
+
+         // Keep our layer hierarchy updated.
+        if (oldChild->firstChild() || oldChild->layer()) {
+            if (!layer) layer = enclosingLayer();            
+            oldChild->removeLayers(layer);
+        }
         // remove the child from any special layout lists
         oldChild->removeFromObjectLists();
         
-        if (oldChild->isPositioned() && childrenInline())
+        if (oldChild->isPosWithStaticDim() && childrenInline())
             dirtyLinesFromChangedChild(oldChild);
 
         // if oldChild is the start or end of the selection, then clear
@@ -473,21 +475,29 @@ void RenderContainer::appendChildNode(RenderObject* newChild)
 
     // Keep our layer hierarchy updated.  Optimize for the common case where we don't have any children
     // and don't have a layer attached to ourselves.
+    RenderLayer* layer = 0;
     if (newChild->firstChild() || newChild->layer()) {
-        RenderLayer* layer = enclosingLayer();
+        layer = enclosingLayer();
         newChild->addLayers(layer, newChild);
     }
 
+    // if the new child is visible but this object was not, tell the layer it has some visible content
+    // that needs to be drawn and layer visibility optimization can't be used
+    if (style()->visibility() != VISIBLE && newChild->style()->visibility() == VISIBLE && !newChild->layer()) {
+        if (!layer) layer = enclosingLayer();
+        layer->setHasVisibleContent(true);
+    }
+
     if (!newChild->isFloatingOrPositioned() && childrenInline())
         dirtyLinesFromChangedChild(newChild);
 
     newChild->setNeedsLayoutAndMinMaxRecalc(); // Goes up the containing block hierarchy.
 
-    // We may supply the static position for an absolute positioned child.
-    if (!normalChildNeedsLayout() && !newChild->isText()) {
-        if (newChild->firstChild() || newChild->isPosWithStaticDim())
+    if (!normalChildNeedsLayout()) {
+        // We may supply the static position for an absolute positioned child.
+        if (newChild->firstChild() || newChild->isPosWithStaticDim() || !newChild->isPositioned())
             setChildNeedsLayout(true);
-        else if (newChild->isPositioned()) {
+        else {
             assert(!newChild->inPosObjectList());
             newChild->containingBlock()->insertPositionedObject(newChild);
         }
@@ -516,19 +526,31 @@ void RenderContainer::insertChildNode(RenderObject* child, RenderObject* beforeC
     child->setPreviousSibling(prev);
     child->setParent(this);
 
-    // Keep our layer hierarchy updated.
-    RenderLayer* layer = enclosingLayer();
-    child->addLayers(layer, child);
+    // Keep our layer hierarchy updated.  Optimize for the common case where we don't have any children
+    // and don't have a layer attached to ourselves.
+    RenderLayer* layer = 0;
+    if (child->firstChild() || child->layer()) {
+        layer = enclosingLayer();
+        child->addLayers(layer, child);
+    }
+
+    // if the new child is visible but this object was not, tell the layer it has some visible content
+    // that needs to be drawn and layer visibility optimization can't be used
+    if (style()->visibility() != VISIBLE && child->style()->visibility() == VISIBLE && !child->layer()) {
+        if (!layer) layer = enclosingLayer();
+        layer->setHasVisibleContent(true);
+    }
 
     if (!child->isFloating() && childrenInline())
         dirtyLinesFromChangedChild(child);
 
     child->setNeedsLayoutAndMinMaxRecalc();
-    // We may supply the static position for an absolute positioned child.
-    if (!normalChildNeedsLayout() && !child->isText()) {
-        if (child->firstChild() || child->isPosWithStaticDim())
+
+    if (!normalChildNeedsLayout()) {
+        // We may supply the static position for an absolute positioned child.
+        if (child->firstChild() || child->isPosWithStaticDim() || !child->isPositioned())
             setChildNeedsLayout(true);
-        else if (child->isPositioned()) {
+        else {
             assert(!child->inPosObjectList());
             child->containingBlock()->insertPositionedObject(child);
         }
