@@ -5,7 +5,9 @@
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000 Dirk Mueller (mueller@kde.org)
  *           (C) 2006 Maksim Orlovich (maksim@kde.org)
- *           (C) 2006 Germain Garand (germain@ebooksfrance.org)
+ *           (C) 2007 Germain Garand (germain@ebooksfrance.org)
+ *           (C) 2007 Mitz Pettel (mitz@webkit.org)
+ *           (C) 2007 Charles Samuels (charles@kde.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,11 +39,13 @@
 #include <kmessagebox.h>
 #include <kreplace.h>
 #include <kreplacedialog.h>
-#include <k3spell.h>
+#include <dialog.h>
+#include <backgroundchecker.h>
 #include <kurlcompletion.h>
 #include <kwindowsystem.h>
 #include <kstandardaction.h>
 #include <kactioncollection.h>
+#include <kdeuiwidgetsproxystyle_p.h>
 
 #include <QAbstractItemView>
 #include <QAbstractTextDocumentLayout>
@@ -67,6 +71,7 @@ using namespace khtml;
 RenderFormElement::RenderFormElement(HTMLGenericFormElementImpl *element)
     : RenderWidget(element)
 //     , m_state(0)
+    , proxyStyle(0)
 {
     // init RenderObject attributes
     setInline(true);   // our object is Inline
@@ -77,15 +82,97 @@ RenderFormElement::~RenderFormElement()
 {
 }
 
+void RenderFormElement::setStyle(RenderStyle *style)
+{
+    RenderWidget::setStyle(style);
+    setPadding();
+}
+
+int RenderFormElement::paddingTop() const
+{
+    return !includesPadding() ? RenderWidget::paddingTop() : 0;
+}
+int RenderFormElement::paddingBottom() const
+{
+    return !includesPadding() ? RenderWidget::paddingBottom() : 0;
+}
+int RenderFormElement::paddingLeft() const
+{
+    return !includesPadding() ? RenderWidget::paddingLeft() : 0;
+}
+int RenderFormElement::paddingRight() const
+{
+    return !includesPadding() ? RenderWidget::paddingRight() : 0;
+}
+
+bool RenderFormElement::includesPadding() const
+{
+    return style()->boxSizing() == BORDER_BOX;
+} 
+
+
+void RenderFormElement::setPadding()
+{
+    if (!includesPadding())
+        return;
+    
+    struct AddPadding : public KdeUiProxyStyle
+    {
+        AddPadding(QWidget *parent)
+            : KdeUiProxyStyle(parent)
+        { }
+        QSize sizeFromContents(
+                ContentsType type, const QStyleOption * option, const QSize &contentsSize,
+                const QWidget * widget
+            ) const
+        {
+            QSize s = KdeUiProxyStyle::sizeFromContents(type, option, contentsSize, widget);
+            return s + QSize(left+right, top+bottom);
+        }
+        QRect subElementRect(
+                SubElement element, const QStyleOption *option, const QWidget *widget
+            ) const
+        {
+            QRect r = KdeUiProxyStyle::subElementRect(element, option, widget);
+            r.adjust(left, top, -right, -bottom);
+            return r;
+        }
+        
+        int left, right, top, bottom;
+    };
+
+
+    AddPadding *style = new AddPadding(widget());
+    style->left = RenderWidget::paddingLeft();
+    style->right = RenderWidget::paddingRight();
+    style->top = RenderWidget::paddingTop();
+    style->bottom = RenderWidget::paddingBottom();
+    
+    widget()->setStyle(style);
+    delete proxyStyle;
+    proxyStyle = style;
+}
+
 short RenderFormElement::baselinePosition( bool f ) const
 {
     return RenderWidget::baselinePosition( f ) - 2 - style()->fontMetrics().descent();
 }
 
+void RenderFormElement::setQWidget( QWidget *w )
+{
+    // sets the Qt Object Name for the purposes
+    // of setPadding() -- this is because QStyleSheet
+    // will propagate children of 'w' even if they are toplevel, like
+    // the "find" dialog or the popup menu
+    w->setObjectName("RenderFormElementWidget");
+    RenderWidget::setQWidget(w);
+}
+
 void RenderFormElement::updateFromElement()
 {
     m_widget->setEnabled(!element()->disabled());
     RenderWidget::updateFromElement();
+    setPadding();
 }
 
 void RenderFormElement::layout()
@@ -102,7 +189,7 @@ void RenderFormElement::layout()
     if ( m_widget )
         resizeWidget(m_width-borderLeft()-borderRight()-paddingLeft()-paddingRight(),
                      m_height-borderTop()-borderBottom()-paddingTop()-paddingBottom());
-
+    
     setNeedsLayout(false);
 }
 
@@ -306,19 +393,26 @@ void RenderSubmitButton::calcMinMaxWidth()
     QStyleOptionButton butOpt;
     butOpt.init(pb);
     butOpt.text = raw;
-    QSize s(pb->style()->sizeFromContents( QStyle::CT_PushButton, &butOpt, ts, pb )
-            .expandedTo(QApplication::globalStrut()));
+    QSize s = pb->style()->sizeFromContents( QStyle::CT_PushButton, &butOpt, ts, pb );
+    
+    s = s.expandedTo(QApplication::globalStrut());
     int margin = pb->style()->pixelMetric( QStyle::PM_ButtonMargin) +
-		 pb->style()->pixelMetric( QStyle::PM_DefaultFrameWidth ) * 2;
+              pb->style()->pixelMetric( QStyle::PM_DefaultFrameWidth ) * 2;
     int w = ts.width() + margin;
+    
     int h = s.height();
     if (pb->isDefault() || pb->autoDefault()) {
-	int dbw = pb->style()->pixelMetric( QStyle::PM_ButtonDefaultIndicator ) * 2;
-	w += dbw;
+        int dbw = pb->style()->pixelMetric( QStyle::PM_ButtonDefaultIndicator ) * 2;
+        w += dbw;
     }
-
     // add 30% margins to the width (heuristics to make it look similar to IE)
-    s = QSize( w*13/10, h ).expandedTo(QApplication::globalStrut());
+    w = w*13/10;
+    
+    // the crazy heuristic code overrides some changes made by the
+    // AddPadding proxy style, so reapply them
+    w += RenderWidget::paddingLeft() + RenderWidget::paddingRight();
+    
+    s = QSize(w,h).expandedTo(QApplication::globalStrut());
 
     setIntrinsicWidth( s.width() );
     setIntrinsicHeight( s.height() );
@@ -341,6 +435,8 @@ short RenderSubmitButton::baselinePosition( bool f ) const
     return RenderFormElement::baselinePosition( f );
 }
 
+
+
 // -------------------------------------------------------------------------------
 
 RenderResetButton::RenderResetButton(HTMLInputElementImpl *element)
@@ -354,30 +450,31 @@ class CompletionWidget: public KCompletionBox
 {
 public:
     CompletionWidget( QWidget *parent = 0 ) : KCompletionBox( parent ) {}
-    virtual void popup() {
+    virtual QPoint globalPositionHint() const
+    {
         QWidget* pw = parentWidget();
         KHTMLWidget* kwp = dynamic_cast<KHTMLWidget*>(pw);
         if (!kwp) {
             qDebug() << "CompletionWidget has no KHTMLWidget parent" << endl;
-            KCompletionBox::popup();
-            return;
+            return KCompletionBox::globalPositionHint();
         }
-        bool blocked = pw->signalsBlocked();
-        QPoint p = pw->pos();
-        pw->blockSignals(true);
-        pw->move( kwp->m_kwp->absolutePos() );
-        pw->blockSignals(blocked);
-
-        KCompletionBox::popup();
-        
-        pw->blockSignals(true);
-        pw->move( p );
-        pw->blockSignals(blocked);
+        QPoint dest;
+        KHTMLView* v = kwp->m_kwp->rootViewPos(dest);
+        QPoint ret;
+        if (v) {
+            ret = v->mapToGlobal( dest + QPoint(0, pw->height()) );
+            int zoomLevel = v->zoomLevel();
+            if (zoomLevel != 100) {
+                ret.setX( ret.x()*zoomLevel/100 );
+                ret.setY( ret.y()*zoomLevel/100 );
+            }
+        }
+        return ret;
     }
 };
 
 LineEditWidget::LineEditWidget(DOM::HTMLInputElementImpl* input, KHTMLView* view, QWidget* parent)
-    : KLineEdit(parent), m_input(input), m_view(view), m_spell(0)
+    : KLineEdit(parent), m_input(input), m_view(view)
 {
     m_kwp->setIsRedirected( true );
     setMouseTracking(true);
@@ -391,8 +488,6 @@ LineEditWidget::LineEditWidget(DOM::HTMLInputElementImpl* input, KHTMLView* view
 
 LineEditWidget::~LineEditWidget()
 {
-    delete m_spell;
-    m_spell = 0L;
 }
 
 void LineEditWidget::slotCheckSpelling()
@@ -400,16 +495,17 @@ void LineEditWidget::slotCheckSpelling()
     if ( text().isEmpty() ) {
         return;
     }
-
-    delete m_spell;
-    m_spell = new K3Spell( this, i18n( "Spell Checking" ), this, SLOT( slotSpellCheckReady( K3Spell *) ), 0, true, true);
-
-    connect( m_spell, SIGNAL( death() ),this, SLOT( spellCheckerFinished() ) );
-    connect( m_spell, SIGNAL( misspelling( const QString &, const QStringList &, unsigned int ) ),this, SLOT( spellCheckerMisspelling( const QString &, const QStringList &, unsigned int ) ) );
-    connect( m_spell, SIGNAL( corrected( const QString &, const QString &, unsigned int ) ),this, SLOT( spellCheckerCorrected( const QString &, const QString &, unsigned int ) ) );
+    Sonnet::Dialog *spellDialog = new Sonnet::Dialog(new Sonnet::BackgroundChecker(this), 0);
+    connect(spellDialog, SIGNAL(replace( const QString&, int,const QString&)), this, SLOT(spellCheckerCorrected( const QString&, int,const QString&)));
+    connect(spellDialog, SIGNAL(misspelling( const QString&, int)), this, SLOT(spellCheckerMisspelling(const QString &,int)));
+    connect(spellDialog, SIGNAL(done(const QString&)), this, SLOT(spellCheckerDone(const QString&)));
+    connect(spellDialog, SIGNAL(cancel()), this, SLOT(spellCheckerFinished()));
+    connect(spellDialog, SIGNAL(stop()), this, SLOT(spellCheckerFinished()));
+    spellDialog->setBuffer(text());
+    spellDialog->show();
 }
 
-void LineEditWidget::spellCheckerMisspelling( const QString &_text, const QStringList &, unsigned int pos)
+void LineEditWidget::spellCheckerMisspelling( const QString &_text, int pos)
 {
     highLightWord( _text.length(),pos );
 }
@@ -425,7 +521,7 @@ void LineEditWidget::highLightWord( unsigned int length, unsigned int pos )
     setSelection ( pos, length );
 }
 
-void LineEditWidget::spellCheckerCorrected( const QString &old, const QString &corr, unsigned int pos )
+void LineEditWidget::spellCheckerCorrected( const QString &old, int pos, const QString &corr )
 {
     if( old!= corr )
     {
@@ -439,12 +535,6 @@ void LineEditWidget::spellCheckerFinished()
 {
 }
 
-void LineEditWidget::slotSpellCheckReady( K3Spell *s )
-{
-    s->check( text() );
-    connect( s, SIGNAL( done( const QString & ) ), this, SLOT( slotSpellCheckDone( const QString & ) ) );
-}
-
 void LineEditWidget::slotSpellCheckDone( const QString &s )
 {
     if( s != text() )
@@ -460,7 +550,7 @@ void LineEditWidget::contextMenuEvent(QContextMenuEvent *e)
 
     if (m_input->autoComplete()) {
         popup->addSeparator();
-        QAction* act = popup->addAction( KIcon("history-clear"), i18n("Clear &History"));
+        QAction* act = popup->addAction( KIcon("edit-clear-history"), i18n("Clear &History"));
         act->setEnabled(compObj() && !compObj()->isEmpty());
         connect(act, SIGNAL(triggered()),
                 this, SLOT(clearHistoryActivated()));
@@ -505,7 +595,7 @@ bool LineEditWidget::event( QEvent *e )
 {
     if (KLineEdit::event(e))
 	return true;
-
+#if 0
     if ( e->type() == QEvent::AccelAvailable && isReadOnly() ) {
         QKeyEvent* ke = (QKeyEvent*) e;
         if ( ke->modifiers() & Qt::ControlModifier ) {
@@ -522,6 +612,7 @@ bool LineEditWidget::event( QEvent *e )
             }
         }
     }
+#endif
     return false;
 }
 
@@ -614,11 +705,12 @@ void RenderLineEdit::calcMinMaxWidth()
     int h = fm.lineSpacing();
     int w = fm.width( 'x' ) * (size > 0 ? size+1 : 17); // "some"
 
-    QStyleOption opt;
-    opt.init(widget());
+    QStyleOptionFrame opt;
+    opt.initFrom(widget());
+    if (static_cast<LineEditWidget*>(widget())->hasFrame())
+        opt.lineWidth = widget()->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, &opt, widget());
 
-    int margin = static_cast<LineEditWidget*>(widget())->hasFrame() ? 4 : 0;
-    s = QSize(w + margin, qMax(h, 14) + margin);
+    s = QSize(w, qMax(h, 14));
     s = widget()->style()->sizeFromContents(QStyle::CT_LineEdit, &opt, s, widget());
     s = s.expandedTo(QApplication::globalStrut());
 
@@ -717,6 +809,26 @@ void RenderLineEdit::setSelectionRange(long start, long end)
 RenderFieldset::RenderFieldset(HTMLGenericFormElementImpl *element)
     : RenderBlock(element)
 {
+    m_intrinsicWidth = 0;
+}
+ 
+void RenderFieldset::calcMinMaxWidth()
+{
+    RenderBlock::calcMinMaxWidth();
+    if (style()->htmlHacks()){ if (RenderObject* legend = findLegend()) {
+        int legendMinWidth = legend->minWidth();
+
+        Length legendMarginLeft = legend->style()->marginLeft();
+        Length legendMarginRight = legend->style()->marginLeft();
+
+        if (legendMarginLeft.isFixed())
+            legendMinWidth += legendMarginLeft.value();
+
+        if (legendMarginRight.isFixed())
+            legendMinWidth += legendMarginRight.value();
+
+        m_intrinsicWidth = qMax((int)m_minWidth, legendMinWidth + paddingLeft() + paddingRight() + borderLeft() + borderRight());
+    }}
 }
 
 RenderObject* RenderFieldset::layoutLegend(bool relayoutChildren)
@@ -738,7 +850,7 @@ RenderObject* RenderFieldset::layoutLegend(bool relayoutChildren)
     return legend;
 }
 
-RenderObject* RenderFieldset::findLegend()
+RenderObject* RenderFieldset::findLegend() const
 {
     for (RenderObject* legend = firstChild(); legend; legend = legend->nextSibling()) {
       if (!legend->isFloatingOrPositioned() && legend->element() &&
@@ -759,6 +871,7 @@ void RenderFieldset::paintBoxDecorations(PaintInfo& pI, int _tx, int _ty)
     int w = width();
     int h = height() + borderTopExtra() + borderBottomExtra();
     int yOff = (legend->yPos() > 0) ? 0 : (legend->height()-borderTop())/2;
+    int legendBottom = _ty + legend->yPos() + legend->height();
     h -= yOff;
     _ty += yOff - borderTopExtra();
 
@@ -766,11 +879,11 @@ void RenderFieldset::paintBoxDecorations(PaintInfo& pI, int _tx, int _ty)
     paintBackground(pI.p, style()->backgroundColor(), style()->backgroundLayers(), cr, _tx, _ty, w, h);
 
     if ( style()->hasBorder() )
-	    paintBorderMinusLegend(pI.p, _tx, _ty, w, h, style(), legend->xPos(), legend->width());
+	    paintBorderMinusLegend(pI.p, _tx, _ty, w, h, style(), legend->xPos(), legend->width(), legendBottom);
 }
 
 void RenderFieldset::paintBorderMinusLegend(QPainter *p, int _tx, int _ty, int w, int h,
-                                            const RenderStyle* style, int lx, int lw)
+                                            const RenderStyle* style, int lx, int lw, int lb)
 {
 
     const QColor& tc = style->borderTopColor();
@@ -786,10 +899,15 @@ void RenderFieldset::paintBorderMinusLegend(QPainter *p, int _tx, int _ty, int w
     bool render_r = rs > BHIDDEN;
     bool render_b = bs > BHIDDEN;
 
+    int borderLeftWidth = style->borderLeftWidth();
+    int borderRightWidth = style->borderRightWidth();
+
     if(render_t) {
-        drawBorder(p, _tx, _ty, _tx + lx, _ty +  style->borderTopWidth(), BSTop, tc, style->color(), ts,
+        if (lx >= borderLeftWidth)
+            drawBorder(p, _tx, _ty, _tx + lx, _ty +  style->borderTopWidth(), BSTop, tc, style->color(), ts,
                    (render_l && (ls == DOTTED || ls == DASHED || ls == DOUBLE)?style->borderLeftWidth():0), 0);
-        drawBorder(p, _tx+lx+lw, _ty, _tx + w, _ty +  style->borderTopWidth(), BSTop, tc, style->color(), ts,
+        if (lx + lw <=  w - borderRightWidth)
+            drawBorder(p, _tx+lx+lw, _ty, _tx + w, _ty +  style->borderTopWidth(), BSTop, tc, style->color(), ts,
                    0, (render_r && (rs == DOTTED || rs == DASHED || rs == DOUBLE)?style->borderRightWidth():0));
     }
 
@@ -812,7 +930,14 @@ void RenderFieldset::paintBorderMinusLegend(QPainter *p, int _tx, int _ty, int w
 	  (ls >= OUTSET) &&
 	  (bs == DOTTED || bs == DASHED || bs == SOLID || bs == INSET);
 
-        drawBorder(p, _tx, _ty, _tx + style->borderLeftWidth(), _ty + h, BSLeft, lc, style->color(), ls,
+        int startY = _ty;
+        if (lx < borderLeftWidth && lx + lw > 0) {
+            // The legend intersects the border.
+            ignore_top = true;
+            startY = lb;
+        }
+
+        drawBorder(p, _tx, startY, _tx + borderLeftWidth, _ty + h, BSLeft, lc, style->color(), ls,
 		   ignore_top?0:style->borderTopWidth(),
 		   ignore_bottom?0:style->borderBottomWidth());
     }
@@ -831,7 +956,14 @@ void RenderFieldset::paintBorderMinusLegend(QPainter *p, int _tx, int _ty, int w
 	  (rs >= DOTTED || rs == INSET) &&
 	  (bs == DOTTED || bs == DASHED || bs == SOLID || bs == INSET);
 
-        drawBorder(p, _tx + w - style->borderRightWidth(), _ty, _tx + w, _ty + h, BSRight, rc, style->color(), rs,
+        int startY = _ty;
+        if (lx < w && lx + lw > w - borderRightWidth) {
+            // The legend intersects the border.
+            ignore_top = true;
+            startY = lb;
+        }
+
+        drawBorder(p, _tx + w - borderRightWidth, startY, _tx + w, _ty + h, BSRight, rc, style->color(), rs,
 		   ignore_top?0:style->borderTopWidth(),
 		   ignore_bottom?0:style->borderBottomWidth());
     }
@@ -881,11 +1013,13 @@ void RenderFileButton::calcMinMaxWidth()
     int w = fm.width( 'x' ) * (size > 0 ? size+1 : 17); // "some"
     KLineEdit* edit = static_cast<KUrlRequester*>( m_widget )->lineEdit();
 
-    QStyleOption opt;
-    opt.init(edit);
+    QStyleOptionFrame opt;
+    opt.initFrom(edit);
+    if (edit->hasFrame())
+        opt.lineWidth = edit->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, &opt, edit);
     QSize s = edit->style()->sizeFromContents(QStyle::CT_LineEdit,
                                              &opt,
-          QSize(w + 2 + 2, qMax(h, 14) + 2 + 2), edit)
+          QSize(w, qMax(h, 14)), edit)
         .expandedTo(QApplication::globalStrut());
     QSize bs = static_cast<KUrlRequester*>( m_widget )->minimumSizeHint() - edit->minimumSizeHint();
 
@@ -954,6 +1088,18 @@ RenderLegend::RenderLegend(HTMLGenericFormElementImpl *element)
 
 // -------------------------------------------------------------------------------
 
+bool ListBoxWidget::event( QEvent * event )
+{
+    // accept all wheel events so that they are not propagated to the view
+    // once either end of the list is reached.
+    bool ret = KListWidget::event(event);
+    if (event->type() == QEvent::Wheel) {
+        event->accept();
+        ret = true;
+    }
+    return ret;
+}
+
 ComboBoxWidget::ComboBoxWidget(QWidget *parent)
     : KComboBox(false, parent)
 {
@@ -966,13 +1112,25 @@ ComboBoxWidget::ComboBoxWidget(QWidget *parent)
 void ComboBoxWidget::showPopup()
 {
     QPoint p = pos();
+    QPoint dest;
+    QWidget* parent = parentWidget();
+    KHTMLView* v = m_kwp->rootViewPos(dest);
+    int zoomLevel = v ? v->zoomLevel() : 100;
+    if (zoomLevel != 100) {
+        dest.setX( dest.x()*zoomLevel/100 );
+        dest.setY( dest.y()*zoomLevel/100 );
+    }
     bool blocked = blockSignals(true);
-    move( m_kwp->absolutePos() );
+    if (v != parent)
+        setParent(v);
+    move( dest );
     blockSignals(blocked);
 
     KComboBox::showPopup();
 
     blocked = blockSignals(true);
+    if (v != parent)
+        setParent(parent);
     move( p );
     blockSignals(blocked);
 }
@@ -1195,11 +1353,18 @@ void RenderSelect::layout( )
         int width = 0;
         int height = 0;
 
+        QAbstractItemModel *m = w->model();
+        QAbstractItemDelegate *d = w->itemDelegate();
+        QStyleOptionViewItem so;
+        so.font = w->font();
+
         for ( int rowIndex = 0 ; rowIndex < w->count() ; rowIndex++ ) {
-            QListWidgetItem* p = w->item(0);
-            width = qMax(width, p->listWidget()->visualItemRect(p).width());
-            height = qMax(height, p->listWidget()->visualItemRect(p).height());
+            QModelIndex mi = m->index(rowIndex, 0);
+            QSize s = d->sizeHint( so, mi);
+            width = qMax(width, s.width());
+            height = qMax(height, s.height());
         }
+
         if ( !height )
             height = w->fontMetrics().height();
         if ( !width )
@@ -1215,6 +1380,9 @@ void RenderSelect::layout( )
             size = qMin(static_cast<KListWidget*>(m_widget)->count(), 10);
 
         width += 2*w->frameWidth() + w->verticalScrollBar()->sizeHint().width();
+        int lhs = m_widget->style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing);
+        if (lhs>0)
+            width += lhs;
         height = size*height + 2*w->frameWidth();
 
         setIntrinsicWidth( width );
@@ -1327,6 +1495,12 @@ void RenderSelect::setOptionsChanged(bool _optionsChanged)
     m_optionsChanged = _optionsChanged;
 }
 
+void RenderSelect::setPadding()
+{
+    if (m_size > 1 || m_multiple)
+        RenderFormElement::setPadding();
+}
+
 ListBoxWidget* RenderSelect::createListBox()
 {
     ListBoxWidget *lb = new ListBoxWidget(view()->widget());
@@ -1358,8 +1532,7 @@ void RenderSelect::updateSelection()
     }
     else {
         bool found = false;
-        int firstOption = listItems.size();
-        i = listItems.size();
+        int firstOption = i = listItems.size();
         while (i--)
             if (listItems[i]->id() == ID_OPTION) {
                 if (found)
@@ -1371,7 +1544,11 @@ void RenderSelect::updateSelection()
                 firstOption = i;
             }
 
-        Q_ASSERT(firstOption == listItems.size() || found);
+        if (!found && firstOption != listItems.size()) {
+            // select first option (IE7/Gecko behaviour)
+            static_cast<HTMLOptionElementImpl*>(listItems[firstOption])->m_selected = true;
+            static_cast<KComboBox*>( m_widget )->setCurrentIndex(firstOption);
+        }
     }
 
     m_selectionChanged = false;
@@ -1519,7 +1696,8 @@ void TextAreaWidget::slotReplaceNext()
 
     if (res == KFind::NoMatch) {
         m_replace->displayFinalDialog();
-        delete m_replace;
+        m_replace->disconnect(this);
+        m_replace->deleteLater(); // we are in a slot connected to m_replace, don't delete it right away
         m_replace = 0;
         ensureCursorVisible();
         //or           if ( m_replace->shouldRestart() ) { reinit (w/o FromCursor) and call slotReplaceNext(); }
@@ -1567,7 +1745,8 @@ void TextAreaWidget::slotFindNext()
 
     if (res == KFind::NoMatch) {
         m_find->displayFinalDialog();
-        delete m_find;
+        m_find->disconnect(this);
+        m_find->deleteLater(); // we are in a slot connected to m_find, don't delete right away
         m_find = 0;
         //or           if ( m_find->shouldRestart() ) { reinit (w/o FromCursor) and call slotFindNext(); }
     } else {
@@ -1619,6 +1798,7 @@ void TextAreaWidget::slotReplace()
 
 bool TextAreaWidget::event( QEvent *e )
 {
+#if 0
     if ( e->type() == QEvent::AccelAvailable && isReadOnly() ) {
         QKeyEvent* ke = (QKeyEvent*) e;
         if ( ke->modifiers() & Qt::ControlModifier ) {
@@ -1635,7 +1815,15 @@ bool TextAreaWidget::event( QEvent *e )
             }
         }
     }
-    return KTextEdit::event( e );
+#endif
+    // accept all wheel events so that they are not propagated to the view
+    // once either end of the widget is reached.
+    bool ret = KTextEdit::event(e);
+    if (e->type() == QEvent::Wheel) {
+        e->accept();
+        ret = true;
+    }
+    return ret;
 }
 
 // -------------------------------------------------------------------------
