@@ -100,9 +100,10 @@ void* InlineBox::operator new(size_t sz, RenderArena* renderArena) throw()
 void InlineBox::operator delete(void* ptr, size_t sz)
 {
     assert(inInlineBoxDetach);
-
+#ifdef KHTML_USE_ARENA_ALLOCATOR
     // Stash size where detach can find it.
     *(size_t *)ptr = sz;
+#endif
 }
 
 static bool needsOutlinePhaseRepaint(RenderObject* o, RenderObject::PaintInfo& i, int tx, int ty) {
@@ -403,7 +404,7 @@ void InlineFlowBox::determineSpacingForFlowBoxes(bool lastLine, RenderObject* en
                     includeRightEdge = true;
             }
             else {
-                if ((!prevLineBox() || !prevLineBox()->isConstructed()) &&
+                if ((!prevLineBox() || prevLineBox()->isConstructed()) &&
                     ((lastLine && !object()->continuation()) ||
                      prevOnLineExists() || onEndChain(endObject)))
                     includeLeftEdge = true;
@@ -1086,9 +1087,9 @@ bool RootInlineBox::nodeAtPoint(RenderObject::NodeInfo& i, int x, int y, int tx,
 BidiStatus RootInlineBox::lineBreakBidiStatus() const
 {
     BidiStatus st;
-    st.eor = m_lineBreakBidiStatusEor;
-    st.last = m_lineBreakBidiStatusLast;
-    st.lastStrong = m_lineBreakBidiStatusLastStrong;
+    st.eor = KDE_CAST_BF_ENUM(QChar::Direction, m_lineBreakBidiStatusEor);
+    st.last = KDE_CAST_BF_ENUM(QChar::Direction, m_lineBreakBidiStatusLast);
+    st.lastStrong = KDE_CAST_BF_ENUM(QChar::Direction, m_lineBreakBidiStatusLastStrong);
     return st;
 }
 
