@@ -50,6 +50,7 @@
 #include "khtmlview.h"
 #include <khtml_part.h>
 #include <QBitmap>
+#include <QPaintEngine>
 
 #include <assert.h>
 using namespace DOM;
@@ -90,8 +91,10 @@ void RenderObject::operator delete(void* ptr, size_t sz)
 {
     assert(baseOfRenderObjectBeingDeleted == ptr);
 
+#ifdef KHTML_USE_ARENA_ALLOCATOR
     // Stash size where detach can find it.
     *(size_t *)ptr = sz;
+#endif
 }
 
 RenderObject *RenderObject::createObject(DOM::NodeImpl* node,  RenderStyle* style)
@@ -241,7 +244,10 @@ bool RenderObject::isHR() const
 {
     return element() && element()->id() == ID_HR;
 }
-
+bool RenderObject::isWordBreak() const
+{
+    return element() && element()->id() == ID_WBR;
+}
 bool RenderObject::isHTMLMarquee() const
 {
     return element() && element()->renderer() == this && element()->id() == ID_MARQUEE;
@@ -252,13 +258,13 @@ void RenderObject::addChild(RenderObject* , RenderObject *)
     KHTMLAssert(0);
 }
 
-RenderObject* RenderObject::removeChildNode(RenderObject* )
+RenderObject* RenderObject::removeChildNode(RenderObject*)
 {
     KHTMLAssert(0);
     return 0;
 }
 
-void RenderObject::removeChild(RenderObject *o )
+void RenderObject::removeChild(RenderObject*)
 {
     KHTMLAssert(0);
 }
@@ -503,6 +509,16 @@ short RenderObject::clientWidth() const
         (layer() ? layer()->verticalScrollbarWidth() : 0);
 }
 
+int RenderObject::clientLeft() const
+{
+    return borderLeft();
+}
+
+int RenderObject::clientTop() const
+{
+    return borderTop();
+}
+
 int RenderObject::clientHeight() const
 {
     return height() - borderTop() - borderBottom() -
@@ -740,8 +756,16 @@ void RenderObject::drawBorder(QPainter *p, int x1, int y1, int x2, int y2,
     if(!c.isValid()) {
         if(invalidisInvert)
         {
-            p->setCompositionMode(QPainter::CompositionMode_Difference);
-            c = Qt::white;
+            // handle 'outline-color: invert'
+            if (p->paintEngine()->hasFeature(QPaintEngine::BlendModes)) {
+                p->setCompositionMode(QPainter::CompositionMode_Difference);
+                c = Qt::white;
+            } else {
+                // 'invert' is not supported on this platform (for instance XRender)
+                // CSS3 UI 8.4: If the UA does not support the 'invert' value then the initial value of
+                // the 'outline-color' property is the 'currentColor' [CSS3COLOR] keyword.
+                c = m_style->color();
+            }
         }
         else {
             if(style == INSET || style == OUTSET || style == RIDGE || style ==
@@ -758,7 +782,7 @@ void RenderObject::drawBorder(QPainter *p, int x1, int y1, int x2, int y2,
     case BNONE:
     case BHIDDEN:
         // should not happen
-        if(invalidisInvert && p->compositionMode() == QPainter::CompositionMode_Xor)
+        if(invalidisInvert && p->compositionMode() == QPainter::CompositionMode_Difference)
             p->setCompositionMode(QPainter::CompositionMode_SourceOver);
 
         return;
@@ -968,7 +992,7 @@ void RenderObject::drawBorder(QPainter *p, int x1, int y1, int x2, int y2,
         break;
     }
 
-    if(invalidisInvert && p->compositionMode() == QPainter::CompositionMode_Xor)
+    if(invalidisInvert && p->compositionMode() == QPainter::CompositionMode_Difference)
         p->setCompositionMode(QPainter::CompositionMode_SourceOver);
 }
 
@@ -1306,6 +1330,15 @@ void RenderObject::setStyle(RenderStyle *style)
                 layer()->stackingContext()->dirtyZOrderLists();
                 layer()->dirtyZOrderLists();
             }
+            // keep layer hierarchy visibility bits up to date if visibility changes
+            if (m_style->visibility() != style->visibility()) {
+                RenderLayer* l = enclosingLayer(); 
+                if (style->visibility() == VISIBLE && l)
+                    l->setHasVisibleContent(true);
+                else if (l && l->hasVisibleContent() && 
+                    (this == l->renderer() || l->renderer()->style()->visibility() != VISIBLE))
+                    l->dirtyVisibleContentStatus();
+            }            
         }
 
         // reset style flags
@@ -1318,11 +1351,12 @@ void RenderObject::setStyle(RenderStyle *style)
 
     // only honor z-index for non-static objects and objects with opacity
     if ( style->position() == STATIC && style->opacity() == 1.0f ) {
-        if ( isRoot() )
-            style->setZIndex( 0 );
-        else
-            style->setHasAutoZIndex();
+        style->setHasAutoZIndex();
     }
+    // force establishment of a stacking context by transparent objects, as those define
+    // the bounds of an atomically painted region.
+    if (style->hasAutoZIndex() && (isRoot() || style->opacity() < 1.0f))
+        style->setZIndex( 0 );
 
     RenderStyle *oldStyle = m_style;
     m_style = style;
@@ -1641,8 +1675,6 @@ RenderArena* RenderObject::renderArena() const
 void RenderObject::detach()
 {
     detachCounters();
-
-    deleteInlineBoxes();
     remove();
 
     // make sure our DOM-node don't think we exist
@@ -2212,34 +2244,31 @@ CounterNode* RenderObject::getCounter(const QString& counter, bool view, bool co
 
 CounterNode* RenderObject::lookupCounter(const QString& counter) const
 {
-    Q3Dict<khtml::CounterNode>* counters = document()->counters(this);
-    if (counters)
-        return counters->find(counter);
-    else
-        return 0;
+    QHash<QString,khtml::CounterNode*>* counters = document()->counters(this);
+    return counters ? counters->value(counter) : 0;
 }
 
 void RenderObject::detachCounters()
 {
-    Q3Dict<khtml::CounterNode>* counters = document()->counters(this);
+    QHash<QString,khtml::CounterNode*>* counters = document()->counters(this);
     if (!counters) return;
 
-    Q3DictIterator<khtml::CounterNode> i(*counters);
+    QHashIterator<QString,khtml::CounterNode*> i(*counters);
 
-    while (i.current()) {
-        (*i)->remove();
-        delete (*i);
-        ++i;
+    while (i.hasNext()) {
+        i.next();
+        i.value()->remove();
+        delete i.value();
     }
     document()->removeCounters(this);
 }
 
 void RenderObject::insertCounter(const QString& counter, CounterNode* val)
 {
-    Q3Dict<khtml::CounterNode>* counters = document()->counters(this);
+    QHash<QString,khtml::CounterNode*>* counters = document()->counters(this);
 
     if (!counters) {
-        counters = new Q3Dict<khtml::CounterNode>(11);
+        counters = new QHash<QString,khtml::CounterNode*>();
         document()->setCounters(this, counters);
     }
 
