@@ -165,10 +165,9 @@ public:
         return isRoot()/* ### */ || isPositioned() || isRelPositioned() || hasOverflowClip() || style()->opacity() < 1.0f;
     }
 
-    // ### rename to overflowClipRect and clipRect
-    virtual QRect getOverflowClipRect(int /*tx*/, int /*ty*/)
+    virtual QRect overflowClipRect(int /*tx*/, int /*ty*/)
 	{ return QRect(0,0,0,0); }
-    virtual QRect getClipRect(int /*tx*/, int /*ty*/) { return QRect(0,0,0,0); }
+    virtual QRect clipRect(int /*tx*/, int /*ty*/) { return QRect(0,0,0,0); }
     bool hasClip() const { return isPositioned() &&  style()->hasClip(); }
     bool hasOverflowClip() const { return m_hasOverflowClip; }
 
@@ -280,6 +279,7 @@ public:
     virtual bool isApplet() const { return false; }
 
     bool isHTMLMarquee() const;
+    bool isWordBreak() const;
 
     bool isAnonymous() const { return m_isAnonymous; }
     void setIsAnonymous(bool b) { m_isAnonymous = b; }
@@ -361,7 +361,7 @@ public:
     void updateBackgroundImages(RenderStyle* oldStyle);
 
     virtual InlineBox* createInlineBox(bool makePlaceHolderBox, bool isRootLineBox);
-    virtual void removeInlineBox(InlineBox* box) {}
+    virtual void removeInlineBox(InlineBox* /*box*/) {}
 
     virtual short lineHeight( bool firstLine ) const;
     virtual short verticalPositionHint( bool firstLine ) const;
@@ -623,6 +623,8 @@ public:
     virtual int offsetLeft() const;
     virtual int offsetTop() const;
     virtual RenderObject* offsetParent() const;
+    int clientLeft() const;
+    int clientTop() const;
     short clientWidth() const;
     int clientHeight() const;
     short scrollWidth() const;
@@ -756,7 +758,7 @@ public:
     bool inPosObjectList() const { return m_inPosObjectList; }
 
     virtual void deleteInlineBoxes(RenderArena* arena=0) {(void)arena;}
-    virtual void dirtyInlineBoxes(bool fullLayout, bool isRootLineBox = false) {}
+    virtual void dirtyInlineBoxes(bool /*fullLayout*/, bool /*isRootLineBox*/ = false) {}
     virtual void dirtyLinesFromChangedChild(RenderObject*) {}
     virtual void detach( );
 
@@ -791,6 +793,7 @@ protected:
 
     virtual QRect viewRect() const;
     void remove() { if (m_parent) m_parent->removeChild(this); }
+    void setDetached() { m_attached = false; }
     void invalidateVerticalPosition();
     bool attemptDirectLayerTranslation();
     void updateWidgetMasks();
