@@ -35,6 +35,7 @@
 #include <QtGui/QActionEvent>
 #include <QtGui/QApplication>
 #include <QtGui/QLineEdit>
+#include <QtGui/QComboBox>
 #include <kglobalsettings.h>
 #include <kurlrequester.h>
 #include <QtCore/QObject>
@@ -50,6 +51,7 @@
 #include "misc/helper.h"
 #include "misc/paintbuffer.h"
 #include "css/cssvalues.h"
+#include "misc/htmltags.h"
 #include <kdebug.h>
 
 bool khtml::allowWidgetPaintEvents = false;
@@ -76,8 +78,7 @@ void RenderReplaced::calcMinMaxWidth()
     kDebug( 6040 ) << "RenderReplaced::calcMinMaxWidth() known=" << minMaxKnown();
 #endif
 
-    m_width = calcReplacedWidth();
-    m_width = calcBoxWidth( m_width );
+    m_width = calcReplacedWidth() + borderLeft() + borderRight() + paddingLeft() + paddingRight();
 
     if ( style()->width().isPercent() || style()->height().isPercent() ||
 		    style()->maxWidth().isPercent() || style()->maxHeight().isPercent() ||
@@ -161,12 +162,6 @@ void RenderWidget::detach()
     detachCounters();
     remove();
 
-    InlineBox* ph = placeHolderBox();
-    if (ph) {
-        ph->detach(renderArena());
-        setPlaceHolderBox( 0 );
-    }
-
     if ( m_widget ) {
         if ( m_view ) {
             m_view->setWidgetVisible(this, false);
@@ -182,6 +177,7 @@ void RenderWidget::detach()
     if ( node() && node()->renderer() == this)
         node()->setRenderer(0);
 
+    setDetached();
     deref();
 }
 
@@ -271,16 +267,25 @@ void RenderWidget::setQWidget(QWidget *widget)
         }
         m_widget = widget;
         if (m_widget) {
-            m_widget->setParent(m_view->widget());
             KHTMLWidget* k = dynamic_cast<KHTMLWidget*>(m_widget);
-            if (k)
+            bool isRedirectedSubFrame = false;
+            if (k) {
                 k->m_kwp->setRenderWidget(this);
+                // enable redirection of every sub-frame that is not a FRAME
+                if (qobject_cast<KHTMLView*>(m_widget) && element() && element()->id() != ID_FRAME) {
+                    k->m_kwp->setIsRedirected( true );
+                    isRedirectedSubFrame = true;
+                }
+            }   
+            m_widget->setParent(m_view->widget());
+            if (isRedirectedSubFrame)
+                static_cast<KHTMLView*>(m_widget)->setHasStaticBackground();
             connect( m_widget, SIGNAL( destroyed()), this, SLOT( slotWidgetDestructed()));
             m_widget->installEventFilter(this);
-
-            if ( isRedirectedWidget() && !qobject_cast<QFrame*>(m_widget))
-                m_widget->setAttribute( Qt::WA_NoSystemBackground );
-
+            if (isRedirectedWidget()) {
+                if (!qobject_cast<QFrame*>(m_widget))
+                    m_widget->setAttribute( Qt::WA_NoSystemBackground );
+            }
             if (m_widget->focusPolicy() > Qt::StrongFocus)
                 m_widget->setFocusPolicy(Qt::StrongFocus);
             // if we've already received a layout, apply the calculated space to the
@@ -531,19 +536,30 @@ void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
     if (!khtmlw)
         m_view->addChild( m_widget, xPos, yPos );
     else
-        m_widget->move( xPos, -500000 +yPos);
+        m_view->addChild( m_widget, xPos, -500000 +yPos);
     m_widget->show();
     if (khtmlw)
         paintWidget(paintInfo, m_widget, xPos, yPos);
 }
 
-static void setInPaintEventFlag(QWidget* w, bool b = true)
+static void setInPaintEventFlag(QWidget* w, bool b = true, bool recurse=true)
 {
       w->setAttribute(Qt::WA_WState_InPaintEvent, b);
-      foreach(QObject* cw, w->children()) {
-          if (cw->isWidgetType()) {
-              static_cast<QWidget*>(cw)->setAttribute(Qt::WA_WState_InPaintEvent, false);
-              setInPaintEventFlag(static_cast<QWidget*>(cw), b);
+
+      if (!recurse)
+          return;
+      if (qobject_cast<KHTMLView*>(w)) {
+          setInPaintEventFlag(static_cast<KHTMLView*>(w)->widget(), b, false);
+          setInPaintEventFlag(static_cast<KHTMLView*>(w)->horizontalScrollBar(), b, false);
+          setInPaintEventFlag(static_cast<KHTMLView*>(w)->verticalScrollBar(), b, false);
+          return;
+      }
+
+      foreach(QObject* o, w->children()) {
+          QWidget* const cw = static_cast<QWidget*>(o);
+          if (o->isWidgetType() && ! cw->isWindow()
+                                && !(cw->windowModality() & Qt::ApplicationModal)) {
+              setInPaintEventFlag(cw, b);
           }
       }
 }
@@ -552,17 +568,26 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
 {
     if (r.isNull() || r.isEmpty() )
         return;
-
+ 
     QPoint thePoint(tx, ty);
-    QMatrix m = p->worldMatrix();
-    thePoint = thePoint * m;
+    QTransform t = p->worldTransform();
+    bool vte = p->viewTransformEnabled();
+    bool wme = p->worldMatrixEnabled();
+    QRect w = p->window();
+    QRect v = p->viewport();
+    if (!buffered && t.isTranslating()) {
+        thePoint.setX( thePoint.x()+ static_cast<int>(t.dx()) );
+        thePoint.setY( thePoint.y()+ static_cast<int>(t.dy()) );
+    }
     QRegion rg = p->clipRegion();
     QPaintDevice *d = p->device();
     QPaintDevice *x = d;
+    qreal op = p->opacity();
+    QPixmap* pm = 0;
     if (buffered) {
         if (!widget->size().isValid())
             return;
-        QPixmap* pm = PaintBuffer::grab(widget->size());
+        pm = PaintBuffer::grab(widget->size());
         QPainter pp(pm);
         pp.setCompositionMode(QPainter::CompositionMode_Clear);
         pp.eraseRect(r);
@@ -575,18 +600,25 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
 
     widget->render( d, (buffered ? QPoint(0,0) : thePoint), r);
 
-    if (!p->isActive())
-        p->begin(x);
-    p->setWorldMatrix( m );
-    p->setClipRegion( rg );
-
-    if (buffered) {
-        // transfer results
-        p->drawPixmap(QPoint(tx, ty), static_cast<QPixmap&>(*d), r);
-        PaintBuffer::release();
-    }
-
     setInPaintEventFlag( widget );
+
+    if (!buffered) {
+        p->begin(x);
+        p->setWorldTransform(t);
+        p->setWindow(w);
+        p->setViewport(v);
+        p->setViewTransformEnabled( vte );
+        p->setWorldMatrixEnabled( wme );
+        if (!rg.isEmpty())
+            p->setClipRegion(rg);
+        if (op < 1.0f)
+            p->setOpacity(op);
+    } else {
+        // transfer results
+        QPoint off(r.x(), r.y());
+        p->drawPixmap(thePoint+off, static_cast<QPixmap&>(*d), r);
+        PaintBuffer::release(pm);
+    }
 }
 
 void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty)
@@ -594,20 +626,44 @@ void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty)
     QPainter* const p = pI.p;
     allowWidgetPaintEvents = true;
 
-    bool buffered = p->combinedMatrix().m22() != 1.0 && !qobject_cast<KHTMLView*>(widget);
+    bool buffered = p->combinedMatrix().m22() != 1.0 || (p->device()->devType() == QInternal::Printer);
 
     QRect rr = pI.r;
     rr.translate(-tx, -ty);
     const QRect r = widget->rect().intersect( rr );
-    copyWidget(r, p, widget, tx, ty, buffered);
-
+    if ( KHTMLView* v = qobject_cast<KHTMLView*>( widget ) ) {
+        QPoint thePoint(tx, ty);
+        if (v->verticalScrollBar()->isVisible()) {
+            QRect vbr = v->verticalScrollBar()->rect();
+            QPoint of = v->verticalScrollBar()->mapTo(v, QPoint(vbr.x(), vbr.y()));
+            vbr.translate( of );
+            vbr &= r;
+            vbr.translate( -of );
+            if (vbr.isValid() && !vbr.isEmpty())
+                copyWidget(vbr, p, v->verticalScrollBar(), tx+of.x(), ty+of.y(), buffered);
+        }
+        if (v->horizontalScrollBar()->isVisible()) {
+            QRect hbr = v->horizontalScrollBar()->rect();
+            QPoint of = v->horizontalScrollBar()->mapTo(v, QPoint(hbr.x(), hbr.y()));
+            hbr.translate( of );
+            hbr &= r;
+            hbr.translate( -of );
+            if (hbr.isValid() && !hbr.isEmpty())
+                copyWidget(hbr, p, v->horizontalScrollBar(), tx+ of.x(), ty+ of.y(), buffered);
+        }
+        QRect vr = (r & v->viewport()->rect());
+        if (vr.isValid() && !vr.isEmpty())
+            v->render(p, vr, thePoint);
+    } else {
+        copyWidget(r, p, widget, tx, ty, buffered);
+    }
     allowWidgetPaintEvents = false;
 }
 
 bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
 {
     // no special event processing if this is a frame (in which case KHTMLView handles it all)
-    if ( qobject_cast<KHTMLView*>( m_widget ) )
+    if ( qobject_cast<KHTMLView*>( m_widget ) || isRedirectedWidget() )
         return false;
     if ( !element() ) return true;
 
@@ -653,6 +709,22 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
 //             if ( ext )  ext->editableWidgetFocused( m_widget );
 //         }
         break;
+    case QEvent::Wheel: {
+       if (widget()->parentWidget() == view()->widget()) {
+            bool vertical = ( static_cast<QWheelEvent*>(e)->orientation() == Qt::Vertical );
+            // don't allow the widget to react to wheel event if
+            // the view is being scrolled by mouse wheel
+            // This does not apply if the webpage has no valid scroll range in the given wheel event orientation.
+            if ( ((vertical && (view()->contentsHeight() > view()->visibleHeight()))  ||
+                  (!vertical && (view()->contentsWidth() > view()->visibleWidth()))) &&
+                   view()->isScrollingFromMouseWheel() )  {
+                static_cast<QWheelEvent*>(e)->ignore();
+                QApplication::sendEvent(view(), e);
+                filtered = true;
+            }
+        }
+        break;
+    }
     case QEvent::KeyPress:
     case QEvent::KeyRelease:
     // TODO this seems wrong - Qt events are not correctly translated to DOM ones,
@@ -661,19 +733,6 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
             filtered = true;
         break;
 
-    case QEvent::Wheel:
-        if (widget()->parentWidget() == view()->widget()) {
-            // don't allow the widget to react to wheel event unless its
-            // currently focused. this avoids accidentally changing a select box
-            // or something while wheeling a webpage.
-            if (qApp->focusWidget() != widget() &&
-                widget()->focusPolicy() <= Qt::StrongFocus)  {
-                static_cast<QWheelEvent*>(e)->ignore();
-                QApplication::sendEvent(view(), e);
-                filtered = true;
-            }
-        }
-        break;
     default:
         break;
     };
@@ -690,6 +749,9 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
 
 void RenderWidget::EventPropagator::sendEvent(QEvent *e) {
     switch(e->type()) {
+    case QEvent::Wheel:
+        wheelEvent ( static_cast<QWheelEvent *> (e) );
+        break;
     case QEvent::MouseButtonPress:
         mousePressEvent(static_cast<QMouseEvent *>(e));
         break;
@@ -803,12 +865,21 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
         absolutePosition(absx, absy);
         absx += borderLeft()+paddingLeft();
         absy += borderTop()+paddingTop();
+
         QPoint p(me.clientX() - absx + m_view->contentsX(),
                  me.clientY() - absy + m_view->contentsY());
 
         QWidget* target = 0;
         target = m_widget->childAt(p);
-
+        
+        if ( includesPadding() && ::qobject_cast<QAbstractScrollArea*>(m_widget) )
+        {
+            QAbstractScrollArea *w = static_cast<QAbstractScrollArea*>(m_widget);
+            // childAt returns the wrong child in this case?
+            if (target == w->viewport())
+                p -= QPoint(RenderWidget::paddingLeft(), RenderWidget::paddingTop());
+        }
+        
         if (m_underMouse != target && ev.id() == EventImpl::MOUSEMOVE_EVENT) {
             if (m_underMouse) {
                 QEvent moe( QEvent::Leave );
@@ -835,7 +906,8 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
             if (!target || (!::qobject_cast<QScrollBar*>(target) && 
                             !::qobject_cast<KUrlRequester*>(m_widget)))
                 target = m_widget;
-            view()->setMouseEventsTarget( target );
+            if ( button == Qt::LeftButton )
+                view()->setMouseEventsTarget( target );
         } else {
             target = view()->mouseEventsTarget();
             if (target) {
@@ -847,11 +919,28 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
                 target = m_widget;
             }
         }
+
         p = target->mapFrom(m_widget, p);
 
         bool needContextMenuEvent = (type == QMouseEvent::MouseButtonPress && button == Qt::RightButton);
+        bool isMouseWheel = (ev.id() == EventImpl::KHTML_MOUSEWHEEL_EVENT);
 
-        QEvent *e = (ev.id() == EventImpl::KHTML_MOUSEWHEEL_EVENT) ?
+        if (isMouseWheel) {
+            // don't allow the widget to react to wheel event if
+            // a) the view is being scrolled by mouse wheel
+            // b) it's an unfocused ComboBox (for extra security against unwanted changes to formulars)
+            // This does not apply if the webpage has no valid scroll range in the given wheel event orientation.
+            if ( ((orient == Qt::Vertical && (view()->contentsHeight() > view()->visibleHeight()))  ||
+                  (orient == Qt::Horizontal && (view()->contentsWidth() > view()->visibleWidth()))) &&
+                  ( view()->isScrollingFromMouseWheel() ||
+                    (qobject_cast<QComboBox*>(m_widget) && 
+                     (!document()->focusNode() || document()->focusNode()->renderer() != this) )))  {
+                ret = false;
+                break;
+            }
+        }
+
+        QEvent *e = isMouseWheel ?
                     static_cast<QEvent*>(new QWheelEvent(p, -me.detail()*40, buttons, state, orient)) :
                     static_cast<QEvent*>(new QMouseEvent(type,    p, button, buttons, state));
         static_cast<EventPropagator *>(target)->sendEvent(e);
@@ -865,7 +954,7 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
             QHoverEvent he( QEvent::HoverMove, p, p );
             QApplication::sendEvent(target, &he);
         }
-        if (ev.id() == EventImpl::MOUSEUP_EVENT && button == Qt::LeftButton) {
+        if (ev.id() == EventImpl::MOUSEUP_EVENT) {
             view()->setMouseEventsTarget( 0 );
         }
         delete e;
@@ -948,6 +1037,11 @@ void RenderWidget::deref()
     if (_ref) _ref--;
 //     qDebug( "deref(%p): width get count is %d", this, _ref);
     if (!_ref) {
+        if (attached()) {
+            _ref++;
+            detach(); // will perform the final deref.
+            return;
+        }
         SharedPtr<RenderArena> guard(m_arena); //Since delete on us gets called -first-,
                                                //before the arena free
         arenaDelete(m_arena.get());
@@ -979,6 +1073,28 @@ QPoint KHTMLWidgetPrivate::absolutePos()
         return QPoint(x, y);
 }
 
+KHTMLView* KHTMLWidgetPrivate::rootViewPos(QPoint& pos)
+{
+    if (!m_rw || !m_rw->widget()) {
+        pos = QPoint();
+        return 0;
+    }
+    pos = absolutePos();
+    KHTMLView* v = m_rw->view();
+    KHTMLView* last = 0;
+    while (v) {
+        last = v;
+        pos.setX( pos.x() - v->contentsX() );
+        pos.setY( pos.y() - v->contentsY() );
+        KHTMLWidget*kw = dynamic_cast<KHTMLWidget*>(v);
+        if (!kw || !kw->m_kwp->isRedirected()) 
+            break;
+        pos += kw->m_kwp->absolutePos();
+        v = v->part()->parentPart() ? v->part()->parentPart()->view() : 0;
+    }
+    return last;
+}
+
 // -----------------------------------------------------------------------------
 
 KHTMLWidget::KHTMLWidget() 
