@@ -37,6 +37,7 @@
 #include <kstandarddirs.h>
 #include <QtGui/QImage>
 #include <QtCore/QFile>
+#include <QtCore/QEventLoop>
 #include <stdio.h>
 
 #include "css/cssstyleselector.h"
@@ -190,7 +191,7 @@ public:
         case CT_PushButton:
             return QSize(size.width(), size.height() - 1);
         case CT_LineEdit:
-	    if (widget && widget->parentWidget() && !qstricmp(widget->parentWidget()->className(), "KUrlRequester"))
+	    if (widget && widget->parentWidget() && !qstricmp(widget->parentWidget()->metaObject()->className(), "KUrlRequester"))
 		return QSize(size.width() + 1, size.height());
             return QSize(size.width() + 2, size.height() + 2);
         case CT_ComboBox:
@@ -216,7 +217,6 @@ public:
 
 #include <kaction.h>
 #include <kcmdlineargs.h>
-#include "khtml_factory.h"
 #include <kio/job.h>
 #include <kmainwindow.h>
 #include <kconfig.h>
@@ -277,6 +277,7 @@ PartMonitor::~PartMonitor()
 {
    if (this == sm_highestMonitor)
 	sm_highestMonitor = 0;
+   qDeleteAll(m_eventLoopStack);
 }
 
 
@@ -289,7 +290,7 @@ void PartMonitor::waitForCompletion()
 
 	sm_highestMonitor = this;
 
-        qApp->enter_loop();
+        enterLoop();
 
         //connect(m_timeout_timer, SIGNAL(timeout()), this, SLOT( timeout() ) );
         //m_timeout_timer->stop();
@@ -297,12 +298,27 @@ void PartMonitor::waitForCompletion()
     }
 
     QTimer::singleShot( 0, this, SLOT( finishTimers() ) );
-    qApp->enter_loop();
+    enterLoop();
+}
+
+void PartMonitor::enterLoop()
+{
+    if (m_eventLoopStack.isEmpty() || m_eventLoopStack.top()->isRunning())
+        m_eventLoopStack.push(new QEventLoop());
+    m_eventLoopStack.top()->exec();
+}
+
+void PartMonitor::exitLoop()
+{
+    while (!m_eventLoopStack.isEmpty() && !m_eventLoopStack.top()->isRunning())
+        delete m_eventLoopStack.pop();
+    if (!m_eventLoopStack.isEmpty())
+        m_eventLoopStack.top()->exit();
 }
 
 void PartMonitor::timeout()
 {
-    qApp->exit_loop();
+    exitLoop();
 }
 
 void PartMonitor::finishTimers()
@@ -314,7 +330,7 @@ void PartMonitor::finishTimers()
         QTimer::singleShot( 10, this, SLOT(finishTimers() ) );
         return;
     }
-    qApp->exit_loop();
+    exitLoop();
 }
 
 void PartMonitor::partCompleted()
@@ -357,9 +373,9 @@ bool RegTestFunction::implementsCall() const
     return true;
 }
 
-ValueImp* RegTestFunction::callAsFunction(ExecState *exec, ObjectImp* /*thisObj*/, const List &args)
+JSValue* RegTestFunction::callAsFunction(ExecState *exec, JSObject* /*thisObj*/, const List &args)
 {
-    ValueImp* result = Undefined();
+    JSValue* result = jsUndefined();
     if ( m_regTest->ignore_errors )
         return result;
 
@@ -433,13 +449,13 @@ KHTMLPartObject::KHTMLPartObject(ExecState *exec, KHTMLPart *_part)
     putDirect("processEvents", new KHTMLPartFunction(exec,m_part,KHTMLPartFunction::ProcessEvents,0), DontEnum);
 }
 
-KJS::ValueImp *KHTMLPartObject::winGetter(KJS::ExecState *, KJS::JSObject*, const KJS::Identifier&, const KJS::PropertySlot& slot)
+KJS::JSValue *KHTMLPartObject::winGetter(KJS::ExecState *, KJS::JSObject*, const KJS::Identifier&, const KJS::PropertySlot& slot)
 {
     KHTMLPartObject* thisObj = static_cast<KHTMLPartObject*>(slot.slotBase());
     return KJS::Window::retrieveWindow(thisObj->m_part);
 }
 
-KJS::ValueImp *KHTMLPartObject::docGetter(KJS::ExecState *exec, KJS::JSObject*, const KJS::Identifier&, const KJS::PropertySlot& slot)
+KJS::JSValue *KHTMLPartObject::docGetter(KJS::ExecState *exec, KJS::JSObject*, const KJS::Identifier&, const KJS::PropertySlot& slot)
 {
     KHTMLPartObject* thisObj = static_cast<KHTMLPartObject*>(slot.slotBase());
     return getDOMNode(exec, thisObj->m_part->document().handle());
@@ -456,7 +472,7 @@ bool KHTMLPartObject::getOwnPropertySlot(KJS::ExecState *exec, const KJS::Identi
         slot.setCustom(this, winGetter);
         return true;
     }
-    return ObjectImp::getOwnPropertySlot(exec, propertyName, slot);
+    return JSObject::getOwnPropertySlot(exec, propertyName, slot);
 }
 
 KHTMLPartFunction::KHTMLPartFunction(ExecState */*exec*/, KHTMLPart *_part, int _id, int length)
@@ -471,15 +487,15 @@ bool KHTMLPartFunction::implementsCall() const
     return true;
 }
 
-ValueImp* KHTMLPartFunction::callAsFunction(ExecState *exec, ObjectImp*/*thisObj*/, const List &args)
+JSValue* KHTMLPartFunction::callAsFunction(ExecState *exec, JSObject*/*thisObj*/, const List &args)
 {
-    ValueImp* result = Undefined();
+    JSValue* result = jsUndefined();
 
     switch (id) {
         case OpenPage: {
 	    if (args[0]->type() == NullType || args[0]->type() == NullType) {
 		exec->setException(Error::create(exec, GeneralError,"No filename specified"));
-		return Undefined();
+		return jsUndefined();
 	    }
 
             QString filename = args[0]->toString(exec).qstring();
@@ -496,11 +512,11 @@ ValueImp* KHTMLPartFunction::callAsFunction(ExecState *exec, ObjectImp*/*thisObj
 	case OpenPageAsUrl: {
 	    if (args[0]->type() == NullType || args[0]->type() == UndefinedType) {
 		exec->setException(Error::create(exec, GeneralError,"No filename specified"));
-		return Undefined();
+		return jsUndefined();
 	    }
 	    if (args[1]->type() == NullType || args[1]->type() == UndefinedType) {
 		exec->setException(Error::create(exec, GeneralError,"No url specified"));
-		return Undefined();
+		return jsUndefined();
 	    }
 
             QString filename = args[0]->toString(exec).qstring();
@@ -733,7 +749,6 @@ int main(int argc, char *argv[])
     kClearDebugConfig();
 
     // create widgets
-    KHTMLFactory *fac = new KHTMLFactory();
     KMainWindow *toplevel = new KMainWindow();
     KHTMLPart *part = new KHTMLPart( toplevel, 0, KHTMLPart::BrowserViewGUI );
 
@@ -749,7 +764,6 @@ int main(int argc, char *argv[])
 	visual = true;
 
     a.setTopWidget(part->widget());
-    a.setMainWidget( toplevel );
     if ( visual )
         toplevel->show();
 
@@ -833,7 +847,6 @@ int main(int argc, char *argv[])
     delete regressionTest;
     delete part;
     delete toplevel;
-    delete fac;
 
     khtml::Cache::clear();
     khtml::CSSStyleSelector::clear();
@@ -853,6 +866,7 @@ RegressionTest::RegressionTest(KHTMLPart *part, const QString &baseDir, const QS
   : QObject(part)
 {
     m_part = part;
+
     m_baseDir = baseDir;
     m_baseDir = m_baseDir.replace( "//", "/" );
     if ( m_baseDir.endsWith( "/" ) )	//krazy:exclude=duoblequote_chars DOM demands chars
@@ -887,6 +901,7 @@ RegressionTest::RegressionTest(KHTMLPart *part, const QString &baseDir, const QS
 
     curr = this;
     m_part->view()->setFrameStyle(QFrame::StyledPanel | QFrame::Plain);
+    resizeTopLevelWidget(800, 598 );
 }
 
 
@@ -1141,7 +1156,7 @@ QImage RegressionTest::renderToImage()
     if (ew * eh > 4000 * 4000) // don't DoS us
         return QImage();
 
-    QImage img( ew, eh, 32 );
+    QImage img( ew, eh, QImage::Format_ARGB32 );
     img.fill( 0xff0000 );
     if (!m_paintBuffer )
         m_paintBuffer = new QPixmap( 512, 128 );
@@ -1435,7 +1450,8 @@ void RegressionTest::doFailureReport( const QString& test, int failures )
 
 void RegressionTest::testStaticFile(const QString & filename)
 {
-    qApp->mainWidget()->resize( 800, 598 ); // restore size
+    qDebug("TESTING:%s", filename.toLatin1().data());
+    resizeTopLevelWidget( 800, 598 ); // restore size
 
     // Set arguments
     KParts::OpenUrlArguments args;
@@ -1465,7 +1481,7 @@ void RegressionTest::testStaticFile(const QString & filename)
             if (comp.value() && comp.value()->type() == ObjectType &&
                comp.value()->toObject(exec)->className() == "Array" )
             {
-                ObjectImp* argArrayObj = comp.value()->toObject(exec);
+                JSObject* argArrayObj = comp.value()->toObject(exec);
                 unsigned int length = argArrayObj->
                                       get(exec, "length")->
                                       toUInt32(exec);
@@ -1482,8 +1498,8 @@ void RegressionTest::testStaticFile(const QString & filename)
         bool success = ( comp2.complType() == ReturnValue || comp2.complType() == Normal );
         QString description = "DOMTS";
         if ( comp2.complType() == Throw ) {
-            KJS::ValueImp*  val = comp2.value();
-            KJS::ObjectImp* obj = val->toObject(exec);
+            KJS::JSValue*  val = comp2.value();
+            KJS::JSObject* obj = val->toObject(exec);
             if ( obj && obj->hasProperty( exec, "jsUnitMessage" ) )
                 description = obj->get( exec, "jsUnitMessage" )->toString( exec ).qstring();
             else
@@ -1561,7 +1577,7 @@ void RegressionTest::evalJS( ScriptInterpreter &interp, const QString &filename,
             QString errmsg = c.value()->toString(exec).qstring();
             if ( !expected_failure ) {
                 int line = -1;
-                ObjectImp* obj = c.value()->toObject(exec);
+                JSObject* obj = c.value()->toObject(exec);
                 if (obj)
                     line = obj->get(exec, "line")->toUInt32(exec);
                 printf( "ERROR: %s (%s) at line:%d\n",qPrintable(filename), qPrintable(errmsg), line);
@@ -1580,19 +1596,20 @@ void RegressionTest::evalJS( ScriptInterpreter &interp, const QString &filename,
     }
 }
 
-class GlobalImp : public ObjectImp {
+class GlobalImp : public JSObject {
 public:
   virtual UString className() const { return "global"; }
 };
 
 void RegressionTest::testJSFile(const QString & filename )
 {
-    qApp->mainWidget()->resize( 800, 598 ); // restore size
+    qDebug("TEST JS:%s", filename.toLatin1().data());
+    resizeTopLevelWidget( 800, 598 ); // restore size
 
     // create interpreter
     // note: this is different from the interpreter used by the part,
     // it contains regression test-specific objects & functions
-    ProtectedPtr<ObjectImp> global(new GlobalImp());
+    ProtectedPtr<JSObject> global(new GlobalImp());
     khtml::ChildFrame frame;
     frame.m_part = m_part;
     ScriptInterpreter interp(global,&frame);
@@ -1824,7 +1841,7 @@ bool RegressionTest::svnIgnored( const QString &filename )
 
 void RegressionTest::resizeTopLevelWidget( int w, int h )
 {
-    qApp->mainWidget()->resize( w, h );
+    m_part->widget()->parentWidget()->resize( w, h );
     // Since we're not visible, this doesn't have an immediate effect, QWidget posts the event
     QApplication::sendPostedEvents( 0, QEvent::Resize );
 }
