@@ -30,6 +30,7 @@
 #include "dom_stringimpl.h"
 #include "dom_nodeimpl.h"
 #include "dom_docimpl.h"
+#include "misc/translator.h"
 #include <rendering/render_layer.h>
 #include <khtmlview.h>
 
@@ -187,6 +188,8 @@ EventImpl::EventId EventImpl::typeToId(DOMString type)
         return KHTML_ECMA_DBLCLICK_EVENT;
     else if ( type == "DOMMouseScroll" )
         return KHTML_MOUSEWHEEL_EVENT;
+    else if ( type == "DOMContentLoaded" )
+        return KHTML_CONTENTLOADED_EVENT;
 
     // ignore: KHTML_CLICK_EVENT
     return UNKNOWN_EVENT;
@@ -271,6 +274,8 @@ DOMString EventImpl::idToType(EventImpl::EventId id)
         return "readystatechange";
     case KHTML_MOUSEWHEEL_EVENT:
         return "DOMMouseScroll"; // adopt the mozilla name for compatibility
+    case KHTML_CONTENTLOADED_EVENT:
+        return "DOMContentLoaded"; // idem
     default:
         return DOMString();
         break;
@@ -492,47 +497,6 @@ bool MouseEventImpl::isMouseEvent() const
 }
 
 //---------------------------------------------------------------------------------------------
-/* This class is used to do remapping between different encodings reasonably compactly */
-
-template<typename L, typename R, typename MemL>
-class IDTranslator
-{
-public:
-    struct Info {
-        MemL l;
-        R    r;
-    };
-
-    IDTranslator(const Info* table) {
-        for (const Info* cursor = table; cursor->l; ++cursor) {
-            m_lToR.insert(cursor->l,  cursor->r);
-            m_rToL.insert(cursor->r,  cursor->l);
-        }
-    }
-
-    L toLeft(R r) {
-        typename QMap<R,L>::iterator i( m_rToL.find(r) );
-        if (i != m_rToL.end())
-            return *i;
-        return L();
-    }
-
-    R toRight(L l) {
-        typename QMap<L,R>::iterator i = m_lToR.find(l);
-        if (i != m_lToR.end())
-            return *i;
-        return R();
-    }
-
-private:
-    QMap<L, R> m_lToR;
-    QMap<R, L> m_rToL;
-};
-
-#define MAKE_TRANSLATOR(name,L,R,MR,table) static IDTranslator<L,R,MR>* s_##name; \
-    static IDTranslator<L,R,MR>* name() { if (!s_##name) s_##name = new IDTranslator<L,R,MR>(table); \
-        return s_##name; }
-//---------------------------------------------------------------------------------------------
 
 /* Mapping between special Qt keycodes and virtual DOM codes */
 IDTranslator<unsigned, unsigned, unsigned>::Info virtKeyToQtKeyTable[] =
@@ -557,8 +521,8 @@ IDTranslator<unsigned, unsigned, unsigned>::Info virtKeyToQtKeyTable[] =
     {KeyEventBaseImpl::DOM_VK_RIGHT,        Qt::Key_Right},
     {KeyEventBaseImpl::DOM_VK_UP,           Qt::Key_Up},
     {KeyEventBaseImpl::DOM_VK_DOWN,         Qt::Key_Down},
-    {KeyEventBaseImpl::DOM_VK_PAGE_DOWN,    Qt::Key_Next},
-    {KeyEventBaseImpl::DOM_VK_PAGE_UP,      Qt::Key_Prior},
+    {KeyEventBaseImpl::DOM_VK_PAGE_DOWN,    Qt::Key_PageDown},
+    {KeyEventBaseImpl::DOM_VK_PAGE_UP,      Qt::Key_PageUp},
     {KeyEventBaseImpl::DOM_VK_F1,           Qt::Key_F1},
     {KeyEventBaseImpl::DOM_VK_F2,           Qt::Key_F2},
     {KeyEventBaseImpl::DOM_VK_F3,           Qt::Key_F3},
@@ -726,10 +690,10 @@ MAKE_TRANSLATOR(keyIdentifiersToVirtKeys, QByteArray, unsigned, const char*, key
 
 /** These are the modifiers we currently support */
 static const IDTranslator<QByteArray, unsigned, const char*>::Info keyModifiersToCodeTable[] = {
-    {"Alt",         Qt::AltButton},
-    {"Control",     Qt::ControlButton},
-    {"Shift",       Qt::ShiftButton},
-    {"Meta",        Qt::MetaButton},
+    {"Alt",         Qt::AltModifier},
+    {"Control",     Qt::ControlModifier},
+    {"Shift",       Qt::ShiftModifier},
+    {"Meta",        Qt::MetaModifier},
     {0,             0}
 };
 
