@@ -48,7 +48,6 @@
 #include <QtCore/QStack>
 //Added by qt3to4:
 #include <QTimerEvent>
-#include <Q3PtrList>
 #include <QtCore/QList>
 #include <kdebug.h>
 #include <klocale.h>
@@ -237,16 +236,20 @@ DOMImplementationImpl *DOMImplementationImpl::instance()
 
 // ------------------------------------------------------------------------
 
-ElementMappingCache::ElementMappingCache():m_dict(257)
+ElementMappingCache::ElementMappingCache():m_dict()
 {
-    m_dict.setAutoDelete(true);
+}
+
+ElementMappingCache::~ElementMappingCache()
+{
+    qDeleteAll( m_dict );
 }
 
 void ElementMappingCache::add(const QString& id, ElementImpl* nd)
 {
     if (id.isEmpty()) return;
 
-    ItemInfo* info = m_dict.find(id);
+    ItemInfo* info = m_dict.value(id);
     if (info)
     {
         info->ref++;
@@ -265,7 +268,8 @@ void ElementMappingCache::set(const QString& id, ElementImpl* nd)
 {
     if (id.isEmpty()) return;
 
-    ItemInfo* info = m_dict.find(id);
+    assert(m_dict.contains(id));
+    ItemInfo* info = m_dict.value(id);
     info->nd = nd;
 }
 
@@ -273,7 +277,8 @@ void ElementMappingCache::remove(const QString& id, ElementImpl* nd)
 {
     if (id.isEmpty()) return;
 
-    ItemInfo* info = m_dict.find(id);
+    assert(m_dict.contains(id));
+    ItemInfo* info = m_dict.value(id);
     info->ref--;
     if (info->ref == 0)
     {
@@ -290,13 +295,13 @@ void ElementMappingCache::remove(const QString& id, ElementImpl* nd)
 bool ElementMappingCache::contains(const QString& id)
 {
     if (id.isEmpty()) return false;
-    return m_dict.find(id);
+    return m_dict.contains(id);
 }
 
 ElementMappingCache::ItemInfo* ElementMappingCache::get(const QString& id)
 {
     if (id.isEmpty()) return 0;
-    return m_dict.find(id);
+    return m_dict.value(id);
 }
 
 typedef QList<DocumentImpl*> ChangedDocuments ;
@@ -304,7 +309,7 @@ K_GLOBAL_STATIC(ChangedDocuments, s_changedDocuments)
 
 // KHTMLView might be 0
 DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
-    : NodeBaseImpl( 0 ), m_domtree_version(0), m_counterDict(257),
+    : NodeBaseImpl( 0 ), m_domtree_version(0), m_counterDict(),
       m_imageLoadEventTimer(0)
 {
     m_document.resetSkippingRef(this); //Make getDocument return us..
@@ -317,7 +322,7 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_view = v;
     m_renderArena.reset();
 
-    KHTMLFactory::ref();
+    KHTMLGlobal::registerDocumentImpl(this);
 
     if ( v ) {
         m_docLoader = new DocLoader(v->part(), this );
@@ -366,7 +371,6 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_inDocument = true;
     m_styleSelectorDirty = false;
     m_styleSelector = 0;
-    m_counterDict.setAutoDelete(true);
 
     m_inStyleRecalc = false;
     m_pendingStylesheets = 0;
@@ -374,8 +378,9 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_async = true;
     m_hadLoadError = false;
     m_docLoading = false;
-    m_inSyncLoad = false;
+    m_inSyncLoad = 0;
     m_loadingXMLDoc = 0;
+    m_documentElement = 0;
     m_cssTarget = 0;
     m_dynamicDomRestyler = new khtml::DynamicDomRestyler();
 }
@@ -385,10 +390,10 @@ void DocumentImpl::removedLastRef()
     if (m_selfOnlyRefCount) {
         /* In this case, the only references to us are from children,
            so we have a cycle. We'll try to break it by disconnecting the
-           children from us; this sucks/is wrong, but it's pretty much 
-           the best we can do without tracing. 
+           children from us; this sucks/is wrong, but it's pretty much
+           the best we can do without tracing.
 
-           Of course, if dumping the children causes the refcount from them to 
+           Of course, if dumping the children causes the refcount from them to
            drop to 0 we can get killed right here, so better hold
            a temporary reference, too
         */
@@ -397,10 +402,10 @@ void DocumentImpl::removedLastRef()
         // we must make sure not to be retaining any of our children through
         // these extra pointers or we will create a reference cycle
         if (m_doctype) {
-            m_doctype->deref(); 
+            m_doctype->deref();
             m_doctype = 0;
         }
-        
+
         if (m_cssTarget) {
             m_cssTarget->deref();
             m_cssTarget = 0;
@@ -415,12 +420,17 @@ void DocumentImpl::removedLastRef()
             m_hoverNode->deref();
             m_hoverNode = 0;
         }
-        
+
         if (m_activeNode) {
             m_activeNode->deref();
             m_activeNode = 0;
         }
 
+        if (m_documentElement) {
+            m_documentElement->deref();
+            m_documentElement = 0;
+        }
+
         removeChildren();
 
         delete m_tokenizer;
@@ -436,9 +446,9 @@ DocumentImpl::~DocumentImpl()
     //you may also have to fix removedLastRef() above - M.O.
     assert( !m_render );
 
-    Q3IntDictIterator<NodeListImpl::Cache> it(m_nodeListCache);
-    for (; it.current(); ++it)
-        it.current()->deref();
+    QHashIterator<long,NodeListImpl::Cache*> it(m_nodeListCache);
+    while (it.hasNext())
+        it.next().value()->deref();
 
     if (m_loadingXMLDoc)
 	m_loadingXMLDoc->deref(this);
@@ -468,10 +478,13 @@ DocumentImpl::~DocumentImpl()
         m_hoverNode->deref();
     if (m_activeNode)
         m_activeNode->deref();
+    if (m_documentElement)
+        m_documentElement->deref();
+    qDeleteAll(m_counterDict);
 
     m_renderArena.reset();
 
-    KHTMLFactory::deref();
+    KHTMLGlobal::deregisterDocumentImpl(this);
 }
 
 
@@ -485,12 +498,25 @@ DOMImplementationImpl *DocumentImpl::implementation() const
     return m_implementation;
 }
 
+void DocumentImpl::childrenChanged()
+{
+    // invalidate the document element we have cached in case it was replaced
+    if (m_documentElement)
+        m_documentElement->deref();
+    m_documentElement = 0;
+}
+
 ElementImpl *DocumentImpl::documentElement() const
 {
-    NodeImpl *n = firstChild();
-    while (n && n->nodeType() != Node::ELEMENT_NODE)
-      n = n->nextSibling();
-    return static_cast<ElementImpl*>(n);
+    if (!m_documentElement) {
+        NodeImpl* n = firstChild();
+        while (n && n->nodeType() != Node::ELEMENT_NODE)
+            n = n->nextSibling();
+        m_documentElement = static_cast<ElementImpl*>(n);
+        if (m_documentElement)
+            m_documentElement->ref();
+    }
+    return m_documentElement;
 }
 
 ElementImpl *DocumentImpl::createElement( const DOMString &name, int* pExceptioncode )
@@ -617,7 +643,7 @@ NodeImpl *DocumentImpl::importNode(NodeImpl *importedNode, bool deep, int &excep
 	result = createDocumentFragment();
     else
 	exceptioncode = DOMException::NOT_SUPPORTED_ERR;
-	
+
     //### FIXME: This should handle Attributes, and a few other things
 
     if(deep && result)
@@ -983,6 +1009,9 @@ ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
     case ID_BR:
         n = new HTMLBRElementImpl(docPtr());
         break;
+    case ID_WBR:
+        n = new HTMLWBRElementImpl(docPtr());
+        break;
     case ID_Q:
         n = new HTMLGenericElementImpl(docPtr(), id);
         break;
@@ -1022,7 +1051,6 @@ ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
     case ID_SUP:
     case ID_SPAN:
     case ID_NOBR:
-    case ID_WBR:
     case ID_BDO:
     case ID_NOFRAMES:
         n = new HTMLGenericElementImpl(docPtr(), id);
@@ -1056,16 +1084,16 @@ QString DocumentImpl::nextState()
 QStringList DocumentImpl::docState()
 {
     QStringList s;
-    for (Q3PtrListIterator<NodeImpl> it(m_maintainsState); it.current(); ++it)
-        s.append(it.current()->state());
+    for (QListIterator<NodeImpl*> it(m_maintainsState); it.hasNext();)
+        s.append(it.next()->state());
 
     return s;
 }
 
 bool DocumentImpl::unsubmittedFormChanges()
 {
-    for (Q3PtrListIterator<NodeImpl> it(m_maintainsState); it.current(); ++it)
-        if (it.current()->state().endsWith('M'))
+    for (QListIterator<NodeImpl*> it(m_maintainsState); it.hasNext();)
+        if (it.next()->state().endsWith('M'))
             return true;
 
     return false;
@@ -1240,7 +1268,7 @@ void DocumentImpl::attach()
     m_styleSelector = new CSSStyleSelector( this, m_usersheet, m_styleSheets, m_url,
                                             !inCompatMode() );
     m_render = new (m_renderArena.get()) RenderCanvas(this, m_view);
-    m_styleSelector->computeFontSizes(m_paintDevice->logicalDpiY(), m_view ? m_view->part()->zoomFactor() : 100);
+    m_styleSelector->computeFontSizes(m_paintDevice->logicalDpiY(), m_view ? m_view->part()->fontScaleFactor() : 100);
     recalcStyle( Force );
 
     RenderObject* render = m_render;
@@ -1853,7 +1881,11 @@ NodeImpl::Id DocumentImpl::getId( NodeImpl::IdType _type, DOMStringImpl* _nsURI,
     bool cs = true; // case sensitive
     if (_type != NodeImpl::NamespaceId) {
         if (_nsURI)
-            nsid = getId( NodeImpl::NamespaceId, 0, 0, _nsURI, false, false, 0 ) << 16;
+            nsid = getId( NodeImpl::NamespaceId, 0, 0, _nsURI, false, false, 0 );
+
+        // for attributes empty and default namespaces are the same
+        if (_type == NodeImpl::AttributeId && nsid == emptyNamespace)
+            nsid = defaultNamespace;
 
         // Each document maintains a mapping of tag name -> id for every tag name encountered
         // in the document.
@@ -1863,12 +1895,12 @@ NodeImpl::Id DocumentImpl::getId( NodeImpl::IdType _type, DOMStringImpl* _nsURI,
         // xhtml is lower case - case sensitive, easy to implement
         if ( cs && (id = lookup(n.toAscii().constData(), _name->l)) ) {
             map->addAlias(_prefix, _name, cs, id);
-            return nsid + id;
+            return makeId(nsid, id);
         }
         // compatibility: upper case - case insensitive
         if ( !cs && (id = lookup(n.toLower().toAscii().constData(), _name->l )) ) {
             map->addAlias(_prefix, _name, cs, id);
-            return nsid + id;
+            return makeId(nsid, id);
         }
     }
 
@@ -1877,23 +1909,23 @@ NodeImpl::Id DocumentImpl::getId( NodeImpl::IdType _type, DOMStringImpl* _nsURI,
     QString name = cs ? n : n.toUpper();
 
     if (!_nsURI) {
-        id = (NodeImpl::Id)(long) map->ids.find( name );
+        id = (NodeImpl::Id)(long)map->ids.value( name );
         if (!id && _type != NodeImpl::NamespaceId) {
-            id = (NodeImpl::Id)(long) map->ids.find( "aliases: " + name );
+            id = (NodeImpl::Id)(long)map->ids.value( "aliases: " + name );
 	}
     } else {
-        id = (NodeImpl::Id)(long) map->ids.find( name );
+        id = (NodeImpl::Id)(long)map->ids.value( name );
         if (!readonly && id && _prefix && _prefix->l) {
             // we were called in registration mode... check if the alias exists
             const QString px = QString::fromRawData( _prefix->s, _prefix->l );
             QString qn("aliases: " + (cs ? px : px.toUpper()) + ":" + name);	//krazy:exclude=doublequote_chars DOM demands chars
-            if (!map->ids.find( qn )) {
+            if (!map->ids.contains( qn )) {
                 map->ids.insert( qn, (void*)id );
             }
         }
     }
 
-    if (id) return nsid + id;
+    if (id) return makeId(nsid, id);
 
     // unknown
     if (readonly) return 0;
@@ -1913,7 +1945,7 @@ NodeImpl::Id DocumentImpl::getId( NodeImpl::IdType _type, DOMStringImpl* _nsURI,
     // and register an alias if needed for DOM1 methods compatibility
     map->addAlias(_prefix, _name, cs, cid);
 
-    return nsid + cid;
+    return makeId(nsid, cid);
  }
 
 NodeImpl::Id DocumentImpl::getId( NodeImpl::IdType _type, DOMStringImpl *_nodeName, bool readonly, bool lookupHTML, int *pExceptioncode)
@@ -2013,7 +2045,7 @@ void DocumentImpl::removeStyleSheet(StyleSheetImpl *sheet, int *exceptioncode)
 
     if (m_addedStyleSheets) {
         bool in_main_list = !sheet->hasOneRef();
-        removed = m_addedStyleSheets->styleSheets.removeRef(sheet);
+        removed = m_addedStyleSheets->styleSheets.removeAll(sheet);
         sheet->deref();
 
         if (m_addedStyleSheets->styleSheets.count() == 0) {
@@ -2061,7 +2093,7 @@ void DocumentImpl::recalcStyleSelector()
 
     assert(m_pendingStylesheets==0);
 
-    Q3PtrList<StyleSheetImpl> oldStyleSheets = m_styleSheets->styleSheets;
+    QList<StyleSheetImpl*> oldStyleSheets = m_styleSheets->styleSheets;
     m_styleSheets->styleSheets.clear();
     QString sheetUsed = view() ? view()->part()->d->m_sheetUsed.replace("&&", "&") : QString();
     bool autoselect = sheetUsed.isEmpty();
@@ -2183,17 +2215,15 @@ void DocumentImpl::recalcStyleSelector()
 
     // Include programmatically added style sheets
     if (m_addedStyleSheets) {
-        Q3PtrListIterator<StyleSheetImpl> it = m_addedStyleSheets->styleSheets;
-        for (; *it; ++it) {
-            if ((*it)->isCSSStyleSheet() && !(*it)->disabled())
-                m_styleSheets->add(*it);
+        foreach (StyleSheetImpl* sh, m_addedStyleSheets->styleSheets) {
+            if (sh->isCSSStyleSheet() && !sh->disabled())
+                m_styleSheets->add(sh);
         }
     }
 
     // De-reference all the stylesheets in the old list
-    Q3PtrListIterator<StyleSheetImpl> it(oldStyleSheets);
-    for (; it.current(); ++it)
-	it.current()->deref();
+    foreach ( StyleSheetImpl* sh, oldStyleSheets)
+        sh->deref();
 
     rebuildStyleSelector();
 }
@@ -2211,9 +2241,9 @@ void DocumentImpl::rebuildStyleSelector()
     m_styleSelectorDirty = false;
 }
 
-void DocumentImpl::setBaseURL(const KUrl& _baseURL) 
-{ 
-    m_baseURL = _baseURL; 
+void DocumentImpl::setBaseURL(const KUrl& _baseURL)
+{
+    m_baseURL = _baseURL;
     if (m_elemSheet)
         m_elemSheet->setHref( baseURL().url() );
 }
@@ -2238,13 +2268,13 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
 {
     // don't process focus changes while detaching
     if( !m_render ) return;
-    
+
     // We do want to blur if a widget is being detached,
-    // but we don't want to emit events since that 
+    // but we don't want to emit events since that
     // triggers updateLayout() and may recurse detach()
     bool widgetDetach = m_focusNode && m_focusNode != this &&
               m_focusNode->renderer() && !m_focusNode->renderer()->parent();
-      
+
     // Make sure newFocusNode is actually in this document
     if (newFocusNode && (newFocusNode->getDocument() != this))
         return;
@@ -2259,7 +2289,7 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
                 oldFocusNode->setActive(false);
 
             oldFocusNode->setFocus(false);
-            
+
             if (!widgetDetach) {
                 oldFocusNode->dispatchHTMLEvent(EventImpl::BLUR_EVENT,false,false);
                 oldFocusNode->dispatchUIEvent(EventImpl::DOMFOCUSOUT_EVENT);
@@ -2327,14 +2357,16 @@ void DocumentImpl::attachNodeIterator(NodeIteratorImpl *ni)
 
 void DocumentImpl::detachNodeIterator(NodeIteratorImpl *ni)
 {
-    m_nodeIterators.remove(ni);
+    int i = m_nodeIterators.indexOf(ni);
+    if (i != -1)
+        m_nodeIterators.removeAt(i);
 }
 
 void DocumentImpl::notifyBeforeNodeRemoval(NodeImpl *n)
 {
-    Q3PtrListIterator<NodeIteratorImpl> it(m_nodeIterators);
-    for (; it.current(); ++it)
-        it.current()->notifyBeforeNodeRemoval(n);
+    QListIterator<NodeIteratorImpl*> it(m_nodeIterators);
+    while (it.hasNext())
+        it.next()->notifyBeforeNodeRemoval(n);
 }
 
 bool DocumentImpl::isURLAllowed(const QString& url) const
@@ -2344,7 +2376,7 @@ bool DocumentImpl::isURLAllowed(const QString& url) const
     KUrl newURL(completeURL(url));
     newURL.setRef(QString());
 
-    if (KHTMLFactory::defaultHTMLSettings()->isAdFiltered( newURL.url() ))
+    if (KHTMLGlobal::defaultHTMLSettings()->isAdFiltered( newURL.url() ))
         return false;
 
     // Prohibit non-file URLs if we are asked to.
@@ -2411,8 +2443,8 @@ CSSStyleDeclarationImpl *DocumentImpl::getOverrideStyle(ElementImpl* /*elt*/, DO
 void DocumentImpl::abort()
 {
     if (m_inSyncLoad) {
-	m_inSyncLoad = false;
-	qApp->exit_loop();
+        assert(m_inSyncLoad->isRunning());
+        m_inSyncLoad->exit();
     }
 
     if (m_loadingXMLDoc)
@@ -2423,8 +2455,8 @@ void DocumentImpl::abort()
 void DocumentImpl::load(const DOMString &uri)
 {
     if (m_inSyncLoad) {
-	m_inSyncLoad = false;
-	qApp->exit_loop();
+        assert(m_inSyncLoad->isRunning());
+        m_inSyncLoad->exit();
     }
 
     m_hadLoadError = false;
@@ -2449,8 +2481,13 @@ void DocumentImpl::load(const DOMString &uri)
     m_loadingXMLDoc->ref(this);
 
     if (!m_async && m_docLoading) {
-	m_inSyncLoad = true;
-	qApp->enter_loop();
+        assert(!m_inSyncLoad);
+	m_inSyncLoad = new QEventLoop();
+	m_inSyncLoad->exec();
+	// returning from event loop:
+	assert(!m_inSyncLoad->isRunning());
+	delete m_inSyncLoad;
+	m_inSyncLoad = 0;
     }
 }
 
@@ -2463,7 +2500,7 @@ void DocumentImpl::loadXML(const DOMString &source)
     dispatchHTMLEvent(EventImpl::LOAD_EVENT,false,false);
 }
 
-void DocumentImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &charset)
+void DocumentImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &/*charset*/)
 {
     if (!m_hadLoadError) {
 	m_url = url.string();
@@ -2472,8 +2509,8 @@ void DocumentImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString
 
     m_docLoading = false;
     if (m_inSyncLoad) {
-	m_inSyncLoad = false;
-	qApp->exit_loop();
+        assert(m_inSyncLoad->isRunning());
+        m_inSyncLoad->exit();
     }
 
     assert(m_loadingXMLDoc != 0);
@@ -2485,8 +2522,8 @@ void DocumentImpl::error(int err, const QString &text)
 {
     m_docLoading = false;
     if (m_inSyncLoad) {
-	m_inSyncLoad = false;
-	qApp->exit_loop();
+        assert(m_inSyncLoad->isRunning());
+        m_inSyncLoad->exit();
     }
 
     m_hadLoadError = true;
@@ -2509,27 +2546,29 @@ void DocumentImpl::error(int err, const QString &text)
 void DocumentImpl::defaultEventHandler(EventImpl *evt)
 {
     // if any html event listeners are registered on the window, then dispatch them here
-    if (!m_windowEventListeners.listeners || evt->propagationStopped())
-        return;
+    if (m_windowEventListeners.listeners && !evt->propagationStopped()) {
 
-    QList<RegisteredEventListener>::iterator it;
+        QList<RegisteredEventListener>::iterator it;
 
-    //Grab a copy in case of clear
-    QList<RegisteredEventListener> listeners = *m_windowEventListeners.listeners;
-    Event ev(evt);
-    for (it = listeners.begin(); it != listeners.end(); ++it) {
-        //Check to make sure it didn't get removed. KDE4: use Java-style iterators
-        if (!m_windowEventListeners.stillContainsListener(*it))
-            continue;
+        //Grab a copy in case of clear
+        QList<RegisteredEventListener> listeners = *m_windowEventListeners.listeners;
+        Event ev(evt);
+        for (it = listeners.begin(); it != listeners.end(); ++it) {
+            //Check to make sure it didn't get removed. KDE4: use Java-style iterators
+            if (!m_windowEventListeners.stillContainsListener(*it))
+                continue;
 
-        if ((*it).id == evt->id()) {
-            // currentTarget must be 0 in khtml for kjs_events to set "this" correctly.
-            // (this is how we identify events dispatched to the window, like window.onmousedown)
-            // ## currentTarget is unimplemented in IE, and is "window" in Mozilla (how? not a DOM node)
-            evt->setCurrentTarget(0);
-            (*it).listener->handleEvent(ev);
-	}
+            if ((*it).id == evt->id()) {
+                // currentTarget must be 0 in khtml for kjs_events to set "this" correctly.
+                // (this is how we identify events dispatched to the window, like window.onmousedown)
+                // ## currentTarget is unimplemented in IE, and is "window" in Mozilla (how? not a DOM node)
+                evt->setCurrentTarget(0);
+                (*it).listener->handleEvent(ev);
+	    }
+        }
     }
+    if ( evt->id() == EventImpl::KHTML_CONTENTLOADED_EVENT && !evt->propagationStopped() && !evt->defaultPrevented() )
+        contentLoaded();
 }
 
 void DocumentImpl::setHTMLWindowEventListener(int id, EventListener *listener)
@@ -2573,9 +2612,8 @@ void DocumentImpl::dispatchImageLoadEventSoon(HTMLImageElementImpl *image)
 void DocumentImpl::removeImage(HTMLImageElementImpl *image)
 {
     // Remove instances of this image from both lists.
-    // Use loops because we allow multiple instances to get into the lists.
-    while (m_imageLoadEventDispatchSoonList.removeRef(image)) { }
-    while (m_imageLoadEventDispatchingList.removeRef(image)) { }
+    m_imageLoadEventDispatchSoonList.removeAll(image);
+    m_imageLoadEventDispatchingList.removeAll(image);
     if (m_imageLoadEventDispatchSoonList.isEmpty() && m_imageLoadEventTimer) {
         killTimer(m_imageLoadEventTimer);
         m_imageLoadEventTimer = 0;
@@ -2598,15 +2636,8 @@ void DocumentImpl::dispatchImageLoadEventsNow()
 
     m_imageLoadEventDispatchingList = m_imageLoadEventDispatchSoonList;
     m_imageLoadEventDispatchSoonList.clear();
-    for (Q3PtrListIterator<HTMLImageElementImpl> it(m_imageLoadEventDispatchingList); it.current(); ) {
-        HTMLImageElementImpl* image = it.current();
-        // Must advance iterator *before* dispatching call.
-        // Otherwise, it might be advanced automatically if dispatching the call had a side effect
-        // of destroying the current HTMLImageElementImpl, and then we would advance past the *next*
-        // item, missing one altogether.
-        ++it;
-        image->dispatchLoadEvent();
-    }
+    for (QLinkedListIterator<HTMLImageElementImpl*> it(m_imageLoadEventDispatchingList); it.hasNext(); )
+        it.next()->dispatchLoadEvent();
     m_imageLoadEventDispatchingList.clear();
 }
 
@@ -2702,7 +2733,7 @@ NodeListImpl::Cache* DOM::DocumentImpl::acquireCachedNodeListInfo(
 
     //Check to see if we have this sort of item cached.
     NodeListImpl::Cache* cached =
-        (type == NodeListImpl::UNCACHEABLE) ? 0 : m_nodeListCache.find(key.hash());
+        (type == NodeListImpl::UNCACHEABLE) ? 0 : m_nodeListCache.value(key.hash());
 
     if (cached) {
         if (cached->key == key) {
@@ -2722,7 +2753,7 @@ NodeListImpl::Cache* DOM::DocumentImpl::acquireCachedNodeListInfo(
 
     if (type != NodeListImpl::UNCACHEABLE) {
         newInfo->ref(); //Add the cache's reference
-        m_nodeListCache.replace(key.hash(), newInfo);
+        m_nodeListCache.insert(key.hash(), newInfo);
     }
 
     return newInfo;
