@@ -52,14 +52,15 @@
 #include <kbuildsycocaprogressdialog.h>
 #include <kconfiggroup.h>
 
-template <> inline
-void KConfigGroup::writeEntry( const QByteArray &key,
-                              const KGlobalSettings::Completion& aValue,
-                              KConfigBase::WriteConfigFlags flags)
+inline void writeEntry( KConfigGroup& group, const char* key,
+                        const KGlobalSettings::Completion& aValue,
+                        KConfigBase::WriteConfigFlags flags = KConfigBase::Normal )
 {
-  writeEntry(key, int(aValue), flags);
+    group.writeEntry(key, int(aValue), flags);
 }
 
+namespace KDEPrivate {
+
 class AppNode
 {
 public:
@@ -84,7 +85,7 @@ public:
     QList<AppNode*> children;
 };
 
-static bool AppNodeLessThan(AppNode *n1, AppNode *n2)
+bool AppNodeLessThan(KDEPrivate::AppNode *n1, KDEPrivate::AppNode *n2)
 {
     if (n1->isDir) {
         if (n2->isDir) {
@@ -102,12 +103,14 @@ static bool AppNodeLessThan(AppNode *n1, AppNode *n2)
     return true;
 }
 
+}
+
 
 class KApplicationModelPrivate
 {
 public:
     KApplicationModelPrivate(KApplicationModel *qq)
-        : q(qq), root(new AppNode())
+        : q(qq), root(new KDEPrivate::AppNode())
     {
     }
     ~KApplicationModelPrivate()
@@ -115,14 +118,14 @@ public:
         delete root;
     }
 
-    void fillNode(const QString &relPath, AppNode *node);
+    void fillNode(const QString &relPath, KDEPrivate::AppNode *node);
 
     KApplicationModel *q;
 
-    AppNode *root;
+    KDEPrivate::AppNode *root;
 };
 
-void KApplicationModelPrivate::fillNode(const QString &_relPath, AppNode *node)
+void KApplicationModelPrivate::fillNode(const QString &_relPath, KDEPrivate::AppNode *node)
 {
    KServiceGroup::Ptr root = KServiceGroup::group(_relPath);
    if (!root || !root->isValid()) return;
@@ -167,7 +170,7 @@ void KApplicationModelPrivate::fillNode(const QString &_relPath, AppNode *node)
          continue;
       }
 
-      AppNode *newnode = new AppNode();
+      KDEPrivate::AppNode *newnode = new KDEPrivate::AppNode();
       newnode->icon = icon;
       newnode->text = text;
       newnode->relPath = relPath;
@@ -176,7 +179,7 @@ void KApplicationModelPrivate::fillNode(const QString &_relPath, AppNode *node)
       newnode->parent = node;
       node->children.append(newnode);
    }
-   qStableSort(node->children.begin(), node->children.end(), AppNodeLessThan);
+   qStableSort(node->children.begin(), node->children.end(), KDEPrivate::AppNodeLessThan);
 }
 
 
@@ -197,7 +200,7 @@ bool KApplicationModel::canFetchMore(const QModelIndex &parent) const
     if (!parent.isValid())
         return false;
 
-    AppNode *node = static_cast<AppNode*>(parent.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(parent.internalPointer());
     return node->isDir && !node->fetched;
 }
 
@@ -212,7 +215,7 @@ QVariant KApplicationModel::data(const QModelIndex &index, int role) const
     if (!index.isValid())
         return QVariant();
 
-    AppNode *node = static_cast<AppNode*>(index.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(index.internalPointer());
 
     switch (role) {
     case Qt::DisplayRole:
@@ -234,7 +237,7 @@ void KApplicationModel::fetchMore(const QModelIndex &parent)
     if (!parent.isValid())
         return;
 
-    AppNode *node = static_cast<AppNode*>(parent.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(parent.internalPointer());
     if (!node->isDir)
         return;
 
@@ -249,7 +252,7 @@ bool KApplicationModel::hasChildren(const QModelIndex &parent) const
     if (!parent.isValid())
         return true;
 
-    AppNode *node = static_cast<AppNode*>(parent.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(parent.internalPointer());
     return node->isDir;
 }
 
@@ -272,9 +275,9 @@ QModelIndex KApplicationModel::index(int row, int column, const QModelIndex &par
     if (row < 0 || column != 0)
         return QModelIndex();
 
-    AppNode *node = d->root;
+    KDEPrivate::AppNode *node = d->root;
     if (parent.isValid())
-        node = static_cast<AppNode*>(parent.internalPointer());
+        node = static_cast<KDEPrivate::AppNode*>(parent.internalPointer());
 
     if (row >= node->children.count())
         return QModelIndex();
@@ -287,7 +290,7 @@ QModelIndex KApplicationModel::parent(const QModelIndex &index) const
     if (!index.isValid())
         return QModelIndex();
 
-    AppNode *node = static_cast<AppNode*>(index.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(index.internalPointer());
     if (node->parent->parent) {
         int id = node->parent->parent->children.indexOf(node->parent);
 
@@ -305,7 +308,7 @@ int KApplicationModel::rowCount(const QModelIndex &parent) const
     if (!parent.isValid())
         return d->root->children.count();
 
-    AppNode *node = static_cast<AppNode*>(parent.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(parent.internalPointer());
     return node->children.count();
 }
 
@@ -314,7 +317,7 @@ QString KApplicationModel::nameFor(const QModelIndex &index) const
     if (!index.isValid())
         return QString();
 
-    AppNode *node = static_cast<AppNode*>(index.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(index.internalPointer());
     return node->text;
 }
 
@@ -323,7 +326,7 @@ QString KApplicationModel::execFor(const QModelIndex &index) const
     if (!index.isValid())
         return QString();
 
-    AppNode *node = static_cast<AppNode*>(index.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(index.internalPointer());
     return node->exec;
 }
 
@@ -332,7 +335,7 @@ bool KApplicationModel::isDirectory(const QModelIndex &index) const
     if (!index.isValid())
         return false;
 
-    AppNode *node = static_cast<AppNode*>(index.internalPointer());
+    KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(index.internalPointer());
     return node->isDir;
 }
 
@@ -559,17 +562,16 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
   {
     // init the history combo and insert it into the URL-Requester
     KHistoryComboBox *combo = new KHistoryComboBox();
-    KLineEdit *lineEdit = new KLineEdit();
+    KLineEdit *lineEdit = new KLineEdit(q);
     lineEdit->setClearButtonShown(true);
     combo->setLineEdit(lineEdit);
     combo->setDuplicatesEnabled( false );
     KConfigGroup cg( KGlobal::config(), QString::fromLatin1("Open-with settings") );
-    int max = cg.readEntry( QString::fromLatin1("Maximum history"), 15 );
+    int max = cg.readEntry( "Maximum history", 15 );
     combo->setMaxCount( max );
-    int mode = cg.readEntry(QString::fromLatin1("CompletionMode"),
-				int(KGlobalSettings::completionMode()));
+    int mode = cg.readEntry( "CompletionMode", int(KGlobalSettings::completionMode()));
     combo->setCompletionMode((KGlobalSettings::Completion)mode);
-    QStringList list = cg.readEntry( QString::fromLatin1("History"), QStringList() );
+    QStringList list = cg.readEntry( "History", QStringList() );
     combo->setHistoryItems( list, true );
     edit = new KUrlRequester( combo, mainWidget );
   }
@@ -580,7 +582,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     edit->button()->hide();
   }
 
-  edit->setUrl( KUrl(_value) );
+  edit->setPath( _value );
   edit->setWhatsThis(i18n(
     "Following the command, you can have several place holders which will be replaced "
     "with the actual values when the actual program is run:\n"
@@ -677,7 +679,7 @@ void KOpenWithDialog::slotSelected( const QString& /*_name*/, const QString& _ex
 {
     kDebug(250)<<"KOpenWithDialog::slotSelected";
     KService::Ptr pService = d->curService;
-    d->edit->setUrl(KUrl(_exec)); // calls slotTextChanged :(
+    d->edit->setPath(_exec); // calls slotTextChanged :(
     d->curService = pService;
 }
 
@@ -875,7 +877,7 @@ void KOpenWithDialogPrivate::_k_slotOK()
      desktop = new KDesktopFile(newPath);
   }
   KConfigGroup cg = desktop->desktopGroup();
-  cg.writeEntry("Type", QString::fromLatin1("Application"));
+  cg.writeEntry("Type", "Application");
   cg.writeEntry("Name", initialServiceName);
   cg.writePathEntry("Exec", fullExec);
   if (terminal->isChecked())
@@ -892,31 +894,29 @@ void KOpenWithDialogPrivate::_k_slotOK()
   cg.writeEntry("InitialPreference", maxPreference + 1);
 
 
-    if (bRemember || saveNewApps)
-  {
-    QStringList mimeList = cg.readEntry("MimeType", QStringList(), ';');
-    if (!qMimeType.isEmpty() && !mimeList.contains(qMimeType))
-      mimeList.append(qMimeType);
-    cg.writeEntry("MimeType", mimeList, ';');
+    if (bRemember || saveNewApps) {
+        // Add mimetype to the service desktop file
+        QStringList mimeList = cg.readXdgListEntry("MimeType");
+        if (!qMimeType.isEmpty() && !mimeList.contains(qMimeType))
+            mimeList.append(qMimeType);
+        cg.writeXdgListEntry("MimeType", mimeList);
 
-    if ( !qMimeType.isEmpty() )
-    {
-      // Also make sure the "auto embed" setting for this mimetype is off
-      KDesktopFile mimeDesktop( KStandardDirs::locateLocal( "mime", qMimeType + ".desktop" ) );
-      mimeDesktop.desktopGroup().writeEntry( "X-KDE-AutoEmbed", false );
-      mimeDesktop.sync();
+        if ( !qMimeType.isEmpty() ) {
+            // Also make sure the "auto embed" setting for this mimetype is off
+            KSharedConfig::Ptr fileTypesConfig = KSharedConfig::openConfig("filetypesrc", KConfig::NoGlobals);
+            fileTypesConfig->group("EmbedSettings").writeEntry(QString("embed-")+qMimeType, false);
+            fileTypesConfig->sync();
+        }
     }
-  }
 
-  // write it all out to the file
-  cg.sync();
-  delete desktop;
+    // write it all out to the file
+    cg.sync();
+    delete desktop;
 
     KBuildSycocaProgressDialog::rebuildKSycoca(q);
 
-  m_pService = KService::serviceByMenuId( menuId );
-
-  Q_ASSERT( m_pService );
+    m_pService = KService::serviceByMenuId( menuId );
+    Q_ASSERT( m_pService );
 
     q->accept();
 }
@@ -954,8 +954,8 @@ void KOpenWithDialog::accept()
         combo->addToHistory(d->edit->url().url());
 
         KConfigGroup cg( KGlobal::config(), QString::fromLatin1("Open-with settings") );
-        cg.writeEntry( QString::fromLatin1("History"), combo->historyItems() );
-        cg.writeEntry(QString::fromLatin1("CompletionMode"), combo->completionMode());
+        cg.writeEntry( "History", combo->historyItems() );
+        writeEntry( cg, "CompletionMode", combo->completionMode() );
         // don't store the completion-list, as it contains all of KUrlCompletion's
         // executables
         cg.sync();
