@@ -131,45 +131,42 @@ KIO_EXPORT QString KIO::itemsSummaryString(uint items, uint files, uint dirs, KI
 
 KIO_EXPORT QString KIO::encodeFileName( const QString & _str )
 {
-  QString str( _str );
+    QString str( _str );
 
-  int i = 0;
-  while ( ( i = str.indexOf( "%", i ) ) != -1 )
-  {
-    str.replace( i, 1, "%%");
-    i += 2;
-  }
-  while ( ( i = str.indexOf( "/" ) ) != -1 )
-      str.replace( i, 1, "%2f");
-  return str;
+    int i = 0;
+    while ( ( i = str.indexOf( '%', i ) ) != -1 ) {
+        str.replace( i, 1, "%%");
+        i += 2;
+    }
+    while ( ( i = str.indexOf( '/' ) ) != -1 )
+        str.replace( i, 1, "%2f");
+    return str;
 }
 
 KIO_EXPORT QString KIO::decodeFileName( const QString & _str )
 {
-  QString str;
+    const int len = _str.length();
+    QString text;
+    text.reserve(len);
+    for ( int i = 0; i < len ; ++i ) {
+        if ( _str[i] == '%' && i+1 < len ) {
+            const QChar nextChar = _str[i+1];
+            if ( nextChar == '%' ) { // %% -> %
+                text.append(QLatin1Char('%'));
+                ++i;
+            } else if ( nextChar == '2' && (i+2<len) && _str[i+2].toLower()=='f' ) { // %2f -> /
+                text.append(QLatin1Char('/'));
+                i += 2;
+            } else {
+                text.append(QLatin1Char('%'));
+            }
+        } else {
+            text.append(_str[i]);
+        }
+    }
+    text.squeeze();
 
-  int i = 0;
-  for ( ; i < _str.length() ; ++i )
-  {
-    if ( _str[i]=='%' )
-    {
-      if ( _str[i+1]=='%' ) // %% -> %
-      {
-        str.append(QLatin1Char('%'));
-        ++i;
-      }
-      else if ( _str[i+1]=='2' && (i+2<_str.length()) && _str[i+2].toLower()=='f' ) // %2f -> /
-      {
-        str.append(QLatin1Char('/'));
-        i += 2;
-      }
-      else
-        str.append(QLatin1Char('%'));
-    } else
-      str.append(_str[i]);
-  }
-
-  return str;
+    return text;
 }
 
 KIO_EXPORT QString KIO::Job::errorString() const
@@ -452,30 +449,34 @@ KIO_EXPORT QStringList KIO::Job::detailedErrorStrings( const KUrl *reqUrl /*= 0L
                                                 KLocale::LongDate );
 
   ret << errorName;
-  ret << QLatin1String( "<qt><p><b>" ) + errorName +
-         QLatin1String( "</b></p><p>" ) + description +
-         QLatin1String( "</p>" );
-  ret2 = QLatin1String( "<qt><p>" );
+  ret << i18nc( "@info %1 error name, %2 description",
+                "<qt><p><b>%1</b></p><p>%2</p></qt>", errorName, description);
+
+  ret2 = QLatin1String( "<qt>" );
   if ( !techName.isEmpty() )
-    ret2 += i18n( "<b>Technical reason</b>: " ) + techName + QLatin1String( "</p>" );
-  ret2 += i18n( "</p><p><b>Details of the request</b>:" );
-  ret2 += i18n( "</p><ul><li>URL: %1</li>" ,  url );
+    ret2 += QLatin1String( "<p>" ) + i18n( "<b>Technical reason</b>: " ) +
+            techName + QLatin1String( "</p>" );
+  ret2 += QLatin1String( "<p>" ) + i18n( "<b>Details of the request</b>:" ) +
+          QLatin1String( "</p><ul>" ) + i18n( "<li>URL: %1</li>", url );
   if ( !protocol.isEmpty() ) {
-    ret2 += i18n( "<li>Protocol: %1</li>" ,  protocol );
+    ret2 += i18n( "<li>Protocol: %1</li>" , protocol );
   }
-  ret2 += i18n( "<li>Date and time: %1</li>" ,  datetime );
-  ret2 += i18n( "<li>Additional information: %1</li></ul>" ,  errorText() );
+  ret2 += i18n( "<li>Date and time: %1</li>", datetime ) +
+          i18n( "<li>Additional information: %1</li>" ,  errorText() ) +
+          QLatin1String( "</ul>" );
   if ( !causes.isEmpty() ) {
-    ret2 += i18n( "<p><b>Possible causes</b>:</p><ul><li>" );
-    ret2 += causes.join( "</li><li>" );
-    ret2 += QLatin1String( "</li></ul>" );
+    ret2 += QLatin1String( "<p>" ) + i18n( "<b>Possible causes</b>:" ) +
+            QLatin1String( "</p><ul><li>" ) + causes.join( "</li><li>" ) +
+            QLatin1String( "</li></ul>" );
   }
   if ( !solutions.isEmpty() ) {
-    ret2 += i18n( "<p><b>Possible solutions</b>:</p><ul><li>" );
-    ret2 += solutions.join( "</li><li>" );
-    ret2 += QLatin1String( "</li></ul>" );
+    ret2 += QLatin1String( "<p>" ) + i18n( "<b>Possible solutions</b>:" ) +
+            QLatin1String( "</p><ul><li>" ) + solutions.join( "</li><li>" ) +
+            QLatin1String( "</li></ul>" );
   }
+  ret2 += QLatin1String( "</qt>" );
   ret << ret2;
+
   return ret;
 }
 
