@@ -151,13 +151,20 @@ QPair<int /*row*/, KDirModelNode*> KDirModelPrivate::nodeForUrl(const KUrl& _url
     url.adjustPath(KUrl::RemoveTrailingSlash);
 
     //kDebug(7008) << url;
+    KUrl nodeUrl = m_dirLister->url();
+    // For a URL without a path, like "applications:" or "settings://",
+    // we want to resolve here "no path" to "/ assumed".
+    // We don't do it before (e.g. in KDirLister) because we want to
+    // give the ioslave a chance for a redirect (e.g. kio_ftp redirects "no path"
+    // to the user's home dir)
+    if (nodeUrl.path().isEmpty())
+        nodeUrl.setPath("/");
 
-    if (url == m_dirLister->url())
+    if (url == nodeUrl)
         return qMakePair(0, static_cast<KDirModelNode *>(m_rootNode));
 
     const QString urlStr = url.url();
     KDirModelDirNode* dirNode = m_rootNode;
-    KUrl nodeUrl = m_dirLister->url();
 
     if ( !urlStr.startsWith(nodeUrl.url()) ) {
         return qMakePair(0, static_cast<KDirModelNode*>(0));
@@ -310,6 +317,8 @@ void KDirModelPrivate::_k_slotNewItems(const KFileItemList& items)
     const KUrl::List urlsBeingFetched = m_urlsBeingFetched.value(dirNode);
     //kDebug(7008) << "urlsBeingFetched for dir" << dirNode << dir << ":" << urlsBeingFetched;
 
+    QList<QModelIndex> emitExpandFor;
+
     KFileItemList::const_iterator it = items.begin();
     KFileItemList::const_iterator end = items.end();
     for ( ; it != end ; ++it ) {
@@ -326,7 +335,7 @@ void KDirModelPrivate::_k_slotNewItems(const KFileItemList& items)
                     //kDebug(7008) << "Listing found" << dirUrl << "which is a parent of fetched url" << urlFetched;
                     const QModelIndex parentIndex = indexForNode(node, dirNode->m_childNodes.count()-1);
                     Q_ASSERT(parentIndex.isValid());
-                    emit q->expand(parentIndex);
+                    emitExpandFor.append(parentIndex);
                     if (dirUrl != urlFetched) {
                         q->fetchMore(parentIndex);
                         m_urlsBeingFetched[node].append(urlFetched);
@@ -339,6 +348,12 @@ void KDirModelPrivate::_k_slotNewItems(const KFileItemList& items)
     m_urlsBeingFetched.remove(dirNode);
 
     q->endInsertRows();
+
+    // Emit expand signal after rowsInserted signal has been emitted,
+    // so that any proxy model will have updated its mapping already
+    Q_FOREACH(const QModelIndex& idx, emitExpandFor) {
+        emit q->expand(idx);
+    }
 }
 
 void KDirModelPrivate::_k_slotDeleteItem(const KFileItem& item)
@@ -350,12 +365,12 @@ void KDirModelPrivate::_k_slotDeleteItem(const KFileItem& item)
     const QPair<int, KDirModelNode*> result = nodeForUrl(item.url()); // O(n*m)
     const int rowNumber = result.first;
     KDirModelNode* node = result.second;
-    Q_ASSERT(node);
     if (!node)
         return;
 
     KDirModelDirNode* dirNode = node->parent();
-    Q_ASSERT(dirNode);
+    if (!dirNode)
+        return;
 
     QModelIndex parentIndex = indexForNode(dirNode); // O(n)
     q->beginRemoveRows( parentIndex, rowNumber, rowNumber );
@@ -416,7 +431,7 @@ QVariant KDirModel::data( const QModelIndex & index, int role ) const
         case Qt::DisplayRole:
             switch (index.column()) {
             case Name:
-                return item.name();
+                return item.text();
             case Size:
                 //
                 //return KIO::convertSize(item->size());
@@ -450,7 +465,8 @@ QVariant KDirModel::data( const QModelIndex & index, int role ) const
         case Qt::TextAlignmentRole:
             if (index.column() == Size) {
                 // use a right alignment for L2R and R2L languages
-                return Qt::AlignRight;
+                const Qt::Alignment alignment = Qt::AlignRight | Qt::AlignVCenter;
+                return int(alignment);
             }
             break;
         case FileItemRole:
@@ -611,7 +627,7 @@ QVariant KDirModel::headerData( int section, Qt::Orientation orientation, int ro
         case Size:
             return i18nc("@title:column","Size");
         case ModifiedTime:
-            return i18nc("@title:column","Modified"); // was called "Date" in kde3's kfiledialog - which one is better?
+            return i18nc("@title:column","Date");
         case Permissions:
             return i18nc("@title:column","Permissions");
         case Owner:
@@ -644,25 +660,33 @@ Qt::ItemFlags KDirModel::flags( const QModelIndex & index ) const
 
     // Allow dropping onto this item?
     if (d->m_dropsAllowed != NoDrops) {
-        KFileItem item = itemForIndex(index);
-        if (item.isDir()) {
+        if(!index.isValid()) {
             if (d->m_dropsAllowed & DropOnDirectory) {
                 f |= Qt::ItemIsDropEnabled;
             }
-        } else { // regular file item
-            if (d->m_dropsAllowed & DropOnAnyFile)
-                f |= Qt::ItemIsDropEnabled;
-            else if (d->m_dropsAllowed & DropOnLocalExecutable) {
-                if (item.isLocalFile()) {
-                    // Desktop file?
-                    if (item.mimeTypePtr()->is("application/x-desktop"))
-                        f |= Qt::ItemIsDropEnabled;
-                    // Executable, shell script ... ?
-                    else if ( QFileInfo( item.localPath() ).isExecutable() )
-                        f |= Qt::ItemIsDropEnabled;
+        } else {
+            KFileItem item = itemForIndex(index);
+            if (item.isNull()) {
+                kWarning(7007) << "Invalid item returned for index";
+            } else if (item.isDir()) {
+                if (d->m_dropsAllowed & DropOnDirectory) {
+                    f |= Qt::ItemIsDropEnabled;
+                }
+            } else { // regular file item
+                if (d->m_dropsAllowed & DropOnAnyFile)
+                    f |= Qt::ItemIsDropEnabled;
+                else if (d->m_dropsAllowed & DropOnLocalExecutable) {
+                    if (item.isLocalFile()) {
+                        // Desktop file?
+                        if (item.mimeTypePtr()->is("application/x-desktop"))
+                            f |= Qt::ItemIsDropEnabled;
+                        // Executable, shell script ... ?
+                        else if ( QFileInfo( item.localPath() ).isExecutable() )
+                            f |= Qt::ItemIsDropEnabled;
+                    }
                 }
             }
-        }
+	}
     }
 
     return f;
@@ -724,18 +748,21 @@ void KDirModel::setDropsAllowed(DropsAllowed dropsAllowed)
 void KDirModel::expandToUrl(const KUrl& url)
 {
     const QPair<int, KDirModelNode*> result = d->nodeForUrl(url, true /*return last parent*/); // O(n*m)
+
     if (!result.second) // doesn't seem related to our base url?
         return;
-    if (result.second == d->m_rootNode) // this -is- our base url, nothing to do
-        return;
     if (!(result.second->item().isNull()) && result.second->item().url() == url) {
         // We have it already, nothing to do
         kDebug(7008) << "have it already item=" <<url /*result.second->item()*/;
         return;
     }
-    //kDebug(7008) << k_funcinfo << url << result.second << result.second->item().url() << endl;
+
     d->m_urlsBeingFetched[result.second].append(url);
 
+    if (result.second == d->m_rootNode) // the root is fetched by default, so it must be currently being fetched
+        return;
+
+    // start a new fetch to look for the next level down the URL
     const QModelIndex parentIndex = d->indexForNode(result.second, result.first);
     Q_ASSERT(parentIndex.isValid());
     fetchMore(parentIndex);
