@@ -417,7 +417,11 @@ QString KFileItemDelegate::Private::elidedText(QTextLayout &layout, const QStyle
             break;
         }
         else if (line.naturalTextWidth() > maxWidth)
+        {
             elided += metrics.elidedText(text.mid(start, length), option.textElideMode, maxWidth);
+            if (!elided.endsWith(QChar::LineSeparator))
+                elided += QChar::LineSeparator;
+        }
         else
             elided += text.mid(start, length);
 
@@ -626,7 +630,7 @@ QBrush KFileItemDelegate::Private::backgroundBrush(const QStyleOptionViewItem &o
     if ((option.state & QStyle::State_MouseOver) && index.column() == KDirModel::Name)
     {
         // Use a lighter version of the highlight color with 1/3 opacity
-        QColor hover = option.palette.color(QPalette::Highlight).light();
+        QColor hover = option.palette.color(QPalette::Highlight);
         hover.setAlpha(88);
 
         background = composite(hover, background);
@@ -842,13 +846,11 @@ void KFileItemDelegate::Private::drawTextItems(QPainter *painter, const QStyleOp
 
     if (!infoLayout.text().isEmpty())
     {
-        QColor color;
-        if (option.state & QStyle::State_Selected)
-        {
-            color = option.palette.color(QPalette::HighlightedText);
-            color.setAlphaF(.5);
-        } else
-            color = option.palette.color(QPalette::Highlight);
+        // TODO - for apps not doing funny things with the color palette,
+        // KColorScheme::InactiveText would be a much more correct choice. We
+        // should provide an API to specify what color to use for information.
+        QColor color = pen.color();
+        color.setAlphaF(0.6);
 
         painter->setPen(color);
         infoLayout.draw(painter, QPoint());
@@ -903,12 +905,17 @@ KFileItemDelegate::KFileItemDelegate(QObject *parent)
     int focusVMargin = QApplication::style()->pixelMetric(QStyle::PM_FocusFrameVMargin);
 
     // Margins for horizontal mode (list views, tree views, table views)
-    d->setHorizontalMargin(Private::TextMargin, focusHMargin, focusVMargin);
+    const int textMargin = focusHMargin * 4;
+    if (QApplication::isRightToLeft())
+        d->setHorizontalMargin(Private::TextMargin, textMargin, focusVMargin, focusHMargin, focusVMargin);
+    else
+        d->setHorizontalMargin(Private::TextMargin, focusHMargin, focusVMargin, textMargin, focusVMargin);
+
     d->setHorizontalMargin(Private::IconMargin, focusHMargin, focusVMargin);
     d->setHorizontalMargin(Private::ItemMargin, 0, 0);
 
     // Margins for vertical mode (icon views)
-    d->setVerticalMargin(Private::TextMargin, 6, 2);
+    d->setVerticalMargin(Private::TextMargin, 2, 2);
     d->setVerticalMargin(Private::IconMargin, focusHMargin, focusVMargin);
     d->setVerticalMargin(Private::ItemMargin, 0, 0);
 
