@@ -218,7 +218,7 @@ bool KRun::displayOpenWithDialog( const KUrl::List& lst, QWidget* window, bool t
     if ( l.exec() )
     {
       KService::Ptr service = l.service();
-      if ( !!service )
+      if ( service )
         return KRun::run( *service, lst, window, tempFiles, suggestedFileName, asn );
 
       kDebug(7010) << "No service set, running " << l.text();
@@ -308,7 +308,8 @@ KRunMX2::subst( int option, const KUrl &url, QStringList &ret )
 {
    switch( option ) {
    case 'u':
-      ret << url.pathOrUrl();
+      ret << ((url.isLocalFile() && url.fragment().isNull() && url.encodedQuery().isNull()) ?
+                 url.toLocalFile()  : url.url());
       break;
    case 'd':
       ret << url.directory();
@@ -376,15 +377,19 @@ QStringList KRun::processDesktopExec(const KService &_service, const KUrl::List&
   KRunMX1 mx1( _service );
   KRunMX2 mx2( _urls );
 
-  if( !mx1.expandMacrosShellQuote( exec ) )
-    goto synerr; // error in shell syntax
+  if( !mx1.expandMacrosShellQuote( exec ) ) { // Error in shell syntax
+    kWarning() << "KRun: syntax error in command" << _service.exec() << ", service" << _service.name();
+    return QStringList();
+  }
 
   // FIXME: the current way of invoking kioexec disables term and su use
 
   // Check if we need "tempexec" (kioexec in fact)
   appHasTempFileOption = tempFiles && _service.property("X-KDE-HasTempFileOption").toBool();
   if( tempFiles && !appHasTempFileOption && _urls.size() ) {
-    result << "kioexec" << "--tempfiles" << exec;
+    const QString kioexec = KStandardDirs::findExe("kioexec");
+    Q_ASSERT(!kioexec.isEmpty());
+    result << kioexec << "--tempfiles" << exec;
     if ( !suggestedFileName.isEmpty() ) {
         result << "--suggestedfilename";
         result << suggestedFileName;
@@ -398,7 +403,9 @@ QStringList KRun::processDesktopExec(const KService &_service, const KUrl::List&
     for( KUrl::List::ConstIterator it = _urls.begin(); it != _urls.end(); ++it )
       if ( !(*it).isLocalFile() && !KProtocolInfo::isHelperProtocol(*it) ) {
         // We need to run the app through kioexec
-        result << "kioexec";
+        const QString kioexec = KStandardDirs::findExe("kioexec");
+        Q_ASSERT(!kioexec.isEmpty());
+        result << kioexec;
         if ( tempFiles )
             result << "--tempfiles";
         if ( !suggestedFileName.isEmpty() ) {
@@ -458,39 +465,33 @@ QStringList KRun::processDesktopExec(const KService &_service, const KUrl::List&
   }
 
   KShell::Errors err;
+  QStringList execlist = KShell::splitArgs(exec, KShell::AbortOnMeta | KShell::TildeExpand, &err);
+  if (err == KShell::NoError && !execlist.isEmpty()) { // mx1 checked for syntax errors already
+    // Resolve the executable to ensure that helpers in lib/kde4/libexec/ are found.
+    // Too bad for commands that need a shell - they must reside in $PATH.
+    const QString exePath = KStandardDirs::findExe(execlist[0]);
+    if (!exePath.isEmpty())
+      execlist[0] = exePath;
+  }
   if (_service.substituteUid()) {
     if (_service.terminal())
       result << "su";
     else
-      result << "kdesu" << "-u";
+      result << KStandardDirs::findExe("kdesu") << "-u";
     result << _service.username() << "-c";
-    KShell::splitArgs(exec, KShell::AbortOnMeta | KShell::TildeExpand, &err);
-    if (err == KShell::FoundMeta) {
+    if (err == KShell::FoundMeta)
       exec = "/bin/sh -c " + KShell::quoteArg(exec);
-    } else if (err != KShell::NoError)
-      goto synerr;
+    else
+      exec = KShell::joinArgs(execlist);
     result << exec;
   } else {
-    result += KShell::splitArgs(exec, KShell::AbortOnMeta | KShell::TildeExpand, &err);
     if (err == KShell::FoundMeta)
       result << "/bin/sh" << "-c" << exec;
-    else if (err != KShell::NoError)
-      goto synerr;
-    else {
-      if (!result.isEmpty()) {
-        // resolve the executable according to the path, which also makes it possible to find kde4/libexec helpers.
-        const QString exePath = KStandardDirs::findExe(result[0]);
-        if (!exePath.isEmpty())
-          result[0] = exePath;
-      }
-    }
+    else
+      result += execlist;
   }
 
   return result;
-
- synerr:
-  kWarning() << "KRun: syntax error in command `" << _service.exec() << "', service `" << _service.name() << "'";
-  return QStringList();
 }
 
 //static
@@ -633,6 +634,10 @@ static bool runTempService( const KService& _service, const KUrl::List& _urls, Q
   {
       args = KRun::processDesktopExec(_service, _urls, tempFiles, suggestedFileName );
   }
+  if (args.isEmpty()) {
+      KMessageBox::sorry(window, i18n("Error processing Exec field in %1", _service.entryPath()) );
+      return false;
+  }
   kDebug(7010) << "runTempService: KProcess args=" << args;
 
   KProcess * proc = new KProcess;
@@ -770,7 +775,8 @@ bool KRun::run( const QString& _exec, const KUrl::List& _urls, QWidget* window,
 
 bool KRun::runCommand( const QString &cmd, QWidget* window )
 {
-  return KRun::runCommand( cmd, QString(), QString(), window, "" );
+  QString bin = binaryName( cmd, true );
+  return KRun::runCommand( cmd, bin, bin, window, "" );
 }
 
 bool KRun::runCommand( const QString& cmd, const QString &execName, const QString & iconName, QWidget* window, const QByteArray& asn )
@@ -942,7 +948,7 @@ void KRun::init()
   kDebug(7010) << "Testing directory (stating)";
 
   // It may be a directory or a file, let's stat
-  KIO::JobFlags flags = d->m_bProgressInfo ? KIO::HideProgressInfo : KIO::DefaultFlags;
+  KIO::JobFlags flags = d->m_bProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo;
   KIO::StatJob *job = KIO::stat( d->m_strURL, KIO::StatJob::SourceSide, 0 /* no details */, flags );
   job->ui()->setWindow (d->m_window);
   connect( job, SIGNAL( result( KJob * ) ),
@@ -992,7 +998,7 @@ void KRun::scanFile()
   }
   kDebug(7010) << this << " Scanning file " << d->m_strURL.url();
 
-  KIO::JobFlags flags = d->m_bProgressInfo ? KIO::HideProgressInfo : KIO::DefaultFlags;
+  KIO::JobFlags flags = d->m_bProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo;
   KIO::TransferJob *job = KIO::get( d->m_strURL, KIO::NoReload /*reload*/, flags );
   job->ui()->setWindow (d->m_window);
   connect(job, SIGNAL( result(KJob *)),
