@@ -2,7 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
- *  Copyright (C) 2003 Apple Computer, Inc.
+ *  Copyright (C) 2003, 2007 Apple Inc. All rights reserved.
  *  Copyright (C) 2003 Peter Kelly (pmk@post.com)
  *  Copyright (C) 2006 Alexey Proskuryakov (ap@nypop.com)
  *
@@ -49,12 +49,40 @@ const unsigned sparseArrayCutoff = 10000;
 
 const ClassInfo ArrayInstance::info = {"Array", 0, 0, 0};
 
+static inline JSValue** allocateStorage(size_t capacity)
+{
+  if (capacity == 0)
+      return 0;
+
+  // store capacity in extra space before the beginning of the storage array to save space
+  JSValue** storage = static_cast<JSValue**>(fastCalloc(capacity + 1, sizeof(JSValue *))) + 1;
+  storage[-1] = reinterpret_cast<JSValue*>(capacity);
+  return storage;
+}
+
+static inline void reallocateStorage(JSValue**& storage, size_t newCapacity)
+{
+  if (!storage) {
+    storage =  allocateStorage(newCapacity);
+    return;
+  }
+
+  // store capacity in extra space before the beginning of the storage array to save space
+  storage = static_cast<JSValue**>(fastRealloc(storage - 1, (newCapacity + 1) * sizeof (JSValue*))) + 1;
+  storage[-1] = reinterpret_cast<JSValue*>(newCapacity);
+}
+
+static inline void freeStorage(JSValue** storage)
+{
+  if (storage)
+    fastFree(storage - 1);
+}
+
 ArrayInstance::ArrayInstance(JSObject *proto, unsigned initialLength)
   : JSObject(proto)
   , length(initialLength)
   , storageLength(initialLength < sparseArrayCutoff ? initialLength : 0)
-  , capacity(storageLength)
-  , storage(capacity ? (JSValue **)fastCalloc(capacity, sizeof(JSValue *)) : 0)
+  , storage(allocateStorage(storageLength))
 {
 }
 
@@ -62,8 +90,7 @@ ArrayInstance::ArrayInstance(JSObject *proto, const List &list)
   : JSObject(proto)
   , length(list.size())
   , storageLength(length)
-  , capacity(storageLength)
-  , storage(capacity ? (JSValue **)fastMalloc(sizeof(JSValue *) * capacity) : 0)
+  , storage(allocateStorage(storageLength))
 {
   ListIterator it = list.begin();
   unsigned l = length;
@@ -74,7 +101,7 @@ ArrayInstance::ArrayInstance(JSObject *proto, const List &list)
 
 ArrayInstance::~ArrayInstance()
 {
-  fastFree(storage);
+  freeStorage(storage);
 }
 
 JSValue* ArrayInstance::getItem(unsigned i) const
@@ -106,7 +133,7 @@ bool ArrayInstance::getOwnPropertySlot(ExecState* exec, const Identifier& proper
       return false;
     if (index < storageLength) {
       JSValue *v = storage[index];
-      if (!v || v->isUndefined())
+      if (!v)
         return false;      
       slot.setValueSlot(this, &storage[index]);
       return true;
@@ -125,7 +152,7 @@ bool ArrayInstance::getOwnPropertySlot(ExecState *exec, unsigned index, Property
     return false;
   if (index < storageLength) {
     JSValue *v = storage[index];
-    if (!v || v->isUndefined())
+    if (!v)
       return false;
     slot.setValueSlot(this, &storage[index]);
     return true;
@@ -236,7 +263,8 @@ void ArrayInstance::resizeStorage(unsigned newLength)
     if (newLength < storageLength) {
       memset(storage + newLength, 0, sizeof(JSValue *) * (storageLength - newLength));
     }
-    if (newLength > capacity) {
+    size_t cap = capacity();
+    if (newLength > cap) {
       unsigned newCapacity;
       if (newLength > sparseArrayCutoff) {
         newCapacity = newLength;
@@ -246,9 +274,9 @@ void ArrayInstance::resizeStorage(unsigned newLength)
           newCapacity = sparseArrayCutoff;
         }
       }
-      storage = (JSValue **)fastRealloc(storage, newCapacity * sizeof (JSValue *));
-      memset(storage + capacity, 0, sizeof(JSValue *) * (newCapacity - capacity));
-      capacity = newCapacity;
+      
+      reallocateStorage(storage, newCapacity);
+      memset(storage + cap, 0, sizeof(JSValue*) * (newCapacity - cap));
     }
     storageLength = newLength;
 }
@@ -289,7 +317,7 @@ void ArrayInstance::mark()
   }
 }
 
-static ExecState *execForCompareByStringForQSort;
+static ExecState* execForCompareByStringForQSort;
 
 static int compareByStringForQSort(const void *a, const void *b)
 {
@@ -305,13 +333,32 @@ static int compareByStringForQSort(const void *a, const void *b)
     return compare(va->toString(exec), vb->toString(exec));
 }
 
-void ArrayInstance::sort(ExecState *exec)
+void ArrayInstance::sort(ExecState* exec)
 {
-    int lengthNotIncludingUndefined = pushUndefinedObjectsToEnd(exec);
-    
+    size_t lengthNotIncludingUndefined = compactForSorting();
+      
+    ExecState* oldExec = execForCompareByStringForQSort;
     execForCompareByStringForQSort = exec;
-    qsort(storage, lengthNotIncludingUndefined, sizeof(JSValue *), compareByStringForQSort);
-    execForCompareByStringForQSort = 0;
+#if HAVE(MERGESORT)
+    // mergesort usually does fewer compares, so it is actually faster than qsort for JS sorts.
+    // however, becuase it requires extra copies of the storage buffer, don't use it for very
+    // large arrays
+    // FIXME: for sorting by string value, the fastest thing would actually be to convert all the
+    // values to string once up front, and then use a radix sort. That would be O(N) rather than 
+    // O(N log N).
+    if (lengthNotIncludingUndefined < sparseArrayCutoff) {
+        JSValue** storageCopy = static_cast<JSValue**>(fastMalloc(capacity * sizeof(JSValue*)));
+        memcpy(storageCopy, storage, capacity * sizeof(JSValue*));
+        mergesort(storageCopy, lengthNotIncludingUndefined, sizeof(JSValue *), compareByStringForQSort);
+        fastFree(storage);
+        storage = storageCopy;
+        execForCompareByStringForQSort = oldExec;
+        return;
+    }
+#endif
+
+    qsort(storage, lengthNotIncludingUndefined, sizeof(JSValue*), compareByStringForQSort);
+    execForCompareByStringForQSort = oldExec;
 }
 
 struct CompareWithCompareFunctionArguments {
@@ -330,7 +377,7 @@ struct CompareWithCompareFunctionArguments {
     JSObject *globalObject;
 };
 
-static CompareWithCompareFunctionArguments *compareWithCompareFunctionArguments;
+static CompareWithCompareFunctionArguments* compareWithCompareFunctionArguments;
 
 static int compareWithCompareFunctionForQSort(const void *a, const void *b)
 {
@@ -353,17 +400,34 @@ static int compareWithCompareFunctionForQSort(const void *a, const void *b)
     return compareResult < 0 ? -1 : compareResult > 0 ? 1 : 0;
 }
 
-void ArrayInstance::sort(ExecState *exec, JSObject *compareFunction)
+void ArrayInstance::sort(ExecState* exec, JSObject* compareFunction)
 {
-    int lengthNotIncludingUndefined = pushUndefinedObjectsToEnd(exec);
-    
+    size_t lengthNotIncludingUndefined = compactForSorting();
+
+    CompareWithCompareFunctionArguments* oldArgs = compareWithCompareFunctionArguments;
     CompareWithCompareFunctionArguments args(exec, compareFunction);
     compareWithCompareFunctionArguments = &args;
-    qsort(storage, lengthNotIncludingUndefined, sizeof(JSValue *), compareWithCompareFunctionForQSort);
-    compareWithCompareFunctionArguments = 0;
+#if HAVE(MERGESORT)
+    // mergesort usually does fewer compares, so it is actually faster than qsort for JS sorts.
+    // however, becuase it requires extra copies of the storage buffer, don't use it for very
+    // large arrays
+    // FIXME: a tree sort using a perfectly balanced tree (e.g. an AVL tree) could do an even
+    // better job of minimizing compares
+    if (lengthNotIncludingUndefined < sparseArrayCutoff) {
+        JSValue** storageCopy = static_cast<JSValue**>(fastMalloc(capacity * sizeof(JSValue*)));
+        memcpy(storageCopy, storage, capacity * sizeof(JSValue*));
+        mergesort(storageCopy, lengthNotIncludingUndefined, sizeof(JSValue *), compareWithCompareFunctionForQSort);
+        fastFree(storage);
+        storage = storageCopy;
+        compareWithCompareFunctionArguments = oldArgs;
+        return;
+    }
+#endif
+    qsort(storage, lengthNotIncludingUndefined, sizeof(JSValue*), compareWithCompareFunctionForQSort);
+    compareWithCompareFunctionArguments = oldArgs;
 }
 
-unsigned ArrayInstance::pushUndefinedObjectsToEnd(ExecState *exec)
+unsigned ArrayInstance::compactForSorting()
 {
     JSValue *undefined = jsUndefined();
 
@@ -388,8 +452,8 @@ unsigned ArrayInstance::pushUndefinedObjectsToEnd(ExecState *exec)
     PropertyNameArrayIterator end = sparseProperties.end();
     for (PropertyNameArrayIterator it = sparseProperties.begin(); it != end; ++it) {
         Identifier name = *it;
-        storage[o] = get(exec, name);
-        JSObject::deleteProperty(exec, name);
+        storage[o] = getDirect(name);
+        _prop.remove(name);
         o++;
     }
     
@@ -856,7 +920,7 @@ JSValue *ArrayProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, cons
     if (id == Some || id == Every)
       result = jsBoolean(id == Every);
     else
-      result = thisObj;
+      result = jsUndefined();
     
     for (unsigned k = 0; k < length && !exec->hadException(); ++k) {
       PropertySlot slot;
@@ -903,7 +967,7 @@ JSValue *ArrayProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, cons
     for (; index < length; ++index) {
         JSValue* e = getProperty(exec, thisObj, index);
         if (!e)
-            e = jsUndefined();
+            continue;
         if (strictEqual(exec, searchElement, e))
             return jsNumber(index);
     }
@@ -929,7 +993,7 @@ JSValue *ArrayProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, cons
     for (; index >= 0; --index) {
         JSValue* e = getProperty(exec, thisObj, index);
         if (!e)
-            e = jsUndefined();
+            continue;
         if (strictEqual(exec, searchElement, e))
             return jsNumber(index);
     }
