@@ -50,6 +50,10 @@
 #include <cstring>
 #include <time.h>
 
+#if PLATFORM(SOLARIS_OS)
+#include <strings.h>
+#endif
+
 #include "error_object.h"
 #include "operations.h"
 
@@ -1018,6 +1022,8 @@ static double parseDate(const UString &date)
 
     CString dateCString = date.UTF8String();
     const char *dateString = dateCString.c_str();
+    if(!dateString)
+      return NaN;
 
     // Skip leading space
     dateString = skipSpacesAndComments(dateString);
@@ -1104,7 +1110,7 @@ static double parseDate(const UString &date)
     }
 
     // Missing delimiter between month and day (like "January29")?
-    if (month == -1 && dateString && wordStart != dateString)
+    if (month == -1 && wordStart != dateString)
         month = findMonth(wordStart);
 
     dateString = skipSpacesAndComments(dateString);
