@@ -4,6 +4,7 @@
  *  Copyright (C) 1999-2002 Harri Porten (porten@kde.org)
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
  *  Copyright (C) 2003 Apple Computer, Inc.
+ *  Copyright (C) 2007 Cameron Zwarich (cwzwarich@uwaterloo.ca)
  *  Copyright (C) 2007 Maksim Orlovich (maksim@kde.org)
  *
  *  This library is free software; you can redistribute it and/or
@@ -26,6 +27,7 @@
 #include "function.h"
 #include <config.h>
 #include "scriptfunction.h"
+#include "dtoa.h"
 #include "internal.h"
 #include "function_object.h"
 #include "lexer.h"
@@ -33,6 +35,7 @@
 #include "operations.h"
 #include "debugger.h"
 #include "context.h"
+#include "PropertyNameArray.h"
 
 #include <stdio.h>
 #include <errno.h>
@@ -110,7 +113,7 @@ JSValue* FunctionImp::callAsFunction(ExecState* exec, JSObject* thisObj, const L
 
   // enter a new execution context
   Context ctx(globalObj, exec->dynamicInterpreter(), thisObj, body.get(),
-                 codeType(), exec->context(), this, &args);
+                 FunctionCode, exec->context(), this, &args);
   ExecState newExec(exec->dynamicInterpreter(), &ctx);
   if (exec->hadException())
     newExec.setException(exec->exception());
@@ -132,7 +135,7 @@ JSValue* FunctionImp::callAsFunction(ExecState* exec, JSObject* thisObj, const L
   // Here, we actually fill stuff in, going by increasing priorities: 
   // first, initialize flags and set to undefined...
   activation->setupLocals();
-  
+
   // Next, assign user supplied arguments to parameters
   passInParameters(&newExec, args);
 
@@ -148,7 +151,7 @@ JSValue* FunctionImp::callAsFunction(ExecState* exec, JSObject* thisObj, const L
       lineno = static_cast<DeclaredFunctionImp*>(this)->body->firstLine();
     }
 
-    bool cont = dbg->callEvent(&newExec,sid,lineno,this,args);
+    bool cont = dbg->enterContext(&newExec,sid,lineno,this,args);
     if (!cont) {
       dbg->imp()->abort();
       return jsUndefined();
@@ -172,7 +175,7 @@ JSValue* FunctionImp::callAsFunction(ExecState* exec, JSObject* thisObj, const L
 
   // The debugger may have been deallocated by now if the WebFrame
   // we were running in has been destroyed, so refetch it.
-  // See http://bugzilla.opendarwin.org/show_bug.cgi?id=9477
+  // See http://bugs.webkit.org/show_bug.cgi?id=9477
   dbg = exec->dynamicInterpreter()->debugger();
 
   if (dbg) {
@@ -182,7 +185,7 @@ JSValue* FunctionImp::callAsFunction(ExecState* exec, JSObject* thisObj, const L
     if (comp.complType() == Throw)
         newExec.setException(comp.value());
 
-    int cont = dbg->returnEvent(&newExec,sid,lineno,this);
+    int cont = dbg->exitContext(&newExec,sid,lineno,this);
     if (!cont) {
       dbg->imp()->abort();
       return jsUndefined();
@@ -512,9 +515,8 @@ const ClassInfo ActivationImp::info = {"Activation", 0, 0, 0};
 
 // ECMA 10.1.6
 ActivationImp::ActivationImp(FunctionImp *function, const List &arguments)
-    : _function(function), _arguments(true), _locals(0)
+    : _function(function), _arguments(arguments), _locals(0)
 {
-  _arguments.copyFrom(arguments);
   // FIXME: Do we need to support enumerating the arguments property?
 }
 
@@ -553,6 +555,16 @@ PropertySlot::GetValueFunc ActivationImp::getArgumentsGetter()
   return ActivationImp::argumentsGetter;
 }
 
+void ActivationImp::getPropertyNames(ExecState* exec, PropertyNameArray& props)
+{
+    // This is used for debugging only -- no way to access from script code.
+    int numLocals = _function->body->numLocals();
+    for (int l = 1; l < numLocals + 1; ++l)
+        props.add(_function->body->getLocalName(l));
+
+    JSObject::getPropertyNames(exec, props);
+}
+
 bool ActivationImp::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot)
 {
     // do this first so property map arguments property wins over the below
@@ -563,7 +575,7 @@ bool ActivationImp::getOwnPropertySlot(ExecState *exec, const Identifier& proper
     int id = _function->body->lookupSymbolID(propertyName);
 
     if (validLocal(id)) {
-      slot.setValueSlot(this, &_locals[id].value, _locals[id].attr & ReadOnly);
+      slot.setValueSlot(this, &_locals[id].value, (_locals[id].attr & ReadOnly) != 0);
       return true;
     }
 
@@ -615,7 +627,6 @@ void ActivationImp::mark()
 {
     if (_function && !_function->marked())
         _function->mark();
-    _arguments.mark();
     for (int pos = 0; pos < numLocals(); ++pos) {
       JSValue *val = _locals[pos].value;
       if (val && !val->marked())
@@ -629,9 +640,11 @@ ActivationImp::~ActivationImp()
     delete[] _locals;
 }
 
-void ActivationImp::createArgumentsObject(ExecState *exec) const
+void ActivationImp::createArgumentsObject(ExecState *exec)
 {
     _locals[0].value = new Arguments(exec, _function, _arguments, const_cast<ActivationImp*>(this));
+     // The arguments list is only needed to create the arguments object, so discard it now
+    _arguments.reset();
 }
 
 // ------------------------------ GlobalFunc -----------------------------------
@@ -644,11 +657,6 @@ GlobalFuncImp::GlobalFuncImp(ExecState* exec, FunctionPrototype* funcProto, int
   putDirect(exec->propertyNames().length, len, DontDelete|ReadOnly|DontEnum);
 }
 
-CodeType GlobalFuncImp::codeType() const
-{
-  return id == Eval ? EvalCode : codeType();
-}
-
 static JSValue *encode(ExecState *exec, const List &args, const char *do_not_escape)
 {
   UString r = "", s, str = args[0]->toString(exec);
@@ -774,6 +782,28 @@ static int parseDigit(unsigned short c, int radix)
     return digit;
 }
 
+double parseIntOverflow(const char* s, int length, int radix)
+{
+    double number = 0.0;
+    double radixMultiplier = 1.0;
+
+    for (const char* p = s + length - 1; p >= s; p--) {
+        if (radixMultiplier == Inf) {
+            if (*p != '0') {
+                number = Inf;
+                break;
+            }
+        } else {
+            int digit = parseDigit(*p, radix);
+            number += digit * radixMultiplier;
+        }
+
+        radixMultiplier *= radix;
+    }
+
+    return number;
+}
+
 static double parseInt(const UString &s, int radix)
 {
     int length = s.size();
@@ -806,6 +836,7 @@ static double parseInt(const UString &s, int radix)
     if (radix < 2 || radix > 36)
         return NaN;
 
+    int firstDigitPosition = p;
     bool sawDigit = false;
     double number = 0;
     while (p < length) {
@@ -818,6 +849,13 @@ static double parseInt(const UString &s, int radix)
         ++p;
     }
 
+    if (number >= mantissaOverflowLowerBound) {
+        if (radix == 10)
+            number = kjs_strtod(s.substr(firstDigitPosition, p - firstDigitPosition).ascii(), 0);
+        else if (radix == 2 || radix == 4 || radix == 8 || radix == 16 || radix == 32)
+            number = parseIntOverflow(s.substr(firstDigitPosition, p - firstDigitPosition).ascii(), p - firstDigitPosition, radix);
+    }
+
     if (!sawDigit)
         return NaN;
 
@@ -891,10 +929,9 @@ JSValue *GlobalFuncImp::callAsFunction(ExecState *exec, JSObject * /*thisObj*/,
           return throwError(exec, SyntaxError, errMsg, errLine, sid, NULL);
 
         // enter a new execution context
-        JSObject *thisVal = static_cast<JSObject *>(exec->context()->thisValue());
         Context ctx(exec->dynamicInterpreter()->globalObject(),
                        exec->dynamicInterpreter(),
-                       thisVal,
+                       0, /* thisVal is inherited in EvalCode */
                        progNode.get(),
                        EvalCode,
                        exec->context());
@@ -903,10 +940,27 @@ JSValue *GlobalFuncImp::callAsFunction(ExecState *exec, JSObject * /*thisObj*/,
         if (exec->hadException())
             newExec.setException(exec->exception());
 
+        if (dbg) {
+            bool cont = dbg->enterContext(&newExec, sid, 0, 0, List::empty());
+            if (!cont) {
+                dbg->imp()->abort();
+                return jsUndefined();
+            }
+        }
+
         // execute the code
         progNode->processDecls(&newExec);
         Completion c = progNode->execute(&newExec);
 
+        dbg = exec->dynamicInterpreter()->debugger();
+        if (dbg) {
+            bool cont = dbg->exitContext(&newExec, sid, 0, 0);
+            if (!cont) {
+                dbg->imp()->abort();
+                return jsUndefined();
+            }
+        }
+
         // if an exception occurred, propogate it back to the previous execution object
         if (newExec.hadException())
           exec->setException(newExec.exception());
