@@ -214,15 +214,17 @@ static inline InterpreterMap &interpreterMap()
 }
     
 Interpreter::Interpreter(JSObject* globalObject)
-    : m_globalExec(this, 0)
-    , m_globalObject(globalObject)
+    : m_globalExec(this, 0),
+      m_globalObject(globalObject),
+      globPkg(0)
 {
     init();
 }
 
 Interpreter::Interpreter()
-    : m_globalExec(this, 0)
-    , m_globalObject(new JSObject())
+    : m_globalExec(this, 0),
+      m_globalObject(new JSObject()),
+      globPkg(0)
 {
     init();
 }
@@ -488,8 +490,21 @@ Completion Interpreter::evaluate(const UString& sourceURL, int startingLineNumbe
         // execute the code
         Context ctx(globalObj, this, thisObj, progNode.get());
         ExecState newExec(this, &ctx);
+
+        if (m_debugger && !m_debugger->enterContext(&newExec, sid, startingLineNumber, 0, List::empty())) {
+            // debugger requested we stop execution.
+            m_debugger->imp()->abort();
+            return Completion(Break);
+        }
+
         progNode->processDecls(&newExec);
         res = progNode->execute(&newExec);
+
+        if (m_debugger && !m_debugger->exitContext(&newExec, sid, startingLineNumber, 0)) {
+            // debugger requested we stop execution.
+            m_debugger->imp()->abort();
+            return Completion(Break);
+        }
     }
     
     m_recursion--;
@@ -812,6 +827,16 @@ void Interpreter::stopTimeoutCheck()
     m_timeoutChecker->stopTimeoutCheck(this);
 }
 
+void Interpreter::restartTimeoutCheck()
+{
+    if (!m_timeoutChecker || !m_startTimeoutCheckCount)
+        return;
+
+    m_timedOut = false;
+    m_timeoutChecker->stopTimeoutCheck(this);
+    m_timeoutChecker->startTimeoutCheck(this);
+}
+
 void Interpreter::pauseTimeoutCheck()
 {
     ASSERT(m_timeoutChecker);
