@@ -216,43 +216,31 @@ static inline const UString& currentSourceURL(ExecState* exec)
     return exec->context()->currentBody()->sourceURL();
 }
 
-Completion Node::createErrorCompletion(ExecState* exec, ErrorType e, const char *msg)
+Completion Node::createErrorCompletion(ExecState* exec, ErrorType e, const UString& msg)
 {
     return Completion(Throw, Error::create(exec, e, msg, lineNo(), currentSourceId(exec), currentSourceURL(exec)));
 }
 
-Completion Node::createErrorCompletion(ExecState *exec, ErrorType e, const char *msg, const Identifier &ident)
+Completion Node::createErrorCompletion(ExecState *exec, ErrorType e, const UString& msg, const Identifier &ident)
 {
     UString message = msg;
     substitute(message, ident.ustring());
     return Completion(Throw, Error::create(exec, e, message, lineNo(), currentSourceId(exec), currentSourceURL(exec)));
 }
 
-Node* Node::createErrorNode(ErrorType e, const char *msg)
-{
-    return new ErrorNode(this, e, msg);
-}
-
-Node* Node::createErrorNode(ErrorType e, const char *msg, const Identifier &ident)
-{
-    UString message = msg;
-    substitute(message, ident.ustring());
-    return new ErrorNode(this, e, message);
-}
-
-JSValue *Node::throwError(ExecState* exec, ErrorType e, const char *msg)
+JSValue* Node::throwError(ExecState* exec, ErrorType e, const UString& msg)
 {
     return KJS::throwError(exec, e, msg, lineNo(), currentSourceId(exec), currentSourceURL(exec));
 }
 
-JSValue *Node::throwError(ExecState* exec, ErrorType e, const char* msg, const char* string) 
+JSValue* Node::throwError(ExecState* exec, ErrorType e, const UString& msg, const char* string) 
 { 
     UString message = msg; 
     substitute(message, string); 
     return KJS::throwError(exec, e, message, lineNo(), currentSourceId(exec), currentSourceURL(exec)); 
 }
 
-JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, JSValue *v, Node *expr)
+JSValue* Node::throwError(ExecState* exec, ErrorType e, const UString& msg, JSValue* v, Node* expr)
 {
     UString message = msg;
     substitute(message, v->toString(exec));
@@ -261,14 +249,14 @@ JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, JSValue
 }
 
 
-JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, const Identifier &label)
+JSValue* Node::throwError(ExecState* exec, ErrorType e, const UString& msg, const Identifier& label)
 {
     UString message = msg;
     substitute(message, label.ustring());
     return KJS::throwError(exec, e, message, lineNo(), currentSourceId(exec), currentSourceURL(exec));
 }
 
-JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, JSValue *v, Node *e1, Node *e2)
+JSValue* Node::throwError(ExecState* exec, ErrorType e, const UString& msg, JSValue* v, Node* e1, Node* e2)
 {
     UString message = msg;
     substitute(message, v->toString(exec));
@@ -277,7 +265,7 @@ JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, JSValue
     return KJS::throwError(exec, e, message, lineNo(), currentSourceId(exec), currentSourceURL(exec));
 }
 
-JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, JSValue *v, Node *expr, const Identifier &label)
+JSValue* Node::throwError(ExecState* exec, ErrorType e, const UString& msg, JSValue* v, Node* expr, const Identifier& label)
 {
     UString message = msg;
     substitute(message, v->toString(exec));
@@ -286,7 +274,7 @@ JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, JSValue
     return KJS::throwError(exec, e, message, lineNo(), currentSourceId(exec), currentSourceURL(exec));
 }
 
-JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, JSValue *v, const Identifier &label)
+JSValue* Node::throwError(ExecState* exec, ErrorType e, const UString& msg, JSValue* v, const Identifier& label)
 {
     UString message = msg;
     substitute(message, v->toString(exec));
@@ -294,7 +282,7 @@ JSValue *Node::throwError(ExecState *exec, ErrorType e, const char *msg, JSValue
     return KJS::throwError(exec, e, message, lineNo(), currentSourceId(exec), currentSourceURL(exec));
 }
 
-JSValue *Node::throwUndefinedVariableError(ExecState *exec, const Identifier &ident)
+JSValue* Node::throwUndefinedVariableError(ExecState* exec, const Identifier& ident)
 {
     return throwError(exec, ReferenceError, "Can't find variable: %s", ident);
 }
@@ -319,6 +307,11 @@ Node *Node::nodeInsideAllParens()
     return this;
 }
 
+void Node::copyDebugInfo(Node* otherNode)
+{
+    m_line = otherNode->m_line;
+}
+
 class VarDeclVisitor: public NodeVisitor {
   private:
     ExecState* m_exec;
@@ -393,6 +386,12 @@ void StatementNode::setLoc(int firstLine, int lastLine)
     m_lastLine = lastLine;
 }
 
+void StatementNode::copyDebugInfo(StatementNode* otherNode)
+{
+    m_line     = otherNode->m_line;
+    m_lastLine = otherNode->m_lastLine;
+}
+
 // return true if the debugger wants us to stop at this point
 bool StatementNode::hitStatement(ExecState* exec)
 {
@@ -403,6 +402,20 @@ bool StatementNode::hitStatement(ExecState* exec)
     return true; // continue
 }
 
+Node* StatementNode::createErrorNode(ErrorType e, const UString& msg)
+{
+    return new ErrorNode(this, e, msg);
+}
+
+Node* StatementNode::createErrorNode(ErrorType e, const UString& msg, const Identifier &ident)
+{
+    UString message = msg;
+    substitute(message, ident.ustring());
+    return new ErrorNode(this, e, message);
+}
+
+
+
 // ------------------------------ NullNode -------------------------------------
 
 JSValue *NullNode::evaluate(ExecState *)
@@ -524,6 +537,7 @@ Reference VarAccessNode::evaluateReference(ExecState* exec) {
 Node* VarAccessNode::optimizeLocalAccess(ExecState *exec, FunctionBodyNode* node)
 {
   int index = node->lookupSymbolID(ident);
+  Node* out = 0;
 
   // If the symbol isn't local, we can still optimize 
   // a bit, and skip the local scope for lookup.
@@ -531,12 +545,16 @@ Node* VarAccessNode::optimizeLocalAccess(ExecState *exec, FunctionBodyNode* node
   // is careful about that)
   if (index == -1) {
     if (ident == exec->propertyNames().arguments)
-      return 0; //Dynamic property..
+      out = 0; //Dynamic property..
     else
-      return new NonLocalVarAccessNode(ident);
+      out = new NonLocalVarAccessNode(ident);
   } else {
-      return new LocalVarAccessNode(ident, index);
+      out = new LocalVarAccessNode(ident, index);
   }
+
+  if (out)
+    out->copyDebugInfo(this);
+  return out;
 }
 
 int VarAccessNode::localID(FunctionBodyNode* node)
@@ -1014,6 +1032,8 @@ JSValue *FunctionCallReferenceNode::evaluate(ExecState *exec)
   KJS_CHECKEXCEPTIONVALUE
 
   JSObject *thisObj = ref.base;
+  if (thisObj->isActivation())
+    thisObj =  exec->dynamicInterpreter()->globalObject();
 
   return func->call(exec, thisObj, argList);
 }
@@ -1177,8 +1197,11 @@ Node* PostfixNode::optimizeLocalAccess(ExecState* /*exec*/, FunctionBodyNode* no
   if (m_loc->isVarAccessNode()) {
     VarAccessNode* ident = static_cast<VarAccessNode*>(m_loc.get());
     int id = ident->localID(node);
-    if (id != -1)
-      return new LocalPostfixNode(m_loc.get(), m_oper, id);
+    if (id != -1) {
+      Node* out = new LocalPostfixNode(m_loc.get(), m_oper, id);
+      out->copyDebugInfo(this);
+      return out;
+    }
   }
   return 0;
 }
@@ -1359,8 +1382,11 @@ Node* PrefixNode::optimizeLocalAccess(ExecState* /*exec*/, FunctionBodyNode* nod
   if (m_loc->isVarAccessNode()) {
     VarAccessNode* ident = static_cast<VarAccessNode*>(m_loc.get());
     int id = ident->localID(node);
-    if (id != -1)
-      return new LocalPrefixNode(m_loc.get(), m_oper, id);
+    if (id != -1) {
+      Node* out = new LocalPrefixNode(m_loc.get(), m_oper, id);
+      out->copyDebugInfo(this);
+      return out;
+    }
   }
   return 0;
 }
@@ -1440,6 +1466,33 @@ void LogicalNotNode::recurseVisit(NodeVisitor *visitor)
 
 // ------------------------ BinaryOperatorNode -------------------------------
 
+static inline JSValue *add(ExecState *exec, JSValue *v1, JSValue *v2)
+{
+    // exception for the Date exception in defaultValue()
+    JSValue *p1 = v1->toPrimitive(exec, UnspecifiedType);
+    JSValue *p2 = v2->toPrimitive(exec, UnspecifiedType);
+    
+    if (p1->isString() || p2->isString()) {
+        UString value = p1->toString(exec) + p2->toString(exec);
+        if (value.isNull()) {
+            JSObject *error = Error::create(exec, GeneralError, "Out of memory");
+            exec->setException(error);
+            return error;
+        } else
+            return jsString(value);
+    }
+    
+    return jsNumber(p1->toNumber(exec) + p2->toNumber(exec));
+}
+
+static inline JSValue *sub(ExecState *exec, JSValue *v1, JSValue *v2)
+{
+    JSValue *p1 = v1->toPrimitive(exec, NumberType);
+    JSValue *p2 = v2->toPrimitive(exec, NumberType);
+
+    return jsNumber(p1->toNumber(exec) - p2->toNumber(exec));
+}
+
 JSValue* BinaryOperatorNode::evaluate(ExecState* exec)
 {
   JSValue* v1 = expr1->evaluate(exec);
@@ -1450,14 +1503,20 @@ JSValue* BinaryOperatorNode::evaluate(ExecState* exec)
 
   switch (oper) {
   case OpMult:
+      // operator *
+      return jsNumber(v1->toNumber(exec) * v2->toNumber(exec));
   case OpDiv:
+      // operator /
+      return jsNumber(v1->toNumber(exec) / v2->toNumber(exec));
   case OpMod:
-      // operators *, / and %
-      return mult(exec, v1, v2, oper);
+      // operator %
+      return jsNumber(fmod(v1->toNumber(exec), v2->toNumber(exec)));
   case OpPlus:
+      // operator +
+      return add(exec, v1, v2);
   case OpMinus:
-      // operators + and -
-      return add(exec, v1, v2, oper);
+      // operator -
+      return sub(exec, v1, v2);
   case OpLShift:
       // operator <<
       return jsNumber(v1->toInt32(exec) << (v2->toUInt32(exec) & 0x1f));
@@ -1621,16 +1680,16 @@ static ALWAYS_INLINE JSValue *valueForReadModifyAssignment(ExecState * exec, JSV
   unsigned int ui;
   switch (oper) {
   case OpMultEq:
-    v = mult(exec, v1, v2, OpMult);
+    v = jsNumber(v1->toNumber(exec) * v2->toNumber(exec));
     break;
   case OpDivEq:
-    v = mult(exec, v1, v2, OpDiv);
+    v = jsNumber(v1->toNumber(exec) / v2->toNumber(exec));
     break;
   case OpPlusEq:
-    v = add(exec, v1, v2, OpPlus);
+    v = add(exec, v1, v2);
     break;
   case OpMinusEq:
-    v = add(exec, v1, v2, OpMinus);
+    v = sub(exec, v1, v2);
     break;
   case OpLShift:
     i1 = v1->toInt32(exec);
@@ -1736,7 +1795,9 @@ Node* AssignNode::optimizeLocalAccess(ExecState* /*exec*/, FunctionBodyNode* nod
     VarAccessNode* ident = static_cast<VarAccessNode*>(m_loc.get());
     int id = ident->localID(node);
     if (id != -1) {
-        return new LocalAssignNode(m_loc.get(), m_right.get(), id);
+      Node* out = new LocalAssignNode(m_loc.get(), m_right.get(), id);
+      out->copyDebugInfo(this);
+      return out;
     }
   }
   return 0;
@@ -2031,6 +2092,7 @@ Node* VarStatementNode::optimizeLocalAccess(ExecState*, FunctionBodyNode* funcBo
       staticVer->initializers.append(init);
     }
   }
+  staticVer->copyDebugInfo(this);
   return staticVer;
 }
 
@@ -2951,7 +3013,7 @@ ProgramNode::ProgramNode(SourceElementsNode *s) : FunctionBodyNode(s)
 
 Completion ErrorNode::execute(ExecState *exec)
 {
-    return createErrorCompletion(exec, SyntaxError, message.cstring().c_str());
+    return createErrorCompletion(exec, SyntaxError, message);
 }
 
 // ------------------------------ PackageNameNode ------------------------------
@@ -3040,7 +3102,7 @@ PackageObject* PackageNameNode::resolvePackage(ExecState* exec,
 	    if (err.isEmpty()) {
 		throwError(exec, GeneralError, "Package not found");
 	    } else {
-		throwError(exec, GeneralError, err.ascii());
+		throwError(exec, GeneralError, err);
 	    }
 	    return 0;
 	}
