@@ -213,8 +213,10 @@ void JSObject::put(ExecState *exec, const Identifier &propertyName, JSValue *val
   if (propertyName == exec->propertyNames().underscoreProto) {
     JSObject* proto = value->getObject();
     while (proto) {
-      if (proto == this)
+      if (proto == this) {
         throwError(exec, GeneralError, "cyclic __proto__ value");
+        return;
+      }
       proto = proto->prototype() ? proto->prototype()->getObject() : 0;
     }
     
@@ -556,6 +558,11 @@ void JSObject::putDirect(const Identifier &propertyName, int value, int attr)
     _prop.put(propertyName, jsNumber(value), attr);
 }
 
+void JSObject::removeDirect(const Identifier &propertyName)
+{
+    _prop.remove(propertyName);
+}
+
 void JSObject::putDirectFunction(InternalFunctionImp* func, int attr)
 {
     putDirect(func->functionName(), func, attr); 
