@@ -60,8 +60,8 @@
 #include <kglobalsettings.h>
 #include <kdebug.h>
 
-static QString details = I18N_NOOP("Settings");
-static QString about = I18N_NOOP("About");
+static const char *details = I18N_NOOP("Settings");
+static const char *about = I18N_NOOP("About");
 
 KPluginSelector::Private::Private(KPluginSelector *parent)
     : QObject(parent)
@@ -73,12 +73,11 @@ KPluginSelector::Private::Private(KPluginSelector *parent)
     pluginDelegate = new PluginDelegate(this);
 
     pluginDelegate->setMinimumItemWidth(200);
-    pluginDelegate->setLeftMargin(20);
-    pluginDelegate->setRightMargin(20);
+    pluginDelegate->setLeftMargin(KDialog::marginHint());
+    pluginDelegate->setRightMargin(KDialog::marginHint());
     pluginDelegate->setSeparatorPixels(8);
 
     QFont title(parent->font());
-    title.setPointSize(title.pointSize() + 2);
     title.setWeight(QFont::Bold);
 
     QFontMetrics titleMetrics(title);
@@ -806,7 +805,6 @@ void KPluginSelector::Private::PluginDelegate::paint(QPainter *painter, const QS
     QFont previousFont(painter->font());
     QFont configureFont(painter->font());
 
-    title.setPointSize(title.pointSize() + 2);
     title.setWeight(QFont::Bold);
 
     if (index.internalPointer())
@@ -842,7 +840,7 @@ void KPluginSelector::Private::PluginDelegate::paint(QPainter *painter, const QS
                 opt.icon = iconLoader->loadIcon("configure", KIconLoader::Small);
                 opt.iconSize = QSize(iconLoader->currentSize(KIconLoader::Small), iconLoader->currentSize(KIconLoader::Small));
             }
-            opt.text = details;
+            opt.text = i18n(details);
             opt.fontMetrics = option.fontMetrics;
             opt.direction = option.direction;
             opt.rect = option.rect;
@@ -878,14 +876,14 @@ void KPluginSelector::Private::PluginDelegate::paint(QPainter *painter, const QS
             opt.icon = iconLoader->loadIcon("dialog-information", KIconLoader::Small);
             opt.iconSize = QSize(iconLoader->currentSize(KIconLoader::Small), iconLoader->currentSize(KIconLoader::Small));
         }
-        opt.text = about;
+        opt.text = i18n(about);
         opt.fontMetrics = option.fontMetrics;
         opt.direction = option.direction;
         opt.rect = option.rect;
         opt.rect = aboutButtonRect(opt);
 
         QStyleOptionButton opt2(opt);
-        opt2.text = details;
+        opt2.text = i18n(details);
         opt2.rect = option.rect;
         opt2.rect = settingsButtonRect(opt2);
 
@@ -968,16 +966,15 @@ void KPluginSelector::Private::PluginDelegate::paint(QPainter *painter, const QS
 
         QFont painterFont = painter->font();
         painterFont.setWeight(QFont::Bold);
-        painterFont.setPointSize(painterFont.pointSize() + 2);
         QFontMetrics metrics(painterFont);
         painter->setFont(painterFont);
 
         opt.fontMetrics = painter->fontMetrics();
 
         QRect auxRect(optionCopy.rect.left() + leftMargin,
-                      optionCopy.rect.bottom() - 2,
+                      optionCopy.rect.bottom() - 1,
                       optionCopy.rect.width() - leftMargin - rightMargin,
-                      2);
+                      1);
 
         QPainterPath path;
         path.addRect(auxRect);
@@ -993,6 +990,8 @@ void KPluginSelector::Private::PluginDelegate::paint(QPainter *painter, const QS
         painter->setBrush(gradient);
         painter->fillPath(path, gradient);
 
+        int iconSize = KIconLoader::global()->currentSize(KIconLoader::Small);
+
         QRect auxRect2(optionCopy.rect.left() + leftMargin,
                        option.rect.top(),
                        optionCopy.rect.width() - leftMargin - rightMargin,
@@ -1010,7 +1009,6 @@ QSize KPluginSelector::Private::PluginDelegate::sizeHint(const QStyleOptionViewI
     Q_UNUSED(option);
 
     QFont title(option.font);
-    title.setPointSize(title.pointSize() + 2);
     title.setWeight(QFont::Bold);
 
     QFontMetrics titleMetrics(title);
@@ -1031,7 +1029,7 @@ QSize KPluginSelector::Private::PluginDelegate::sizeHint(const QStyleOptionViewI
         return QSize(minimumItemWidth, qMax((separatorPixels * 2) + iconHeight, (separatorPixels * 3) + qMax(titleMetrics.height(), opt.rect.height()) + currentMetrics.height()));
     }
 
-    return QSize(minimumItemWidth, separatorPixels + titleMetrics.height() + 2);
+    return QSize(minimumItemWidth, separatorPixels + titleMetrics.height() + 1);
 }
 
 void KPluginSelector::Private::PluginDelegate::setIconSize(int width, int height)
@@ -1097,7 +1095,7 @@ QRect KPluginSelector::Private::PluginDelegate::settingsButtonRect(const QStyleO
     const QFontMetrics &fontMetrics = option.fontMetrics;
 
     QStyleOptionButton aboutOption(option);
-    aboutOption.text = about;
+    aboutOption.text = i18n(about);
 
     retRect.setHeight(KApplication::style()->sizeFromContents(QStyle::CT_PushButton, &option, QSize(fontMetrics.width(caption) + option.iconSize.width(), qMax(fontMetrics.height(), option.iconSize.height()))).height());
     retRect.setWidth(KApplication::style()->sizeFromContents(QStyle::CT_PushButton, &option, QSize(fontMetrics.width(caption) + option.iconSize.width(), qMax(fontMetrics.height(), option.iconSize.height()))).width());
@@ -1332,7 +1330,7 @@ bool KPluginSelector::Private::PluginDelegate::eventFilter(QObject *watched, QEv
             }
 
             QStyleOptionButton opt;
-            opt.text = about;
+            opt.text = i18n(about);
             opt.fontMetrics = listView->viewOptions().fontMetrics;
             opt.direction = listView->layoutDirection();
             if (KGlobalSettings::showIconsOnPushButtons())
@@ -1343,7 +1341,7 @@ bool KPluginSelector::Private::PluginDelegate::eventFilter(QObject *watched, QEv
             opt.rect = aboutButtonRect(opt);
 
             QStyleOptionButton opt2(opt);
-            opt2.text = details;
+            opt2.text = i18n(details);
             opt2.rect = listView->visualRect(currentIndex);
             opt2.rect = settingsButtonRect(opt2);
 
@@ -1393,7 +1391,7 @@ bool KPluginSelector::Private::PluginDelegate::eventFilter(QObject *watched, QEv
             }
 
             QStyleOptionButton opt;
-            opt.text = about;
+            opt.text = i18n(about);
             opt.fontMetrics = listView->viewOptions().fontMetrics;
             opt.direction = listView->layoutDirection();
             if (KGlobalSettings::showIconsOnPushButtons())
@@ -1404,7 +1402,7 @@ bool KPluginSelector::Private::PluginDelegate::eventFilter(QObject *watched, QEv
             opt.rect = aboutButtonRect(opt);
 
             QStyleOptionButton opt2(opt);
-            opt2.text = details;
+            opt2.text = i18n(details);
             opt2.rect = listView->visualRect(currentIndex);
             opt2.rect = settingsButtonRect(opt2);
 
@@ -1520,7 +1518,7 @@ void KPluginSelector::Private::PluginDelegate::updateCheckState(const QModelInde
 
     // Option button for the about button
     QStyleOptionButton opt;
-    opt.text = about;
+    opt.text = i18n(about);
     if (KGlobalSettings::showIconsOnPushButtons())
     {
         opt.iconSize = QSize(iconLoader->currentSize(KIconLoader::Small), iconLoader->currentSize(KIconLoader::Small));
@@ -1532,7 +1530,7 @@ void KPluginSelector::Private::PluginDelegate::updateCheckState(const QModelInde
 
     // Option button for the settings button
     QStyleOptionButton opt2(opt);
-    opt2.text = details;
+    opt2.text = i18n(details);
     opt2.rect = option.rect;
     opt2.rect = settingsButtonRect(opt2);
 
@@ -1549,9 +1547,15 @@ void KPluginSelector::Private::PluginDelegate::updateCheckState(const QModelInde
             configDialog = new KDialog(parent->parent);
             configDialog->setLayoutDirection(listView->layoutDirection());
             configDialog->setWindowTitle(pluginInfo.name());
-            KTabWidget *newTabWidget = new KTabWidget(configDialog);
 
-            tabWidgets.insert(index.row(), newTabWidget);
+            // The number of KCModuleProxies in use determines whether to use a tabwidget
+            KTabWidget *newTabWidget = 0;
+            // widget to use for the setting dialog's main widget,
+            // either a KTabWidget or a KCModuleProxy
+            QWidget * mainWidget = 0;
+            // widget to use as the KCModuleProxy's parent.
+            // The first proxy is owned by the dialog itself
+            QWidget * moduleProxyParentWidget = configDialog;
 
             foreach(KService::Ptr servicePtr, services)
             {
@@ -1559,10 +1563,25 @@ void KPluginSelector::Private::PluginDelegate::updateCheckState(const QModelInde
                 {
                     KCModuleInfo moduleinfo(servicePtr);
                     model->setParentComponents(index, moduleinfo.service()->property("X-KDE-ParentComponents").toStringList());
-                    KCModuleProxy *currentModuleProxy = new KCModuleProxy(moduleinfo, newTabWidget);
+                    KCModuleProxy *currentModuleProxy = new KCModuleProxy(moduleinfo, moduleProxyParentWidget);
                     if (currentModuleProxy->realModule())
                     {
-                        newTabWidget->addTab(currentModuleProxy, servicePtr->name());
+                        if ( mainWidget && !newTabWidget ) {
+                            // we already created one KCModuleProxy, so we need a tab widget.
+                            // Move the first proxy into the tab widget and ensure this and subsequent
+                            // proxies are in the tab widget
+                            newTabWidget = new KTabWidget(configDialog);
+                            moduleProxyParentWidget = newTabWidget;
+                            mainWidget->setParent( newTabWidget );
+                            newTabWidget->addTab(mainWidget, (qobject_cast<KCModuleProxy*>( mainWidget ))->moduleInfo().moduleName() );
+                            mainWidget = newTabWidget;
+                            tabWidgets.insert(index.row(), newTabWidget);
+                        }
+                        if ( newTabWidget ) {
+                            newTabWidget->addTab(currentModuleProxy, servicePtr->name());
+                        } else {
+                            mainWidget = currentModuleProxy;
+                        }
                     }
 
                     if (!modulesDialogs.contains(index.row()))
@@ -1576,7 +1595,12 @@ void KPluginSelector::Private::PluginDelegate::updateCheckState(const QModelInde
 
             configDialog->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Default);
 
-            configDialog->setMainWidget(newTabWidget);
+            QWidget *showWidget = new QWidget(configDialog);
+            QVBoxLayout *layout = new QVBoxLayout;
+            showWidget->setLayout(layout);
+            layout->addWidget(mainWidget);
+            layout->insertSpacing(-1, KDialog::marginHint());
+            configDialog->setMainWidget(showWidget);
         }
         else
         {
@@ -1627,30 +1651,38 @@ void KPluginSelector::Private::PluginDelegate::updateCheckState(const QModelInde
 
             QWidget *aboutWidget = new QWidget(aboutDialog);
             QVBoxLayout *layout = new QVBoxLayout;
-            layout->setSpacing(0);
             aboutWidget->setLayout(layout);
+            layout->insertStretch(-1);
+
+            QGridLayout *gridLayout = new QGridLayout;
+            gridLayout->setSpacing(KDialog::spacingHint());
+            gridLayout->setColumnStretch(1, 1);
+            gridLayout->setRowStretch(1, 1);
+            layout->addLayout(gridLayout);
 
             if (!pluginInfo.comment().isEmpty())
             {
-                QLabel *description = new QLabel(i18n("Description:\n\t%1", pluginInfo.comment()), aboutWidget);
-                layout->addWidget(description);
-                layout->addSpacing(20);
+                QLabel *description = new QLabel(i18n("Description:"), aboutWidget);
+                QLabel *description2 = new QLabel(pluginInfo.comment(), aboutWidget);
+                description2->setWordWrap(true);
+                description2->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, QSizePolicy::Label));
+                gridLayout->addWidget(description, 0, 0, Qt::AlignLeft | Qt::AlignTop);
+                gridLayout->addWidget(description2, 0, 1, Qt::AlignLeft | Qt::AlignTop);
             }
 
             if (!pluginInfo.author().isEmpty())
             {
-                QLabel *author = new QLabel(i18n("Author:\n\t%1", pluginInfo.author()), aboutWidget);
-                layout->addWidget(author);
-                layout->addSpacing(20);
+                QLabel *author = new QLabel(i18n("Author:"), aboutWidget);
+                QLabel *author2 = new QLabel(pluginInfo.author(), aboutWidget);
+                gridLayout->addWidget(author, 1, 0, Qt::AlignLeft | Qt::AlignTop);
+                gridLayout->addWidget(author2, 1, 1, Qt::AlignLeft | Qt::AlignTop);
             }
 
             if (!pluginInfo.email().isEmpty())
             {
                 QLabel *authorEmail = new QLabel(i18n("E-Mail:"), aboutWidget);
-                KUrlLabel *sendEmail = new KUrlLabel("mailto:" + pluginInfo.email(), '\t' + pluginInfo.email());
+                KUrlLabel *sendEmail = new KUrlLabel("mailto:" + pluginInfo.email(), pluginInfo.email());
 
-                sendEmail->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
-                sendEmail->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
                 sendEmail->setGlowEnabled(false);
                 sendEmail->setUnderline(false);
                 sendEmail->setFloatEnabled(true);
@@ -1660,18 +1692,15 @@ void KPluginSelector::Private::PluginDelegate::updateCheckState(const QModelInde
 
                 QObject::connect(sendEmail, SIGNAL(leftClickedUrl(QString)), this, SLOT(invokeMailer(QString)));
 
-                layout->addWidget(authorEmail);
-                layout->addWidget(sendEmail);
-                layout->addSpacing(20);
+                gridLayout->addWidget(authorEmail, 2, 0, Qt::AlignLeft | Qt::AlignTop);
+                gridLayout->addWidget(sendEmail, 2, 1, Qt::AlignLeft | Qt::AlignTop);
             }
 
             if (!pluginInfo.website().isEmpty())
             {
                 QLabel *website = new QLabel(i18n("Website:"), aboutWidget);
-                KUrlLabel *visitWebsite = new KUrlLabel(pluginInfo.website(), '\t' + pluginInfo.website());
+                KUrlLabel *visitWebsite = new KUrlLabel(pluginInfo.website(), pluginInfo.website());
 
-                visitWebsite->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
-                visitWebsite->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
                 visitWebsite->setGlowEnabled(false);
                 visitWebsite->setUnderline(false);
                 visitWebsite->setFloatEnabled(true);
@@ -1681,30 +1710,32 @@ void KPluginSelector::Private::PluginDelegate::updateCheckState(const QModelInde
 
                 QObject::connect(visitWebsite, SIGNAL(leftClickedUrl(QString)), this, SLOT(invokeBrowser(QString)));
 
-                layout->addWidget(website);
-                layout->addWidget(visitWebsite);
-                layout->addSpacing(20);
+                gridLayout->addWidget(website, 3, 0, Qt::AlignLeft | Qt::AlignTop);
+                gridLayout->addWidget(visitWebsite, 3, 1, Qt::AlignLeft | Qt::AlignTop);
             }
 
             if (!pluginInfo.version().isEmpty())
             {
-                QLabel *version = new QLabel(i18n("Version:\n\t%1", pluginInfo.version()), aboutWidget);
+                QLabel *version = new QLabel(i18n("Version:"), aboutWidget);
+                QLabel *version2 = new QLabel(pluginInfo.version(), aboutWidget);
 
-                layout->addWidget(version);
-                layout->addSpacing(20);
+                gridLayout->addWidget(version, 4, 0, Qt::AlignLeft | Qt::AlignTop);
+                gridLayout->addWidget(version2, 4, 1, Qt::AlignLeft | Qt::AlignTop);
             }
 
             if (!pluginInfo.license().isEmpty())
             {
-                QLabel *license = new QLabel(i18n("License:\n\t%1", pluginInfo.license()), aboutWidget);
+                QLabel *license = new QLabel(i18n("License:"), aboutWidget);
+                QLabel *license2 = new QLabel(pluginInfo.license(), aboutWidget);
 
-                layout->addWidget(license);
-                layout->addSpacing(20);
+                gridLayout->addWidget(license, 5, 0, Qt::AlignLeft | Qt::AlignTop);
+                gridLayout->addWidget(license2, 5, 1, Qt::AlignLeft | Qt::AlignTop);
             }
 
             layout->insertStretch(-1);
 
             aboutDialog->setMainWidget(aboutWidget);
+            aboutDialog->setMinimumSize(400, 250);
 
             aboutDialogs.insert(index.row(), aboutDialog);
         }
