@@ -91,6 +91,7 @@
 
 #include <ksslall.h>
 #include <kopenssl.h>
+#include <ksslcertificatebox.h>
 
 #include "certexport.h"
 #include "kdatetimedlg.h"
@@ -271,7 +272,7 @@ QString whatstr;
   grid->setMargin( KDialog::marginHint() );
   grid->setSpacing( KDialog::spacingHint() );
   // no need to parse kdeglobals.
-  config = new KConfig("cryptodefaults", KConfig::CascadeConfig);
+  config = new KConfig("cryptodefaults", KConfig::NoGlobals);
   policies = new KConfig("ksslpolicies", KConfig::SimpleConfig);
   pcerts = new KConfig("ksslcertificates", KConfig::SimpleConfig);
   authcfg = new KConfig("ksslauthmap", KConfig::SimpleConfig);
@@ -321,21 +322,21 @@ QString whatstr;
 #endif
 
   mWarnOnEnter = new QCheckBox(i18n("Warn on &entering SSL mode"), tabSSL);
-  connect(mWarnOnEnter, SIGNAL(clicked()), SLOT(configChanged()));
+  connect(mWarnOnEnter, SIGNAL(toggled(bool)), SLOT(configChanged()));
   grid->addWidget(mWarnOnEnter, 5, 0);
   whatstr = i18n("If selected, you will be notified when entering an SSL"
                 " enabled site");
   mWarnOnEnter->setWhatsThis( whatstr);
 
   mWarnOnLeave = new QCheckBox(i18n("Warn on &leaving SSL mode"), tabSSL);
-  connect(mWarnOnLeave, SIGNAL(clicked()), SLOT(configChanged()));
+  connect(mWarnOnLeave, SIGNAL(toggled(bool)), SLOT(configChanged()));
   grid->addWidget(mWarnOnLeave, 5, 1);
   whatstr = i18n("If selected, you will be notified when leaving an SSL"
                 " based site.");
   mWarnOnLeave->setWhatsThis( whatstr);
 
   mWarnOnUnencrypted = new QCheckBox(i18n("Warn on sending &unencrypted data"), tabSSL);
-  connect(mWarnOnUnencrypted, SIGNAL(clicked()), SLOT(configChanged()));
+  connect(mWarnOnUnencrypted, SIGNAL(toggled(bool)), SLOT(configChanged()));
   grid->addWidget(mWarnOnUnencrypted, 6, 0);
   whatstr = i18n("If selected, you will be notified before sending"
                 " unencrypted data via a web browser.");
@@ -380,15 +381,14 @@ QString whatstr;
   QFrame *eFrame = new QFrame(tabOSSL);
   QVBoxLayout *egrid = new QVBoxLayout(eFrame);
   mUseEGD = new QCheckBox(i18n("Use EGD"), eFrame);
-  connect(mUseEGD, SIGNAL(clicked()), SLOT(slotUseEGD()));
+  connect(mUseEGD, SIGNAL(toggled(bool)), SLOT(slotUseEGD()));
   mUseEFile = new QCheckBox(i18n("Use entropy file"), eFrame);
-  connect(mUseEFile, SIGNAL(clicked()), SLOT(slotUseEFile()));
+  connect(mUseEFile, SIGNAL(toggled(bool)), SLOT(slotUseEFile()));
   vbox->addWidget(eFrame);
   egrid->addWidget(mUseEGD);
   egrid->addWidget(mUseEFile);
   QFrame *egdframe = new QFrame(tabOSSL);
-  QGridLayout *grid2 = new QGridLayout(egdframe, 2, 2, KDialog::marginHint(),
-                                                       KDialog::spacingHint());
+  QGridLayout *grid2 = new QGridLayout(egdframe);
   mEGDLabel = new QLabel(i18n("Path to EGD:"), egdframe);
   grid2->addWidget(mEGDLabel, 0, 0);
   mEGDPath = new KUrlRequester(egdframe);
@@ -463,8 +463,8 @@ QString whatstr;
   grid->addWidget(yourSSLPass, 5, 5);
 
   grid->addWidget(new KSeparator(Qt::Horizontal, tabYourSSLCert), 6, 0, 1, 6);
-  ySubject = KSSLInfoDialog::certInfoWidget(tabYourSSLCert, QString());
-  yIssuer = KSSLInfoDialog::certInfoWidget(tabYourSSLCert, QString());
+  ySubject = new KSslCertificateBox(tabYourSSLCert);
+  yIssuer = new KSslCertificateBox(tabYourSSLCert);
   grid->addWidget(ySubject, 7, 0, 5, 3);
   grid->addWidget(yIssuer, 7, 3, 5, 3);
   whatstr = i18n("This is the information known about the owner of the certificate.");
@@ -622,8 +622,8 @@ QString whatstr;
       otherSSLRemove->setEnabled(false);
 
   grid->addWidget(new KSeparator(Qt::Horizontal, tabOtherSSLCert), 8, 0, 1, 6);
-  oSubject = KSSLInfoDialog::certInfoWidget(tabOtherSSLCert, QString());
-  oIssuer = KSSLInfoDialog::certInfoWidget(tabOtherSSLCert, QString());
+  oSubject = new KSslCertificateBox(tabOtherSSLCert);
+  oIssuer = new KSslCertificateBox(tabOtherSSLCert);
   grid->addWidget(oSubject, 9, 0, 5, 3 );
   grid->addWidget(oIssuer, 9, 3, 5, 3);
   whatstr = i18n("This is the information known about the owner of the certificate.");
@@ -727,8 +727,8 @@ QString whatstr;
   connect(caSSLRestore, SIGNAL(clicked()), SLOT(slotCARestore()));
   grid->addWidget(caSSLRestore, 2, 7);
 
-  caSubject = KSSLInfoDialog::certInfoWidget(tabSSLCA, QString());
-  caIssuer = KSSLInfoDialog::certInfoWidget(tabSSLCA, QString());
+  caSubject = new KSslCertificateBox(tabSSLCA);
+  caIssuer = new KSslCertificateBox(tabSSLCA);
   grid->addWidget(caSubject, 4, 0, 3, 4 );
   grid->addWidget(caIssuer, 4, 4, 3, 4);
 
@@ -848,16 +848,16 @@ void KCryptoConfig::configChanged()
 
 void KCryptoConfig::load()
 {
-  KConfigGroup cg(config, QByteArray(""));
+  KConfigGroup cg(config, "");
 #ifdef KSSL_HAVE_SSL
   otherCertDelList.clear();
   yourCertDelList.clear();
   authDelList.clear();
   caDelList.clear();
-  cg.changeGroup("Warnings");
-  mWarnOnEnter->setChecked(cg.readEntry("OnEnter", false));
-  mWarnOnLeave->setChecked(cg.readEntry("OnLeave", true));
-  mWarnOnUnencrypted->setChecked(cg.readEntry("OnUnencrypted", true));
+  KConfigGroup cgWarning(config, "Warnings");
+  mWarnOnEnter->setChecked(cgWarning.readEntry("OnEnter", false));
+  mWarnOnLeave->setChecked(cgWarning.readEntry("OnLeave", true));
+  mWarnOnUnencrypted->setChecked(cgWarning.readEntry("OnUnencrypted", true));
 
 #if 0 // NOT IMPLEMENTED IN KDE 2.0
   mWarnOnMixed->setChecked(cg.readEntry("OnMixed", true));
@@ -868,21 +868,21 @@ void KCryptoConfig::load()
   mWarnRevoked->setChecked(cg.readEntry("WarnRevoked", true));
 #endif
 
-  cg.changeGroup("EGD");
+  KConfigGroup cgEGD(config,"EGD");
   slotUseEGD();  // set the defaults
-  if (cg.readEntry("UseEGD", false)) {
+  if (cgEGD.readEntry("UseEGD", false)) {
     mUseEGD->setChecked(true);
     slotUseEGD();
-  } else if (cg.readEntry("UseEFile", false)) {
+  } else if (cgEGD.readEntry("UseEFile", false)) {
     mUseEFile->setChecked(true);
     slotUseEFile();
   }
-  mEGDPath->setPath(cg.readPathEntry("EGDPath"));
+  mEGDPath->setPath(cgEGD.readPathEntry("EGDPath", QString()));
 
 
 #ifdef KSSL_HAVE_SSL
-  cg.changeGroup("OpenSSL");
-  oPath->setUrl(cg.readPathEntry("Path"));
+  KConfigGroup cgSSL( config, "OpenSSL");
+  oPath->setUrl(cgSSL.readPathEntry("Path", QString()));
 #endif
 
   KConfigGroup sslV3(config, "SSLv3");
@@ -897,7 +897,7 @@ void KCryptoConfig::load()
 
   otherSSLBox->clear();
   for (QStringList::Iterator i = groups.begin(); i != groups.end(); ++i) {
-    if ((*i).isEmpty() || *i == "<default>" || *i == "General") continue;
+    if (*i == "General") continue;
     KConfigGroup _cg(policies, *i);
     KSSLCertificate *cert = KSSLCertificate::fromString(_cg.readEntry("Certificate", QString()).toLocal8Bit());
     if (cert) {
@@ -913,7 +913,6 @@ void KCryptoConfig::load()
 
   yourSSLBox->clear();
   for (QStringList::Iterator i = groups.begin(); i != groups.end(); ++i) {
-    if ((*i).isEmpty() || *i == "<default>") continue;
     KConfigGroup _cg(pcerts, *i);
     YourCertItem *j = new YourCertItem(yourSSLBox,
                      _cg.readEntry("PKCS12Base64"),
@@ -946,7 +945,6 @@ void KCryptoConfig::load()
   for (QStringList::Iterator i = groups.begin();
                              i != groups.end();
                              ++i) {
-    if ((*i).isEmpty() || *i == "<default>") continue;
     KConfigGroup _cg(authcfg, *i);
     KSSLCertificateHome::KSSLAuthAction aa = KSSLCertificateHome::AuthDont;
     if (_cg.readEntry("send", false) == true)
@@ -962,12 +960,11 @@ void KCryptoConfig::load()
   }
 
   groups = _signers->list();
-  KConfig sigcfg( "ksslcalist", KConfig::CascadeConfig );
+  KConfig sigcfg( "ksslcalist", KConfig::NoGlobals );
   caList->clear();
   for (QStringList::Iterator i = groups.begin();
                              i != groups.end();
                              ++i) {
-    if ((*i).isEmpty() || *i == "<default>") continue;
     if (!sigcfg.hasGroup(*i)) continue;
     KConfigGroup _cg(&sigcfg, *i);
     if (!_cg.hasKey("x509")) continue;
@@ -990,17 +987,17 @@ void KCryptoConfig::load()
 
 void KCryptoConfig::save()
 {
-  KConfigGroup cg(config, QByteArray(""));
+  KConfigGroup cg(config, "");
 #ifdef KSSL_HAVE_SSL
-  cg.changeGroup("Warnings");
-  cg.writeEntry("OnEnter", mWarnOnEnter->isChecked());
-  cg.writeEntry("OnLeave", mWarnOnLeave->isChecked());
-  cg.writeEntry("OnUnencrypted", mWarnOnUnencrypted->isChecked());
+  KConfigGroup cgWarning(config, "Warnings");
+  cgWarning.writeEntry("OnEnter", mWarnOnEnter->isChecked());
+  cgWarning.writeEntry("OnLeave", mWarnOnLeave->isChecked());
+  cgWarning.writeEntry("OnUnencrypted", mWarnOnUnencrypted->isChecked());
 
-  cg.changeGroup("EGD");
-  cg.writeEntry("UseEGD", mUseEGD->isChecked());
-  cg.writeEntry("UseEFile", mUseEFile->isChecked());
-  cg.writePathEntry("EGDPath", mEGDPath->url().path());
+  KConfigGroup cgEGD(config,"EGD");
+  cgEGD.writeEntry("UseEGD", mUseEGD->isChecked());
+  cgEGD.writeEntry("UseEFile", mUseEFile->isChecked());
+  cgEGD.writePathEntry("EGDPath", mEGDPath->url().path());
 
 #if 0  // NOT IMPLEMENTED IN KDE 2.0
   cg.writeEntry("OnMixed", mWarnOnMixed->isChecked());
@@ -1012,23 +1009,23 @@ void KCryptoConfig::save()
 #endif
 
 #ifdef KSSL_HAVE_SSL
-  cg.changeGroup("OpenSSL");
-  cg.writePathEntry("Path", oPath->url().path());
+  KConfigGroup cgSSL(config,"OpenSSL");
+  cgSSL.writePathEntry("Path", oPath->url().path());
 #endif
 
   int ciphercount = 0;
-  cg.changeGroup("SSLv3");
+  KConfigGroup cgSSLV3(config,"SSLv3");
   CipherItem *item = static_cast<CipherItem *>(SSLv3Box->firstChild());
   while ( item ) {
     if (item->isOn()) {
-      cg.writeEntry(item->configName(), true);
+      cgSSLV3.writeEntry(item->configName(), true);
       ciphercount++;
-    } else cg.writeEntry(item->configName(), false);
+    } else cgSSLV3.writeEntry(item->configName(), false);
 
     item = static_cast<CipherItem *>(item->nextSibling());
   }
 
-  KSSLCertificateCache _cc;
+//###   KSSLCertificateCache _cc;
 
   if (ciphercount == 0)
     KMessageBox::information(this, i18n("If you do not select at least one"
@@ -1038,7 +1035,7 @@ void KCryptoConfig::save()
   for (OtherCertItem *x = otherCertDelList.first(); x != 0; x = otherCertDelList.next()) {
      KSSLX509Map cert(x->configName());
      QString thisCN = cert.getValue("CN");
-     _cc.removeByCN(thisCN);
+//###      _cc.removeByCN(thisCN);
      otherCertDelList.remove(x);
   }
   // Go through the non-deleted ones and save them
@@ -1049,7 +1046,7 @@ void KCryptoConfig::save()
      KSSLX509Map cert(x->configName());
      QString thisCN = cert.getValue("CN");
      QDateTime expires = x->getExpires();
-     _cc.modifyByCN(thisCN, (KSSLCertificateCache::KSSLCertificatePolicy)x->getPolicy(), x->isPermanent(), expires);
+//###      _cc.modifyByCN(thisCN, (KSSLCertificateCache::KSSLCertificatePolicy)x->getPolicy(), x->isPermanent(), expires);
   }
 
   // SSL Personal certificates code
@@ -1099,18 +1096,18 @@ void KCryptoConfig::save()
   if (doGen) genCAList();
 
 
-  cg.changeGroup("Auth");
-  QString whichAuth = cg.readEntry("AuthMethod", "none");
+  KConfigGroup cgAuth(config,"Auth");
+  QString whichAuth = cgAuth.readEntry("AuthMethod", "none");
   if (defCertBG->selected() == defSend)
-    cg.writeEntry("AuthMethod", "send");
+    cgAuth.writeEntry("AuthMethod", "send");
   else if (defCertBG->selected() == defPrompt)
-    cg.writeEntry("AuthMethod", "prompt");
+    cgAuth.writeEntry("AuthMethod", "prompt");
   else
-    cg.writeEntry("AuthMethod", "none");
+    cgAuth.writeEntry("AuthMethod", "none");
 
   if (defCertBox->currentIndex() == 0)
-     cg.writeEntry("DefaultCert", QString());
-  else cg.writeEntry("DefaultCert", defCertBox->currentText());
+     cgAuth.writeEntry("DefaultCert", QString());
+  else cgAuth.writeEntry("DefaultCert", defCertBox->currentText());
 
   for (HostAuthItem *x = authDelList.first(); x != 0; x = authDelList.next()) {
      authcfg->deleteGroup(x->configName());
@@ -1129,7 +1126,7 @@ void KCryptoConfig::save()
         static_cast<HostAuthItem *>(hostAuthList->firstChild());
                                                               x;
              x = static_cast<HostAuthItem *>(x->nextSibling())) {
-     KConfigGroup _cg(authcfg, KResolver::domainToAscii(x->configName()));
+     KConfigGroup _cg(authcfg, KResolver::domainToAscii(x->configName()).constData());
      _cg.writeEntry("certificate", x->getCertName());
      _cg.writeEntry("prompt", (x->getAction() == KSSLCertificateHome::AuthPrompt));
      _cg.writeEntry("send", (x->getAction() == KSSLCertificateHome::AuthSend));
@@ -1340,13 +1337,13 @@ void KCryptoConfig::slotPolicyChanged(int id) {
 OtherCertItem *x = static_cast<OtherCertItem *>(otherSSLBox->selectedItem());
 if (!x) return;
 
-   if (id == policyGroup->id(policyAccept)) {
-     x->setPolicy(KSSLCertificateCache::Accept);
-   } else if (id == policyGroup->id(policyReject)) {
-     x->setPolicy(KSSLCertificateCache::Reject);
-   } else if (id == policyGroup->id(policyPrompt)) {
-     x->setPolicy(KSSLCertificateCache::Prompt);
-   }
+//###    if (id == policyGroup->id(policyAccept)) {
+//###      x->setPolicy(KSSLCertificateCache::Accept);
+//###    } else if (id == policyGroup->id(policyReject)) {
+//###      x->setPolicy(KSSLCertificateCache::Reject);
+//###    } else if (id == policyGroup->id(policyPrompt)) {
+//###      x->setPolicy(KSSLCertificateCache::Prompt);
+//###    }
 
    configChanged();
 }
@@ -1394,7 +1391,7 @@ QString iss = QString();
          QPalette cspl;
          iss = cert->getIssuer();
          cspl = validFrom->palette();
-         if (QDateTime::currentDateTime(Qt::UTC) < cert->getQDTNotBefore()) {
+         if (QDateTime::currentDateTime() < cert->getQDTNotBefore()) {
             cspl.setColor(QPalette::Foreground, QColor(196,33,21));
          } else {
             cspl.setColor(QPalette::Foreground, QColor(42,153,59));
@@ -1402,7 +1399,7 @@ QString iss = QString();
          validFrom->setPalette(cspl);
 
          cspl = validUntil->palette();
-         if (QDateTime::currentDateTime(Qt::UTC) > cert->getQDTNotAfter()) {
+         if (QDateTime::currentDateTime() > cert->getQDTNotAfter()) {
             cspl.setColor(QPalette::Foreground, QColor(196,33,21));
          } else {
             cspl.setColor(QPalette::Foreground, QColor(42,153,59));
@@ -1412,7 +1409,7 @@ QString iss = QString();
          validFrom->setText(cert->getNotBefore());
          validUntil->setText(cert->getNotAfter());
          untilDate->setText(x ? KGlobal::locale()->formatDateTime(x->getExpires())
-                              : KGlobal::locale()->formatDateTime(QDateTime::currentDateTime(Qt::UTC)));
+                              : KGlobal::locale()->formatDateTime(QDateTime::currentDateTime()));
          untilDate->setEnabled(x && !x->isPermanent());
          pHash->setText(cert->getMD5DigestText());
          delete cert;
@@ -1422,17 +1419,17 @@ QString iss = QString();
          pHash->clear();
       }
 
-      switch(x->getPolicy()) {
-      case KSSLCertificateCache::Accept:
-        policyGroup->setButton(policyGroup->id(policyAccept));
-      break;
-      case KSSLCertificateCache::Reject:
-        policyGroup->setButton(policyGroup->id(policyReject));
-      break;
-      case KSSLCertificateCache::Prompt:
-        policyGroup->setButton(policyGroup->id(policyPrompt));
-      break;
-      }
+//###       switch(x->getPolicy()) {
+//###       case KSSLCertificateCache::Accept:
+//###         policyGroup->setButton(policyGroup->id(policyAccept));
+//###       break;
+//###       case KSSLCertificateCache::Reject:
+//###         policyGroup->setButton(policyGroup->id(policyReject));
+//###       break;
+//###       case KSSLCertificateCache::Prompt:
+//###         policyGroup->setButton(policyGroup->id(policyPrompt));
+//###       break;
+//###       }
 
       cachePerm->setChecked(x->isPermanent());
       cacheUntil->setChecked(!x->isPermanent());
@@ -1460,8 +1457,8 @@ QString iss = QString();
    }
 
 
-   oSubject->setValues(x ? x->getSub() : QString());
-   oIssuer->setValues(iss);
+   //### oSubject->setValues(x ? x->getSub() : QString());
+   //### oIssuer->setValues(iss);
 
 }
 
@@ -1642,12 +1639,12 @@ QString iss;
 
    // update the info
    iss = pkcs->getCertificate()->getIssuer();
-   ySubject->setValues(x->getName());
-   yIssuer->setValues(iss);
+   //### ySubject->setValues(x->getName());
+   //### yIssuer->setValues(iss);
    QPalette cspl;
    KSSLCertificate *cert = pkcs->getCertificate();
    cspl = yValidFrom->palette();
-   if (QDateTime::currentDateTime(Qt::UTC) < cert->getQDTNotBefore()) {
+   if (QDateTime::currentDateTime() < cert->getQDTNotBefore()) {
 	   cspl.setColor(QPalette::Foreground, QColor(196,33,21));
    } else {
 	   cspl.setColor(QPalette::Foreground, QColor(42,153,59));
@@ -1655,7 +1652,7 @@ QString iss;
    yValidFrom->setPalette(cspl);
 
    cspl = yValidUntil->palette();
-   if (QDateTime::currentDateTime(Qt::UTC) > cert->getQDTNotAfter()) {
+   if (QDateTime::currentDateTime() > cert->getQDTNotAfter()) {
 	   cspl.setColor(QPalette::Foreground, QColor(196,33,21));
    } else {
 	   cspl.setColor(QPalette::Foreground, QColor(42,153,59));
@@ -1687,7 +1684,7 @@ QString iss;
          KSSLCertificate *cert = pkcs->getCertificate();
          iss = cert->getIssuer();
          cspl = yValidFrom->palette();
-         if (QDateTime::currentDateTime(Qt::UTC) < cert->getQDTNotBefore()) {
+         if (QDateTime::currentDateTime() < cert->getQDTNotBefore()) {
             cspl.setColor(QPalette::Foreground, QColor(196,33,21));
          } else {
             cspl.setColor(QPalette::Foreground, QColor(42,153,59));
@@ -1695,7 +1692,7 @@ QString iss;
          yValidFrom->setPalette(cspl);
 
          cspl = yValidUntil->palette();
-         if (QDateTime::currentDateTime(Qt::UTC) > cert->getQDTNotAfter()) {
+         if (QDateTime::currentDateTime() > cert->getQDTNotAfter()) {
             cspl.setColor(QPalette::Foreground, QColor(196,33,21));
          } else {
             cspl.setColor(QPalette::Foreground, QColor(42,153,59));
@@ -1714,8 +1711,8 @@ QString iss;
       yHash->clear();
    }
 
-   ySubject->setValues(x ? x->getName() : QString());
-   yIssuer->setValues(iss);
+   //### ySubject->setValues(x ? x->getName() : QString());
+   //### yIssuer->setValues(iss);
 }
 
 
@@ -1872,8 +1869,7 @@ void KCryptoConfig::slotCAImport() {
 			char *cr;
 			cr = new char[qf.size()+1];
 			qf.read(cr, qf.size());
-			QByteArray qba;
-			qba.duplicate(cr, qf.size());
+			QByteArray qba(cr, qf.size());
 			certtext = KCodecs::base64Encode(qba);
 			delete [] cr;
 		}
@@ -1976,12 +1972,11 @@ void KCryptoConfig::slotCARestore() {
 
 
   QStringList groups = _signers->list();
-  KConfig sigcfg( "ksslcalist", KConfig::CascadeConfig );
+  KConfig sigcfg( "ksslcalist", KConfig::NoGlobals );
 
   for (QStringList::Iterator i = groups.begin();
                              i != groups.end();
                              ++i) {
-    if ((*i).isEmpty() || *i == "<default>") continue;
     if (!sigcfg.hasGroup(*i)) continue;
     KConfigGroup _cg(&sigcfg, *i);
     if (!_cg.hasKey("x509")) continue;
@@ -2003,10 +1998,10 @@ void KCryptoConfig::slotCAItemChanged() {
  CAItem *x = static_cast<CAItem *>(caList->selectedItem());
  if (x) {
     caSSLRemove->setEnabled(true);
-    caSubject->setValues(x->getName());
+    //### caSubject->setValues(x->getName());
     KSSLCertificate *cert = KSSLCertificate::fromString(x->getCert().toLocal8Bit());
     if (!cert) {
-       caIssuer->setValues(QString());
+       //### caIssuer->setValues(QString());
        caSite->setEnabled(false);
        caEmail->setEnabled(false);
        caCode->setEnabled(false);
@@ -2021,7 +2016,7 @@ void KCryptoConfig::slotCAItemChanged() {
        caSite->setChecked(x->getSite());
        caEmail->setChecked(x->getEmail());
        caCode->setChecked(x->getCode());
-       caIssuer->setValues(cert->getIssuer());
+       //### caIssuer->setValues(cert->getIssuer());
        cHash->setText(cert->getMD5DigestText());
        delete cert;
     }
@@ -2030,8 +2025,8 @@ void KCryptoConfig::slotCAItemChanged() {
     caSite->setEnabled(false);
     caEmail->setEnabled(false);
     caCode->setEnabled(false);
-    caSubject->setValues(QString());
-    caIssuer->setValues(QString());
+    caSubject->clear();
+    caIssuer->clear();
     cHash->clear();
  }
 }
